c $Header: /homes/jacob/foam-cvs/ccm2lnd/ccm2lnd_drv.F,v 1.1.1.1 2002/03/20 22:21:09 jacob Exp $
#include "params.h"
c
c  This program calls the FOAM version of the CCM2 land model.
c  
c  This code evolved from CCM2's srfint.F
c
c This subroutine implements a bucket hydrology scheme to replace the 
c prescribed surface wetness and snowdepth used in CCM2.
c
c Excess surface moisture or snowdepth (exws) is handled
c by FOAM's river model.
c  
C---------------------srfint.F code history----------------------------
C
C Original version:  CCM1
C Standardized:      L. Buja, June 1992
C Reviewed:          B. Briegleb, J. Hack, August 1992
C
C-------------------ccm2lnd_drv.F code history------------------------
c  Initial version of FOAM's land model:  R. Jacob,Oct, 1995.
c  Refactoring:  R. Jacob, Jan, 2000
c  Switch snow albedo formula to Genesis version,  R. Jacob, S. Vavrus, Jan, 2001
c  Lower Genesis snow albedoes by 0.1.  Restrict zenith angle effect
c  factor to be less the 1.0.  Treat land ice snow albedo differently.
c           S. Vavrus, Aug, 2001
c
c--------------------------------------------------------------------
      subroutine ccm2lnd_drv(
c  atmosphere input variables
     $ psm1  ,pmidm1, ubot, vbot, tbot, qbot,thbot,
     $ zbot,srfrad,
c  surface input
     $ snowing,ltheat,oprec,doalb,coszrs,
c  by common block: ioro,lwupfh,sst
c surface input/output
     $ ts,
c  by common block: tssub
c hydrology output
     $ snwmlt,exws,
c  by common block: sndpth,ws
c surface output
     $ fbt,oalbs,oalbl,oalbsd,oalbld,
c  by common block:  delts
C overlap grid output
     $ qflx,lhflx,shflx,taux,tauy)


      implicit none

c------------------------------------------------
#include "pmgrid.com"
c------------------------------------------------
#include "comctl.com"
c------------------------------------------------
#include "pspect.com"
c------------------------------------------------
#include "comtim.com"
c------------------------------------------------
#include "comtsc.com"
c------------------------------------------------
#include "comparll.com"
c------------------------------------------------
#include "pcoupled.com"
c------------------------------------------------
#include "pccm2lnd.com"
c------------------------------------------------
#include "pradalb.com"
c------------------------------------------------
#include "pcplrstr.com"
c------------------------------------------------

c atmosphere input
      real psm1(p_glon,p_glat) ! surface pressure
      real pmidm1(p_glon,p_glat) ! bottom level pressure
      real ubot(p_glon,p_glat) ! bottom level u wind
      real vbot(p_glon,p_glat) ! bottom level v wind
      real tbot(p_glon,p_glat) ! bottom level temp
      real qbot(p_glon,p_glat) ! bottom level specific humidity
      real thbot(p_glon,p_glat) ! bottom level potential temp
      real zbot(p_glon,p_glat) ! bottom level height above surface
      real srfrad(p_glon,p_glat) ! total downward radiation at surface.

c  surface input
      logical snowing(uwlon,uwlat)  !  true if its snowing
      real ltheat(uwlon,uwlat)    ! latent heat
      real oprec(uwlon,uwlat)   ! Atmos. precip fields on the ocean grid
      real coszrs(uwlon,uwlat)  !cosine of the zenith angle
      logical doalb             ! update albedos flag

c input/output
      real ts(uwlon,uwlat)    ! non-adjusted srf/sub temps
c hydrology output
      real snwmlt(uwlon,uwlat)     ! heat flow between ocean and sea ice
      real exws(uwlon,uwlat)     ! excess surface moisture

c  surface output
      real fbt(uwlon,uwlat)     ! heat flow between ocean and sea ice
      real oalbs(uwlon,uwlat)   ! Srf alb for direct rad  0.2-0.7 micron
      real oalbl(uwlon,uwlat)   ! Srf alb for direct rad  0.7-5.0 
      real oalbsd(uwlon,uwlat)  ! Srf alb for diffuse rad 0.2-0.7
      real oalbld(uwlon,uwlat)  ! Srf alb for diffuse rad 0.7-5.0 

c overlap grid output
      real qflx(uwlon,uwlat,maxoverlaps)        ! water flux
      real lhflx(uwlon,uwlat,maxoverlaps)       ! latent heat flux
      real shflx(uwlon,uwlat,maxoverlaps)       ! sensible heat flux
      real taux(uwlon,uwlat,maxoverlaps)        ! x wind stress
      real tauy(uwlon,uwlat,maxoverlaps)	! y wind stress



c  local variables
      integer isrftyp(uwlon)  ! surface "soil" type
      real wet(uwlon)     ! surface wetness (fraction)
      real z0(uwlon)      ! surface roughness lenght for fluxes

      real alpha(uwlon)   ! ts term from top layer linear equ
      real beta(uwlon)    ! rhs term from top layer linear equ
      real gamma(uwlon)   ! t2 term from top layer linear equ
      real eexc(uwlon)    ! excess thermal energy after snow melt
      real tpthck(uwlon)  ! top layer geometrical thickness
      real tsbsf(uwlon,plevmx)    ! non-adjusted srf/sub temps
      real oldmst(uwlon)  ! old ws value
      logical logic(uwlon)	! surface wetness logical
      real tsin(uwlon)    ! save ts
      real ofnt(uwlon)    ! net surface flux for input conditions
      real odfntdt(uwlon) ! derivative of net surface flux

c albedo local variables
      real tlslo	    ! low threshold temp for snow albedo
      real tlshi	    ! high threshold temp for snow albedo
      real snwsh,           ! high temp snow albedo for 0.2-0.7 micro-meters
     $     snwlh,           ! high temp snow albedo for 0.7-5.0 micro-meters
     $     snwll,           ! low temp snow albedo for 0.7-5.0 micro-meters
     $     snwsl            ! low temp snow albedo for 0.2-0.7 micro-meters
c GENESIS snow abledo values
c SJV:  below values are: original - 0.1.
c   a change which warms up the land surface.
      parameter (snwsl  = 0.80,
     $ 		 snwsh  = 0.45,
     $           snwll  = 0.50,
     $           snwlh  = 0.25)

      real salbs(uwlon),   ! Snow alb for direct rad  0.2-0.7 micro-ms
     $     salbl(uwlon),   ! Snow alb for direct rad  0.7-5.0 micro-ms
     $     salbsd(uwlon),  ! Snow alb for diffuse rad  0.2-0.7 micro-ms
     $     salbld(uwlon),  ! Snow alb for diffuse rad  0.7-5.0 micro-ms
     $     rs,             ! Empirical fact strng znth angl dependence
     $     rw,             ! Empirical fact weak  znth angl dependence
     $     frsnow(uwlon),  ! Horizontal fraction of snow cover
     $     snwhgt,         ! Physical snow height
     $     rghsnw          ! Roughness for horizontal snow cover fractn


c overlap grid local variables
      real fnt(uwlon,uwlat,maxoverlaps)  ! net srf flx tmp (ts)
      real dfntdt(uwlon,uwlat,maxoverlaps)  ! net srf flx tmp deriv (ts)

      real lhf(uwlon,uwlat,maxoverlaps) ! initial latent heat flux
      real shf(uwlon,uwlat,maxoverlaps) ! initial sensible heat flux
      real dlhf(uwlon,uwlat,maxoverlaps)        ! ts deriv for latent heat flux
      real dshf(uwlon,uwlat,maxoverlaps)        ! ts deriv for sensible heat flu


c physical variables
      real rhoice     ! density of ice
      real rhoh2o     ! density of water
      real snwmax     ! maximum snow depth in liquid water equiv

c externals
      external posrfflx,posrftsb
      real cvmgt
#if ( defined SGI) || ( defined INTEL)
      external cvmgt
#endif

      integer indx(uwlon)       ! Longitude index array (land)
      integer npts              ! Number of land points
      integer ii              ! Longitude indices 


      integer n,i,j,k,l,m
      real emelt,mflux,tmp,wsnorm,zfac
      real snrun
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Begin land model calculations
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      
      rhoh2o= 1.e3
      rhoice = .92*rhoh2o
      snwmax = 1.0

C  See if theres any land points here to do
      npts = 0
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
         if(ioro(i,j) .eq. 1) then
          npts =npts +1
         endif
       enddo
      enddo
      enddo
      if(npts .eq. 0) return


      do n=1,2
      do j=plats(n),plate(n)

       npts = 0
       do i=plons,plone
         if(ioro(i,j) .eq. 1) then
          npts =npts +1
          indx(npts)=i
         endif
       enddo


      do ii=1,npts
          i=indx(ii)
c  Update surface type
          isrftyp(i) = ovegtyp(i,j)

C set the roughness length everywhere
          z0(i) = orghnss(i,j)

       enddo

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Hydrology Model:  Part 1 of 3
C----------------------------------------
C  distribute precip onto either soil moisture or
C  snow depth.  Calculate wetness factor for
C  latent heat flux calculation
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      do ii=1,npts
          i=indx(ii)
c
c  update surface wetness/snowdepth with precip fields
c

           if (.not.snowing(i,j)) then
               ws(i,j) = ws(i,j) + oprec(i,j) * dtime
c
c  if below freezing, add precip to snow depth (liquid water equiv)
c
           else if (snowing(i,j)) then
               sndpth(i,j) = sndpth(i,j) + oprec(i,j) * dtime
           endif

c
c compute runoff from rain if soil moisture has overflowed
c the bucket depth
c
         if(ws(i,j) .gt. wssat) then
           exws(i,j)= ws(i,j) - wssat
           ws(i,j) = wssat
         endif


c modify latent heat for snow covered, sea ice or land ice surfaces

         if( (sndpth(i,j) .gt. 0.)  .or. (isrftyp(i) .eq. 3)) then
           ltheat(i,j) = latvap + latice
         endif

c  Update surface wetness fraction
c  First be sure the ocean points are zero

        logic(i) = (ioro(i,j) .eq. 1) .and. (sndpth(i,j) .le. 0.)

c use 1.e-6 to avoid underflow error in exp calculation
        if ( ws(i,j) .lt. 1.e-6) then
           wet(i) = cvmgt(0.0,1.,logic(i))
        else if (ws(i,j) .eq. wssat) then
           wet(i) = cvmgt(1.0, 1. , logic(i))
        else
           wsnorm = (1. - ws(i,j)/wssat)**2

           wet(i) = cvmgt( amax1(0.0, 
     $        amin1((exp( wsnorm/(wsnorm -1.)) )**1.25, 1.))
     $                  ,1.,logic(i))
        endif

      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End of hydrology model part one
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



C Calculate fluxes of heat,water, and momentum between
C land and atmosphere.

#ifdef PROFILE
      call mark_event(22,1)
#endif 

      call posrfflx(indx,npts,j,z0,psm1,pmidm1,ubot,
     $       vbot, tbot, qbot, thbot, zbot,
     $       srfrad, lwuphf(1,j),ts(1,j),ltheat(1,j), wet, 
     $       fnt,dfntdt, shf, dshf, lhf, dlhf,
     $       taux, tauy)


#ifdef PROFILE
      call mark_event(22,0)
#endif



c
c  contruct the fnt dfntdt fields for the surface resolution
c
      do ii=1,npts
          i=indx(ii)
          ofnt(i) = 0.
          odfntdt(i) = 0.

          do k=1,overlaps(j,i)

            ofnt(i) = ofnt(i) + fnt(i,j,k)*cweight(j,i,k)
   
            odfntdt(i) = odfntdt(i) + dfntdt(i,j,k)*cweight(j,i,k)
          enddo
          ofnt(i) = ofnt(i)/uwarea(j)
          odfntdt(i) = odfntdt(i)/uwarea(j)
      enddo


c
c use diffusion equation in sub-surface along with
c backward implicit scheme to forecast surface temp
c
      do ii=1,npts
          i=indx(ii)
           tsin(i) = ts(i,j)
        do k=1,plevmx
           tsbsf(i,k) = tssub(i,j,k)
        enddo
       enddo

#ifdef PROFILE
      call mark_event(21,1)
#endif 

      call posrftsb(indx,npts,isrftyp,ofnt,odfntdt,
     $     sndpth(1,j),
     $     sst(1,j),tsbsf,alpha,beta,
     $     gamma,tpthck,fbt(1,j))


#ifdef PROFILE
      call mark_event(21,0)
#endif

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Hydrology Model:  Part 2 of 3
C----------------------------------------
C  adjust predicted suface temp down if it can melt snow
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c
c Melt snow with the warm ground and cool off the ground
c
c if snow cover present and forecasted temp is above
c melting temp, then energy available for snow melt
c

      do ii=1,npts
          i=indx(ii)
        if(sndpth(i,j) .gt. 0. .and. (tsbsf(i,1) .gt. tmelt)) then
         emelt = -gamma(i) + beta(i) -(alpha(i)*tmelt)
         snwmlt(i,j) = (emelt*dtime*tpthck(i))/(latice*rhoice)
         eexc(i) = emelt - sndpth(i,j)*latice*rhoice/
     $             (tpthck(i)*dtime)
         tsbsf(i,1) = tmelt
        endif
      enddo

c
c Don't melt more snow than available
c
      do ii=1,npts
          i=indx(ii)
       if(snwmlt(i,j) .gt. sndpth(i,j)) then
         snwmlt(i,j) = sndpth(i,j)
         tsbsf(i,1) = tmelt + eexc(i)/alpha(i)
       endif
      enddo

c  update snow cover and surface moisture with snow melt
c     
      do ii=1,npts
          i=indx(ii)
          sndpth(i,j) = sndpth(i,j) - snwmlt(i,j)
          ws(i,j) = ws(i,j) + snwmlt(i,j)

c  this is the last moisture mod before evap. so save it.
          oldmst(i) = ws(i,j) + sndpth(i,j)
      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End of hydrology model part 2 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c
c  modification to update surface temperatures:
c if land (permanent) ice (3) and
c if temps of any layer is over melting point,
c reset temperature to melting point.

      do ii=1,npts
          i=indx(ii)
        if(isrftyp(i) .eq. 3) then
         do k=1,plevmx
           tsbsf(i,k) = amin1(tsbsf(i,k),tmelt)
         enddo
        endif
      enddo


c
c  update surface temps over land surfaces only
c    
      do ii=1,npts
          i=indx(ii)
          ts(i,j) = tsbsf(i,1)
      enddo

c
c update subsurface temps
c
      do k=1,plevmx
      do ii=1,npts
          i=indx(ii)
          tssub(i,j,k) = tsbsf(i,k)
      enddo
      enddo


c
c  find delts over land 
c  
      do ii=1,npts
          i=indx(ii)
         delts(i,j) = ts(i,j) - tsin(i)
      enddo

c
c Define latent and sensible heat flux on the overlap grid
c
      do ii=1,npts
          i=indx(ii)
       do k=1,overlaps(j,i)
          lhflx(i,j,k) = lhf(i,j,k) + dlhf(i,j,k)*delts(i,j)
          qflx(i,j,k) = lhflx(i,j,k)/ltheat(i,j)
          shflx(i,j,k) = shf(i,j,k) + dshf(i,j,k)*delts(i,j)
       enddo

      enddo

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Hydrology Model:  Part 3 of 3
C----------------------------------------
C  evaporate water and/or sublimate snow
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c
c evaporate water from wet land and snow covered surfaces
c
      do ii=1,npts
          i=indx(ii)
c on land only

c first find the total moisture flux in this grid cell
c qflx is in kg/m^2 s
         tmp = 0.
         mflux = 0.
         do k=1,overlaps(j,i)
           tmp = tmp + qflx(i,j,k)*cweight(j,i,k)
         enddo
c     now find the total moisture evaporated in meters
crlj There is a factor of rhoh2o missing from these qflx caculations.
crlj  Otherwise the units don't work.
         mflux = tmp*dtime/(uwarea(j)*rhoh2o)

          if(sndpth(i,j) .gt. 0.) then
c first evaporate the snow.
            sndpth(i,j) = sndpth(i,j) - mflux
c if evaporation more than available snow, reduce surface moisture
            if(sndpth(i,j) .lt. 0.) then
c   note:  ws may be driven below zero here.  Thats intended.
              ws(i,j) = ws(i,j) + sndpth(i,j)
              sndpth(i,j) = 0.0
            endif

          else
c evaporate the surface moisture
            ws(i,j) = ws(i,j) - mflux
          endif

c if more evaporation than available surface moisture, set surface
c moisture to zero and reduce evaporative loss accordingly.
c maximum possible evap is a function of the total ws and sndpth
c NOTE:  this needs to be lt and not le because otherwise if
c  ws started out as 0 but sndpth was positive and some was
c  lost do to sublimation, le would incorrectly reset the qflx to oldmst.
c  if sublimation is greater then sndpth, ws will be set to negative and
c  this condition will be invoked
c  

          if(ws(i,j) .lt. 0.0) then
             ws(i,j) = 0.0
             do k=1,overlaps(j,i)
               qflx(i,j,k) = oldmst(i)*rhoh2o/dtime
c        change lhflx to reflect changes in qflx
	       lhflx(i,j,k) = qflx(i,j,k)*ltheat(i,j)
             enddo
          endif
      enddo

c
c if bucket moisture is still above maximum after
c  evaporation/snowmelt, add excess to exws.
c
c also add snowdepth above preset maximum to exws to mimic ice flow
c off the Antractic/Greenland ice sheets and prevent unbounded
c growth in snow depth.
      do ii=1,npts
          i=indx(ii)
	  snrun = 0.
	  tmp = amax1(ws(i,j)-wssat,0.)
	  ws(i,j) = ws(i,j) - tmp

 	   if (sndpth(i,j) .gt. snwmax) then
	      snrun = sndpth(i,j) - snwmax
	      sndpth(i,j) = snwmax
           endif
	   exws(i,j) = exws(i,j) + tmp + snrun
      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End of hydrology model part 3 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c
c  Update albedos
c
      if(doalb) then
c New low and high albedo thresholds set here
       tlslo = tmelt -5.
       tlshi = tmelt

       do ii=1,npts
        i=indx(ii)
C
C Use empirical factors to adjust surface albedos for zenith angle
C effects, distinquishing between strong and weakly dependent surfaces:
C
        if(coszrs(i,j).gt.0.0) then
         rs = 1.4/(1. + .8*coszrs(i,j))
         rw = 1.1/(1. + .2*coszrs(i,j))
c SJV: try eliminating albedo amplification when zenith angle is high
	 rs = amin1(rs,1.0)
         rw = amin1(rw,1.0)

         oalbs(i,j)  = oalbvss(i,j)*ofrctst(i,j)*rs +
     $              oalbvsw(i,j)*(1. - ofrctst(i,j))*rw
         oalbl(i,j)  = oalbnis(i,j)*ofrctst(i,j)*rs +
     $              oalbniw(i,j)*(1. - ofrctst(i,j))*rw
         oalbsd(i,j) = oalbvss(i,j)*ofrctst(i,j) +
     $              oalbvsw(i,j)*(1. - ofrctst(i,j))
         oalbld(i,j) = oalbnis(i,j)*ofrctst(i,j) +
     $              oalbniw(i,j)*(1. - ofrctst(i,j))

        endif

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Albedo adjustment for snowcover
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C
C
         if( sndpth(i,j).gt.0. .and. coszrs(i,j).gt.0. ) then

C  set visible snow albedos
	  zfac = amax1(0., 1.5/(1. + 4. * coszrs(i,j)) - 0.5)

	  tmp = (snwsh*(tssub(i,j,1)-tlslo) + snwsl*(tlshi -tssub(i,j,1)))
     $     / (tlshi - tlslo)
	  tmp = amin1(snwsl, amax1(snwsh, tmp))
	  salbs(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
          salbsd(i) = tmp

C  set near-infrared (longwave) snow albedos
	  tmp = (snwlh*(tssub(i,j,1)-tlslo) + snwll*(tlshi -tssub(i,j,1)))
     $     / (tlshi - tlslo)
	  tmp = amin1(snwll, amax1(snwlh, tmp))
	  salbl(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
          salbld(i) = tmp

c SJV: alter snow albedos over land ice (consistent with GENESIS formula)
           if (isrftyp(i) .eq. 3) then
C  set visible snow albedos
             tmp = (0.7*(tssub(i,j,1)-tlslo) + 0.9*(tlshi -
     $       tssub(i,j,1))) / (tlshi - tlslo)
             tmp = amin1(0.9, amax1(0.7, tmp))
             salbs(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
             salbsd(i) = tmp
C  set near-infrared (longwave) snow albedos
             tmp = (0.5*(tssub(i,j,1)-tlslo) + 0.7*(tlshi -
     $       tssub(i,j,1))) / (tlshi - tlslo)
             tmp = amin1(0.7, amax1(0.5, tmp))
             salbl(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
             salbld(i) = tmp
           endif


         endif
C
C Compute fraction of horizonal surface covered with snow:
C
         if(sndpth(i,j) .le. 0.) then
            snwhgt    = 0.0
            rghsnw    = 0.0
            frsnow(i) = 0.0
         else
C Change snow fraction to use the correct
C snow equivalent depth factor and also
C use total roughness.  RLJ, RG
C           snwhgt    = 20. * sndpth(i,j)
C           rghsnw    = amax1(orghnss(i,j),0.25)
C           frsnow(i) = snwhgt/(rghsnw + snwhgt)

            snwhgt    = snwedp * sndpth(i,j)
            rghsnw    = orghnss(i,j)
            frsnow(i) = snwhgt/(rghsnw + snwhgt)
         end if
C
C  Now modify albedoes for snowcover
C
         if( sndpth(i,j).gt.0. .and. coszrs(i,j).gt.0. ) then
          oalbs(i,j)  = oalbs(i,j) *(1.-frsnow(i)) +
     $      salbs(i) *frsnow(i)
          oalbl(i,j)  = oalbl(i,j) *(1.-frsnow(i)) +
     $      salbl(i) *frsnow(i)
          oalbsd(i,j) = oalbsd(i,j)*(1.-frsnow(i)) +
     $      salbsd(i)*frsnow(i)
          oalbld(i,j) = oalbld(i,j)*(1.-frsnow(i)) +
     $      salbld(i)*frsnow(i)
         endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C End of albedo adjustment for snowcover.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

       enddo
      endif

C end of loop over latitude
      enddo
C end of loop over hemispheres
      enddo

      return
      end
