#include "params.h"
      subroutine posrfflx(indx,npts,jlat,z0, psm1    ,pmidm1  ,ubot ,
     $                  vbot    ,tbot    ,qbot    ,thbot   ,zbot    ,
     $                  srfrad  ,lwuphf  ,ts      ,ltheat  ,wet     ,
     $                  fnt     ,dfntdt  ,shf     ,dshf    ,lhf     ,
#ifdef FOAM_LPJ
     $                  dlhf    ,taux    ,tauy    ,fbare)
#else
     $                  dlhf    ,taux    ,tauy )
#endif
c-----------------------------------------------------------------------
c
c compute surface fluxes, flux temperature derivatives, and exchange
c coefficiant for momentum,heat, and neutral conditions.
c
c computes surface fluxes over all surfaces.
c
c  Hacked for the coupled model by Robert Jacob
c-----------------------------------------------------------------------
      implicit none
c-----------------------------------------------------------------------
#include "pmgrid.com"
c-----------------------------------------------------------------------
#include "pagrid.com"
c-----------------------------------------------------------------------
#include "comtsc.com"
c-----------------------------------------------------------------------
#include "crdalb.com"
c-----------------------------------------------------------------------
#include "pcoupled.com"
c-----------------------------------------------------------------------

c
c input arguments
c
c
      integer indx(uwlon)       ! Longitude index array (land)
      integer npts              ! Number of land points
      integer ii              ! Longitude indices 
      integer jlat		! latitude

      real psm1(p_glon,p_glat),        ! surface pressure
     $     z0(uwlon),          ! surface roughness length for heat
     $     pmidm1(p_glon,p_glat), ! level pressures
     $     ubot(p_glon,p_glat),        ! bottom level u wind
     $     vbot(p_glon,p_glat),        ! bottom level v wind
     $     tbot(p_glon,p_glat),        ! bottom level temperature
     $     qbot(p_glon,p_glat),        ! bottom level specific humidity
     $     thbot(p_glon,p_glat),       ! bottom level potential temperature
     $     zbot(p_glon,p_glat),        ! bottom level height above surface
     $     srfrad(p_glon,p_glat),      ! surface radiative heat flux     
     $     ts(uwlon),          ! surface temp
     $     ltheat(uwlon),      ! latent heat for surface
     $     lwuphf(uwlon),      ! latent heat for surface
     $     wet(uwlon)          ! surface wetness (fraction)
#ifdef FOAM_LPJ
      real fbare(uwlon)        ! bareground fraction for sfc evap-lpj
#endif
c
c output arguments
c
      real fnt(uwlon,uwlat,maxoverlaps),         ! net surface flux for input conditions
     $     dfntdt(uwlon,uwlat,maxoverlaps),      ! net srf flx tmp (ts) derivative 
     $     shf(uwlon,uwlat,maxoverlaps),         ! initial sensible heat flux
     $     dshf(uwlon,uwlat,maxoverlaps),        ! ts derivative for sensible heat flux
     $     lhf(uwlon,uwlat,maxoverlaps),         ! initial latent heat flux
     $     dlhf(uwlon,uwlat,maxoverlaps),        ! ts derivative for latent heat flux
     $     taux(uwlon,uwlat,maxoverlaps),        ! x surface stress (N)
     $     tauy(uwlon,uwlat,maxoverlaps)        ! y surface stress (N)
c
c local workspace
c
      real zqs(uwlon,maxoverlaps),         ! surface saturation specific humidity
     $     ts2(uwlon),         ! ts squared
     $     esk(uwlon)          ! saturation vapor pressure
c
      real zb(uwlon,maxoverlaps),          ! RO*CH*CP*|V|
     $     zd(uwlon,maxoverlaps),          ! RO*CH*DW*L*|V|
     $     zbu(uwlon,maxoverlaps),         ! RO*CD*CP*|V|
     $     vmag(p_glon,p_glat)         ! surface wind magnitude
c
      real dfhdts(uwlon,maxoverlaps),      ! derivative of Fh w.r.t. Ts
     $     dridts(uwlon,maxoverlaps),      ! d(Ri0)/d(Ts)
     $     du(uwlon,maxoverlaps),          ! denominator for unstable coefficients
     $     fm(uwlon,maxoverlaps),          ! stability function for cm
     $     fmu,                ! unstable  function for cm
     $     fh(uwlon,maxoverlaps),          ! stability function for ch
     $     fhu,                ! unstable  function for ch
     $     ri0(uwlon,maxoverlaps),         ! surface layer Richardson number
     $     zz0(uwlon,maxoverlaps),         ! (zbot+z0) / z0
     $     denom,              !
     $     ddudts,             !
     $     dchdts,             !
     $     dqsdt               !
      real cm(uwlon,maxoverlaps)      ! sfc exchange coeff for mom
      real ch(uwlon,maxoverlaps)      ! sfc exchange coeff for heat
      real cn(uwlon,maxoverlaps)      ! sfc neutral exchange coeff 

c
      real rh2o,ltmp,omeps,tmp
      integer j,l,m,ii,jj,k,n,i
      integer rank,istat,shape(2),dspdata

c-----------------------------------------------------------------------
#include "eslookup.com"
c-----------------------------------------------------------------------
      omeps  = 1.0 - epsilo
c

      do ii=1,npts
        i=indx(ii)

      do k=1,overlaps(jlat,i)
	 l = ctoulon(jlat,i,k)
	 m = ctoulat(jlat,i,k)

         vmag(l,m) = amax1(1.0,sqrt(ubot(l,m)**2+vbot(l,m)**2))
c
c calculate bulk richardson number
c
         ri0(i,k)  = gravit * zbot(l,m) * (thbot(l,m)-ts(i)) /
     $	 		(thbot(l,m)*vmag(l,m)**2)
c
c calculate neutral drag coefficients
c
         zz0(i,k)  = (zbot(l,m) + z0(i)) / z0(i)
	 ltmp = alog(zz0(i,k))
         cn(i,k)   = (xkar / alog(zz0(i,k)))**2
c
c unstable branch (ri < 0)
c
         du(i,k)   = 1. / (1. + 75. * cn(i,k)  
     $              * sqrt(zz0(i,k)*abs(ri0(i,k))))
         fmu     = 1. - 10. * ri0(i,k) * du(i,k)
         fhu     = 1. - 15. * ri0(i,k) * du(i,k)
c
c stable branch (ri >= 0)
c
         fm(i,k)   = 1.0 / (1.0 + 10.0*ri0(i,k) * 
     $	 (1.0 + 8.0*ri0(i,k)))
         fh(i,k)   = fm(i,k)
c
c merge stable and unstable values
c
         if( ri0(i,k) .lt. 0. ) then
           fm(i,k) = fmu
           fh(i,k) = fhu
         endif
         cm(i,k)   = cn(i,k) * fm(i,k)
         ch(i,k)   = cn(i,k) * fh(i,k)
c
c (dw=modulated evaporability)
c (drag coefficient for momentum => cd;
c for moisture => ch)
c
         tmp     = vmag(l,m)*pmidm1(l,m)/(rair*tbot(l,m))
         zbu(i,k)  = tmp*cm(i,k)
         zb(i,k)   = tmp*cpair*ch(i,k)
         zd(i,k)   = tmp*ltheat(i)*ch(i,k)*wet(i)
c
c surface stress
c
         taux(i,jlat,k) = -zbu(i,k)*ubot(l,m)
         tauy(i,jlat,k) = -zbu(i,k)*vbot(l,m)
c
      enddo
      enddo

#ifdef DIAGNOSTICS
      write(6,*) 'done with 100 loops'
#endif


c
c compute net surface flux (positive for downwards flux)
c at the current surface temperature:
c
      do ii=1,npts
        i=indx(ii)
      do k=1,overlaps(jlat,i)

	 l = ctoulon(jlat,i,k)
	 m = ctoulat(jlat,i,k)
c
         ts2(i)    = ts(i)*ts(i)
c
c interpolate saturation vapor pressure for temp. zts
c call establ(esk(i),zts(i))
c
         tmp       = tlim(ts(i))
         esk(i)    = estbl4(tmp)
         denom     = psm1(l,m) - 
     $	             omeps*esk(i)
         zqs(i,k)    = epsilo*esk(i)/denom
c
c define net surface flux components at current surface temperatu
c (positive downwards into surface)...
c
         shf(i,jlat,k)    = zb(i,k)*(ts(i)-
     $	 	thbot(l,m))
         lhf(i,jlat,k)    = zd(i,k)*(zqs(i,k)-
     $   	 qbot(l,m))

c net surface flux at current surface temperature:
c (positive downwards into surface)...
c
#ifdef FOAM_LPJ
         fnt(i,jlat,k)    = srfrad(l,m)
     $   - lwuphf(i) - shf(i,jlat,k) - lhf(i,jlat,k)*
     $   fbare(i)
#else
         fnt(i,jlat,k)    = srfrad(l,m)
     $   - lwuphf(i) - shf(i,jlat,k) - lhf(i,jlat,k)
#endif

c
c
c compute net surface flux surface temperature derivative
c at the current surface temperature:
c
c
c derivative of richardson number w.r.t. ts
c
         if(ri0(i,k).ge.rimax) then
            dridts(i,k) = 0.
         else
            dridts(i,k) = -gravit*
     $	    zbot(l,m)/
     $     (thbot(l,m)*
     $     vmag(l,m)**2)
         endif
c
c derivative of fh w.r.t ts: unstable branch (ri < 0)
c
         if (ri0(i,k) .lt. 0.0) then
            ddudts = -75./2. * cn(i,k) * sqrt(zz0(i,k) / 
     $	    abs(ri0(i,k)))*dridts(i,k)

            dfhdts(i,k) = -15.*du(i,k) *
     $           (dridts(i,k) - ri0(i,k)*du(i,k)*ddudts)
c
c derivative of fh w.r.t. ts: stable branch (ri >= 0)
c
         else
            dfhdts(i,k) = -10.0 * fh(i,k)**2 * dridts(i,k)
     $           * (1.0 + 16.0*ri0(i,k))
         endif
c
c derivatives of surface fluxes w.r.t. ts
c

         dchdts    = cn(i,k) * dfhdts(i,k) / ch(i,k)
         dqsdt     = clrh2o*zqs(i,k)*
     $	 psm1(l,m)/(denom*ts2(i))


         dshf(i,jlat,k)   = zb(i,k)*(1.0 + dchdts*(ts(i) - 
     $	 thbot(l,m)))
         dlhf(i,jlat,k)   = zd(i,k)*(dqsdt + dchdts*(zqs(i,k) - 
     $	 qbot(l,m)))
c
c exclude changes in radiation as surface temperature is forecast
c
#ifdef FOAM_LPJ
         dfntdt(i,jlat,k) = -(dshf(i,jlat,k) + dlhf(i,jlat,k)*
     $  fbare(i))
#else
         dfntdt(i,jlat,k) = -(dshf(i,jlat,k) + dlhf(i,jlat,k))
#endif
c
      enddo
      enddo
c
c done
c
      return
      end
