c
#include "params.h"
c
c This subroutine is the driver for the portions
c of NCAR's CSIM 2.2.6 included in FOAM.
c 
c
c This routine is based on THERMO.F from CSIM 2.2.6
c rewritten and adapted for FOAM by R. Jacob, March, 2000 
c Additional debugging/development by S. Vavrus, July, 2001.
c  
c
      subroutine csmice_drv(
c surface input
     $ifnt,ifntdt,ofsns,coszrs,doalb,
     $PminE,
c surface input/output
     $ts,
c surface output
     $fbt,oalbs,oalbl,oalbsd,oalbld,
c hydrology output
     $fwfice,snwmlt)
C
C
C
      implicit none
c------------------------------------------------
#include "pmgrid.com"
c------------------------------------------------
#include "comtim.com"
c------------------------------------------------
#include "comtsc.com"
c------------------------------------------------
#include "pcoupled.com"
c------------------------------------------------
#include "pcplrstr.com"
c------------------------------------------------

c  surface input
      real ifnt(uwlon,uwlat)    ! surface heat flux forcing
      real ifntdt(uwlon,uwlat)  ! derivative of fnt
      real ofsns(uwlon,uwlat) ! surface absorbed solar flux
      real coszrs(uwlon,uwlat)  !cosine of the zenith angle
      real fwfice(uwlon,uwlat)    ! moisture flux to ocean
      real snwmlt(uwlon,uwlat)     ! snow melt in meters

      real PminE(uwlon,uwlat)   ! Atmos. precip fields on the ocean grid
      logical doalb             ! update albedos flag


c input/output
      real ts(uwlon,uwlat)    ! surface temp
c output
      real fbt(uwlon,uwlat)    ! flux between ice/ocean

      real oalbs(uwlon,uwlat)   ! Srf alb for direct rad  0.2-0.7 micron
      real oalbl(uwlon,uwlat)   ! Srf alb for direct rad  0.7-5.0
      real oalbsd(uwlon,uwlat)  ! Srf alb for diffuse rad 0.2-0.7
      real oalbld(uwlon,uwlat)  ! Srf alb for diffuse rad 0.7-5.0




c  local variables

      real TI(2)

c physical variables
      real rhoice     ! density of ice
      real rhoh2o     ! density of water
      real snwmax     ! maximum snow depth in liquid water equiv
      real Qi         ! latent heat of fusion, surface ice
      real Ci         ! specific heat of ice
      real minice     ! minimum ice thickness
      real hmin	      ! thickness for 0-layer model
      real maxice
      real minsnow     ! minimum snow thickness on ice
      real Tb	      ! temp at bottom of ice
      real Fb         ! leftover meltp after ice closes
      real tfrez      ! freezing temp in K
      real solout     ! net solar exiting bottom
      real sdf        ! snow density factor for thermo calculations
      real idf        ! ice density factor for thermo calculations
      real rhos       ! density of snow
      real iceform    ! minimum ice thickness at formation
      real taui       ! Harvey constant for ice maxmimum formula
      real ficemax    ! Maximum ice fraction
      real ficemin
      real firsticemax
      real absicemax
      real df	      ! ice area fraction increase
      real rat,fx     ! ratio of redistributed ice fraction
      real fwfs      ! freshwater flux due to snow.
      real fwfi      ! freshwater flux due to ice
      real sct,hflx

      real dh,hi,dhi,dhs
      real dhtotal
      real xh,dhx,dfi,xhf
      real oldthick
      real oldfice
      real qstor2
      real oldthick2(uwlon,uwlat)

      real ficenew,hinew,ficeold,ficein,thickin
      real t1in,t2in

      real timelt     ! temperature to hold ice at when its melting.

      real fracsnow   ! fractional area of snow
      real zfac       

c ice albedo variables
c New parameters
      real tlslo           ! low threshold temp for snow albedo
      real tlshi           ! high threshold temp for snow albedo

      real snwsh,          ! high temp snow albedo for 0.2-0.7 micro-meters
     $     snwlh,          ! high temp snow albedo for 0.7-5.0 micro-meters
     $     snwll,          ! low temp snow albedo for 0.7-5.0 micro-meters
     $     snwsl           ! low temp snow albedo for 0.2-0.7 micro-meters

      real sices           ! sea ice albedo for 0.2-0.7 micro-meters
      real sicel           ! sea ice albedo for 0.7-5.0 micro-meters
      real snws            ! snow    albedo for 0.2-0.7 micro-meters
      real snwl            ! snow    albedo for 0.7-5.0 micro-meters
 
      real salbs(uwlon),   ! Snow alb for direct rad  0.2-0.7 micro-ms
     $     salbl(uwlon),   ! Snow alb for direct rad  0.7-5.0 micro-ms
     $     salbsd(uwlon),  ! Snow alb for diffuse rad  0.2-0.7 micro-ms
     $     salbld(uwlon),  ! Snow alb for diffuse rad  0.7-5.0 micro-ms
     $     frsnow(uwlon),  ! Horizontal fraction of snow cover
     $     snwhgt,         ! Physical snow height
     $     rghsnw          ! Roughness for horizontal snow cover fractn

c Old parameters
c      real sicesm          ! melting ice albedo for 0.2-0.7 micro-meters
c      real sicelm          ! melting ice albedo for 0.7-5.0 micro-meters
c      real snwsm           ! melting sno albedo for 0.2-0.7 micro-meters
c      real snwlm           ! melting sno albedo for 0.7-5.0 micro-meters

c SJV: set standard ice albedo = 0.5
c      parameter (sices  = 0.70 ,
      parameter (sices  = 0.5,
c     $           sicel  = 0.50 ,
     $           sicel  = 0.5 ,
     $           snwsl  = 0.90 ,
     $           snwsh  = 0.55 ,
     $           snwll  = 0.60 ,
     $           snwlh  = 0.35 )


c externals
      real cvmgt
#if ( defined SGI) || ( defined INTEL)
      external cvmgt
#endif

      integer indx(uwlon)       ! Longitude index array (land)
      integer npts,npts2,npts3              ! Number of land points
      integer ii,jj              ! Longitude indices 
      integer iw,jw


      integer n,i,j,k,l,m
      real snowfx
      real hsnow,hice
      real hiold
      real tsurf              ! surface temp
      real tinter2
      real tice1,tice2
      real ticem
      real tmidsnow           ! snow layer midpoint temp
      real tmp,wsnorm
      real snrun,hsold,Sref,tscom
      real hsnew,hsx

c function for finding freshwater flux after a
c change in height or ice fraction. 
      real h2oflx
      real hinewx,finewx,hioldx,fioldx,Six,rhox,dtx
      h2oflx(hinewx,finewx,hioldx,fioldx,Six,rhox,dtx)=
     $((Six-Sref)*rhox*((hinewx*finewx)-(hioldx*fioldx))/dtx)/Sref

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Begin ice model calculations
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      

      rhoh2o= 1.e3
      idf = .905
      rhoice = idf*rhoh2o
      rhos  = 333.0
      sdf = rhoh2o/rhos
      snwmax = 1.0
      Sref = 34.7
      Qi = latice*rhoice !  J/ m3
      Ci = 2.1e3  ! J/Kg/K
      minice = .01  ! m
      hmin = .25
      maxice = 3.
      firsticemax = 5.
      absicemax = 20.

      iceform = .2  !meters
c SJV: change ficemax to 1.0 to reduce ice growth in leads
c      ficemax = .98
      ficemax = 1.0

      ficemin = 0.00000001
      taui = 3.  ! meters
      minsnow = .0001  !m
      tfrez  = 271.36  ! -1.8 C
      Tb = tfrez
      timelt = tmelt - 0.1
      iw=0
      jw=0


C  See if its all land here.
      npts = 0
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
         if(ioro(i,j) .ne. 1) then
          npts =npts +1
         endif
       enddo
      enddo
      enddo
      if(npts .eq. 0) return


c  see if all points are  open ocean and negative melt potential
c  if so, return.
      npts = 0
      npts2 = 0
      npts3 = 0
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone

	if(ioro(i,j) .ne. 1) then
         if((ioro(i,j) .eq. 0) .and. (meltp(i,j) .le. 0)) then
          npts3 =npts3 +1
         endif
	 npts2=npts2+1
        endif

       enddo
      enddo
      enddo
      if(npts3 .eq. npts2) return

c  open ocean and possibly making sea ice or sea ice already here.


c handle evaporation.
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone

c only do this where there's ice
       if((fice(i,j).gt.0.).and.(PminE(i,j).lt.0.))then

c take evap out of snow first.
         if(sndpth(i,j).gt.0.)then
c    from the amount of snow in meters that can be evaporated
	  dhs=(-1.0)*PminE(i,j)*dtime/rhoh2o

c  if there's enough snow, reduce it.
	  if(sndpth(i,j).gt.dhs) then
	   sndpth(i,j)=sndpth(i,j)-dhs
	   PminE(i,j)=0.

c  check to see if snow dropped below minsnow
           if(sndpth(i,j)*sdf .lt.minsnow)then
 	     fwfs=h2oflx(0.,0.,sndpth(i,j),
     $	          fice(i,j),0.,rhoh2o,dtime)
 	     fwfice(i,j)=fwfice(i,j)+fwfs
 	     sndpth(i,j)=0.
	     tsnow(i,j)=tmelt
           endif
c if not enough snow, reduce PminE.
          else
	   PminE(i,j)=PminE(i,j)+(sndpth(i,j)*rhoh2o/dtime)
	   sndpth(i,j)=0.
	   tsnow(i,j)=tmelt

c  form amount of ice in meters to be sublimated
	   dhi=(-1.0)*PminE(i,j)*dtime/rhoice

c    if enough ice, reduce it
	   if(thickn(i,j).gt.dhi) then
	      thickn(i,j)=thickn(i,j)-dhi
	      PminE(i,j)=0.
c  if not enough ice, take rest out of ocean
	   else
	      PminE(i,j)=PminE(i,j)+(thickn(i,j)*rhoice/dtime)

	      fwfs=h2oflx(0.,0.,PminE(i,j)*dtime/rhoh2o,
     $	        fice(i,j),0.,rhoh2o,dtime)
c	      write(0,*)'evaporation',nstep,i,j,fwfs,fice(i,j),
c    $          thickn(i,j),PminE(i,j),sndpth(i,j),ts(i,j)
	      fwfice(i,j)=fwfice(i,j)+fwfs
c   ice is gone
	      PminE(i,j)=0.
	      thickn(i,j)=0.
	      fice(i,j)=0.
	      ioro(i,j)=0
	      qstor(i,j)=0.
	      tice(i,j,1)=0.
	      tice(i,j,2)=0.
	      tinter(i,j)=0.
	      tsnow(i,j)=0.
	      ts(i,j)=sst(i,j)
	      tssub(i,j,1)=ts(i,j)
	      tssub(i,j,2)=ts(i,j)
	      tssub(i,j,3)=ts(i,j)
	      tssub(i,j,4)=ts(i,j)
	   endif
	  endif
	  
c if no snow, take out of ice/ocean
         else
	  dhi=(-1.0)*PminE(i,j)*dtime/rhoice

c    if enough ice, reduce it
	  if(thickn(i,j).gt.dhi) then
	     thickn(i,j)=thickn(i,j)-dhi
	     PminE(i,j)=0.
c  if not enough ice, take rest out of ocean
	  else
	     PminE(i,j)=PminE(i,j)+(thickn(i,j)*rhoice/dtime)

	     fwfs=h2oflx(0.,0.,PminE(i,j)*dtime/rhoh2o,
     $	        fice(i,j),0.,rhoh2o,dtime)
c	     write(0,*)'evaporation2',nstep,i,j,fwfs
	     fwfice(i,j)=fwfice(i,j)+fwfs
c   ice is gone
	     PminE(i,j)=0.
	     thickn(i,j)=0.
	     fice(i,j)=0.
	     ioro(i,j)=0
	     qstor(i,j)=0.
	     tice(i,j,1)=0.
	     tice(i,j,2)=0.
	     tinter(i,j)=0.
	     tsnow(i,j)=0.
	     ts(i,j)=sst(i,j)
	     tssub(i,j,1)=ts(i,j)
	     tssub(i,j,2)=ts(i,j)
	     tssub(i,j,3)=ts(i,j)
	     tssub(i,j,4)=ts(i,j)
          endif
	 endif

c      if((sndpth(i,j).gt.0.).and.(sndpth(i,j).lt.minsnow))then
c       write(0,*)'snow Warning',nstep,i,j,sndpth(i,j),fwfs,
c    $	thickn(i,j),PminE(i,j)
c      endif

       if(sndpth(i,j).lt.0.)then
       write(0,*)'negative snow',nstep,i,j,sndpth(i,j),ioro(i,j)
       endif
       if((thickn(i,j).gt.0.).and.(thickn(i,j).lt.minice))then
        write(0,*)'Ice warning',nstep,i,j,thickn(i,j),ioro(i,j)
       endif


       endif
       enddo
      enddo
      enddo


c  now take care of rest of ice
      do n=1,2
      do j=plats(n),plate(n)

      npts = 0
      npts3 = 0

       do i=plons,plone

	if(ioro(i,j) .ne. 1) then
         if((ioro(i,j) .eq. 0) .and. (meltp(i,j) .le. 0)) then
          npts3 =npts3 +1
         else
	  npts= npts+1
	  indx(npts)=i
         endif
        endif

       enddo

ccccccccccccccccccccccccccccccccccccccccc
c  Begin Ice calculations
c  these are only done on points which are sea ice
c  or open ocean with ice forming.
ccccccccccccccccccccccccccccccccccccccccc
      do ii=1,npts
          i=indx(ii)


c assume all moisture flux on ice is snow.
        snowfx=amax1(0.,PminE(i,j))

        oldthick2(i,j)=thickn(i,j)

 	if((i.eq.iw).and.(j.eq.jw)) then
  	 write(0,*)'   ',coszrs(i,j),calday
  	 write(0,*)nstep,ioro(i,j),sst(i,j),thickn(i,j),
     $	 fice(i,j),meltp(i,j)
	 debb(14)=coszrs(i,j)
	 debb(15)=0.
	 debb(16)=0.
	 debb(17)=0.
	 debb(18)=0.
	 debb(19)=0.
	debb(20)=0.
	debb(21)=0.
	debb(22)=0.
	debb(23)=0.
        endif

	 
c  heat available to melt ice laterally in J/m2
        dh = -meltp(i,j)*dtime
	Fb = 0.

ccccccccccccccccccccc
c  form ice in leads
c  could be open ocean with sea ice forming
c   or ice with more ice forming
ccccccccccccccccccccc
      if (meltp(i,j) .gt. 0.) then
 	  if((i.eq.iw).and.(j.eq.jw)) then
	   write(0,*)'IN Form ',meltp(i,j),fice(i,j)
          endif

c SJV: skip over lateral ice growth if there are no leads
      if (fice(i,j) .lt. 1.0) then

	  ficein=fice(i,j)
	  thickin=thickn(i,j)
	  t1in=tice(i,j,1)
	  t2in=tice(i,j,2)

c   find fractional increase
	  df=(meltp(i,j)/(Qi*iceform))*dtime

c  find new ice cover
          ficenew=fice(i,j)+df

c SJV: try to arrest ice buildup when ice fraction exceeds 1.0
          if (ficenew .gt. 1.0) then
             ficenew = 1.0
             df = 1.0-fice(i,j)
          endif

c find new ice thickness
	  hinew =(thickn(i,j)*fice(i,j) + df*iceform)/ficenew
 	  if((i.eq.iw).and.(j.eq.jw)) then
 	    write(0,*)'form 1',nstep,df,hinew,ficenew,fice(i,j),
     $	      thickn(i,j),ts(i,j),oalbs(i,j),oalbl(i,j)
	  debb(15)=df
          endif

c redistrbute snow
	  sndpth(i,j)=(fice(i,j)*sndpth(i,j))/ficenew

c here is the only place that open ocean turns to ice.
	  if((ioro(i,j) .eq.0) .and. (fice(i,j) .gt. 0.)) then
	     qstor(i,j)=0.
	     ioro(i,j)=2
	     ts(i,j)=tfrez
	     tinter(i,j)=tfrez
	     tsnow(i,j)=tmelt
	     tice(i,j,1)=tfrez
	     tice(i,j,2)=tfrez
	     tssub(i,j,1)=tfrez
	     tssub(i,j,2)=tfrez
	     tssub(i,j,3)=tfrez
	     tssub(i,j,4)=tfrez
          endif

c average temp
          tice(i,j,1)=(fice(i,j)*thickn(i,j)*tice(i,j,1)+
     $            df*iceform*tfrez)/(ficenew*hinew)
          tice(i,j,2)=(fice(i,j)*thickn(i,j)*tice(i,j,2)+
     $            df*iceform*tfrez)/(ficenew*hinew)


c finally update thickness and fice
          fice(i,j)=ficenew
	  thickn(i,j)=hinew


c make sure fice is less than ficemax
c following Harvey(1988)

          if(fice(i,j) .gt. ficemax) then
	   ficeold=fice(i,j)
	   if(hinew .le. 1.) then
	     fice(i,j)=ficemax
           else
	     fice(i,j)=1. - (1.-ficemax)*
     $	     exp(-(hinew-1.0)/taui)
           endif
	   if(fice(i,j).gt.ficeold)fice(i,j)=ficeold

	   rat = ficeold/fice(i,j)

c set new ice thickness
           thickn(i,j)=hinew*rat

c redistribute snow again
	   sndpth(i,j)=sndpth(i,j)*rat

	  endif


          if(sndpth(i,j)*sdf .lt.minsnow)then
            fwfs=h2oflx(0.,0.,sndpth(i,j),
     $	          fice(i,j),0.,rhoh2o,dtime)
 	    fwfice(i,j)=fwfice(i,j)+fwfs
 	    sndpth(i,j)=0.
	    tsnow(i,j)=tmelt
          endif

c renormalize snowfx.  This is done assuming that ocean p-e
c was already added with the old value of fice.  Now that
c the open ocean area has changed, snowfx needs to be changed too
          snowfx=snowfx*(ficein/fice(i,j))

 	  if((i.eq.iw).and.(j.eq.jw)) then
  	    write(0,*)'form',nstep,df,hinew,ficenew,fice(i,j),
     $	      thickn(i,j),ts(i,j),sndpth(i,j),tice(i,j,1),
     $		tice(i,j,2)
	  debb(16)=thickn(i,j)-thickin
	  debb(17)=fice(i,j)-ficein
	  debb(18)=tice(i,j,1)-t1in
          endif
         
      else
c RLJ attempt to account for melt present when fice=1.0
c  by setting in equal to Fb. 
  	 Fb = -meltp(i,j)
 	  if((i.eq.iw).and.(j.eq.jw)) then
	    write(0,*)'SET Fb',Fb
	  debb(19)=Fb
          endif
      endif
      endif
       
ccccccccccccccccccccc
c melt ice in leads
c  this will only happen where there's ice.
ccccccccccccccccccccc
      if(dh .gt. 0.) then

c RLJ if ice thickness is greater than 3m, than all melting
c energy should go to decreasing thickness by passing
c it in as Fb.
       if(thickn(i,j) .lt. firsticemax) then

	ficein=fice(i,j)

	ticem=(tice(i,j,1)+tice(i,j,2))/2.0

c find heat required to melt this column
	xh = rhoice*thickn(i,j)*(Ci*(tmelt-ticem) + latice)

c find minimum. if its dh then all heat is used to melt some ice
c  		if its xh then heat melted all ice
        xhf = xh*fice(i,j)

        dhx = amin1(dh,xhf)

        ficeold = fice(i,j)
	dfi = dhx/xh
c SJV: don't let dfi be < 0, or else spurious lateral growth occurs
        if (dfi .lt. 0.0) dfi = 0.0

	fice(i,j)=amax1(0.,fice(i,j)-dfi)

c SJV: try altering fbt as a test
	fbt(i,j) = (-1.0*dhx/dtime) + qstor(i,j)*dfi/dtime

c add to freshwater flux
	fwfs=h2oflx(sndpth(i,j),fice(i,j),sndpth(i,j),
     $    ficeold,0.0,rhoh2o,dtime)
c	write(0,*)'melting snow',nstep,i,j,fwfs,sndpth(i,j)

	fwfi=h2oflx(thickn(i,j),fice(i,j),thickn(i,j),
     $    ficeold,0.0,rhoice,dtime)
c	write(0,*)'melting ice',nstep,i,j,fwfi,thickn(i,j)

        fwfice(i,j)=fwfice(i,j)+fwfs+fwfi

	if(fice(i,j) .lt. ficemin) then
	     fice(i,j)=0.
	     ioro(i,j)=0
c  note:  no snowmelt.  Its zeroed each timestep.
	     thickn(i,j)=0.
	     sndpth(i,j)=0.
	     tice(i,j,1)=0.
	     tice(i,j,2)=0.
	     tinter(i,j)=0.
	     tsnow(i,j)=0.
	     qstor(i,j)=0.

	     ts(i,j)=sst(i,j)
	     tssub(i,j,1)=ts(i,j)
	     tssub(i,j,2)=ts(i,j)
	     tssub(i,j,3)=ts(i,j)
	     tssub(i,j,4)=ts(i,j)
	endif

        if(fice(i,j).gt.0.) then
c renormalize snowfx.  This is done assuming that ocean p-e
c was already added with the old value of fice.  Now that
c the open ocean area has changed, snowfx needs to be changed too
          snowfx=snowfx*(ficein/fice(i,j))
        else
c ice is gone so add precip to fwf
	 fwfs=h2oflx(0.,0.,snowfx*dtime/rhoh2o,ficein,
     $     0.0,rhoh2o,dtime)
c 	 write(0,*)'melted all ice',nstep,i,j,fwfs
	 snowfx=0.
	 fwfice(i,j)=fwfice(i,j)+fwfs
        endif

 	if((i.eq.iw).and.(j.eq.jw)) then
 	 write(0,*)'lateral melt',nstep,dhx,dfi,xh,dh,
     $	 thickn(i,j),fice(i,j),fbt(i,j)
	debb(20)=dh
	debb(21)=xh
	debb(22)=xhf
	debb(23)=dfi
        endif

       else
	Fb=-meltp(i,j)
       endif

      endif


      oldthick2(i,j)=thickn(i,j)
ccccccccccccccccccccc
c vertical energy balance
ccccccccccccccccccccc
      if(ioro(i,j).eq.2) then

      hsnow=sndpth(i,j)
      hice=thickn(i,j)
      tice1=tice(i,j,1)
      tice2=tice(i,j,2)
      TI(1)=tice(i,j,1)
      TI(2)=tice(i,j,2)
      tsurf=ts(i,j)
      tinter2=tinter(i,j)
      tmidsnow=tsnow(i,j)
      qstor2=qstor(i,j)
      hsold=hsnow
      hiold=hice
      sct = -ofsns(i,j)
      hflx=ifnt(i,j)
c convert snowfx to m/s
      snowfx = snowfx/rhoh2o

      if( hice.lt. hmin) then
      call ticezl(i,j,
c surface input
     $ hflx,ifntdt,sct,snowfx,Fb,
c surface input/output
     $ tsurf,tinter2,tmidsnow,tice1,tice2,
     $ hice,hsnow,
c surface output
     $ fbt,snwmlt)

      else

      call tice3l(i,j,
c surface input
     $ hflx,ifntdt,sct,snowfx,Fb,
c surface input/output
     $ tsurf,tinter2,tmidsnow,TI,
     $ hice,hsnow,qstor2,
c surface output
     $ fbt,snwmlt)

      tice1=TI(1)
      tice2=TI(2)

      endif
      
      qstor(i,j)=qstor2

c     if(snwmlt(i,j).lt.0.) then
c       write(0,*)'neg snowmelt',nstep,i,j,snwmlt(i,j),
c     $ thickn(i,j)
c     endif
c freshwater flux due to snowmelt
      hsnew = hsold-snwmlt(i,j)
      fwfs=h2oflx(hsnew,fice(i,j),hsold,
     $  fice(i,j),0.0,rhoh2o,dtime)

      fwfice(i,j)=fwfice(i,j)+fwfs

c ice grew, determine freshwater flux
      if(hice.gt.hiold) then
	fwfi=h2oflx(hice,fice(i,j),hiold,
     $ fice(i,j),0.0,rhoice,dtime)

        fwfice(i,j)=fwfice(i,j)+fwfi
      endif

c ice thinned, determine freshwater flux
c  redistribute following Harvey(1988)
      if(hice.lt.hiold) then

         ficeold=fice(i,j)
         rat= 1. - ((hiold-hice)/(2.0*hiold))
c SJV: set rat=1 to eliminate lateral ice melt due to vertical melting
c         rat = 1.0

         fice(i,j)= fice(i,j)*rat
         hice=hice/rat

c handle case where ice melted completely
         if(hice .lt. 0.) then
	  hice = 0.
	  fice(i,j)=0.
         endif
         fwfi=h2oflx(hice,fice(i,j),hiold,
     $     ficeold,0.0,rhoice,dtime)

         fwfice(i,j)=fwfice(i,j)+fwfi

c  the freshwater flux due to snow
c  note: if fice went to 0 above, then
c  all snow will get added as freshwater.
c  if fice was just adjusted, then instead
c  of adjusting snowdepth, take snow off
c as a freshwater flux.
         fwfs=h2oflx(hsnow,fice(i,j),hsnow,
     $      ficeold,0.0,rhoh2o,dtime)
         fwfice(i,j)=fwfice(i,j)+fwfs
      endif


c ensure icethickness is greater than minimum
      ficeold=fice(i,j)
      if(hice .lt. minice) then
	fice(i,j)=fice(i,j)*(hice/minice)
	hice=minice
c CSIM 2.2.6 does this.  Handle below instead.
c	hsnow=hsnow/max(hice/minice,1e-10)
c  wait to adjust snow after fice check
      endif

c finally load in the new temps, thickness      
      ts(i,j)=tsurf

c ice temps are the same for zero or single layer ice
      if(hice .lt. (2.0*hmin))tice2=tice1

c      if(tice1.lt. 200.)
c     $ write(0,*)'LOW TICE1',i,j,nstep,tice1,ts(i,j),tinter2,tice2

      tice(i,j,1)=tice1
      tice(i,j,2)=tice2
      tinter(i,j)=tinter2

      tsnow(i,j)=tmidsnow
      tssub(i,j,1)=ts(i,j)
      tssub(i,j,2)=tinter(i,j)
      tssub(i,j,3)=tice(i,j,1)
      tssub(i,j,4)=tice(i,j,2)
      sndpth(i,j)=hsnow
      thickn(i,j)=hice

c make ice go away if fice falls below minimum
c note:  freshwater melt here is not done.
c
      if(fice(i,j).lt.ficemin) then
	fice(i,j)=0.
	thickn(i,j)=0.
	ts(i,j)=sst(i,j)
	tice(i,j,1)=0.
	tice(i,j,2)=0.
	tinter(i,j)=0.
	tsnow(i,j)=0.
	qstor(i,j)=0.
	tssub(i,j,1)=ts(i,j)
	tssub(i,j,2)=ts(i,j)
	tssub(i,j,3)=ts(i,j)
	tssub(i,j,4)=ts(i,j)
	ioro(i,j)=0
	sndpth(i,j)=0.
      else
c resolve the snowdepth adjustment from above
c ice thickness adjustment
	sndpth(i,j)=sndpth(i,j)*(ficeold/fice(i,j))
      endif


      endif

cccccccccccccccccccccccccccccccccccccc
c  end of vertical temperature balance
cccccccccccccccccccccccccccccccccccccc

      if(fice(i,j).lt.ficemin) then
	fice(i,j)=0.
        ioro(i,j)=0
	thickn(i,j)=0.
	sndpth(i,j)=0.
	tice(i,j,1)=0.
	tice(i,j,2)=0.
	tinter(i,j)=0.
	tsnow(i,j)=0.
	qstor(i,j)=0.
	ts(i,j)=sst(i,j)
	tssub(i,j,1)=ts(i,j)
	tssub(i,j,2)=ts(i,j)
	tssub(i,j,3)=ts(i,j)
	tssub(i,j,4)=ts(i,j)
      endif

c RLJ make sure fice is 1.0 if thickness goes above 3m
c
c     if((fice(i,j).lt. 1.0).and.(thickn(i,j).ge. firsticemax))then
c	ficeold=fice(i,j)
c	fice(i,j)=1.0
c	thickn(i,j)=thickn(i,j)*(1.0/ficeold)
c     endif
	 

c try to keep ice below a maximum thickness
c when fice is small
c     if(fice(i,j).lt.ficemax)then
cif(thickn(i,j).gt.maxice) then
c	   oldfice=fice(i,j)
c	   rat=thickn(i,j)/maxice
c	   if(rat*fice(i,j).lt.ficemax) then
c	     thickn(i,j)=maxice
c	     fice(i,j)=rat*fice(i,j)
c	     sndpth(i,j)=sndpth(i,j)/rat
c   check for small snow values
c             if(sndpth(i,j).lt.minsnow)then
c              fwfs=h2oflx(0.,0.,sndpth(i,j),
c    $	          fice(i,j),0.,rhoh2o,dtime)
c	      fwfice(i,j)=fwfice(i,j)+fwfs
c	      sndpth(i,j)=0.
c            endif
c   endif
c	   if(fice(i,j).gt.1.) then
c	     write(0,*)'fice',nstep,i,j,fice(i,j),rat,
c    $ 			oldfice
c          endif
c        endif
c     endif

c check for snow maximum

      if(sndpth(i,j) .gt. snwmax) then
       fwfs=h2oflx(0.,0.,sndpth(i,j)-snwmax,
     $            fice(i,j),0.,rhoh2o,dtime)
       fwfice(i,j)=fwfice(i,j)+fwfs
       sndpth(i,j)=snwmax
      endif

c sea ice thickness clamp
      if(thickn(i,j) .gt. absicemax) then
       fwfs=h2oflx(0.,0.,thickn(i,j)-20.,
     $            fice(i,j),0.,rhoh2o,dtime)
       fwfice(i,j)=fwfice(i,j)+fwfs
       thickn(i,j)=20.
      endif
      if((i.eq.iw).and.(j.eq.jw)) then
       write(0,*)"END",fwfice(i,j),fbt(i,j),thickn(i,j),fice(i,j)
      endif

c sea ice thickness clamp
c     if((ioro(i,j) .eq. 2) .and. thickn(i,j).gt. 6.92) then
c     write(0,*)'thickw',nstep,i,j,thickn(i,j),ts(i,j),sst(i,j)
c     endif

c ice temp check
      if((ioro(i,j).eq. 2).and.(ts(i,j) .gt. 300.)) then
       write(0,*) 'HIGH ICE TEMP',nstep,i,j,ts(i,j),thickn(i,j)
       write(0,*) ifnt(i,j),ifntdt(i,j)
      endif
      if((ioro(i,j).eq. 2).and.(ts(i,j) .lt. 150.)) then
       write(0,*) 'LOW ICE TEMP',nstep,i,j,ts(i,j),tice(i,j,1),
     $ tice(i,j,2),fice(i,j),sndpth(i,j)
       write(0,*) thickn(i,j),ifnt(i,j),ifntdt(i,j),tsnow(i,j),
     $ tinter(i,j)
      endif
       


      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End ice growth, temp forecast
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  Update albedos
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if(doalb) then
c New temperature thresholds for sea ice
       tlslo = tmelt -5.
       tlshi = tmelt

       do ii=1,npts
        i=indx(ii)
C
C  Set sea ice albedos
C
       if(ioro(i,j) .eq. 2) then
         if( coszrs(i,j).gt.0.0 ) then

          oalbs(i,j)  = sices
          oalbl(i,j)  = sicel
c SJV: modify ice albedos to be functions of temperature and thickness
          if (tssub(i,j,1) .lt. tmelt) then
             oalbs(i,j) = oalbs(i,j) + 0.015*(tmelt-tssub(i,j,1))
             oalbs(i,j) = amin1(oalbs(i,j),0.7)
          endif
          if (thickn(i,j) .lt. 1.0) then
            oalbs(i,j)=sqrt(amax1(thickn(i,j),0.0))*
     $      (oalbs(i,j)-0.1)+0.1
          endif
          oalbl(i,j) = oalbs(i,j) 
c New direct albedos
          oalbsd(i,j) = oalbs(i,j)
          oalbld(i,j) = oalbl(i,j)

          if((i.eq.iw).and.(j.eq.jw)) then
            write(0,*) 'albedos =',oalbs(i,j),oalbl(i,j),
     $                 oalbld(i,j), oalbsd(i,j)
          endif
c Old direct albedos
c          oalbsd(i,j) = sices
c          oalbld(i,j) = sicel
c Try not lowering ice thicknesses less than 1 m
c	  if(thickn(i,j) .lt. 1.) then
c	  oalbs(i,j)=oalbs(i,j) + .10855*alog(thickn(i,j))
c	  oalbl(i,j)=oalbl(i,j) + .10855*alog(thickn(i,j))
c	  oalbsd(i,j)=oalbsd(i,j) + .10855*alog(thickn(i,j))
c 	  oalbld(i,j)=oalbld(i,j) + .10855*alog(thickn(i,j))
c	  endif

c Old code
c if ice is melting, reset albedo
c          if ( ts(i,j) .ge. timelt ) then
c              oalbs(i,j) = sicesm
c              oalbl(i,j) = sicelm
c              oalbsd(i,j) = sicesm
c              oalbld(i,j) = sicelm
c          endif

c if snow is present, reset albedo
c Old code
c	  if(sndpth(i,j) .gt. 0.) then
c New code
      if(sndpth(i,j).gt.0. .and. coszrs(i,j).gt.0.) then

C  set visible snow albedos
        zfac = amax1(0., 1.5/(1. + 4. * coszrs(i,j)) - 0.5)

        tmp = (snwsh*(tssub(i,j,1)-tlslo) + snwsl*(tlshi -
     $  tssub(i,j,1))) / (tlshi - tlslo)
        tmp = amin1(snwsl, amax1(snwsh, tmp))
        salbs(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
        salbsd(i) = tmp

C  set near-infrared (longwave) snow albedos
        tmp = (snwlh*(tssub(i,j,1)-tlslo) + snwll*(tlshi -
     $  tssub(i,j,1))) / (tlshi - tlslo)
        tmp = amin1(snwll, amax1(snwlh, tmp))
        salbl(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
        salbld(i) = tmp
      endif
C
C Compute fraction of horizonal surface covered with snow:
C
         if(sndpth(i,j) .le. 0.) then
            snwhgt    = 0.0
            rghsnw    = 0.0
            frsnow(i) = 0.0
         else
C Change snow fraction to use the correct
C snow equivalent depth factor and also
C use total roughness.  RLJ, BG
C           snwhgt    = 20. * sndpth(i,j)
C           rghsnw    = amax1(zzsice,0.25)
C           frsnow(i) = snwhgt/(rghsnw + snwhgt)

            snwhgt    = snwedp * sndpth(i,j)
            rghsnw    = zzsice
            frsnow(i) = snwhgt/(rghsnw + snwhgt)
         end if
C
C  Now modify albedoes for snowcover
C
         if( sndpth(i,j).gt.0. .and. coszrs(i,j).gt.0. ) then
          oalbs(i,j)  = oalbs(i,j) *(1.-frsnow(i)) +
     $      salbs(i) *frsnow(i)
          oalbl(i,j)  = oalbl(i,j) *(1.-frsnow(i)) +
     $      salbl(i) *frsnow(i)
          oalbsd(i,j) = oalbsd(i,j)*(1.-frsnow(i)) +
     $      salbsd(i)*frsnow(i)
          oalbld(i,j) = oalbld(i,j)*(1.-frsnow(i)) +
     $      salbld(i)*frsnow(i)
         endif

c estimate fractional area of snow cover.
c  .1 m liquid water equivalent is considered half coverage

c Old code
c	    fracsnow= sndpth(i,j)/(sndpth(i,j) + 0.1)
c 	    oalbs(i,j) = snws*fracsnow + 
c     $	      sices*(1.-fracsnow)
c	    oalbl(i,j) = snwl*fracsnow + 
c     $	      sicel*(1.-fracsnow)
c	    oalbsd(i,j) = snws*fracsnow + 
c     $	      sices*(1.-fracsnow)
c	    oalbld(i,j) = snwl*fracsnow + 
c     $	      sicel*(1.-fracsnow)
c if snow is melting reset albedo
c	    if (ts(i,j) .ge. tmelt) then
c	     oalbs(i,j) = snwsm*fracsnow + 
c     $	        sicesm*(1.-fracsnow)
c	     oalbl(i,j) = snwlm*fracsnow + 
c     $	        sicelm*(1.-fracsnow)
c	     oalbsd(i,j) = snwsm*fracsnow + 
c     $	        sicesm*(1.-fracsnow)
c	     oalbld(i,j) = snwlm*fracsnow + 
c     $	        sicelm*(1.-fracsnow)
c	    endif

c          endif

         endif
       endif

       enddo
      endif


      enddo
      enddo

      return
      end
