c
#include "params.h"
c
c  The Semtner 0-layer ice model
c
c  This routine is based on ticezl.F from
c  NCAR's CSIM 2.2.6
c
c  rewritten and adapted for FOAM by R. Jacob, March, 2000
c

      subroutine ticezl(i,j,
c surface input
     $hflx,ifntdt,solin,snowfx,Fb,
c surface input/output
     $ tsurf,tinter2,tmidsnow,tice1,tice2,
     $ hice,hsnow,
c surface output
     $ fbt,snwmlt)


      implicit none
c------------------------------------------------
#include "pmgrid.com"
c------------------------------------------------
#include "comtim.com"
c------------------------------------------------
#include "comtsc.com"
c------------------------------------------------
#include "pcoupled.com"
c------------------------------------------------
#include "pcplrstr.com"
c------------------------------------------------
c input
      integer indx(uwlon)
      integer npts
      real hflx    ! surface heat flux forcing
      real ifntdt(uwlon,uwlat)  ! derivative of fnt
      real snowfx
      real solin ! surface absorbed solar flux
      real tsurf	      ! surface temp
      real tinter2	      ! snow/ice interface
      real tmidsnow	      ! snow layer midpoint temp
      real hice		      ! ice thickness
      real hsnow	      ! snow thickness
      real tice1
      real tice2
      real Fb         ! heat flux from ocean (always 0.)

c input/output
      real ts(uwlon,uwlat)    ! surface temp
c output
      real fbt(uwlon,uwlat)    ! flux between ice/ocean
      real snwmlt(uwlon,uwlat)     ! snow melt in meters

c physical variables
      real rhoice     ! density of ice
      real rhoh2o     ! density of water
      real snwmax     ! maximum snow depth in liquid water equiv
      real Qi         ! latent heat of fusion, surface ice
      real Qs         ! latent heat of fusion, snow
      real Ci         ! specific heat of ice
      real Qb         ! latent heat of fusion, bottom ice
      real minice     ! minimum ice thickness
      real minsnow     ! minimum snow thickness on ice
      real Yk         ! conductivity of ice
      real Sk         ! conductivity of snow
      real Tb         ! temp at bottom of ice
      real tfrez      ! freezing temp in K
      real fpensol    ! fraction of solar radiation penetrating ice
      real solout     ! net solar exiting bottom
      real coptic     ! bulk shortwave extinction
      real sdf        ! snow density factor for thermo calculations
      real idf        ! ice density factor for thermo calculations
      real rhos       ! density of snow
      real dh,hi,fa,fo,fs,dhi,dhb
      real dhtotal
      real con,xh,dhx,dfi
      real meltw,oldthick,oldthick2
      real dthick,hsold,dsnow
      real denom,dtemp
      real timelt     ! temperature to hold ice at when its melting.
      real tpast

      integer iw,jw,i,j,ii


      rhoh2o= 1.e3
      idf = .905
      rhoice = idf*rhoh2o
      rhos  = 333.0
      sdf = rhoh2o/rhos
      snwmax = 1.0
      Qi = latice*rhoice !  J/ m3
      Qs = 110.2e6  ! J/m3
      Ci = 2.1e3  ! J/Kg/K
      Qb =Qi
      Yk = 2.03  ! J/(m s K)
      Sk = 0.31  ! J/(m s K)
c     Fb = 0.  !
      minice = .01  ! m
      minsnow = .0001  !m
      fpensol = .3
      coptic = 1.5  ! m-1
      tfrez  = 271.36  ! -1.8 C
      Tb = tfrez
      timelt = tmelt - 0.1
      iw=0
      jw=0

      fo=0.
      dhi=0.
      dhb=0.

      tpast=tsurf

ccccccccccccccccccccccccccccccccccccccccc
c            snow cover on thick enough ice
ccccccccccccccccccccccccccccccccccccccccc
c if surface has snow cover, all energy goes to changing
c snow temp.
       if(hsnow*sdf .gt. minsnow) then

c forcing terms on temperature
           con=Sk/(hsnow*sdf + (hice*Sk/Yk))

	   hflx=hflx+solin
	   dtemp=(-hflx + con*(Tb-tpast))/
     $	     (con - ifntdt(i,j))

           tsurf=tpast+dtemp

c  hold snow temp to freezing for melting
	   if(tsurf .gt. tmelt) tsurf = tmelt

c find temperature at snow/ice interface
	   tinter2=(hice*Sk*tsurf + 
     $         hsnow*sdf*Yk*Tb       )/
     $		(hice*Sk + sdf*hsnow*Yk)

c  compute fluxes
c change heat flux to positive upward
	   fa= hflx
	   fs= Sk*(tinter2 - tsurf)/(sdf*hsnow)
	   fo=fs

c update temp and thickness of snow

c  to cold to melt so add snow.

c  note:  use lt instead of ne here  -rlj
           if(tsurf .lt. tmelt) then
	      hsnow=hsnow+snowfx*dtime
c  get rid of snow if there's too little
	      if(hsnow*sdf .lt.minsnow) then
		snwmlt(i,j)=snwmlt(i,j)+hsnow
		hsnow=0.
		tmidsnow=tmelt
              endif

	   else

c  snow is melting
           snwmlt(i,j)=snwmlt(i,j)+ snowfx*dtime

c see if snow actually melts
	     hsold=hsnow
	     if(fs.gt.fa) then
		dsnow=(dtime*(fs-fa)/Qs)/sdf
c  see if all snow melts
		if(dsnow.ge.hsnow) then
		  snwmlt(i,j)=snwmlt(i,j)+hsnow
		  hsnow=0.
		  tmidsnow=tmelt
                else
	          hsnow=hsnow - dsnow
	          snwmlt(i,j)=snwmlt(i,j)+ dsnow
		endif
             endif
	     if(hsnow*sdf.lt.minsnow) then
		snwmlt(i,j)=snwmlt(i,j)+hsnow
		hsnow=0.
		tmidsnow=tmelt
             endif
	   endif

           if((i.eq.iw).and.(j.eq.jw)) then
 	     write(0,*)'ZL snow,thick',nstep,tsurf,tinter2,tmelt,dtemp,
     $	     hice,hsnow,snwmlt(i,j),snowfx,fs,fa,
     $       hflx,ifntdt(i,j)
           endif

       else
cccccccccccccccccccccccccccccccccccccccccccccc
c else calculate new ts and let snow accumulate
cccccccccccccccccccccccccccccccccccccccccccccc
	     con=Yk/hice
        
	     denom = 4. * stebol * tsurf*tsurf*tsurf
	     solout = solin*fpensol*
     $	        exp(-coptic*hice)

	     hflx=hflx+solin-solout
	     dtemp=(-hflx + con*(Tb-tsurf))/
     $	       (con - ifntdt(i,j)+ denom)

c add solar out to flux between sea ice and ocean
	     fbt(i,j) = fbt(i,j)- solout*fice(i,j)

             tinter2=tsurf+dtemp

c  hold ice temp to 273.15 (-0.1 C) for melting
	     if(tinter2 .gt. timelt) then
	       tinter2 = timelt
	     endif
	     tsurf=tinter2

c  compute fluxes
	     fa= hflx
c  
c heat flux through ice; positive upward
	     fo=Yk*(Tb - tinter2)/hice

c if to cold to melt
c then form snow on ice
	     if(tinter2 .lt. timelt) then
	       dhi=0.
               if(snowfx.gt.0.)then
	        hsnow= hsnow + snowfx*dtime
               endif
c if melting from top
c figure out how much ice melts and let precip pass
c  through.
             else
	       if(snowfx.gt.0.) then
	        snwmlt(i,j)= snwmlt(i,j) + snowfx*dtime
	       endif
	       dhi = 0.
c add a check to make sure dhi will be negative  -rlj
	       if(fa.lt.fo) then
	          dhi= dtime*(fa-fo)/Qi
	       endif
	     endif

             if((i.eq.iw).and.(j.eq.jw)) then
 	        write(0,*)'ZL thick,no snow',nstep,tsurf,tinter2,
     $	        hice,hsnow,snowfx,fa,fo
             endif

ccccccccccccccccccccccc
c  end of snow-free ice temp calc
ccccccccccccccccccccccc

       endif
ccccccccccccccccccccccc
c  end of temp, seaice and snow update,
c    snowfree or not
ccccccccccccccccccccccc


c ablation or accretion at bottom
       dhb = dtime*(fo - Fb)/Qb

       dhtotal=dhi+dhb
       if(dhtotal  .gt. 1.) then
	 write(0,*)'dhwatch',nstep,i,j,dhi,dhb,tsurf,sst(i,j),
     $  hsnow,fo
       endif

c  new ice thickness
       oldthick=hice
       hice=hice+dhi+dhb

       if(hice .ge. minice) then

c  if snow exists, assume vertically constant heat flux
         if(hsnow.gt.0.)tmidsnow=0.5*(tsurf+tinter2)

         tice1=0.5*(tinter2 + Tb)
       else
c ice is too thin.  resolve later but set tice1
         if(hsnow .ne. 0.)  then
           tice1=0.5*(tinter2 + Tb)
         endif
       endif

       tice2=tice1

       if((i.eq.iw).and.(j.eq.jw)) then
 	 write(0,*)'ZL vert',nstep,tsurf,hice,
     $	 hsnow,dhi,dhb,snowfx
       endif

cccccccccccccccccccccccccccccccccccccc
c  end of vertical temperature balance
cccccccccccccccccccccccccccccccccccccc

       return
       end
