
#include "params.h"
/*************************************************************/
/*                FOAM v1.5                                  */
/*                                                           */
/*************************************************************/
/*                                                           */
/*                                                           */
/* Original version:    R. Jacob, C. Schafer, May,1995       */
/* parallel coupler:    Oct, 1995                            */
/* change to R15 atmos:  Jul, 1996                           */ 
/* add CCM3 physics;     Dec, 1996 (version 1.0)             */
/* Port to SGI Origin:     Dec, 1998                         */
/* add remaining CCM3.6.6 physics, bug fixes    Dec 1999     */
/*                                                           */
/*                                                           */
/* $Name: F1_5_14 $ */
/* $Id: foam.c,v 1.9 2002/08/27 20:43:32 jacob Exp $ */ 
/*                                                           */
/*  Note on parallel comm
This is the main program for the coupled ocean/atmosphere
model. split_comm is used as the communicator within each
model, while there is one intercommunicator, coupler_ocean,
which handles the communication between the ocean and coupler.
These communicators are carried over to each of the models
by treating them as external variables.
Chad Schafer, 10/5/95
*/

#include <stdio.h>
#include "mpi.h"

#ifdef UPSHOT
#include "mpe.h"
  char logfilename[110];
#endif

MPI_Comm split_comm, coupler_ocean;

char buff[256];

#ifdef PROFILE
  FILE *profile_file;
#endif
 
  int rsfrq,hsfrq,rnlng,finished;
  char time_inv[256];
  char param[50],prefix[100];
#ifdef SIMPLE_OUTPUT
  char storage[100];
  char lastoutname[256];
  FILE *endfile;
#endif
  FILE *fp;

#ifdef LINUX
MAIN_(int argc, char *argv[])
#else
main(int argc, char *argv[])
#endif
{
  int numprocs,myid,split_point,color,myoid;
  int ocean_step_num,total_ocean_steps;
  char buff2[256];
  
  int timeinvlen;



  MPI_Init(&argc,&argv);

  /* check and process arguments */
  if(argc < 2){
  printf("FOAM needs one argument: the run parameters file\n Aborting\n");
  MPI_Finalize();
  exit(1);
  }

#ifdef TCP_TEST
nexus_disable_tcp_poll();
#endif

  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
  MPI_Comm_rank(MPI_COMM_WORLD,&myid);

/* check number of procs */
#ifdef COLOCATED_OCEAN
  if( numprocs != (PP_NPROC_LT*PP_NPROC_LN)){
  printf("FOAM: Wrong number of processors.  Got %d, want %d for colocated ocean and atmosphere. Exiting\n",
  numprocs,(PP_NPROC_LT*PP_NPROC_LN));
  MPI_Finalize();
  exit(1);
  }
#else
  if( numprocs != (Nrows*Ncols)+(PP_NPROC_LT*PP_NPROC_LN)){
  printf("FOAM: Wrong number of processors.  Got %d, want %d for ocean + %d for rest = %d total. Exiting\n",
  numprocs,(Nrows*Ncols),(PP_NPROC_LT*PP_NPROC_LN),
  (Nrows*Ncols)+(PP_NPROC_LT*PP_NPROC_LN));
  MPI_Finalize();
  exit(1);
  }
#endif

#ifdef PROFILE
  mark_event_C(1,1);
#endif

#ifdef UPSHOT
  MPE_Init_log();
  if (myid == 0) {
    MPE_Describe_state(15,16,"foammain","lavender");
    MPE_Describe_state(1,2,"Atm","green:vlines3");
    MPE_Describe_state(3,4,"Coupler","red:gray3");
    MPE_Describe_state(5,6,"Ocean","blue:light_gray");
    MPE_Describe_state(17,18,"CplOcn","firebrick");
    MPE_Describe_state(7,8,"Ocn2Cpl","yellow:gray");
    MPE_Describe_state(9,10,"Cpl2Ocn","purple4:dimple3");
    MPE_Describe_state(11,12,"OvrlpGath","cyan:vlines4");
    MPE_Describe_state(13,14,"Bcast","maroon:white");
    MPE_Describe_state(19,20,"OcnGath","SeaGreen:white");
    MPE_Describe_state(21,22,"GenGath","navy:white");
  }
  MPE_Log_event(15,myid,"startfoam");
#endif

  fp=fopen(argv[1],"r");
  fscanf(fp,"%s %d\n",param,&rsfrq);
  fscanf(fp,"%s %d\n",param,&hsfrq);
  fscanf(fp,"%s %s\n",param,prefix);
#ifdef SIMPLE_OUTPUT
  fscanf(fp,"%s %s\n",param,storage);
#endif
  fscanf(fp,"%s %s\n",param,time_inv);
  fscanf(fp,"%s %d\n",param,&finished);
  fscanf(fp,"%s %d\n",param,&rnlng);
  fclose(fp);

  if(rsfrq > rnlng) rsfrq=rnlng;
  if (myid == 0) {
  fprintf(stdout,"\n************************************************\n");
  fprintf(stdout,"              This is FOAM 1.5                  \n\n");
    fprintf(stdout,"Restart Frequency: %d\n",rsfrq);
    fprintf(stdout,"History Frequency: %d\n",hsfrq);
    fprintf(stdout,"Run Length: %d\n",rnlng);
    fprintf(stdout,"Prefix: %s\n",prefix);
    fprintf(stdout,"Finished: %d\n",finished);
#ifdef SIMPLE_OUTPUT
    /*fprintf(stdout,"Storage: %s\n",storage);*/
    fprintf(stdout,"Storage directory currently not supported.\n Writing output to subdirectories history and restart under %s\n",prefix);
#endif
    fprintf(stdout,"Time invariant data directory: %s\n",time_inv);
  }

  timeinvlen=strlen(time_inv);

/*  if(myid == 0){
  sprintf(buff,"ln -sf %s data",time_inv);
  fprintf(stdout,"Executing system command %s\n",buff);	
  system(buff);
  }
  */

#ifdef SIMPLE_OUTPUT
  sprintf(lastoutname,"");

  if (myid == 0) {
    sprintf(buff,"ln -sf restart/atmos/restart.atmos.%7.7d %s/rest",finished,prefix);
    system(buff);

    sprintf(buff,"ln -sf restart/atmos/restart.atmos.%7.7d.A %s/rest.A",finished,prefix);
    system(buff);

#ifdef COLOCATED_OCEAN
/*    sprintf(buff,"ln -sf restart/ocean/restart.ocean.%7.7d %s/om3.restart",finished,prefix);
    system(buff);
*/
#endif

  }

/* since all coupler nodes read the restart file, set file links on
  each node for systems like chiba.mcs which don't have a true global file
  system to run from. 
    sprintf(buff,"ln -sf restart/coupl/restart.coupl.%7.7d %s/coupler.restart",finished,prefix);
    system(buff); */

#endif
  if (myid == 0) {
  fprintf(stdout,"     Starting component models\n\n");
  }

/**************************************************************/
/*  Colocated case:  run all components on the same nodes */
/**************************************************************/
#ifdef COLOCATED_OCEAN

MPI_Comm_dup(MPI_COMM_WORLD,&split_comm);
MPI_Comm_dup(MPI_COMM_WORLD,&coupler_ocean);

#ifdef UPSHOT
MPE_Log_event(16,myid,"startcomponents");
#endif

sprintf(prefix,"%s/",prefix);

#ifdef UPSHOT
MPE_Log_event(5,myid,"startOcn");
#endif

ocean_init(myid);

#ifdef UPSHOT
MPE_Log_event(6,myid,"EndOcn");
#endif

#ifdef UPSHOT
MPE_Log_event(1,myid,"startAtm");
#endif

#ifdef RS6000
pccm3uw(&rsfrq,&hsfrq,&rnlng,&finished,time_inv,&timeinvlen,&split_comm);
#else
# ifdef T3E
PCCM3UW(&rsfrq,&hsfrq,&rnlng,&finished,time_inv,&timeinvlen,&split_comm);
# else
pccm3uw_(&rsfrq,&hsfrq,&rnlng,&finished,time_inv,&timeinvlen,&split_comm);
# endif
#endif

#ifdef UPSHOT
MPE_Log_event(2,myid,"EndAtm");
#endif

#ifdef UPSHOT
MPE_Log_event(5,myid,"startOcn");
#endif

ocean_end();

#ifdef UPSHOT
MPE_Log_event(6,myid,"EndOcn");
#endif


/**************************************************************/
/*  Default case:  run atmosphere/land/sea ice on one subdomain
		   run ocean on the rest    */
/**************************************************************/

#else

split_point=PP_NPROC_LT*PP_NPROC_LN;

if (myid < split_point)
  color=0;  /* atmosphere/coupler node */
else
  color=1;  /* ocean node */

MPI_Comm_split(MPI_COMM_WORLD,color,myid,&split_comm);
#ifdef UPSHOT
MPE_Log_event(16,myid,"startcomponents");
#endif

if (color == 0) {

  MPI_Intercomm_create(split_comm,0,MPI_COMM_WORLD,split_point,98,&coupler_ocean);

#ifdef UPSHOT
MPE_Log_event(1,myid,"startAtm");
#endif

#ifdef RS6000
  pccm3uw(&rsfrq,&hsfrq,&rnlng,&finished,time_inv,&timeinvlen,&split_comm);
#else
# ifdef T3E
  PCCM3UW(&rsfrq,&hsfrq,&rnlng,&finished,time_inv,&timeinvlen,&split_comm);
# else
  pccm3uw_(&rsfrq,&hsfrq,&rnlng,&finished,time_inv,&timeinvlen,&split_comm);
# endif
#endif
#ifdef UPSHOT
MPE_Log_event(2,myid,"endAtm");
#endif

}
else {
  MPI_Intercomm_create(split_comm,0,MPI_COMM_WORLD,0,98,&coupler_ocean);
#ifdef UPSHOT
MPE_Log_event(5,myid,"startOcn");
#endif

  MPI_Comm_rank(split_comm,&myoid);

/*  if (myoid == 0) {
    sprintf(buff,"ln -sf restart/ocean/restart.ocean.%7.7d %s/om3.restart",finished,prefix);
    system(buff);
    }
*/

  sprintf(prefix,"%s/",prefix);
  ocean_init(myid);
 
  total_ocean_steps = rnlng * OCNTSPDAY;

  for(ocean_step_num = 1; ocean_step_num <= total_ocean_steps; ocean_step_num++)
  {
    ocean_step();
  }
   ocean_end();
#ifdef UPSHOT
MPE_Log_event(6,myid,"EndOcn");
#endif
}

#endif

#ifdef UPSHOT
MPE_Log_event(15,myid,"restartfoam");
#endif

#ifdef SIMPLE_OUTPUT
  MPI_Barrier(MPI_COMM_WORLD);
  if (myid == 0) 
  {
    sprintf(buff,"%s/Finished",prefix);
    endfile=fopen(buff,"w");  
    close(endfile);
  }

#endif

#ifdef PROFILE
  mark_event_C(1,0);

  sprintf(buff,"%s/Profile.%4.4d",prefix,myid);
  profile_file=fopen(buff,"w");
/* write the data */
  mark_event_C(0,2);
  close(profile_file);
#endif

#ifdef UPSHOT
  MPE_Log_event(16,myid,"endfoam");
  sprintf(logfilename,"%s/Logfile",prefix);
  MPE_Finish_log(logfilename);
#endif
  if (myid == 0) {
  sprintf(buff,"pccm.out.%7.7d",finished+rnlng);
  sprintf(buff2,"mv pccm.out.0000 %s", buff);

  system(buff2);
  fprintf(stdout,"\n            FOAM has ended                    \n");
  fprintf(stdout,"************************************************\n");
 }

  MPI_Finalize();
  exit(0);
}
