#include "params.h"
/* $Header: /homes/jacob/foam-cvs/om3coupled/om3cplslab.c,v 1.5 2002/08/27 22:39:18 jacob Exp $ */
/*version for parallel machines*/

/* 
slab model replacement for om3coupled.c
Can act as a mixed layer or supply fixed sst's to atmosphere.
*/

/*
   Define file names for restart init and save files if needed.
   This should become part of an integrated model control procedure
   ASAP.
*/
#define RestartRun
#define RestartSave
#define RestartRunName "om3.restart"
#define RestartSaveName "om3.save"



#include <stdio.h>
#include <fcntl.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include "mpi.h"


#define MixDepth 50.
#define Np (Ncols*Nrows)
#define Lx 4.0e7  /* Global domain from 0W - 360W */
#define Ly 2.0e7  /* 90S - 90N */
#define Nx 128
#define Ny 128
#define Nz 16
#define Mx ((Nx/Ncols)+2)
#define My ((Ny/Nrows)+2)
#define Mz Nz

#define Delx (Lx / Nx)
#define Dely (Ly / Ny)
#define Diff_Mks 8.0e3
#define Diff_Ratio 0.5
#define G 0.50
#define N_Split 3
#define Sub_Split 8

#define Delt (6.0*3600.0)

int num_nodes, my_address;
int row,col,north,south,east,west;
int ofile;
int hold_tme,milliclock();
extern int hold_tme;
extern char time_inv[256];


float uvts_levels[Mz],w_levels[Mz]; /* read from geometry file */
float h[Mz][My][Mx]; /* level thickneses, 0 if no water */

float strx[My][Mx],stry[My][Mx]; /* wind stress */ 
float annual_salt[Mz][My][Mx],annual_temp[Mz][My][Mx]; /* mean salt and temp for spin up */

float u[Mz][My][Mx],ul[Mz][My][Mx],ux[Mz][My][Mx],dtu[Mz][My][Mx]; /* State Vars */
float v[Mz][My][Mx],vl[Mz][My][Mx],vx[Mz][My][Mx],dtv[Mz][My][Mx];
float t[Mz][My][Mx],tl[Mz][My][Mx],tx[Mz][My][Mx],dtt[Mz][My][Mx];
float s[Mz][My][Mx],sl[Mz][My][Mx],sx[Mz][My][Mx],dts[Mz][My][Mx];
float ptop[My][Mx],ptopl[My][Mx],ptopx[My][Mx]; /* surface pressure */
/* end of state */

float w[Mz][My][Mx],p[Mz][My][Mx],dzt[Mz][My][Mx],dzs[Mz][My][Mx]; /* derived state variables */
float ubaro[My][Mx],vbaro[My][Mx],dtubaro[My][Mx],dtvbaro[My][Mx]; /* barotropic split variables */
float ubstar[My][Mx],vbstar[My][Mx],ptstar[My][Mx]; /* Euler step temps */

float f[My],taux[My][Mx],tauy[My][Mx]; /* Derived geometries */
float salt_for[My][Mx],temp_for[My][Mx];

float to_coupler[2][My-2][Mx-2], from_coupler[4][My-2][Mx-2];

float base_norm[Mz]; /* base stratification for diagnostic studies */
float base_t[Nz]={0.659178,0.851181,1.226342,1.767641,
		  2.272589,2.576623,2.979796,3.560002,
		  4.410163,5.626800,7.577657,9.099754,
		  10.793540,12.250330,13.498031,13.787509};
float base_s[Nz]={34.487881,34.498528,34.534710,34.549000,
		  34.524113,34.501549,34.473404,34.445309,
		  34.432766,34.461227,34.590309,34.686813,
		  34.674068,34.547871,34.286461,34.085960};

float uwmask[Ny][Nx];
float sstinit[Ny][Nx];
float work[My][Mx];
char buff[256];
char resname[256];
char hisname[256];
char srbuff[Mx*My*8*10];
FILE *of,*ef;

/* sea ice formation variables */
#define izmxice 1
float qice[Mz][My][Mx];
float qflx[My][Mx];
float Cp,Lathf,Tf,potice;
float rhoref, Sref;
float CtoK; /* celcius to kelvin */


FILE *hfile;

MPI_Datatype mpi_edge_type;

extern MPI_Comm split_comm, coupler_ocean;
extern char prefix[100], storage[100];
extern int rnlng,hsfrq,rsfrq,finished;

#ifdef OMPICS
unsigned char picbuf[My-2][Mx-2];
#endif

void init_state(); /* initialization routines */
void read_slab_geometry();
void read_stress();
void read_annual_temp();
void read_annual_salt();

#ifdef SST_FROM_FILE
void read_sst_data();
float osstint(int j, int i, int timestep);
float sstm[12][My-2][Mx-2];

/* set step to Jan 1 */
#define Start_step 0
/*#define Start_step 244*4.*/
#define Restore (1.0/(30.0 * 24.0 * 3600.0))
#endif


void time_step(); /* model run time code */
float density(float s, float t, int iz);
void wrap_q(float a[My][Mx]);
void wrap_q3(float a[My][Mx], float b[My][Mx], float c[My][Mx]);
void wrap_qz(float a[Mz][My][Mx]);

void ocean_to_coupler(int nfields,float data[][My-2][Mx-2],int tag);
void ocean_from_coupler(int nfields,float data[][My-2][Mx-2],int tag);

#ifdef OMPICS
void write_picture(float a[My][Mx],float vmax); /* movie and history file output
routines */
void fwrite_picture(float a[My][Mx],float vmax);
#endif
void write_history(float a[Mz][My][Mx]);
void write_history_slice(float a[Mz][My][Mx], int iz);

float f_max(float a[My][Mx]); /* non-timestep utility routines */
void node_timer_clear(int n);
void node_timer_start(int n);
void node_timer_stop(int n);
float node_timer_busy(int n);
void load_restart(char* resname);
void read_restart_slice(float a[My][Mx], FILE *tfile);
void save_restart();
void write_restart_slice(float a[My][Mx], FILE *tfile);

int istep,iblock,N_Blocks,N_Steps,N_File;
#ifdef SST_FROM_FILE
   int step; /*Holds total number of steps since beginning of calendar year */
#endif
int history_rate,restart_rate,history_count,restart_count;
int overall_proc,just_did_restart;
float max_u, max_v, max_t, max_p, max_s;
float max_an_t,max_an_s, max_sx, max_sy;
float max_tf, max_sf, max_tx, max_ty;


void ocean_init(overall_proc)
{
   int relfrq;

   int ellen,elcnt,stride;

   N_Steps=((int)((hsfrq*24.0*3600.0/Delt)+0.499));
   N_Blocks=rnlng/hsfrq;

   relfrq = rsfrq/hsfrq;

#ifndef COLOCATED_OCEAN
/* In the colocated case, only one buffer can be attached, and the one
used in the atmos model is kept because it is necessarily larger */
   MPI_Buffer_attach(srbuff, Mx*My*8*10);
#endif

   ellen = sizeof(float); /* make mti type for e-w edge */
   elcnt = My;
   stride = Mx * ellen;
   MPI_Type_vector(elcnt, ellen, stride, MPI_CHAR, &mpi_edge_type);
   MPI_Type_commit(&mpi_edge_type);


   /* Get self address and number of nodes.*/
   

   fflush(stdout);


   MPI_Comm_rank(split_comm, &my_address);
   MPI_Comm_size(split_comm, &num_nodes);

#ifdef UPSHOT
   overall_proc=PP_NPROC_LT*PP_NPROC_LN+my_address+1;
   MPE_Log_event(5,overall_proc,"StrtOcn");
#endif


   /* open stdout file on sphome for root node, flush on other nodes */


   if(my_address <= 0 )
   {
     sprintf(buff,"%som3.out.%d_%d.%d",prefix,Nrows,Ncols,my_address);
     if((of = freopen(buff, "w", stdout)) == NULL)  exit(89);
   }
   else
   {
/*     sprintf(buff,"%som3.out.%d_%d.%d",prefix,Nrows,Ncols,my_address);
     if((of = freopen(buff, "w", stdout)) == NULL)  exit(89); */

     if((of = freopen("/dev/null", "w", stdout)) == NULL) exit(89);
   }
#ifndef COLOCATED_OCEAN
   sprintf(buff,"%som3.err.%d_%d.%d",prefix,Nrows,Ncols,my_address);
   if((ef = freopen(buff, "w", stderr)) == NULL)  exit(89); 
#endif



   /* open picture file */

#ifdef OMPICS 
   if(my_address == 0) {
       sprintf(buff,"%som3_%d_%d.cmpics",prefix,Nrows,Ncols);
       ofile = global_open(buff, O_CREAT | O_WRONLY, 0666);
   }
#endif







   /* validate machine geometry */

   if(Np != num_nodes) {
     printf("Node number mismatch - needed %d got %d ...exiting\n",Np,num_nodes);
     exit(89);
   }
   else
     printf("THIS IS THE SLAB MODEL VERSION OF OM3\n Started on %d nodes.\n",num_nodes);
   fflush(of);


   /* Calculate node geometery and neighbor node numbers */

   if(my_address == 0)
      printf("Ncols,Nrows %d %d Mx,My: %d %d\n",Ncols,Nrows,Mx,My);

   row = my_address / Ncols;
   col = my_address % Ncols;

#ifdef SST_FROM_FILE
     printf("Using SST from file instead of computed slab temps\n");
   read_sst_data();
#endif

   if(col == 0) west = my_address + (Ncols-1); /* 2-D torus */
   else west = my_address - 1;

   if(col == Ncols-1) east = my_address - (Ncols-1);
   else east = my_address + 1;

   if(row == 0) south = my_address + (Nrows-1)*Ncols;
   else south = my_address - Ncols;

   if(row == Nrows-1)  north = my_address - (Nrows-1)*Ncols;
   else north = my_address + Ncols;

   /* Initialization processing */

   node_timer_clear(0);
   node_timer_start(0);

   read_slab_geometry();
#ifndef COUPLED
   read_stress();
   read_annual_temp();
   read_annual_salt();
#endif

   node_timer_stop(0);

   if(my_address == 0)
      printf("Forcing and geometry read time=%f [secs]\n",node_timer_busy(0));

#ifdef OMPICS
   write_picture(strx,0.1);
   write_picture(stry,0.1);
   write_picture(&(annual_temp[Nz-1][0][0]),10.0);
   write_picture(&(annual_salt[Nz-1][0][0]),2.0);
#endif

   max_sx = f_max(strx);
   max_sy = f_max(stry);
   max_an_t = f_max(&(annual_temp[Nz-1][0]));
   max_an_s = f_max(&(annual_salt[Nz-1][0]));

   if(my_address == 0)
   {
	 printf("Max sx: %f, sy: %f, ann. temp, %f, ann. salt %f\n",
            max_sx,max_sy,max_an_t,max_an_s);
   }
   fflush(of);

   node_timer_clear(0);
   node_timer_start(0);

   init_state();

   node_timer_stop(0);

   if(my_address == 0)
      printf("State init time=%f [secs] ,\n",node_timer_busy(0));
   fflush(of);

#ifdef RestartRun
# ifdef OCEAN_ALONE
   sprintf(resname,"restart/restart.ocean.%7.7d", finished);
# else
   sprintf(resname,"restart/ocean/restart.ocean.%7.7d", finished);
# endif
   load_restart(resname);
   if(my_address == 0)
   {
     printf("Model initialized from restart file: %s\n",resname);
   }
   fflush(of);
#endif

   max_u = f_max(&(u[Nz-1][0]));
   max_v = f_max(&(v[Nz-1][0]));
   max_t = f_max(&(t[Nz-1][0]));
   max_p = f_max(&(ptop[0]));

   if(my_address == 0)
   {
      printf("Max u: %f, v: %f, t: %f, p: %f\n",max_u,max_v,max_t,max_p);
      printf("Delt = %f seconds, Block time = %f days.\n",Delt,Delt*N_Steps/(3600.0*24.0));
   }
   fflush(of);

   istep = 0;
   restart_rate = rsfrq * (int)(3600.*24./Delt);
   history_rate = hsfrq * (int)(3600.*24./Delt);
   restart_count = 0;
   history_count = 0;

}  /* This is the end of ocean_init */

#ifndef COLOCATED_OCEAN
void ocean_step()
#else
# ifdef RS6000
void ocean_step()
# else
#  ifdef T3E
void OCEAN_STEP()
#  else
void ocean_step_()
#  endif
# endif
#endif

{
   int hiswrite = 0;
   int reswrite = 0;
   int suffix;
   int ix,iy,iz;

   istep = istep + 1;
/* This section tests whether it is time for history and restart writes */
  if ((istep % restart_rate) == 0)
   {
     restart_count++;
     reswrite = restart_count;
   }

   if ((istep % history_rate) == 0)
   {
     history_count++;
     hiswrite = history_count;
   }

      node_timer_clear(0);
      node_timer_start(0);

/*   for(iblock = 0; iblock < N_Blocks; iblock++)
   {
      for(istep = 0; istep < N_Steps; istep++)
      { */
#ifdef UPSHOT
        MPE_Log_event(6,overall_proc,"EndOcn");
        MPE_Log_event(9,overall_proc,"StrtRecv");
#endif
        ocean_from_coupler(4,from_coupler,200);

#ifdef UPSHOT
        MPE_Log_event(10,overall_proc,"EndRecv");
        MPE_Log_event(5,overall_proc,"StrtOcn");
#endif


/* Compute forcing parameters here. */
        for(iy = 1; iy < My-1; iy++)
        {
           for(ix = 1; ix < Mx-1; ix++)
           {
                  if(h[Nz-1][iy][ix] > 1.0)
                  {

                     temp_for[iy][ix] = from_coupler[0][iy-1][ix-1]/(rhoref * h[Nz-1][iy][ix] * Cp);
                     salt_for[iy][ix] = -s[Nz-1][iy][ix]*from_coupler[1][iy-1][ix-1]/(rhoref * h[Nz-1][iy][ix]);
                  }
                  else
                  {
                     temp_for[iy][ix] = 0.0;
                     temp_for[iy][ix] = 0.0;
                  }
           }
        }

        wrap_q(taux);
        wrap_q(tauy);
        wrap_q(temp_for);
        wrap_q(salt_for);

       max_tf = f_max(&(temp_for[0]));
       max_sf = f_max(&(salt_for[0]));
       max_tx = f_max(&(taux[0]));
       max_ty = f_max(&(tauy[0]));

       if(my_address == 0)
       {
             printf("Max tempf: %e, saltf: %e, taux: %e, tauy: %e\n",max_tf,max_sf,max_tx,max_ty);
       }
       fflush(of);

#ifdef OMPICS_EX
         write_picture(&(temp_for[0][0]),fabs(max_tf));
         write_picture(&(salt_for[0][0]),fabs(max_sf));
         write_picture(&(taux[0][0]),fabs(max_tx));
         write_picture(&(tauy[0][0]),fabs(max_ty));
#endif

/* Return ocean state back to coupler */
        for(iy = 1; iy < My-1; iy++)
        {
           for(ix = 1; ix < Mx-1; ix++)
           {
#ifdef SST_FROM_FILE
/* drive the atmosphere directly with the climo sst */
              step = Start_step + finished * 4 + (istep + iblock * N_Steps);
              to_coupler[0][iy-1][ix-1] = osstint(ix-1,iy-1,step)+273.12;/* SST */
              to_coupler[1][iy-1][ix-1] = 0.0; /* melting potential */
#else
              to_coupler[0][iy-1][ix-1] = t[Nz-1][iy][ix]+273.12;/* SST */
# ifdef CSIMICE
              to_coupler[1][iy-1][ix-1] = qflx[iy][ix]; /* melt potential */
# else
              to_coupler[1][iy-1][ix-1] = 0.;
# endif
#endif
           }
        }
/* zero qice and qflx */
   for (iz =  0; iz < Mz; iz++)
   for(iy = 0; iy < My; iy++)
    for(ix = 0; ix < Mx; ix++)
     qice[iz][iy][ix]=0.;

   for(iy = 0; iy < My; iy++)
    for(ix = 0; ix < Mx; ix++)
     qflx[iy][ix]=0.;


#ifdef UPSHOT
        MPE_Log_event(6,overall_proc,"EndOcn");
        MPE_Log_event(7,overall_proc,"StrtSend");
#endif
      
        ocean_to_coupler(2,to_coupler,300);

#ifdef UPSHOT
        MPE_Log_event(8,overall_proc,"EndSend");
        MPE_Log_event(5,overall_proc,"StrtOcn");
#endif

	    time_step();
	    fflush(of);
    /*  }  This is the end of NStep loop */

       max_t = f_max(&(t[Nz-1][0]));
       max_s = f_max(&(s[Nz-1][0]));
       max_u = f_max(&(u[Nz-1][0]));
       max_v = f_max(&(v[Nz-1][0]));
       if(my_address == 0)
             printf("istep %d Max t: %e, s: %e, u: %e, v: %e \n",istep,max_t,max_s,max_u,max_v);

     if(hiswrite >0) {
      node_timer_stop(0);
      if(my_address == 0)
	    printf("Timestep calculation time=%f [secs],\n",node_timer_busy(0));
      node_timer_clear(0);
      fflush(of);

      max_u = f_max(&(u[Nz-1][0]));
      max_v = f_max(&(v[Nz-1][0]));
      max_t = f_max(&(t[Nz-1][0]));
      max_p = f_max(&(ptop[0]));

      if(my_address == 0)
      {
	    printf("Max u: %f, v: %f, t: %f, p: %f\n",max_u,max_v,max_t,max_p);
      }
      fflush(of);

     suffix = finished + hsfrq * hiswrite;

/* Write surface history file */
     if(my_address == 0) {

#ifdef SIMPLE_OUTPUT
        sprintf(hisname,"history.osstp.%7.7d", suffix);
        sprintf(buff,"history/ocean/%s",hisname);
#else
        sprintf(buff,"%shistory.ocean.%7.7d",prefix,suffix);
#endif

        hfile = fopen(buff,"wb");

/* there's always one time slice per file */
        fprintf(hfile,"%d\n",N_File);
        printf("Opening Surface History File: %s\n",buff);
     }


/* this is the slab model so just write slices */
      if(iblock >= 0)
      {

        write_history_slice(u,Mz-1);
        write_history_slice(v,Mz-1);
        write_history_slice(w,Mz-2);
        write_history_slice(t,Mz-1);
        write_history_slice(s,Mz-1);
        write_history_slice(p,Mz-1);

      }

      if (my_address==0)
      {
        fclose(hfile);
      }

  /* }  The NBlock loop */

#ifdef OMPICS
   if(my_address==0)
   {
      close(ofile);
   }
#endif

fflush(of);

#ifdef UPSHOT
        MPE_Log_event(6,overall_proc,"EndOcn");
#endif
  } /* End of if history write */



      if(reswrite > 0)
      {
#ifdef RestartSave
          suffix = finished + rsfrq * reswrite;
          save_restart(suffix);
#ifdef SIMPLE_OUTPUT
          just_did_restart = suffix;
#endif
#endif
      }

} /* This is the end of ocean_step */


void ocean_end()
{
fflush(of);

#ifdef UPSHOT
        MPE_Log_event(6,overall_proc,"EndOcn");
#endif

}


float f_max(a)
float a[My][Mx];
{
   int ix,iy,maxix,maxiy;
   float max,gmax,gmin,maxamp;

   maxamp = -1.0;

   for(iy = 1; iy < My-1; iy++)
   {
      for(ix = 1; ix < Mx-1; ix++)
      {
	     if((fabs(a[iy][ix]) > maxamp) &&(h[Nz-1][iy][ix] > 1.0))
         {
             max = a[iy][ix];
             maxamp = fabs(a[iy][ix]);
             maxix = ix;
             maxiy = iy;
         }
      }
   }
/*
   if(my_address == 0) printf("Max ix,iy: %d %d %f\n",maxix,maxiy,max);
*/

   MPI_Allreduce(&max, &gmax, 1, MPI_FLOAT, MPI_MAX, split_comm);
   MPI_Allreduce(&max, &gmin, 1, MPI_FLOAT, MPI_MIN, split_comm);



   if(gmax > -gmin) return gmax;
   else return gmin;
}

float f_total(a)
float a[My][Mx];
{
   int ix,iy;
   float total,gtotal;

   total = 0.0;

   for(iy = 1; iy < My-1; iy++)
   {
      for(ix = 1; ix < Mx-1; ix++)
      {
         {
            if(h[Nz-1][iy][ix] > 1.0)
              total = total + a[iy][ix];
         }
      }
   }
   MPI_Allreduce(&total,&gtotal,1,MPI_FLOAT, MPI_SUM, split_comm);


   return gtotal;
}

#ifdef OMPICS

/*
 * We write a picture one node at a time in cm5 order.
 * In this (non piofs) version each write is staged throught the
 * root node.
 */
void write_picture(float a[My][Mx],float vmax)
{
   int ix,iy;
   int len,pixel_val;
   float val,scl;
   char *bptr,buf[200];

   fflush(of);

   /* write header with Nx, Ny, Ncols, Nrows */

   sprintf(buf,"%d %d %d %d\n",Nx,Ny,Ncols,Nrows);
   len = strlen(buf);
   if(my_address == 0) write(ofile,buf,len);

   scl = 40.0/(vmax);

   for(iy = 1; iy < My-1; iy++)
   {
     for(ix = 1; ix < Mx-1; ix++) /* fill buffer in parallel */
     {
	   val = scl * a[iy][ix];
	   pixel_val = ((int) fabs(val+0.5)) % 40;

	   if(val > 0)
         pixel_val = 41 + pixel_val; /* red colors in std map */
	   else
         pixel_val = 81 + pixel_val; /* blue colrs in std map */

	   if(h[Nz-1][iy][ix] > 1.0)
         picbuf[iy-1][ix-1] = pixel_val;
       else
         picbuf[iy-1][ix-1] = 0; /* zero out dry cells */
    }
  }
  global_write();
}

void fwrite_picture(float a[My][Mx],float vmax)
{
   int ix,iy;
   int len,pixel_val;
   float val,scl;
   char *bptr,buf[200];

   /* write header with Nx, Ny, Ncols, Nrows */
   sprintf(buf,"%d %d %d %d\n",Nx,Ny,Ncols,Nrows);
   len = strlen(buf);
   if(my_address == 0) write(ofile,buf,len);

   scl = 40.0/(vmax);

   for(iy = 1; iy < My; iy++)
   {
      for(ix = 1; ix < Mx-1; ix++) /* fill buffer in parallel */
      {
        if(h[Nz-1][iy][ix+1] > 1.0)
	      work[iy][ix] = 0.5 * (a[iy][ix] + a[iy][ix+1]);
        else
          work[iy][ix] = a[iy][ix];
      }
   }

   for(iy = 1; iy < My-1; iy++)
   {
     for(ix = 1; ix < Mx-1; ix++) /* fill buffer in parallel */
     {
       if(h[Nz-1][iy+1][ix] > 1.0)
	     val = scl * 0.5 * (work[iy][ix] + work[iy+1][ix]);
       else
         val = scl * work[iy][ix];
	 
	   pixel_val = ((int) fabs(val+0.5)) % 40;

	   if(val > 0)
         pixel_val = 41 + pixel_val; /* red colors in std map */
	   else
         pixel_val = 81 + pixel_val; /* blue colrs in std map */

	   if(h[Nz-1][iy][ix] > 1.0)
         picbuf[iy-1][ix-1] = pixel_val;
       else
         picbuf[iy-1][ix-1] = 0; /* zero out dry cells */
     }
  }
  global_write();
}

global_write()
{
   int i, j, k, l, ik, jl, ierr;
   int tt,number;
   unsigned char t1, t2;
   MPI_Status status;



   if(my_address == 0) {
       write(ofile, picbuf, (Mx-2)*(My-2));
       for(i=0; i<Nrows; i++)
       {
	     for(j=0; j<Ncols; j++)
         {
          if (i != 0 || j!= 0) 
          {
            MPI_Recv(picbuf, (Mx-2)*(My-2), MPI_CHAR, i*Ncols+j,
		      99, split_comm, &status);
/*            MPI_Get_count(&status,MPI_CHAR,&number);
            printf("Source:%d\n",status.MPI_SOURCE);
            printf("Tag:%d\n",status.MPI_TAG);
            printf("Size:%d\n",number);
            printf("Receiving:8192\n"); */
       	    write(ofile, picbuf, (Mx-2)*(My-2));
          }
	     }
       }
   }
   else
   {
     ierr = MPI_Send(picbuf, (Mx-2)*(My-2), MPI_CHAR, 0, 99, split_comm);
/*     printf("Sending:8192\n"); */
     if(ierr != 0) {
       printf("Bsend error %d\n", ierr);
       exit(1);
     }
   }
}
#endif

/* History write routines */

void write_history(float a[Mz][My][Mx])
{
  int iz;

  for(iz = 0; iz < Mz; iz++)
  {
     write_history_slice(a,iz);
  }
}

unsigned char hist_out[My][Mx][2];

void write_history_slice(float a[Mz][My][Mx], int iz)
{
  int ix,iy,ival,ic;
  float xx,lxmin,lxmax,xmin,xmax,scale;
  int jrow,jcol,jy;

  unsigned char row_out[Mx-2][2];
  MPI_Status status;
  int ierr;


  lxmin = 1.0e10;
  lxmax = -1.0e10;

  for(iy = 1; iy < My-1; iy++)
  {
    for (ix = 1; ix < Mx-1; ix++)
    {
      if(h[iz][iy][ix] > 1.0)
      {
         xx = a[iz][iy][ix];
         if(xx < lxmin) lxmin = xx;
         if(xx > lxmax) lxmax = xx;
      }
    }
  }


   MPI_Allreduce(&lxmax, &xmax, 1, MPI_FLOAT, MPI_MAX, split_comm);
   MPI_Allreduce(&lxmin, &xmin, 1, MPI_FLOAT, MPI_MIN, split_comm);



  if(my_address == 0)
  {
     fprintf(hfile,"%f %f\n",xmin,xmax);
  }
  xx = xmax-xmin;
  if(xx < 1.0e-10) xx=1.0e-10;
  scale = 60000.0/xx;

  for(iy = 1; iy < My-1; iy++)
  {
    for (ix = 1; ix < Mx-1; ix++)
    {
      if(h[iz][iy][ix] > 1.0)
      {
         xx = a[iz][iy][ix];
         ival = 100 + scale*(xx-xmin);
      }
      else ival = 0;

      hist_out[iy][ix][0] = ival/256;
      hist_out[iy][ix][1] = ival % 256;
    }
  }
   for(jy = 0; jy < Ny; jy++) /* write xy field in natural order*/
   {
     jrow = jy/(My-2);
     iy = jy - jrow*(My-2) + 1;
     for(jcol = 0; jcol < Ncols; jcol++) /* write in column order */
     {
        if((col == jcol) && (row == jrow)) /* Select source processor */
        {
           ierr = MPI_Bsend(&(hist_out[iy][1][0]), 2*(Mx-2),
                    MPI_CHAR, 0, 99, split_comm);
/*         printf("Sending:256\n"); */
           if(ierr != 0)
           {
              printf("Bsend error in write_history_slice: %d\n", ierr);
              fflush(of);
              exit(1);
           }
        }
        if(my_address == 0)
        {
           MPI_Recv(row_out, 2*(Mx-2), MPI_CHAR, jrow*Ncols+jcol,
		      99, split_comm, &status);
/*           printf("Receiving:256\n"); */
        
           fwrite(row_out,sizeof(char),2*(Mx-2),hfile);
        }
        MPI_Barrier( split_comm);

     }
   }
}

void read_slab_geometry()
{
   int ix,iy,iz,jx,jy,jz;
   int lon,lat;
   FILE *tfile;
   int tt;
   unsigned char in[2*Nx], *inptr, t1, t2;

   if(my_address == 0)
   {
      sprintf(buff,"%s%s",time_inv,"/mask.128x128");
      tfile = fopen(buff,"r");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on slab geometry failed.\n");
      exit(89);
   }

     for(jy = 0; jy < Ny; jy++)
       for(iy = 0; iy < Nx; iy++) uwmask[jy][iy]=0.;

   if(my_address == 0){
     for(jy = 0; jy < Ny; jy++)
       for(iy = 0; iy < Nx; iy++)
	 fscanf(tfile," %d %d %f%*c",&lat,&lon,&uwmask[jy][iy]);
    }	 

   MPI_Bcast(uwmask,Ny*Nx,MPI_FLOAT,0,split_comm);


   /* read uwmask and use it to make 3d topo field */
   for(jz = Nz-1; jz >= 0; jz--) 
   {
     for(jy = 0; jy < Ny; jy++)
     {
      iy = jy - row*(My-2) + 1;
      for(jx = 0; jx < Nx; jx++)
      {
         ix = jx - col*(Mx-2) + 1;
         if((ix >= 1) && (ix < Mx-1) && (iy >= 1) && (iy < My-1))
         {
	   if(uwmask[jy][jx] < 1.)
            h[jz][iy][ix] = MixDepth;
	    
         }
      }
     }
    }

   if(my_address == 0) fclose(tfile);

   wrap_qz(h);


/* make sure the north pole sees land to north and south pole
  sees land to south to prevent interaction from pole to pole
  caused by the torus communication topology */

/* see if this processor is handling the north pole. */
   if((My-2)*(row+1)==Ny){
    for(iz = 0; iz < Mz; iz++)
     for(ix = 0; ix < Mx; ix++)
      h[iz][My-1][ix]=0.;
   }

/* see if this processor is handling the south pole. */
   if(row==0){
    for(iz = 0; iz < Mz; iz++)
     for(ix = 0; ix < Mx; ix++)
      h[iz][0][ix]=0.;
   }

}

void read_stress()
{
   int ix,iy,jx,jy;
   FILE *tfile;
   int tt;
   unsigned char in[2*Nx], *inptr, t1, t2;

   if(my_address == 0)
   {
      sprintf(buff,"%s%s",time_inv,"/om3.stress");
      tfile = fopen(buff,"r");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on stress failed.\n");
      exit(89);
   }


   for(jy = 0; jy < Ny; jy++) /* read x-stress field */
   {
      iy = jy - row*(My-2) + 1;
      if(my_address == 0) fread(in,sizeof(char),2*Nx,tfile);
      MPI_Bcast(in,2*Nx,MPI_CHAR,0,split_comm);
      inptr = in;

      for(jx = 0; jx < Nx; jx++)
      {
         ix = jx - col*(Mx-2) + 1;
         t1 = *inptr++; t2 = *inptr++;
         tt = t1*256 + t2;
         if((ix >= 0) && (ix < Mx) && (iy >= 0) && (iy < My))
            strx[iy][ix] = ((float)tt/10000.0)-0.3;
      }
   }

   for(jy = 0; jy < Ny; jy++) /* read y-stress field */
   {
      iy = jy - row*(My-2) + 1;
      if(my_address == 0) fread(in,sizeof(char),2*Nx,tfile);
      MPI_Bcast(in,2*Nx,MPI_CHAR,0,split_comm);
      inptr = in;

      for(jx = 0; jx < Nx; jx++)
      {
         ix = jx - col*(Mx-2) + 1;
	     t1 = *inptr++; t2 = *inptr++;
         tt = t1*256 + t2;
         if((ix >= 0) && (ix < Mx) && (iy >= 0) && (iy < My))
            stry[iy][ix] = ((float)tt/10000.0)-0.3;
      }
   }
   if(my_address == 0) fclose(tfile);
}

void read_annual_temp()
{
   int ix,iy,jx,jy,jz;
   FILE *tfile;
   int tt;
   float ftt;
   unsigned char in[2*Nx], *inptr, t1, t2;

   if(my_address == 0)
   {
      sprintf(buff,"%s%s",time_inv,"/om3.temp16");
      tfile = fopen(buff,"r");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on temp failed.\n");
      exit(89);
   }

   for(jz = Nz-1; jz >= 0; jz--) /* read 3-D temp field */
   {
     for(jy = 0; jy < Ny; jy++)
     {
      iy = jy - row*(My-2) + 1;
      if(my_address == 0) fread(in,sizeof(char),2*Nx,tfile);
      MPI_Bcast(in,2*Nx,MPI_CHAR,0,split_comm);
      inptr = in;
      for(jx = 0; jx < Nx; jx++)
      {
         ix = jx - col*(Mx-2) + 1;
	     t1 = *inptr++; t2 = *inptr++;
         tt = t1*256 + t2;
         ftt=((float)tt-10000.0)/100.0;
         if((ix >= 0) && (ix < Mx) && (iy >= 0) && (iy < My))
         {
            annual_temp[jz][iy][ix] = ftt;
         }
      }
     }
   }
   if(my_address == 0) fclose(tfile);
   wrap_qz(annual_temp);
}

void read_annual_salt()
{
   int ix,iy,jx,jy,jz;
   FILE *tfile;
   int tt;
   float ftt;
   unsigned char in[2*Nx], *inptr, t1, t2;

   if(my_address == 0)
   {
      sprintf(buff,"%s%s",time_inv,"/om3.salt16");
      tfile = fopen(buff,"r");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on temp failed.\n");
      exit(89);
   }

   for(jz = Nz-1; jz >= 0; jz--) /* read 3-D salinity field */
   {
     for(jy = 0; jy < Ny; jy++)
     {
      iy = jy - row*(My-2) + 1;
      if(my_address == 0) fread(in,sizeof(char),2*Nx,tfile);
      MPI_Bcast(in,2*Nx,MPI_CHAR,0,split_comm);
      inptr = in;
      for(jx = 0; jx < Nx; jx++)
      {
         ix = jx - col*(Mx-2) + 1;
	     t1 = *inptr++; t2 = *inptr++;
         tt = t1*256 + t2;
         ftt=((float)tt-10000.0)/100.0;
         if((ix >= 0) && (ix < Mx) && (iy >= 0) && (iy < My))
         {
            annual_salt[jz][iy][ix] = ftt;
         }
      }
     }
   }
   if(my_address == 0) fclose(tfile);
   wrap_qz(annual_salt);
}

void init_state()
{
   int ix,iy,iz,jx,jy,mx,my;
   float x,y,yeq,lat,lon,omega;
   FILE* tfile;
   float lbase_t[Mz],lbase_s[Mz],lbase_norm[Mz]; /* node arrays for base stratification*/
   Lathf = 333600.;  /* latent heat of fusion */
   Cp = 4.1e3;    /* specific heat of seawater */
   Tf = -1.8;  /* freezing point of seawater */
   Sref = 34.7;  /* reference salinity */
   rhoref = 1030.;  /* reference density */

   CtoK = 273.15;  /* celcius to kelvin */


   omega = (366.0/365.0) * 2.0*3.14159 / (3600.0*24.0);

    for (iy = 0; iy < My; iy++) /* convective adjustment */
    {
      for (ix = 0; ix < Mx; ix++)
      { int mixing;

        mixing = 1;
        while(mixing)
        {
          mixing = 0;
	      for (iz = Mz-2; iz >= 0; iz--)
	      { float d1 ,d2 , snew, tnew;
            if((h[iz][iy][ix] > 1.0) && (h[iz+1][iy][ix]) > 1.0)
            {
               d1 = density(annual_salt[iz][iy][ix],annual_temp[iz][iy][ix],iz);
               d2 = density(annual_salt[iz+1][iy][ix],annual_temp[iz+1][iy][ix],iz);
               if(d2 > (d1 + 1.0e-7))
               {
                  mixing = 1;

                  snew = (annual_salt[iz][iy][ix]*h[iz][iy][ix] +
                          annual_salt[iz+1][iy][ix]*h[iz+1][iy][ix])/
                         (h[iz][iy][ix] + h[iz+1][iy][ix]);
                  annual_salt[iz][iy][ix] = annual_salt[iz+1][iy][ix] = snew;

                  tnew = (annual_temp[iz][iy][ix]*h[iz][iy][ix] +
                          annual_temp[iz+1][iy][ix]*h[iz+1][iy][ix])/
                         (h[iz][iy][ix] + h[iz+1][iy][ix]);
                  annual_temp[iz][iy][ix] = annual_temp[iz+1][iy][ix] = tnew;
               }
            }
	      }
	    }
      }
    }


  for(iz = 0; iz < Mz; iz++)
  {
    lbase_t[iz] = 0.0;
    lbase_s[iz] = 0.0;
    lbase_norm[iz] = 0.0;
  }

  for(iz = 0; iz < Mz; iz++)
  {
    for(iy = 1; iy < My-1; iy++)
    {
      for(ix = 1; ix < Mx-1; ix++)
      {
        if(h[iz][iy][ix] > 1.0)
        {
           lbase_t[iz] += annual_temp[iz][iy][ix];
           lbase_s[iz] += annual_salt[iz][iy][ix];
           lbase_norm[iz] += 1.0;
        }
      }
    }
  }

  for(iz = 0; iz < Mz; iz++)
  {
    /*MPI_Allreduce(&(lbase_t[iz]),&(base_t[iz]),1,MPI_FLOAT, MPI_SUM, split_comm);*/
    /*MPI_Allreduce(&(lbase_s[iz]),&(base_s[iz]),1,MPI_FLOAT, MPI_SUM, split_comm);*/
    MPI_Allreduce(&(lbase_norm[iz]),&(base_norm[iz]),1,MPI_FLOAT, MPI_SUM, split_comm);


    /*base_t[iz] = base_t[iz]/base_norm[iz];*/
    /*base_s[iz] = base_s[iz]/base_norm[iz];*/
  }

  for(iz = 0; iz < Mz; iz++)
  {
    for(iy = 0; iy < My; iy++)
    {
      for(ix = 0; ix < Mx; ix++)
      {
	    ul[iz][iy][ix] = u[iz][iy][ix] = ux[iz][iy][ix] = 0.0;
	    vl[iz][iy][ix] = v[iz][iy][ix] = vx[iz][iy][ix] = 0.0;
	    tl[iz][iy][ix] = t[iz][iy][ix] = tx[iz][iy][ix] = base_t[iz];
	    sl[iz][iy][ix] = s[iz][iy][ix] = sx[iz][iy][ix] = base_s[iz];
       }
     }
  }

#ifndef RestartRun
   if(my_address == 0)
   {
      sprintf(buff,"%s%s",time_inv,"/sstinit.ascii");
      tfile = fopen(buff,"r");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on initial sst failed.\n");
      exit(89);
   }

     for(jy = 0; jy < Ny; jy++)
       for(iy = 0; iy < Nx; iy++) sstinit[jy][iy]=0.;

   if(my_address == 0){
     for(jy = 0; jy < Ny; jy++)
       for(iy = 0; iy < Nx; iy++)
	 fscanf(tfile," %d %d %f%*c",&lat,&lon,&sstinit[jy][iy]);

    fclose(tfile);
    }	 

   MPI_Bcast(sstinit,Ny*Nx,MPI_FLOAT,0,split_comm);

     for(jy = 0; jy < Ny; jy++)
     {
      iy = jy - row*(My-2) + 1;
      for(jx = 0; jx < Nx; jx++)
      {
         ix = jx - col*(Mx-2) + 1;
         if((ix >= 1) && (ix < Mx-1) && (iy >= 1) && (iy < My-1))
         {
	   if(uwmask[jy][jx] < 1.)
            t[Nz-1][iy][ix] = sstinit[jy][iy];
	    
         }
      }
     }
#endif

  for(iy = 0; iy < My; iy++)
  {
    for(ix = 0; ix < Mx; ix++)
    {
	   ptopl[iy][ix] = ptop[iy][ix] = ptopx[iy][ix] = 0.0;
           qflx[iy][ix]=0.0;
     }
   }

   for (iz =  0; iz < Mz; iz++)
   for(iy = 0; iy < My; iy++)
    for(ix = 0; ix < Mx; ix++)
     qice[iz][iy][ix]=0.;

   for(iy = 0; iy < My; iy++)
   {
      jy = row*(My-2) + (iy-1);
      yeq = Dely * ((float)Ny-1.0)/2.0;
      y = Dely * jy;
      lat = (2.0*3.14159/4.0e7) * (y-yeq);
      f[iy] = 2.0*omega*sin(lat);

      for(ix = 0; ix < Mx; ix++)
      {
	    if(h[Nz-1][iy][ix] > 1.0)
	    {
	       taux[iy][ix] = strx[iy][ix]/(rhoref * h[Nz-1][iy][ix]);
	       tauy[iy][ix] = stry[iy][ix]/(rhoref * h[Nz-1][iy][ix]);
	    }
	    else
	    {
	       taux[iy][ix] = 0.0;
	       tauy[iy][ix] = 0.0;
	    }
      }
   }
   wrap_q(taux);
   wrap_q(tauy);
}

void load_restart(char* resname)
{
   int jz;
   FILE *tfile;

   if(my_address == 0)
   {
      sprintf(buff,"%s%s",prefix,resname);
      tfile = fopen(buff,"r");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on restart read failed.\n");
      fflush(of);
      exit(89);
   }

   /* model state fields */
      read_restart_slice(&(u[Nz-1][0]),tfile);
      read_restart_slice(&(ul[Nz-1][0]),tfile);
      read_restart_slice(&(v[Nz-1][0]),tfile);
      read_restart_slice(&(vl[Nz-1][0]),tfile);
      read_restart_slice(&(t[Nz-1][0]),tfile);
      read_restart_slice(&(tl[Nz-1][0]),tfile);
      read_restart_slice(&(s[Nz-1][0]),tfile);
      read_restart_slice(&(sl[Nz-1][0]),tfile);
   read_restart_slice(&(ptop[0]),tfile);
   read_restart_slice(&(ptopl[0]),tfile);
#ifdef READ1_5
   read_restart_slice(&(qflx[0]),tfile);
#endif


   if(my_address == 0) fclose(tfile);
}

void read_restart_slice(float a[My][Mx], FILE *tfile)
{
   int ix,iy,jx,jy;
   float row_in[Nx];

   for(jy = 0; jy < Ny; jy++) /* read xy field */
   {
     iy = jy - row*(My-2) + 1;
     if(my_address == 0) fread(row_in,sizeof(float),Nx,tfile);
     MPI_Bcast(row_in,Nx,MPI_FLOAT,0,split_comm);
     for(jx = 0; jx < Nx; jx++)
     {
       ix = jx - col*(Mx-2) + 1;
       if((ix > 0) && (ix < Mx-1) && (iy > 0) && (iy < My-1))
       {
          a[iy][ix] = row_in[jx];
       }
     }
   }
   wrap_q(a);
}

void save_restart(int suffix)
{
   int jz;
   FILE *tfile;

   if(my_address == 0)
   {
#ifdef SIMPLE_OUTPUT
      sprintf(buff,"restart/ocean/restart.ocean.%7.7d",suffix);
#else
      sprintf(buff,"%srestart.ocean.%7.7d",prefix,suffix);
#endif
      tfile = fopen(buff,"w");
   }
   if((tfile == 0) && (my_address == 0))
   {
      printf("fopen on restart write failed.\n");
      fflush(of);
      exit(1);
   }
   else if (my_address == 0)
   {
      printf("open ok \n");
      fflush(of);
   }

   /* model state fields */
      write_restart_slice(&(u[Nz-1][0]),tfile);
      write_restart_slice(&(ul[Nz-1][0]),tfile);
      write_restart_slice(&(v[Nz-1][0]),tfile);
      write_restart_slice(&(vl[Nz-1][0]),tfile);
      write_restart_slice(&(t[Nz-1][0]),tfile);
      write_restart_slice(&(tl[Nz-1][0]),tfile);
      write_restart_slice(&(s[Nz-1][0]),tfile);
      write_restart_slice(&(sl[Nz-1][0]),tfile);
   write_restart_slice(&(ptop[0]),tfile);
   write_restart_slice(&(ptopl[0]),tfile);
   write_restart_slice(&(qflx[0]),tfile);


   if(my_address == 0) fclose(tfile);
}

void write_restart_slice(float a[My][Mx], FILE *tfile)
{
   int jrow,jcol,jy,iy;

   float row_out[Mx-2];
   MPI_Status status;
   int ierr;


   for(jy = 0; jy < Ny; jy++) /* write xy field in natural order*/
   {
     jrow = jy/(My-2);
     iy = jy - jrow*(My-2) + 1;
     for(jcol = 0; jcol < Ncols; jcol++) /* write in column order */
     {
        if((col == jcol) && (row == jrow)) /* Select source processor */
        {
           ierr = MPI_Bsend(&(a[iy][1]), Mx-2, MPI_FLOAT, 0, 99, split_comm);
/*           printf("Sending:128\n"); */
           if(ierr != 0)
           {
              printf("Bsend error in write_restart_slice: %d\n", ierr);
              fflush(of);
              exit(1);
           }
        }
        if(my_address == 0)
        {
           MPI_Recv(row_out, Mx-2, MPI_FLOAT, jrow*Ncols+jcol,
		      99, split_comm, &status);
/*           printf("Receiving:128\n"); */
           fwrite(row_out,sizeof(float),Mx-2,tfile);
        }
        MPI_Barrier( split_comm);

     }
   }
}

/* ----------------------------------------------------------------------
   Procedure ocean_to_coupler
      This procedure sends the set of fields to the coupler.
-----------------------------------------------------------------------*/


void ocean_to_coupler(int nfields,float data[][My-2][Mx-2],int tag)
{
   int size_array, ierr;

   size_array=(Mx-2)*(My-2)*nfields*sizeof(float);

#ifdef COLOCATED_OCEAN
   ierr=MPI_Bsend(data,size_array,MPI_CHAR,0,tag,coupler_ocean);
#else
   ierr=MPI_Send(data,size_array,MPI_CHAR,0,tag,coupler_ocean);
#endif

   if (ierr != MPI_SUCCESS) {
      fprintf(stderr,"Error in ocean_to_coupler\n");
   }
}



/* ----------------------------------------------------------------------
   Procedure ocean_from_coupler
      This procedure receives the set of fields from the coupler.
-----------------------------------------------------------------------*/

void ocean_from_coupler(int nfields,float data[][My-2][Mx-2],int tag)
{
   MPI_Status status;
   int size_array, ierr, wordsz;

   size_array=(Mx-2)*(My-2)*nfields*sizeof(float);

   ierr=MPI_Recv(data,size_array,MPI_CHAR,0,tag,coupler_ocean,&status);

   if (ierr != MPI_SUCCESS) {
      fprintf(stderr,"Error in ocean_from_coupler\n");
   }
}


void time_step()
{

   int ix, iy, iz, isplit, isub_split;
   float delt_spl, tdx, tdy;
   float dx, dy;
   float gdxdt, gdydt, rgdxdt, rgdydt;
   float dcx,dcy,dcxm,dcym;

   dcxm = Diff_Mks/(Delx * Delx);
   dcym = Diff_Mks/(Dely * Dely);
   dcx = Diff_Ratio*Diff_Mks/(Delx * Delx);
   dcy = Diff_Ratio*Diff_Mks/(Dely * Dely);

 /* Second order conservative dissipation on scalars*/

 for (iz = Mz-1; iz < Mz; iz++)
 {
   for (iy = 1; iy < My-1; iy++)
   {
      for (ix = 1; ix < Mx-1; ix++) 
      {
         dts[iz][iy][ix] = 0.0;
         dtt[iz][iy][ix] = 0.0;

         if(h[iz][iy][ix] > 1.0)
         {
            if(h[iz][iy][ix-1] > 1.0)
            {
              dts[iz][iy][ix] -= dcx * (sl[iz][iy][ix] - sl[iz][iy][ix-1]);
              dtt[iz][iy][ix] -= dcx * (tl[iz][iy][ix] - tl[iz][iy][ix-1]);
            }

	        if(h[iz][iy][ix+1] > 1.0)
            {
	          dts[iz][iy][ix] -= dcx * (sl[iz][iy][ix] - sl[iz][iy][ix+1]);
	          dtt[iz][iy][ix] -= dcx * (tl[iz][iy][ix] - tl[iz][iy][ix+1]);
            }

            if(h[iz][iy-1][ix] > 1.0)
            {
	          dts[iz][iy][ix] -= dcy * (sl[iz][iy][ix] - sl[iz][iy-1][ix]);
	          dtt[iz][iy][ix] -= dcy * (tl[iz][iy][ix] - tl[iz][iy-1][ix]);
            }

	        if(h[iz][iy+1][ix] > 1.0)
            {
	          dts[iz][iy][ix] -= dcy * (sl[iz][iy][ix] - sl[iz][iy+1][ix]);
	          dtt[iz][iy][ix] -= dcy * (tl[iz][iy][ix] - tl[iz][iy+1][ix]);
            }
         }
      }
   }
 }

 /* Surface forcing on T,S */

 for (iy = 1; iy < My-1; iy++)
 {
   for (ix = 1; ix < Mx-1; ix++)
   {
	 if(h[Nz-1][iy][ix] >1.0)
	 {
#ifdef SST_FROM_FILE
          dtt[Nz-1][iy][ix] += Restore * (osstint(ix-1,iy-1,step)-tl[Nz-1][iy][ix]);
#else
	     dtt[Nz-1][iy][ix] += temp_for[iy][ix];
#endif
         dts[Nz-1][iy][ix] += salt_for[iy][ix];
     }
   }
 }

  /* setup vx, wx, tx, sx for time split step */

   for (iz = Mz-1; iz < Mz; iz++)
   {
     for (iy = 0; iy < My; iy++)
     {
       for (ix = 0; ix < Mx; ix++)
       {
	      ux[iz][iy][ix] = ul[iz][iy][ix];

	      vx[iz][iy][ix] = vl[iz][iy][ix];

	      tx[iz][iy][ix] = tl[iz][iy][ix] + 2.0*Delt * dtt[iz][iy][ix];

	      sx[iz][iy][ix] = sl[iz][iy][ix] + 2.0*Delt * dts[iz][iy][ix];
       }
     }
   }
   wrap_qz(tx); /* only need to be wrapped once */
   wrap_qz(sx);

#ifdef CSIMICE
/* ************************************************************
SEA ICE.
    this algorithm computes ocean heat flux to the sea-ice. it forms
    the necessary ice in the ocean and adjusts the potential
    temperature and salinity fields accordingly. the logic of this
    subroutine is based on William Large''s 1-d model and is based
    on a version from the NCOM model written by Ghokan Danabasoglu.

    If ice forms in lower layers but layers above are warm - the heat is
    used to melt the ice.  total ice heat flux is accumulated.
    WARNING: unless a monotone advection scheme is in place,
    advective errors could lead to temps that are far below freezing
    in some locations and this scheme will form lots of ice.
    ice formation should be limited to the top layer (izmxice=1)
    if the advection scheme is not monotone.

    NOTE:  conversion to C and incorporation into Om3 by R. Jacob
********************************************************** */

/* do sea ice model izmxice levels below and including surface */
     for (iy = 0; iy < My; iy++)
     {
      for (ix = 0; ix < Mx; ix++)
      {
   for(iz = Mz-izmxice; iz < Mz; iz++)
    {
        if (h[iz][iy][ix] > 1.0)
        {

/* find water flux due to ice formation in kg/m2s */
/* positive when  tx < Tf */
           potice = (rhoref*Cp/Lathf)*
              h[iz][iy][ix]*(Tf - tx[iz][iy][ix]);

              if( potice < qice[iz-1][iy][ix])potice=qice[iz-1][iy][ix];

              qice[iz][iy][ix] += qice[iz-1][iy][ix]-potice;

/* convert water flux to temperature forcing */
              tx[iz][iy][ix] +=  potice*
                 Lathf/(h[iz][iy][ix]*rhoref*Cp);

/* convert water flux to salinity forcing */
              sx[iz][iy][ix] += potice*(Sref)/(rhoref*h[iz][iy][ix]);
        }
      }
     }
    }

/* let any residual heat in the upper layer melt formed ice */
     for (iy = 0; iy < My; iy++)
     {
      for (ix =0; ix < Mx; ix++)
      {
        if(h[Nz-1][iy][ix] > 1.0)
        {
           potice = (rhoref*Cp/Lathf)*
              h[Nz-1][iy][ix]*(Tf - tx[Nz-1][iy][ix]);

              if( potice < qice[Nz-1][iy][ix])potice=qice[Nz-1][iy][ix];

              tx[Nz-1][iy][ix] +=  potice*
                 Lathf/(h[Nz-1][iy][ix]*rhoref*Cp);

              sx[Nz-1][iy][ix] += potice*(Sref)/(rhoref*h[Nz-1][iy][ix]);
              qice[Nz-1][iy][ix] -= potice;
        }
      }
     }

/* tx may still be above freezing so determine melt potential */
     for (iy = 0; iy < My; iy++)
     {
      for (ix = 0; ix < Mx; ix++)
      {
        if(h[Nz-1][iy][ix] > 1.0)
        {

           potice = (rhoref*Cp/Lathf)*
              h[Nz-1][iy][ix]*(Tf - tx[Nz-1][iy][ix]);
           qflx[iy][ix]= Lathf*(potice-qice[Nz-1][iy][ix])/Delt;
        }
      }
     }

/*   end of sea ice */
/********************************************************** */

/*  Salinity clamp */
/*  Temp warning */

   for (iz =  0; iz < Mz; iz++)
   {
     for (iy = 1; iy < My-1; iy++)
     {
      for (ix = 1; ix < Mx-1; ix++)
      {

      if(tx[iz][iy][ix] < Tf){
      fprintf(stderr,"OM3 temp warning: %d %d %d %d %f\n",istep,iz,iy,ix,tx[iz][iy][ix]);
      }

      if(sx[iz][iy][ix] < 0.0) sx[iz][iy][ix] = 0.0;
      }
     }
   }

#else
/* Don't let t get lower then -1.92. T Clamp*/
/* And don't let s get lower than 0.  S Clamp*/
   for (iz =  0; iz < Mz; iz++)
   {
     for (iy = 0; iy < My; iy++)
     {
      for (ix = 0; ix < Mx; ix++)
      {
      if(tx[iz][iy][ix] < -1.92) tx[iz][iy][ix] = -1.92;
      if(sx[iz][iy][ix] < 0.0) sx[iz][iy][ix] = 0.0;
      }
     }
   }
#endif

/* Time filter */

   for (iz =  0; iz < Mz; iz++)
   {
     for (iy = 0; iy < My; iy++)
     {
      for (ix = 0; ix < Mx; ix++)
      {
	 ul[iz][iy][ix] = 0.65 * u[iz][iy][ix]
	     + 0.175 * (ul[iz][iy][ix] + ux[iz][iy][ix]);
	 u[iz][iy][ix] = ux[iz][iy][ix];

	 vl[iz][iy][ix] = 0.65 * v[iz][iy][ix]
	     + 0.175 * (vl[iz][iy][ix] + vx[iz][iy][ix]);
	 v[iz][iy][ix] = vx[iz][iy][ix];

	 tl[iz][iy][ix] = 0.65 * t[iz][iy][ix]
	     + 0.175 * (tl[iz][iy][ix] + tx[iz][iy][ix]);
	 t[iz][iy][ix] = tx[iz][iy][ix];

	 sl[iz][iy][ix] = 0.65 * s[iz][iy][ix]
	     + 0.175 * (sl[iz][iy][ix] + sx[iz][iy][ix]);
	 s[iz][iy][ix] = sx[iz][iy][ix];
      }
     }
   }

   for (iy = 0; iy < My; iy++)
   {
     for (ix = 0; ix < Mx; ix++)
     {
	    ptopl[iy][ix] = 0.65 * ptop[iy][ix]
	      + 0.175 * (ptopl[iy][ix] + ptopx[iy][ix]);
	    ptop[iy][ix] = ptopx[iy][ix];
      }
    }
}

float density(float s, float t, int iz)
{
   float den;

/*
 * simple approximate density for salt water.  To be replaced with
 * std polynomical formula when convection details are important.
*/
   den = 0.2*(s-base_s[iz]) - 0.2*(t-base_t[iz]);
   return den/rhoref;
}

/* Begin sp2 communications routines. */

float in_xz[Mz][Mx], out_xz[Mz][Mx];
float in_yz[Mz][My], out_yz[Mz][My];

void wrap_qz(float a[Mz][My][Mx])
{
   int ix, iy, iz;
   int tag=0;

/* rec from south, send north */
   for(iz = 0; iz < Mz; iz++)
   {
     for(ix = 0; ix < Mx; ix++)
     {
       out_xz[iz][ix] = a[iz][My-2][ix];
     }
   }
   send_and_receive(south, tag, in_xz, Mx*Mz*sizeof(float), north, out_xz);
   for(iz = 0; iz < Mz; iz++)
   {
     for(ix = 0; ix < Mx; ix++)
     {
       a[iz][0][ix] = in_xz[iz][ix];
     }
   }

/* rec from north, send south */
   for(iz = 0; iz < Mz; iz++)
   {
     for(ix = 0; ix < Mx; ix++)
     {
       out_xz[iz][ix] = a[iz][1][ix];
     }
   }
   send_and_receive(north, tag, in_xz, Mx*Mz*sizeof(float), south, out_xz);
   for(iz = 0; iz < Mz; iz++)
   {
     for(ix = 0; ix < Mx; ix++)
     {
       a[iz][My-1][ix] = in_xz[iz][ix];
     }
   }

/* rec from west, send east */
   for(iz = 0; iz < Mz; iz++)
   {
     for(iy = 0; iy < My; iy++)
     {
       out_yz[iz][iy] = a[iz][iy][Mx-2];
     }
   }
   send_and_receive(west, tag, in_yz, My*Mz*sizeof(float), east, out_yz);
   for(iz = 0; iz < Mz; iz++)
   {
     for(iy = 0; iy < My; iy++)
     {
       a[iz][iy][0] = in_yz[iz][iy];
     }
   }

/* rec from east, send west */
   for(iz = 0; iz < Mz; iz++)
   {
     for(iy = 0; iy < My; iy++)
     {
       out_yz[iz][iy] = a[iz][iy][1];
     }
   }
   send_and_receive(east, tag, in_yz, My*Mz*sizeof(float), west, out_yz);
   for(iz = 0; iz < Mz; iz++)
   {
     for(iy = 0; iy < My; iy++)
     {
       a[iz][iy][Mx-1] = in_yz[iz][iy];
     }
   }
}

float in_3x[3][Mx], out_3x[3][Mx];
float in_3y[3][My], out_3y[3][My];

void wrap_q3(float a[My][Mx], float b[My][Mx], float c[My][Mx])
{
   int ix, iy;
   int tag=0;

/* rec from south, send north */
   for(ix = 0; ix < Mx; ix++)
   {
     out_3x[0][ix] = a[My-2][ix];
     out_3x[1][ix] = b[My-2][ix];
     out_3x[2][ix] = c[My-2][ix];
   }
   send_and_receive(south, tag, in_3x, Mx*3*sizeof(float), north, out_3x);
   for(ix = 0; ix < Mx; ix++)
   {
     a[0][ix] = in_3x[0][ix];
     b[0][ix] = in_3x[1][ix];
     c[0][ix] = in_3x[2][ix];
   }

/* rec from north, send south */
   for(ix = 0; ix < Mx; ix++)
   {
     out_3x[0][ix] = a[1][ix];
     out_3x[1][ix] = b[1][ix];
     out_3x[2][ix] = c[1][ix];
   }
   send_and_receive(north, tag, in_3x, Mx*3*sizeof(float), south, out_3x);
   for(ix = 0; ix < Mx; ix++)
   {
     a[My-1][ix] = in_3x[0][ix];
     b[My-1][ix] = in_3x[1][ix];
     c[My-1][ix] = in_3x[2][ix];
   }

/* rec from west, send east */
   for(iy = 0; iy < My; iy++)
   {
     out_3y[0][iy] = a[iy][Mx-2];
     out_3y[1][iy] = b[iy][Mx-2];
     out_3y[2][iy] = c[iy][Mx-2];
   }
   send_and_receive(west, tag, in_3y, My*3*sizeof(float), east, out_3y);
   for(iy = 0; iy < My; iy++)
   {
     a[iy][0] = in_3y[0][iy];
     b[iy][0] = in_3y[1][iy];
     c[iy][0] = in_3y[2][iy];
   }

/* rec from east, send west */
   for(iy = 0; iy < My; iy++)
   {
     out_3y[0][iy] = a[iy][1];
     out_3y[1][iy] = b[iy][1];
     out_3y[2][iy] = c[iy][1];
   }
   send_and_receive(east, tag, in_3y, My*3*sizeof(float), west, out_3y);
   for(iy = 0; iy < My; iy++)
   {
     a[iy][Mx-1] = in_3y[0][iy];
     b[iy][Mx-1] = in_3y[1][iy];
     c[iy][Mx-1] = in_3y[2][iy];
   }
}

void wrap_q(a)
float a[My][Mx];
{
   int src, dest, msg_len, tag=0, ellen, elcnt, stride;
   char *msg_in, *msg_out;

   msg_len = Mx * sizeof(float);

   msg_in = (void *) &(a[0][0]); /* rec from south, send north */
   msg_out = (void *) &(a[My-2][0]);

   send_and_receive(south, tag, msg_in, msg_len, north, msg_out);

   msg_in = (void *) &(a[My-1][0]); /* rec from north, send south */
   msg_out = (void *) &(a[1][0]);

   send_and_receive(north, tag, msg_in, msg_len, south, msg_out);

   msg_in = (void *) &(a[0][0]);  /* rec from west, send east */
   msg_out = (void *) &(a[0][Mx-2]);
/* 
 * defined in mpi_edge_type
 *
 * ellen = sizeof(float);
 * elcnt = My;
 * stride = Mx * ellen;
*/

   send_and_receive_v(west, tag, msg_in, ellen, stride, elcnt, east, msg_out);

   msg_in = (void *) &(a[0][Mx-1]);
   msg_out = (void *) &(a[0][1]);
/* 
 * defined in mpi_edge_type
 *
 * ellen = sizeof(float);
 * elcnt = My;
 * stride = Mx * ellen;
*/

   send_and_receive_v(east, tag, msg_in, ellen, stride, elcnt, west, msg_out);
}

send_and_receive(
      int source_node, int tag, void *inbuffer, int buf_len,
      int dest_node, void *outbuffer)
{
    MPI_Status status;
    int result;

#ifdef DEBUG
    if(source_node != dest_node) {
	printf("SEND_RECEIVE: Source and dest node differ: %d %d\n",
	       source_node, dest_node);
	exit(1);
    }
#endif

    result = MPI_Sendrecv(outbuffer, buf_len, MPI_CHAR, dest_node, tag,
			  inbuffer, buf_len, MPI_CHAR, source_node, tag,
			  split_comm, &status);

    if(result != MPI_SUCCESS) {
	printf("SEND_RECEIVE: Return code %d\n",result);
	exit(1);
    }
}

send_and_receive_v(
      int source_node, int tag, void *inbuffer,
      int elem_len, int stride, int elem_count,
      int dest_node, void *outbuffer)
{
    MPI_Status status;
    int result;

#ifdef DEBUG
    if(source_node != dest_node) {
	printf("SEND_RECEIVE_V: Source and dest node differ: %d %d\n",
	       source_node, dest_node);
	exit(1);
    }
#endif

    result = MPI_Sendrecv(outbuffer, 1, mpi_edge_type, dest_node, tag,
			  inbuffer, 1, mpi_edge_type, source_node, tag,
			  split_comm, &status);

    if(result != MPI_SUCCESS) {
	printf("SEND_RECEIVE_V: Return code %d\n",result);
	exit(1);
    }
}

/* End sp2 communications routines */




int global_open(char *file, int flags, int mode)
{
    int fd;

    if(my_address == 0) {
	if((fd = open(file, flags, mode)) < 0) {
	    printf("File open error: %s\n",file);
	    exit(1);
	}
	printf("File open: %s\n",file);
	return(fd);
    }
    else
	return(-1);
}



/* Begin sp2 timing routines. */
/*********** DUMMY ************************************************/
int timers_initialized = 0;

void node_timer_clear(int n)
{
    if(!timers_initialized) {
      UTP_init(100);
      timers_initialized = 1;
    }
    UTP_reset_timer(n);
}

void node_timer_start(int n)
{
    if(!timers_initialized) {
      UTP_init(100);
      timers_initialized = 1;
    }
    UTP_start_timer(n);
}

void node_timer_stop(int n)
{
    if(!timers_initialized) {
      UTP_init(100);
      timers_initialized = 1;
    }
    UTP_stop_timer(n);
}

float node_timer_busy(int n)
{
    double time;
    float result;
    int precision;
    UTP_get_accum_time(n, &time, &precision);
    result = (float) time;
    return(result);
}
/* End sp2 timing routines */

#ifdef SST_FROM_FILE

void read_sst_data()

{
  int iy,ix,month,advance;
  FILE *sstfile;

  sprintf(buff,"%s%s",time_inv,"/sstshea5c");
  sstfile = fopen(buff,"r");
   if((sstfile == 0) && (my_address == 0))
   {
      printf("fopen on sstfile failed.\n");
      exit(89);
   }
   if(my_address == 0)printf("Reading SST from file\n");

  for (month=0; month < 12; month++)
  {
    advance = (month*Nx*Ny) + (col*(Mx-2)+row*(My-2)*Nx);
    fseek(sstfile,advance*sizeof(float),SEEK_SET);
    for (iy = 0; iy < My-2; iy++)
    {
      fread(&(sstm[month][iy][0]),sizeof(float),Mx-2,sstfile);
      advance = Nx - (Mx-2);
      fseek(sstfile,advance*sizeof(float),SEEK_CUR);
    }
  }
  fclose(sstfile);
}


float osstint(int j, int i, int timestep)
{
  int prevstep, nextstep;
  float sst_value;
  float fact;
  int month, sid;

  sid = 86400 / Delt;
  timestep = timestep - 15*sid;
  if (timestep < 0) {
    timestep = timestep + 360*sid;
  }
  timestep = timestep % (360 * sid);
  month = timestep / (30 * sid);
  prevstep = month * 30 * sid;
  nextstep = prevstep + 30 * sid;

  fact = ((float)timestep - (float)prevstep) / ((float)nextstep - (float)prevstep);
  sst_value = sstm[month][i][j] + fact * (sstm[(month+1)%12][i][j] - sstm[month][i][j]);
  return(sst_value);
}

#endif

