#include "params.h"
      subroutine srfflx(lat     ,ioro    ,psm1    ,pmidm1  ,ubot    ,
     $                  vbot    ,tbot    ,qbot    ,thbot   ,zbot    ,
     $                  srfrad  ,ts      ,ltheat  ,wet     ,fnt     ,
     $                  dfntdt  ,shf     ,dshf    ,lhf     ,dlhf    ,
     $                  taux    ,tauy    )
C-----------------------------------------------------------------------
C
C Compute surface to atmosphere fluxes, flux surface temperature partial
C derivatives, and exchange coefficiants for momentum,heat, and neutral
C conditions.
C
C Computes all quantities over all surface types.
C
C Note:  Code was isolated from CCM1 'TSCALC' by B. Briegleb.
C        Formulas for exchange coefficiants changed to those given
C        in Holtslag, A.A.M. and B.A. Boville, Local versus
C        nonlocal boundary-layer diffusion in a global climate
C        model, Submitted to Journal of Climate December 1991
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Briegleb
C Standardized:      L. Buja, June 1992
C Reviewed:          B. Briegleb, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comtsc.com"
C-----------------------------------------------------------------------
#include "crdalb.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer lat,             ! Latitude index (south to north)
     $        ioro(plond)      ! Land/ocean/sea-ice flag
C
      real psm1(plond),        ! Surface pressure
     $     pmidm1(plond,plev), ! Level pressures
     $     ubot(plond),        ! Bottom level u wind
     $     vbot(plond),        ! Bottom level v wind
     $     tbot(plond),        ! Bottom level temperature
     $     qbot(plond),        ! Bottom level specific humidity
     $     thbot(plond),       ! Bottom level potential temperature
     $     zbot(plond),        ! Bottom level height above surface
     $     srfrad(plond),      ! Solar absorbed plus down longwave flux
     $     ts(plond),          ! Surface temperature
     $     ltheat(plond),      ! Latent heat for given srf conditions
     $     wet(plond)          ! Sfc wetness factor (fractn sat sfc)
C
C Output arguments
C
      real fnt(plond),         ! Net surface flux for input conditions
     $     dfntdt(plond),      ! Net surface flux ts partial derivative
     $     shf(plond),         ! Initial sensible heat flux
     $     dshf(plond),        ! Ts partial deriv for sensible heat flux
     $     lhf(plond),         ! Initial latent heat flux
     $     dlhf(plond),        ! Ts partial derivtv for latent heat flux
     $     taux(plond),        ! X surface stress (N)
     $     tauy(plond)         ! Y surface stress (N)
C
C---------------------------Local variables-----------------------------
C
      integer i                ! Longitude index
      real dchdts,             ! Partial derivat ch with respect to ts
     $     ddudts,             ! Partial derivat du with respect to ts
     $     dfhdts(plond),      ! fh ts partial derivative
     $     dqsdt,              ! Partial derivat esk with respect to ts
     $     dridts(plond),      ! Richardson number ts partial derivat
     $     du(plond),          ! Denominator for unstable coefficients
     $     esk(plond),         ! Saturation vapor pressure
     $     fh(plond),          ! Actual stability coeff on cn (for ch)
     $     fhu,                ! Coeff. on cn (for ch), unstable conds
     $     fm(plond),          ! Actual stability coeff on cn (for cm)
     $     fmu                 ! Coeff. on cn (for cm), unstable conds
      real lwuphf,             ! Longwave up flux with input srf ts
     $     omeps,              ! 1 - epsilon (i.e., 1 - 0.622)
     $     ri0(plond),         ! Surface layer Richardson number
     $     tmp,                ! Temporary
     $     ts2(plond),         ! ts squared
     $     vmag(plond),        ! Surface wind magnitude
     $     z0(plond),          ! Srf aerodynmc roughness length for heat
     $     zb(plond),          ! RO*CH*CP*|V|
     $     zbu(plond),         ! RO*CM*CP*|V|
     $     zd(plond),          ! RO*CH*WET*L*|V|
     $     zqs(plond),         ! Surface saturation specific humidity
     $     zz0(plond),         ! (zbot+z0) / z0
     $     cm(plond),          ! Surface exchange coeff for momentum
     $     ch(plond),          ! Surface exchange coefficient for heat
     $     cn(plond)           ! Surface neutral exchange coefficient
C
C-----------------------------------------------------------------------
C
      omeps  = 1.0 - epsilo
      do i=1,plon
C
C Define aerodynamic roughness parameter appropriate for surface
C
         if (ioro(i).eq.0 ) then
            z0(i)  = zzocen
         else if ( ioro(i).eq.2) then
            z0(i)  = zzsice
         else
            z0(i)  = rghnss(i,lat)
         end if
      end do
CDIR$ IVDEP
C
C Compute exchange coefficiants
C
      do 100 i=1,plon
C
C Calculate surface wind; minimum allowed is 1 m/s
C
         vmag(i) = amax1(1.0,sqrt(ubot(i)**2+vbot(i)**2))
C
C Calculate bulk richardson number
C
         ri0(i) = gravit*zbot(i)*(thbot(i)-ts(i))/(thbot(i)*vmag(i)**2)
C
C Calculate neutral exchange coefficient
C
         zz0(i)  = (zbot(i) + z0(i)) / z0(i)
         cn(i)   = (xkar / alog(zz0(i)))**2
C
C Unstable branch (ri < 0)
C
         du(i)   = 1./(1. + 75.*cn(i)*sqrt(zz0(i)*abs(ri0(i))))
         fmu     = 1. - 10.*ri0(i)*du(i)
         fhu     = 1. - 15.*ri0(i)*du(i)
C
C Stable branch (ri >= 0)
C
         fm(i)   = 1.0 / (1.0 + 10.0*ri0(i) * (1.0 + 8.0*ri0(i)))
         fh(i)   = fm(i)
C
C Merge stable and unstable values
C
         if( ri0(i) .lt. 0. ) then
           fm(i) = fmu
           fh(i) = fhu
         end if
C
C Exchange coefficient for momentum => cm; for heat and moisture => ch
C
         cm(i)   = cn(i) * fm(i)
         ch(i)   = cn(i) * fh(i)
C
         tmp     = vmag(i)*pmidm1(i,plev)/(rair*tbot(i))
         zbu(i)  = tmp*cm(i)
         zb(i)   = tmp*cpair*ch(i)
         zd(i)   = tmp*ltheat(i)*ch(i)*wet(i)
C
C Surface stress components
C
         taux(i) = -zbu(i)*ubot(i)
         tauy(i) = -zbu(i)*vbot(i)
  100 continue
C
CDIR$ IVDEP
C
C Compute net surface flux (Flux is positive for downwards flux)
C First, compute saturation mixing ratio, zqs (call to vector routine)
C
      call vqsat(ts, psm1, esk, zqs, plon)       
      do 200 i=1,plon
         ts2(i)    = ts(i)*ts(i)
C
C Define net surface flux components at current surface temperature
C (positive downwards into surface)
C
         shf(i)    = zb(i)*(ts(i) - thbot(i))
         lhf(i)    = zd(i)*(zqs(i) - qbot(i))
         lwuphf    = stebol*(ts2(i)*ts2(i))
C
C Net surface flux at current surface temperature:
C
         fnt(i)    = srfrad(i) - lwuphf - shf(i) - lhf(i)
C
C Compute net surface flux surface temperature derivative at the current
C surface temperature:
C Derivative of Richardson number with respect to ts
C
         if(ri0(i).ge.rimax) then
            dridts(i) = 0.
         else
            dridts(i) = -gravit*zbot(i)/(thbot(i)*vmag(i)**2)
         end if
C
C Derivative of fh with respect to ts: unstable branch (ri < 0)
C
         if (ri0(i) .lt. 0.0) then
            ddudts = -75./2.*cn(i)*sqrt(zz0(i)/abs(ri0(i)))*dridts(i)
            dfhdts(i) = -15.*du(i)*(dridts(i) - ri0(i)*du(i)*ddudts)
C
C Derivative of fh with respect to ts: stable branch (ri >= 0)
C
         else
            dfhdts(i) = -10.0*fh(i)**2*dridts(i)*(1.0 + 16.0*ri0(i))
         end if
C
C Derivatives of surface fluxes of heat and moisture with respect to ts
C
         dchdts  = cn(i) * dfhdts(i) / ch(i)
         dqsdt = clrh2o*zqs(i)*psm1(i)/((psm1(i) - omeps*esk(i))*ts2(i))
         dshf(i) = zb(i)*(1.0 + dchdts*(ts(i) - thbot(i)))
         dlhf(i) = zd(i)*(dqsdt + dchdts*(zqs(i) - qbot(i)))
C
C Compute partial derivative of net surface flux (fnt) with with respect
C to ts.  Ignore changes due to radiation.
C
         dfntdt(i) = -(dshf(i) + dlhf(i))
  200 continue
C 
      return
      end
