#include "params.h"
      subroutine vqsat(t       ,p       ,es      ,qs      ,len    )
C-----------------------------------------------------------------------
C
C Utility procedure to look up and return saturation vapor pressure from 
C precomputed table, calculate and return saturation specific humidity 
C (g/g), for input vectors of temperature and pressure (length len)
C
C----------------------------Code History-------------------------------
C
C Original version:  J. Hack
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer len              ! Vector length 
      real t(len),             ! Temperature
     $     p(len)              ! Pressure
C
C Output arguments
C
      real es(len),            ! Saturation vapor pressure
     $     qs(len)             ! Saturation specific humidity
C
C---------------------------Local workspace-----------------------------
C
      real omeps               ! 1 - 0.622
      integer i                ! Local vector index
C
C-----------------------------------------------------------------------
#include "eslookup.com"
C-----------------------------------------------------------------------
C
      omeps = 1.0 - epsqs
      do i=1,len
         es(i) = estblf(t(i))
C
C Saturation specific humidity
C
         qs(i) = epsqs*es(i)/(p(i) - omeps*es(i))
C
C The following check is to avoid the generation of negative values
C that can occur in the upper stratosphere and mesosphere
C
         qs(i) = amin1(1.0,qs(i))
C
         if (qs(i) .lt. 0.0) then
            qs(i) = 1.0
            es(i) = p(i)
         end if
      end do
C
      return
      end
