#include <params.h>
      subroutine hycoef(siga    ,sigb)
C-----------------------------------------------------------------------
C
C Defines the locations of model interfaces from input data in the
C hybrid coordinate scheme.  Actual pressure values of model level
C interfaces are determined elsewhere from the fields set here.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville
C Standardized:      L. Bath, June 1992
C Reviewed:          B. Boville, D. Williamson, August 1992
C
C-----------------------------------------------------------------------
c
c $Id: hycoef.F,v 1.1.1.1 2002/03/20 22:16:27 jacob Exp $
c $Author: jacob $
c
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C-----------------------------------------------------------------------
C
C the following fields are set:
C hyai     fraction of reference pressure used for interface pressures
C hyam     fraction of reference pressure used for midpoint pressures
C hybi     fraction of surface pressure used for interface pressures
C hybm     fraction of surface pressure used for midpoint pressures
C hybd     difference of hybi's
C hypi     reference state interface pressures
C hypm     reference state midpoint pressures
C hypd     reference state layer thicknesses
C hypdln   reference state layer thicknesses (log p)
C hyalph   distance from interface to level (used in integrals)
C prsfac   log pressure extrapolation factor (used to compute psl)
C
C------------------------------Parameters-------------------------------
#include <pmgrid.com>
C------------------------------Commons----------------------------------
#include <comhyb.com>
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real siga(2*plev+1),      ! Sigma arrays read from header
     $     sigb(2*plev+1)
C
C---------------------------Local workspace-----------------------------
C
      integer k                 ! Level index
      real amean,bmean,atest,btest,eps
      logical flag
#ifdef FORCEHY
      real ainew(19),           ! A values of interface
     $     binew(19)            ! B values of interface
      save ainew,binew
      data ainew /
     $     2.514994703233242E-03, 7.103605195879936E-03,
     $     1.904259435832500E-02, 4.607560858130455E-02,
     $     8.181858807802200E-02, 7.869805395603180E-02,
     $     7.463172823190689E-02, 6.955304741859436E-02,
     $     6.339056789875031E-02, 5.621771141886711E-02,
     $     4.815292730927467E-02, 3.949233517050743E-02,
     $     3.058454766869545E-02, 2.193340472877026E-02,
     $     1.403667870908976E-02, 7.458650507032871E-03,
     $     2.646875567734241E-03, 0.000000000000000E+00,
     $     0.000000000000000E+00
     $     /
      data binew /
     $     0.000000000000000E+00, 0.000000000000000E+00,
     $     0.000000000000000E+00, 0.000000000000000E+00,
     $     0.000000000000000E+00, 3.756976872682571E-02,
     $     8.652626723051071E-02, 1.476711630821228E-01,
     $     2.218644469976425E-01, 3.082221150398254E-01,
     $     4.053182005882263E-01, 5.095875263214111E-01,
     $     6.168329119682312E-01, 7.209885120391846E-01,
     $     8.160612583160400E-01, 8.952575325965881E-01,
     $     9.531889557838440E-01, 9.850560426712036E-01,
     $     1.000000000000000E+00
     $     /
#endif
C
C-----------------------------------------------------------------------
C
      eps = 1.e-05
      if (plevp .ne. 19) then
         write(*,*) '*** error: must redefine A"s and B"s at interfaces'
         call endrun
      endif
      nprlev = 0
      ps0 = 1.0e5               ! Base state surface pressure (pascals)
      psr = 1.0e5               ! Reference surface pressure (pascals)
C
C Set layer locations
C
      write(6,'(a)')'1 Layer Locations (*1000) '
#ifdef FORCEHY
      write(0,*)'hycoef: NOTE:  Forcing ccm3.2 A,B values into model'
      write(6,*)'hycoef: NOTE:  Forcing ccm3.2 A,B values into model'
#endif
      do k=1,plev
C
C Interfaces. Set nprlev to the interface above, the first time a 
C nonzero surface pressure contribution is found. "nprlev" 
C identifies the lowest pure pressure interface.
C
#ifdef FORCEHY
         siga(2*k-1) = ainew(k)
         sigb(2*k-1) = binew(k)
#endif
         hyai(k) = siga(2*k-1)
         hybi(k) = sigb(2*k-1)
         write(6,9800)k, hyai(k), hybi(k), hyai(k)+hybi(k)
         if (nprlev.eq.0 .and. hybi(k).ne.0.0) nprlev = k - 1
C
C Midpoints
C

#ifdef FORCEHY
         siga(2*k) = ( ainew(k+1) + ainew(k) )*0.5
         sigb(2*k) = ( binew(k+1) + binew(k) )*0.5
#endif
         hyam(k) = siga(2*k)
         hybm(k) = sigb(2*k)
         write(6,9810) hyam(k), hybm(k), hyam(k)+hybm(k)
      end do
C
#ifdef FORCEHY
      siga(2*plevp-1) = ainew(plevp)
      sigb(2*plevp-1) = binew(plevp)
#endif
      hyai(plevp) = siga(2*plevp-1)
      hybi(plevp) = sigb(2*plevp-1)
      write(6,9800)plevp,hyai(plevp),hybi(plevp),hyai(plevp)+hybi(plevp)
C
C Set nprlev if no nonzero b's have been found. All interfaces are 
C pure pressure. A pure pressure model requires other changes as well. 
C
      if (nprlev.eq.0) nprlev = plev + 2
C
C Set delta sigma part of layer thickness and reference state midpoint 
C pressures
C
      do k=1,plev
         hybd(k) = hybi(k+1) - hybi(k)
         hypm(k) = hyam(k)*ps0 + hybm(k)*psr
      end do
C
C Reference state interface pressures
C
      do k=1,plevp
         hypi(k) = hyai(k)*ps0 + hybi(k)*psr
      end do
C
C Reference state layer thicknesses
C
      do k=1,plev
         hypd(k) = hypi(k+1) - hypi(k)
      end do
C
      write(6,9820)
      do k=1,plev
         write(6,9830) k, hypi(k)
         write(6,9840) hypm(k), hypd(k)
      end do
      write(6,9830) plevp, hypi(plevp)
C
C Calculate the log pressure extrapolation factor
C
         prsfac = alog( hyam(plev) + hybm(plev)) /
     $        alog((hyam(plev) + hybm(plev)) / 
     $        (hyam(plev-1) + hybm(plev-1)))
C
C Test that A's and B's at full levels are arithmetic means of A's and
C B's at interfaces
C
      flag = .false.
      do k = 1,plev
         amean = ( hyai(k+1) + hyai(k) )*0.5
         bmean = ( hybi(k+1) + hybi(k) )*0.5
         if(amean .eq. 0. .and. hyam(k) .eq. 0.) then
            atest = 0.
         else
            atest = abs( amean - hyam(k) )/
     $             ( 0.5*( abs(amean + hyam(k)) ) )
         endif
         if(bmean .eq. 0. .and. hybm(k) .eq. 0.) then
            btest = 0.
         else
            btest = abs( bmean - hybm(k) )/
     $             ( 0.5*( abs(bmean + hybm(k)) ) )
         endif
         if(atest .gt. eps) then
            flag = .true.
            write(*,9850) k
         endif
         if(btest .gt. eps) then
            flag = .true.
            write(*,9860) k
         endif
      end do
#ifndef USEOLDHY
      if (flag) then
         write(*,9870)
         write(0,9870)
         call endrun
      endif
#endif
      return
 9800 format( 1x, i3, 3p, 3(f10.4,10x) )
 9810 format( 1x, 3x, 3p, 3(10x,f10.4) )
 9820 format(1x,'reference pressures (Pa)')
 9830 format(1x,i3,f15.4)
 9840 format(1x,3x,15x,2f15.4)
 9850 format(' Routine "hycoef":  Input vertical grid level is',
     $       ' specified in an inconsistent manner:'/
     $       ' The hybrid coordinate parameter A at mid-level ',i4,
     $       ' must be the arithmetic mean of the A-values of the',
     $       ' bracketing interfaces')
 9860 format(' Routine "hycoef":  Input vertical grid level is',
     $       ' specified in an inconsistent manner:'/
     $       ' The hybrid coordinate parameter B at mid-level ',i4,
     $       ' must be the arithmetic mean of the B-values of the',
     $       ' bracketing interfaces')
 9870 format(' Aborting run in routine hycoef:  because of errors in',
     $       ' the vertical grid specification.'/' The thermodynamic',
     $       ' and momentum equations are not energetically',
     $       ' consistent.'/
     $       ' Remove this abort statement from the code to proceed',
     $       ' (at own risk)'/
     $       ' or redefine the model levels on the initial conditions',
     $       ' history tape.'/
     $       'FOAM USERS:  You are probably using a restart or',/
     $       ' initial condition file with the old values.'/' To force',
     $       ' the new values in, define FORCEHY in params.h',/
     $       'Or, to force the model to proceed with the old values,',
     $       ' define USEOLDHY in params.h.')

      end
