#include "params.h"
      subroutine inti
C-----------------------------------------------------------------------
C
C Set constants and call initialization procedures for physics/dynamics
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Rosinski
C Standardized:      J. Rosinski, June 1992
C Reviewed:          B. Boville, P. Rasch, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
C
C---------------------------Local workspace-----------------------------
C
      real wssat,            ! Bucket moisture capacity
     $     xlat,             ! Latitude (radians)
     $     clrh2o            ! Ratio of latent heat to gas constant (h2o)
      integer
     $     irow              ! Latitude pair index
C
C------------------------------Externals--------------------------------
C
      external
     $     vdinti,           ! Vertical diffusion
     $     rfinti,           ! Rayleigh friction
     $     mgwinti,           ! Gravity wave drag
     $     cainti,           ! Convective adjustments
     $     radini,           ! Radiation
     $     tsinti,           ! Surface temperature calculation
     $     esinti,           ! Saturation vapor pressure lookup table
     $     mfinti,           ! Moist convective mass flux procedure
     $     hdinti            ! Horizontal diffusion
C
C-----------------------------------------------------------------------
C
C Initialize constants 
C
      clrh2o = latvap/rh2o   ! Passed to several initialization routines
      tmass0 = 98222.        ! Globally averaged sfc. partial pressure
                             ! of dry air (i.e., dry mass of atmosphere)
C
C Latitude array (list of latitudes in radians)
C
      do irow=1,plat/2
         xlat = asin(sit(irow))
         clat(irow) = -xlat
         clat(plat - irow + 1) = xlat
      end do
C
C Call time independent initialization routines for parameterizations.
C
      call mgwinti(cpair   ,cpwv    ,gravit  ,rair    )
C
      call vdinti(cpwv    ,cpair   ,gravit  ,rair    )
C
      call rfinti(cpwv    ,cpair   )
      wssat = .15
      call tsinti(tmelt   ,cpair   ,epsilo  ,gravit  ,latvap  ,
     $            rair    ,stebol  ,wssat   ,clrh2o  ,dtime   )
C
      call radini(gravit  ,cpair   ,epsilo  ,stebol  )
C
      call cainti(cappa   ,cpair   ,epsilo  ,gravit  ,latvap  ,
     $            rhoh2o  ,wssat   ,rair    ,clrh2o  ,tmelt   )
C
      call esinti(epsilo  ,latvap  ,latice  ,rh2o    ,cpair   )
C
      call mfinti(rair    ,cpair   ,gravit  ,latvap  , rhoh2o )
C
      call hdinti(rearth  ,dtime   )
C
      return
      end
