#include "params.h"
#ifdef COUPLED
      subroutine parlyz(pccm_comm)
#else
      subroutine parlyz
#endif
c------------------------------------------------------------
c
c This routine is for the parallel code only and
c Sets communication parameters and begin initializations.
c
c------------------------------------------------------------
      implicit none
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
      integer top,ord,dir,iplon,iplat,loglon,loglat,dummy
      integer nprocs, idum2,mdum
      integer ring_map
      external ring_map
#ifdef COUPLED
      integer pccm_comm
#endif
#if defined(PP_MPI)
      real combuf(SYSBUF)
#else
      real combuf(1)
#endif
      common /sbuffer/ combuf
c------------------------------------------------------------
c
c ... The following commands initialize the values of m_pin and nprocs
c
      nprocs = p_nproc
#ifdef COUPLED
      call open0f(nprocs, m_pin, idum2, pccm_comm)
      call comminit(2*nprocs, SYSBUF*R_SIZE, combuf, SYSBUF, pccm_comm)
#else
      call open0f(nprocs, m_pin, idum2)
      call comminit(2*nprocs, SYSBUF*R_SIZE, combuf, SYSBUF)
#endif

      call check0f(-1)
#if defined(INTEL)
      call flushpipe
      call denormal
#endif
c
c ... Set default data type
c
#if defined(WORD4)
      call setdataf('real')
#else
      call setdataf('double')
#endif

c
c .. Check parameters are ok and set up processor numbering.
c
      if(nprocs .ne. p_nproc_ln*p_nproc_lt) then
         write(6,1344) nprocs,p_nproc_ln*p_nproc_lt
         stop
      end if
c
c ... Procln and p_nproc_lt must be powers of two
c
      loglon = 0
      iplon = 1
      do while (iplon .lt. p_nproc_ln)
        iplon = 2*iplon
        loglon = loglon + 1
      end do
c
      if (p_nproc_ln .ne. iplon) then
        write(6,1346)
        stop
      endif
c
      loglat = 0
      iplat = 1
      do while (iplat .lt. p_nproc_lt)
        iplat = 2*iplat
        loglat = loglat + 1
      end do
      if (p_nproc_lt .ne. iplat) then
        write(6,1348)
        stop
      endif
c
c .. Number of latitudes/longitudes must be multiple of p_nproc_ln/p_nproc_lt
c
      if (p_glon .ne. (plon*p_nproc_ln)) then
        write(6,2345)
        stop
      endif
c
c     if ((plon .ne. 2*(plon/2)) .and. plon .ne. 1 ) then
c       write(6,2347)
c       stop
c     endif
c
      if (p_glat .ne. (plat*p_nproc_lt)) then
        write(6,2346)
        stop
      endif
c
c     if (plat .ne. 2*p_lato2) then
c       write(6,2348)
c       stop
c     endif
c
c ... Parameters are OK.  Write decomposition information
c
      if( m_pin .eq. 0 ) then
         write(6,1345) m_pin,p_nproc_lt,p_nproc_ln
      end if
c
c ... Compute location in machine (m_pin_ln, m_pin_lt, m_glon, m_glat, ...)
c     Following code gives embedding in MESH.
c
      call mesh_map_init(1, p_nproc_ln, p_nproc_lt, mapmesh)
      call mesh_index(m_pin, p_nproc_ln, p_nproc_lt, mapmesh, 
     $  m_pin_ln, m_pin_lt)
      call row_map_init(m_pin_lt, p_nproc_ln, p_nproc_lt, 
     $  mapmesh, maprow)
      call column_map_init(m_pin_ln, p_nproc_ln, p_nproc_lt, 
     $  mapmesh, mapcol)
      m_glat  = p_glat/(2*p_nproc_lt)*m_pin_lt+1
      m_glon  = -1
c     calculate neighbors (up, down, right, left) in toroidal mesh
      m_pin_up  = ring_map(m_pin_lt+1,p_nproc_lt,mapcol)
      m_pin_dwn = ring_map(m_pin_lt-1,p_nproc_lt,mapcol)
      m_pin_rht = ring_map(m_pin_ln+1,p_nproc_ln,maprow)
      m_pin_lft = ring_map(m_pin_ln-1,p_nproc_ln,maprow)
c
c ...  Initialize mtype
c
      m_type = 10000


      return
c
 1344   format('Numprocs (',i2,') != p_nproc_ln*p_nproc_lt (',i2,')')
 1346   format("Error: p_nproc_ln not a power of two")
 1348   format("Error: p_nproc_lt not a power of two")
 2345   format("Error: p_glon not a multiple of p_nproc_ln")
 2347   format("Error: plon not a multiple of 2")
 2346   format("Error: p_glat not a multiple of p_nproc_lt")
 2348   format("Error: plat not a multiple of 2")
c
 1345   format('PCCM Proc',I3,':Lat/long decomposition: ',
     $                 I3,' by ',I3,' processor(s)')
      end



