#include "params.h"
      subroutine scan2(cwava)
C-----------------------------------------------------------------------
C
C Second gaussian latitude scan, converts from spectral coefficients to
C grid point values, from poles to equator, with read/calculate/write cycle.
C The latitude pair loop in this routine is multitasked.
C
C The grid point values of ps, t, q, u, v, z (vorticity), and d (divergence)
C are calculated and stored for each latitude from the spectral coefficients.
C In addition, the pressure-surface corrections to the horizontal diffusion
C are applied and the global integrals of the constituent fields are
C computed for the mass fixer.
C
C Computational note: The main loop over latitude pairs in this routine
C is multitasked.
C
C-----------------------------------------------------------------------
      implicit none
#include "pmgrid.com"
C---------------------------------------------------------------------
#include "pagrid.com"
C---------------------------------------------------------------------
#include "comcon.com"
C---------------------------------------------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comslt.com"
C-----------------------------------------------------------------------
#include "com3d.com"
C-----------------------------------------------------------------------
#ifdef PP_PCCM
#include "pspect.com"
#include "commap.com"
#include "comparll.com"
#endif
C-----------------------------------------------------------------------
c
c Input arguments
c
      real cwava
C
C     !  Local workspace
C
#ifdef BLOCK_FFT
#include "block_fft.com"
       real fft_buffer( plond, max_vects_bwd )
       integer ip, nvect, isign
# ifdef CRAY
      real work1((plon+1)*pcray)  ! Workspace for fft (Cray)
      real work2(1)
# else
#  if (defined(PP_PCCM))
#   if (defined(PP_FFTX))
c     transpose parallel fft work space
      real work1(plond,max_vects_bwd)
      real work2(p_glon+2,((max_vects_bwd)/p_nproc_ln + 1)*2) 
#   else
c     distributed parallel fft work space
      real work1(plond,(max_vects_bwd))
      real work2(1)
#   endif
#  else
      real work1((plon+1)*max_vects_bwd)   ! sequential definition
      real work2(1)
#  endif
# endif
#endif
#ifdef PP_PCCM
c
c hemispheric binary tree sums of hw quantities (see comment
c in the file src/scan1.F).
      real hw2a(pcnst,2)
     $    ,hw2b(pcnst,2)
     $    ,hw3a(pcnst,2)
     $    ,hw3b(pcnst,2)
      real hw2al (pcnst,plat)
     $    ,hw2bl (pcnst,plat)
     $    ,hw3al (pcnst,plat)
     $    ,hw3bl (pcnst,plat)
#else
      real hw2a(pcnst)
     $    ,hw2b(pcnst)
     $    ,hw3a(pcnst)
     $    ,hw3b(pcnst)
      real hw2al (pcnst,plat/2)
     $    ,hw2bl (pcnst,plat/2)
     $    ,hw3al (pcnst,plat/2)
     $    ,hw3bl (pcnst,plat/2)
#endif
c declarations inserted by implnone
      integer irow,itmp,k,m
#ifdef PP_PCCM
      integer kk, mlen
      real repro_tmp(   plat + 2*plat*plev)
      real repro2_tmp(  plat + 2*plat*plev)
      real f_locsum
      integer mm
c
c -- hemispheric binary tree sums of tmass (see comment above)
c
      real tmassf_1, tmassf_2
      real rbuf(2 + 8*pcnst)
#endif
C
C     ! Initialize moisture and mass integrals
C
#ifdef PP_PCCM
      do 5 m=1,pcnst
         hw2a(m,1) = 0.
         hw2b(m,1) = 0.
         hw3a(m,1) = 0.
         hw3b(m,1) = 0.
         hw2a(m,2) = 0.
         hw2b(m,2) = 0.
         hw3a(m,2) = 0.
         hw3b(m,2) = 0.
    5 continue
#else
      do 5 m=1,pcnst
         hw2a(m) = 0.
         hw2b(m) = 0.
         hw3a(m) = 0.
         hw3b(m) = 0.
    5 continue
#endif
CMIC$ DO ALL SHARED(CWAVA, N3M1, N3, TMASS, TQ, DQ, PS, U3, V3, T3, Q3, 
CMIC$1   QFCST, PDELA, HW2AL, HW2BL, HW3AL, HW3BL) PRIVATE(IROW, K)
#ifdef PP_PCCM
      call resetr( tmass, plat     , 0.0 )
      call resetr(    tq, plat*plev, 0.0 )
      call resetr(    dq, plat*plev, 0.0 )
      call resetr( hw2al, pcnst*plat, 0.0 )
      call resetr( hw2bl, pcnst*plat, 0.0 )
      call resetr( hw3al, pcnst*plat, 0.0 )
      call resetr( hw3bl, pcnst*plat, 0.0 )
# ifdef BLOCK_FFT
      ip = 1
      do irow=1,plat/2
         call spgrdrv_a(irow,cwava,ps(1,1,n3m1),
     $                  u3(i1,1,1,1,n3m1),v3(i1,1,1,1,n3m1),
     $                  t3(i1,1,1,1,n3m1),q3(i1,1,1,1,1,n3m1),
     $                  q3(i1,1,1,1,1,n3),qfcst(i1,1,1,1,1),
     $                  u3(i1,1,1,1,n3),v3(i1,1,1,1,n3),pdela,
     $                  hw2al,hw2bl,hw3al,hw3bl,
     $                  fft_buffer, ip )
      enddo
      nvect = ip - 1
      isign = +1
      if (nvect .gt. max_vects_bwd) then
#    ifdef PP_ERROR
	write(0,*) ' scan2 block_fft: too many vectors: ',nvect,' > ',
     &             max_vects_bwd
#    endif
	call abortf('scan2')
      endif
      call rftlon(fft_buffer, work1, work2, plond, plon, nvect, isign)

      ip = 1
      do irow=1,plat/2
         call spgrdrv_c(irow,cwava,ps(1,1,n3m1),
     $                  u3(i1,1,1,1,n3m1),v3(i1,1,1,1,n3m1),
     $                  t3(i1,1,1,1,n3m1),q3(i1,1,1,1,1,n3m1),
     $                  q3(i1,1,1,1,1,n3),qfcst(i1,1,1,1,1),
     $                  u3(i1,1,1,1,n3),v3(i1,1,1,1,n3),pdela,
     $                  hw2al,hw2bl,hw3al,hw3bl,
     $                  fft_buffer, ip )
      enddo
# else
      do 20 irow=1,plat/2
         call spgrdrv(irow,cwava,ps(1,1,n3m1),
     $                u3(i1,1,1,1,n3m1),v3(i1,1,1,1,n3m1),
     $                t3(i1,1,1,1,n3m1),q3(i1,1,1,1,1,n3m1),
     $                q3(i1,1,1,1,1,n3),qfcst(i1,1,1,1,1),
     $                u3(i1,1,1,1,n3),v3(i1,1,1,1,n3),pdela,
     $                hw2al,hw2bl,hw3al,hw3bl)
   20 continue
# endif
#else
      do 20 irow=1,plat/2
         tmass(irow) = 0.
         do 10 k=1,plev
            tq(irow,k) = 0.
            dq(irow,k) = 0.
   10    continue
         call spgrdrv(irow,cwava,ps(1,1,n3m1),
     $                u3(i1,1,j1,n3m1),v3(i1,1,j1,n3m1),
     $                t3(i1,1,j1,n3m1),q3(i1,1,1,j1,n3m1),
     $                q3(i1,1,1,j1,n3),qfcst(i1,1,1,j1),
     $                u3(i1,1,j1,n3),v3(i1,1,j1,n3),pdela,
     $                hw2al,hw2bl,hw3al,hw3bl)
   20 continue
#endif
#ifdef PP_PCCM
C
C     ! Normalize integrals
C
       if(doslt) then
c
c  Accumulate tmassf
c   (note that local integration was done in mint called from spegrd)
c	 
	 call rowsum0( tmass, repro_tmp, plat, m_type )
	 m_type = m_type + 2*p_nproc
c       
         tmassf_1 = f_locsum(tmass(1),plat/2,repro_tmp)
         tmassf_2 = f_locsum(tmass(plat/2+1),plat/2,repro_tmp)

C
C     !  hw2/hw3 computation now complete (SPEGRD) => can compute alpha
C
         
	 call rowsum0( hw2al, repro_tmp, pcnst*plat, m_type)
         m_type = m_type + 2*p_nproc
	 call rowsum0( hw2bl, repro_tmp, pcnst*plat, m_type)
         m_type = m_type + 2*p_nproc
	 call rowsum0( hw3al, repro_tmp, pcnst*plat, m_type)
         m_type = m_type + 2*p_nproc
	 call rowsum0( hw3bl, repro_tmp, pcnst*plat, m_type)
         m_type = m_type + 2*p_nproc

         do 50 m=1,pcnst

	    do irow = 1,plat
	       repro2_tmp(irow)=hw2al(m,irow)
	    enddo
	    hw2a(m,1) = f_locsum( repro2_tmp(1), plat/2, repro_tmp )
	    hw2a(m,2) = f_locsum( repro2_tmp(plat/2+1), 
     $                                           plat/2, repro_tmp )

	    do irow = 1,plat
	       repro2_tmp(irow)=hw2bl(m,irow)
	    enddo
	    hw2b(m,1) = f_locsum( repro2_tmp(1), plat/2, repro_tmp )
	    hw2b(m,2) = f_locsum( repro2_tmp(plat/2+1), 
     $                                           plat/2, repro_tmp )

	    do irow = 1,plat
	       repro2_tmp(irow)=hw3al(m,irow)
	    enddo
	    hw3a(m,1) = f_locsum( repro2_tmp(1), plat/2, repro_tmp )
            hw3a(m,2) = f_locsum( repro2_tmp(plat/2+1), 
     $                                           plat/2, repro_tmp )

	    do irow = 1,plat
	       repro2_tmp(irow)=hw3bl(m,irow)
	    enddo
	    hw3b(m,1) = f_locsum( repro2_tmp(1), plat/2, repro_tmp )
            hw3b(m,2) = f_locsum( repro2_tmp(plat/2+1), 
     $                                           plat/2, repro_tmp )

   50    continue

         rbuf(1) = tmassf_1
	 rbuf(2) = tmassf_2
         mm = 2
         do 51 m=1,pcnst
             rbuf(mm+1) = hw2a(m,1)
             rbuf(mm+2) = hw2b(m,1)
             rbuf(mm+3) = hw3a(m,1)
             rbuf(mm+4) = hw3b(m,1)
             rbuf(mm+5) = hw2a(m,2)
             rbuf(mm+6) = hw2b(m,2)
             rbuf(mm+7) = hw3a(m,2)
             rbuf(mm+8) = hw3b(m,2)
             mm = mm + 8
51       continue

	 call colsum0(rbuf,r_tmp,mm,m_type)
         m_type = m_type + 2*p_nproc

         tmassf = rbuf(1) + rbuf(2)
         mm = 2
         do 52 m=1,pcnst
             hw2a(m,1) = rbuf(mm+1) + rbuf(mm+5)
             hw2b(m,1) = rbuf(mm+2) + rbuf(mm+6)
             hw3a(m,1) = rbuf(mm+3) + rbuf(mm+7)
             hw3b(m,1) = rbuf(mm+4) + rbuf(mm+8)
             mm = mm + 8
52       continue
         tmassf = tmassf*.5/float(p_glon)
         qmassf = hw1(1)*gravit
         fixmas = (tmass0 + qmassf)/tmassf
c
         do 53 m=1,pcnst
c
c            hw2(m)    = hw2a(m,1) + fixmas*hw2b(m,1)
c            hw3(m)    = hw3a(m,1) + fixmas*hw3b(m,1)
c
c *** THE NEXT 2 LINES OF CODE ONLY FOR MACHINE ACCURATE ANSWERS
c
            hw2(m)    = hw2a(m,1) + hw2b(m,1)
            hw3(m)    = hw3a(m,1) + hw3b(m,1)
            alpha(m)  = ( hw1(m) - hw2(m) )/hw3(m)
   53    continue
#ifdef PP_ERROR
         write(0,*)' ***scan2: hw2 = ',hw2
         write(0,*)' ***scan2: hw3 = ',hw3
#endif
      else
       if (p_nproc_ln.gt.1) then
	 kk = 1
	 do irow = 1,plat
	    repro2_tmp(kk) = tmass(irow)
	    kk = kk + 1
            do k=1,plev
	       repro2_tmp(kk) = tq(irow,k)
	       kk = kk + 1
	       repro2_tmp(kk) = dq(irow,k)
	       kk = kk + 1
	    enddo
	 enddo
	 mlen = kk - 1 
c
	 call rowsum0( repro2_tmp, repro_tmp, mlen, m_type )
	 m_type = m_type + 2*p_nproc
c
	 kk = 1
	 do irow = 1,plat
	    tmass(irow) = repro2_tmp(kk) * w(irow)
	    kk = kk + 1
            do k=1,plev
	       tq(irow,k) = repro2_tmp(kk)
	       kk = kk + 1
	       dq(irow,k) = repro2_tmp(kk)
	       kk = kk + 1
	    enddo
	 enddo
       endif

       tmassf = f_locsum( tmass, plat, repro_tmp )
       do k = 1, plev
	  tqf(k) = f_locsum( tq(1,k), plat, repro_tmp )
	  dqf(k) = f_locsum( dq(1,k), plat, repro_tmp )
       enddo

       if (p_nproc_lt.gt.1) then
	 kk = 1
	 repro2_tmp(kk) = tmassf
	 kk = kk + 1
	 do k = 1, plev
	    repro2_tmp(kk) = tqf(k)
	    kk = kk + 1
	    repro2_tmp(kk) = dqf(k)
	    kk = kk + 1
	 enddo
	 mlen = kk - 1

	 call colsum0( repro2_tmp, repro_tmp, mlen, m_type )
	 m_type = m_type + 2*p_nproc

	 kk = 1
	 tmassf = repro2_tmp(kk) * 0.5/float(p_glon)
	 kk = kk + 1
	 do k = 1, plev
	    tqf(k) = repro2_tmp(kk) * 0.5/float(p_glon)
	    kk = kk + 1
	    dqf(k) = repro2_tmp(kk) * 0.5/float(p_glon)
	    kk = kk + 1
	 enddo
       else
         tmassf = tmassf * 0.5/float(p_glon)
	 do k = 1, plev
	    tqf(k) = tqf(k) * 0.5/float(p_glon)
	    dqf(k) = dqf(k) * 0.5/float(p_glon)
	 enddo
       endif
c
       qmassf = 0.0
       do k = 1, plev
	  qmassf = qmassf + tqf(k)
       enddo
       qmassf = qmassf * gravit

      end if
#else
C
C     ! Normalize integrals
C
      tmassf = 0.
      do 40 irow=1,plat/2
         tmassf = tmassf + tmass(irow)
   40 continue
      tmassf = tmassf*.5/float(plon)
      if(doslt) then
         qmassf = hw1(1)*gravit
         fixmas = (tmass0 + qmassf)/tmassf
C
C     !  hw2/hw3 computation now complete (SPEGRD) => can compute alpha
C
         do 50 m=1,pcnst
            do 55 irow=1,plat/2
               hw2a(m) = hw2a(m) + hw2al (m,irow)
               hw2b(m) = hw2b(m) + hw2bl (m,irow)
               hw3a(m) = hw3a(m) + hw3al (m,irow)
               hw3b(m) = hw3b(m) + hw3bl (m,irow)
   55       continue
c            hw2(m)    = hw2a(m) + fixmas*hw2b(m)
c            hw3(m)    = hw3a(m) + fixmas*hw3b(m)
c
c *** THE NEXT 2 LINES OF CODE ONLY FOR MACHINE ACCURATE ANSWERS
c
            hw2(m)    = hw2a(m) + hw2b(m)
            hw3(m)    = hw3a(m) + hw3b(m)
            alpha(m)  = ( hw1(m) - hw2(m) )/hw3(m)
   50    continue
c         write(*,*)' ***scan2: hw2 = ',hw2
c         write(*,*)' ***scan2: hw3 = ',hw3
      else
         qmassf = 0.
         do 30 k=1,plev
            tqf(k) = 0.
            dqf(k) = 0.
            do 25 irow=1,plat/2
               tqf(k) = tqf(k) + tq(irow,k)
               dqf(k) = dqf(k) + dq(irow,k)
   25       continue
            tqf(k) = tqf(k)*.5/float(plon)
            dqf(k) = dqf(k)*.5/float(plon)
            qmassf = qmassf + tqf(k)
   30    continue
         qmassf = qmassf*gravit
      end if
#endif
C
C     !  Toggle n3,n3m1 units for 3-d arrays
C
      itmp = n3
      n3 = n3m1
      n3m1 = itmp
      return
      end

