#include "params.h"
      subroutine wrapup(lhbuf)
C-----------------------------------------------------------------------
C
C This routine will close and dispose to mass store any full history tape
C or any history tape that has data on it when restarts are being written.
C It will then dispose restart datasets according to the lrewrt variable.
C Finally it will assign new history and restart names.  If a partially full
C history tape was disposed (for restart purposes) then wrapup will open that
C unit back up and position it for appending new data. 
C
C--------------------------Code history---------------------------------
C
C Original version  L. Bath
C Standardized:     L. Bath, June 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comloc.com"
C-----------------------------------------------------------------------
#include "comlun.com"
C-----------------------------------------------------------------------
#include "commss.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comparll.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer lhbuf        ! Length of history tape buffer
C
C---------------------------Local variables-----------------------------
C
      logical lwrite,      ! Was history tape written on this iter?
     $        lremov,      ! Rm link after dispose? (subject to setting
     $        lln,         ! Make link to /usr/tmp ?     of ldebug)
     $        lfill,       ! Is history tape ready to dispose?
     $        hdispose,    ! Primary tape disposed
     $        rdispose     ! Regeneration tapes disposed
      character chtem*80,
     $          fname*16,  ! File name for regeneration files
     $          uniqend*3, ! unique file extention (coded)
     $          bbd*12	   ! Suffix temp
      integer ktape,       ! History tape number
     $        kt,i,        ! Indices
     $        reten        ! Retention Time for mass store
      integer nf           ! File counter
      integer itape        ! file index
      integer iwsiz        ! prealloc size
      real    tday         ! Model day number for printout
      integer ierr
#ifdef SIMPLE_OUTPUT
      character holdnhfil*11
#endif
C
C----------------------------Externals----------------------------------
      logical whenwr
      integer lenchr 
      integer suffix
      external whenwr,     ! Is it time to write the history tape?
     $         assign,     ! System assign
     $         bldcom,     ! Build the mass store comment field
     $         savdis,     ! Send output file to the mass store
     $         wsds,       ! Write master regeneration dataset
     $         lenchr,     ! Actual length of a character string
     $         prealc,     ! Preallocate disk space
     $         bldfln,
     $         ljust,
     $         rgnfls,
     $         endrun
C
C-----------------------------------------------------------------------
C
      hdispose = .false.
      rdispose = .false.
      lln = ldebug
#ifndef NOOUT
      do 400 ktape=1,mtapes
         lfill = .false.
         lwrite = whenwr(nstep,ktape)
C
C Find out if tape is full
C
         if (lwrite.and.nfils(ktape).ge.mfilt(ktape))
     $       lfill = .true.
C
C Dispose history tape to mass store if 
C    1) tape is filled or 
C    2) this is the end of run and tape has data on it or
C    3) restarts are being put out and history tape has data on it
C
         if ( (lfill).or.
     $        (nlend.and.nfils(ktape).ge.1).or.
     $        (lrewrt.and.nfils(ktape).ge.1) ) then
C
C Dispose history tape
C
            if (ktape.eq.1) hdispose = .true.
C
C If this is a monthly average run, set date and day variables 
C to reflect a one month average in the comment field
C
            if (ninavg(ktape).eq.'Q') then
               ndcurf(ktape) = ndcur
               nscurf(ktape) = nscur
               ncdatf(ktape) = ncdate
               ncsecf(ktape) = ncsec
            end if
            lremov = .true.
            if (nlend.or..not.lfill) lremov = .false.
C
C Dispose history tape.  If this is the 0 timestep data of a monthly 
C average do not dispose, just close primary unit (this data is 
C effectively thrown away) 
C
            if (ninavg(ktape).eq.'Q' .and. nstep.le.1) then
               close (hunit(1))
            else
               call savdis(hunit(ktape),hpath   ,nhfil(ktape),
     $                     cpath(ktape),nswrps  ,
     $                  nsmvn   ,irt     ,msscom  ,lln     ,lremov  ,
     $                  async)
C
C Print information concerning model output.
C Model day number = iteration number of history tape data * 2*delta-t
C divided by 2*(seconds in the day)
C
               tday = (nstep-1)*twodt/172800.
               if (ktape.eq.1) then
                  write(6,9001)
               else
                  write(6,9002) ktape-1
               end if
               write(6,9003)nstep,nfils(ktape),tday
               write(6,9004)
            end if 
C
C Reset file limit if needed
C
            if ((nlfilt).and.(ktape.eq.1)) then
               mfilt(ktape) = mfilt(ktape) - 1
               nlfilt = .false.
            end if
         end if
  400 continue                  !  End of mtapes loop
C
C Store header info and write restart data if 
C 1. a primary tape has been disposed and restarts have been written
C    (if nrefrq is nonzero the model will dispose some primarys without 
C     corresponding restarts so we have to check lrewrt also)
C 2. midmonth restarts have been written for a monthly average case
C
       if ((ninavg(1).ne.'Q'.and.lrewrt.and.hdispose).or.
     $     (ninavg(1).eq.'Q'.and.lrewrt)) then
         rdispose=.true.
         do kt=1,mtapes
            ppath(kt) = cpath(kt)
         end do
         lremov = .true.
         if (nlend) lremov = .false.
C
         if (nsvsn.ne.'        ') then
            call wsds(lhbuf)
C
C Set up unique extention for monthly average restarts.
C
            if (ninavg(1).eq.'Q') then
               uniqend = '.'//char(97+(rg3num-1)/26)//
     $              char(97+mod((rg3num-1),26))
C
C Monthly average mid-month restarts will be given a 7 day retention 
C period unless the user specifies a retention period of 0 via the namelist.
C End of month and end of run restarts will have the namelist retention.
C
               if (nlend.or.hdispose.or.rirt.eq.0) then
                  reten=rirt
               else
                  reten=7
               end if
            else
               uniqend = ' '//' '//' '
               reten=rirt
            endif
            do i=1,16
                  fname(i:i) = ' '
            end do 
	    write(bbd,'(i5)') m_pin_lt+10000
C
C If this is a monthly average then there is a unique ending appended
C to the local copy of the regeneration datasets.  
C
            fname=nrfil(1:lenchr(nrfil))//uniqend//'.'//bbd(2:5)
C
C The unique extention is removed from the month-end datasets when they 
C are saved to the Mass Store.  The absence of the unique ending 
C distinguishes a month end restart from the temporary mid-month restarts
C
            if (hdispose.and.ninavg(1).eq.'Q') then
               chtem = nrpath(1:nrpthl)//fname(1:lenchr(fname)-3)
            else
               chtem = nrpath(1:nrpthl)//fname
            end if 
            call savdis(nrg     ,rpath   ,fname   ,chtem   ,
     $           nswrps  ,
     $           nrmvn   ,reten ,' '     ,lln     ,
     $           lremov  ,
     $           async)
            do nf=1,nnrg1
               do i=1,16
                  fname(i:i) = ' '
               end do 
               fname = nrfil(1:lenchr(nrfil)) // rg1ext(nf) //
     $                 uniqend
C
C The unique extention is removed from the month-end datasets when they 
C are saved to the Mass Store.  The absence of the unique ending 
C distinguishes a month end restart from the temporary mid-month restarts
C
               if (hdispose.and.ninavg(1).eq.'Q') then
                  chtem = nrpath(1:nrpthl) //fname(1:lenchr(fname)-3)
               else             
                  chtem = nrpath(1:nrpthl) // fname
               end if 
               call savdis(nrg1(nf),rpath   ,fname   ,chtem   ,
     $              nswrps  ,
     $              nrmvn   ,reten ,' '     ,lln     ,
     $              lremov  ,
     $              async)
            end do
            write(6,9004)
            if (aeres) then
               do nf=1,nnrg2
                  do i=1,16
                     fname(i:i) = ' '
                  end do 
                  fname = nrfil(1:lenchr(nrfil)) // rg2ext(nf) //
     $                 uniqend
C
C The unique extention is removed from the month-end datasets when they 
C are saved to the Mass Store.  The absence of the unique ending 
C distinguishes a month end restart from the temporary mid-month restarts
C
                  if (hdispose.and.ninavg(1).eq.'Q') then
                     chtem = nrpath(1:nrpthl) //
     $                    fname(1:lenchr(fname)-3)
                  else
                     chtem = nrpath(1:nrpthl) // fname
                  end if
                  call savdis(nrg2(nf),rpath   ,fname   ,
     $                 chtem   ,nswrps  ,
     $                 nrmvn   ,reten   ,' '     ,
     $                 lln     ,lremov  ,
     $                 async)
               end do
               write(6,9004)
            end if
C
C Write and dispose restart dataset
C
            do i=1,80
               chtem(i:i) = ' '
            end do 
C
C The unique extention is removed from the month-end datasets when they 
C are saved to the Mass Store.  The absence of the unique ending 
C distinguishes a month end restart from the temporary mid-month restarts
C
            if (hdispose.and.ninavg(1).eq.'Q') then
               chtem = nrpath(1:nrpthl)//nrfil
            else
               chtem = nrpath(1:nrpthl)//nrfil//uniqend
            end if

c I am commenting out these lines. They are only used in the Mass Store
c implementation, and without them, I will not get that annoying little
c fort.7 file all of the time. Chad Schafer - 6/5/96
c	    if ( m_pin .eq. mapmesh(0,0) ) then
c             rewind nsds
c             write (nsds,'(a)') chtem(1:lenchr(chtem))
c	    endif
            do i=1,80
               chtem(i:i) = ' '
            end do 
            chtem = nrpath(1:nrpthl)//nsvsn(1:lenchr(nsvsn))
            call savdis(nsds    ,rpath   ,nsvsn   ,chtem   ,
     $           nswrps  ,
     $           nrmvn   ,rirt    ,' '     ,lln     ,
     $           .false. ,
     $           async)
            write(6,9004)
C           
C dispose hbuf regeneration dataset. uniqend is a 2 letter extension
C added to the temporary local regeneration files to make them unique.
C           
            do itape=1,mtapes
               lwrite = whenwr(nstep,itape)
               if (ninavg(1).eq.'Q'.and..not.lwrite) then
                  do nf=1,nnrg3(itape)
                     fname = nrfil(1:lenchr(nrfil)) // 
     $                    rg3ext(nf,itape) // uniqend 
C
C The unique extention is removed from the month-end datasets when they 
C are saved to the Mass Store.  The absence of the unique ending 
C distinguishes a month end restart from the temporary mid-month restarts
C
                     if (hdispose.and.ninavg(1).eq.'Q') then
                        chtem = nrpath(1:nrpthl) // 
     $                       nrfil(1:lenchr(nrfil)) // rg3ext(nf,itape)
                     else
                        chtem = nrpath(1:nrpthl) // 
     $                       nrfil(1:lenchr(nrfil)) // rg3ext(nf,itape)
     $                       //uniqend
                     endif
                     call savdis(nrg3(nf,itape),rpath   ,fname   ,
     $                    chtem,  nswrps  ,
     $                    nrmvn   ,reten  ,' '     ,lln     ,
     $                    lremov  ,async  )
                     write(6,9004)
                  end do
               end if
            end do
         end if                 !  End of restart block
      end if                    !  End of lrewrt block
      if (hdispose.and.ninavg(1).eq.'Q'.and.nstep.gt.1) rg3num=0
                      
C
C History tapes have been disposed now its time to assign and open
C new history file names
C
      do 450 ktape=1,mtapes
         lfill = .false.
         lwrite = whenwr(nstep,ktape)
C
C Find out if tape is full
C
         if (lwrite.and.nfils(ktape).ge.mfilt(ktape))
     $        lfill = .true.
C        
C If a history tape is full and not end of run, build new datasets
C
         if (lfill.and..not.nlend) then
#ifdef SIMPLE_OUTPUT
              holdnhfil=nhfil(ktape)
#endif
	    if ( .not. parhist ) then
              call bldfln(nhfil(ktape))
	    else
              call bldfln_a(nhfil(ktape), m_pin_lt)
	    endif
            cpath(ktape) = nhpath(1:nhpthl)//nhfil(ktape)
	    iwsiz = plenhi + plenhc + plenhr + plat*nrlen(ktape)
	    if ( .not. parhist ) then
	      iwsiz = iwsiz*mfilt(ktape)*p_nproc
	      if ( m_pin .eq. mapmesh(0,0) ) then
		call prealc(nhfil(ktape),  iwsiz)
		close( hunit(ktape) )
#ifdef SIMPLE_OUTPUT
               call move_atmos_his(holdnhfil,m_pin_lt,nstep)

                open(unit=hunit(ktape),file=
     $               'history/atmos/'//nhfil(ktape),
     $               form='UNFORMATTED',status='UNKNOWN',
     $               iostat=ierr)
#else
                open(unit=hunit(ktape),file=nhfil(ktape),
     $               form='UNFORMATTED',status='UNKNOWN',
     $               iostat=ierr)
#endif
                if(ierr.ne.0) then
                  write(6,*)'WRAPUP: Failed to open file ',
     $                       nhfil(ktape),' ierr=',ierr
                  call endrun
                endif
                write(6,*)'WRAPUP: Successfully opened file ',
     $                nhfil(ktape),' on hunit(ktape)= ',hunit(ktape)
	      endif
	    else
	      iwsiz = iwsiz*mfilt(ktape)
	      if ( m_pin_ln .eq. mapmesh(0,0) ) then
		call prealc(nhfil(ktape),  iwsiz)
		close( hunit(ktape) )
#ifdef SIMPLE_OUTPUT
               call move_atmos_his(holdnhfil,m_pin_lt,nstep)

                open(unit=hunit(ktape),file=
     $               'history/atmos/'//nhfil(ktape),
     $               form='UNFORMATTED',status='UNKNOWN',
     $               iostat=ierr)
#else
                open(unit=hunit(ktape),file=nhfil(ktape),
     $               form='UNFORMATTED',status='UNKNOWN',
     $               iostat=ierr)
#endif
                if(ierr.ne.0) then
                  write(6,*)'WRAPUP: Failed to open file ',
     $                       nhfil(ktape),' ierr=',ierr
                  call endrun
                endif
                write(6,*)'WRAPUP: Successfully opened file ',
     $                nhfil(ktape),' on hunit(ktape)= ',hunit(ktape)
	      endif
	    endif
            nfils(ktape) = 0
         end if         !  End of second lfill block
C
  450 continue         !  End of mtapes loop
#ifdef COUPLED
      if(lrewrt.and.( m_pin .eq. mapmesh(0,0))) then
        suffix = (nstep-1)/TSPDAY
	call restart_clean(suffix)
      endif
#endif
#endif
      return
 9001 format('    Primary history tape')
 9002 format('    Auxiliary history tape number ',i3)
 9003 format(
     $             '    Output at NSTEP     = ',i10/
     $             '    Number of time samples on this tape = ',i10/
     $             '    Model Day           = ',f10.2)
 9004 format('---------------------------------------')
      end

