#include "params.h"
      subroutine cmplr(hes     ,w       ,nc)
C-----------------------------------------------------------------------
C
C Compute w, eigenvalues of upper Hessenburg matrix hes
C
C---------------------------Code history--------------------------------
C
C This routine is of unknown lineage. It is only used to provide the
C equivalent depths of the reference atmosphere for a diagnostic print
C in SETTAU and has no effect on the model simulation. Therefore it can 
C be replaced at any time with a functionally equivalent, but more 
C understandable, procedure.  Consequently, the internal commenting has 
C not been brought up to CCM2 standards.                               
C
C Original version:  CCM1
C Standardized:      L. Bath, June 1992
C Reviewed:          D. Williamson, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer nc         ! Dimension of input and output matrices
      complex hes(nc,nc) ! Upper hessenberg matrix from comhes
C
C Output arguments
C
      complex w(nc)      ! Weights
C
C---------------------------Local variables-----------------------------
C
      integer itest,
     $        nfail,     ! Limit for number of iterations to convergence
     $        ntest,
     $        n,j,m,
     $        i,         ! Eigenvalue
     $        its,       ! Iteration counter
     $        l,
     $        l1,m1,n1,i1
      real a,
     $     sr,
     $     si,
     $     tr,
     $     ti,
     $     xr,
     $     yr,
     $     zr,
     $     xi,
     $     yi,
     $     areal,
     $     eps
      complex s,
     $        t,
     $        x,
     $        y,
     $        z,
     $        u
      external endrun
C
C------------------------------Data-------------------------------------
C
      data itest/0/
      save a,eps,sr,itest
C
      nfail = 30
      if (itest.eq.0) then
         a = 1
    5    continue
         eps = a
         sr = 1 + a
         a = a/2.0
         if (sr.ne.1.0) go to 5
         itest = 1
      end if
      if (nc.le.0) then
         write(6,*)'CMPLR: Entered with incorrect dimension  '
         write(6,*)'NC=',NC
         call endrun
      end if
      ntest = 10
      n = nc
      t = 0.0
   10 continue
      if (n.eq.0) go to 300
      its = 0
   20 continue
      if (n.ne.1) then
         do l1=2,n
            l = n + 2 - l1
            if (cabs(hes(l,l-1)).le.
     $           eps*(cabs(hes(l-1,l-1))+cabs(hes(l,l))))
     $           go to 50
         end do
      end if
      l = 1
   50 continue
      if (l.ne.n) then
         if (its.eq.nfail) then
            i = nc - n + 1
            write(6,*)'CMPLR: Failed to converge in ',nfail,
     $                ' iterations'
            write(6,*)'Eigenvalue=',i
            call endrun
         end if
         if (its.eq.ntest) then
            ntest = ntest + 10
            sr = hes(n,n-1)
            si = hes(n-1,n-2)
            sr = abs(sr)+abs(si)
            u = (0.0,-1.0)*hes(n,n-1)
            tr = u
            u = (0.0,-1.0)*hes(n-1,n-2)
            ti = u
            tr = abs(tr) + abs(ti)
            s = cmplx(sr,tr)
         else
            s = hes(n,n)
            x = hes(n-1,n)*hes(n,n-1)
            if (cabs(x).ne.0.0) then
               y = 0.5*(hes(n-1,n-1)-s)
               u = y*y + x
               z = csqrt(u)
               u = conjg(z)*y
               areal = u
               if (areal.lt.0.0) z = -z
               x = x/(y+z)
               s = s - x
            end if
         end if
         do i=1,n
            hes(i,i) = hes(i,i) - s
         end do
         t = t + s
         its = its + 1
         j = l + 1
         xr = cabs(hes(n-1,n-1))
         yr = cabs(hes(n,n-1))
         zr = cabs(hes(n,n))
         n1 = n - 1
         if ((n1.ne.1).and.(n1.ge.j)) then
            do 80 m1=j,n1
               m = n1 + j - m1
               yi = yr
               yr = cabs(hes(m,m-1))
               xi = zr
               zr = xr
               xr = cabs(hes(m-1,m-1))
               if (yr.le.eps*zr/yi*(zr+xr+xi)) go to 100
   80       continue
         end if
         m = l
  100    continue
         m1 = m + 1
         do 150 i=m1,n
            x = hes(i-1,i-1)
            y = hes(i,i-1)
            if (cabs(x).lt.cabs(y)) then
               i1 = i - 1
               do j=i1,n
                  z = hes(i-1,j)
                  hes(i-1,j) = hes(i,j)
                  hes(i,j) = z
               end do
               z = x/y
               w(i) = 1.0
            else
               z = y/x
               w(i) = -1.0
            end if
            hes(i,i-1) = z
            do j=i,n
               hes(i,j) = hes(i,j) - z*hes(i-1,j)
            end do
  150    continue
         m1 = m + 1
         do 190 j=m1,n
            x = hes(j,j-1)
            hes(j,j-1) = 0.0
            areal = w(j)
            if (areal.gt.0.0) then
               do i=l,j
                  z = hes(i,j-1)
                  hes(i,j-1) = hes(i,j)
                  hes(i,j) = z
               end do
            end if
            do i=l,j
               hes(i,j-1) = hes(i,j-1) + x*hes(i,j)
            end do
  190    continue
         go to 20
      end if
      w(n) = hes(n,n) + t
      n = n - 1
      go to 10
  300 continue
      return
      end
