#include "params.h"
      subroutine gauaw(a       ,w       ,k)
C-----------------------------------------------------------------------
C
C Calculate abscissa a(k) and weights w(k)
C for the gaussian integration.  the zeros of the
C bessel function j0 are used to get starting
C approximations for the abscissas.
C
C-------------------------------arguments-------------------------------
c inserted by implnone
      implicit none
c declarations inserted by implnone
#ifdef WORD4
      double precision avsp,fk,fn,pk,pkm1,pkm2,pkmrk,sp,xz
c declarations inserted by implnone
      integer is,iter,k,kk,l,n
      double precision a(k),     !  abscissa
     $     w(k)      !  weights
C------------------------------local variables--------------------------
      double precision pi, eps, c
#else
      real avsp,fk,fn,pk,pkm1,pkm2,pkmrk,sp,xz
c declarations inserted by implnone
      integer is,iter,k,kk,l,n
      real a(k),     !  abscissa
     $     w(k)      !  weights
C------------------------------local variables--------------------------
      real pi, eps, c
#endif
      data pi /3.14159265358979/
C
      eps = 1.e-13
C
C The value eps, used for convergence tests in the
C iterations, can be changed.
C newton iteration is used to find the abscissas.
C
      c = (1.-(2./pi)**2)*0.25
      fk = k
      kk = k/2
      call bsslzr(a,kk)
      do 30 is=1,kk
         xz = cos(a(is)/sqrt((fk+0.5)**2+c))
C
C This is the first approximation to xz
C
         iter = 0
   10    pkm2 = 1.
         pkm1 = xz
         iter = iter + 1
         if (iter.gt.10) then
C
C error exit
C
            write(6,*)'GAUAW:Error exit,no convergence in 10 iterations'
            call endrun
         endif
C
C computation of the legendre polynomial
C
         do 20 n=2,k
            fn = n
            pk = ((2.*fn-1.)*xz*pkm1-(fn-1.)*pkm2)/fn
            pkm2 = pkm1
            pkm1 = pk
   20    continue
         pkm1 = pkm2
         pkmrk = (fk*(pkm1-xz*pk))/(1.-xz**2)
         sp = pk/pkmrk
         xz = xz - sp
         avsp = abs(sp)
         if (avsp.gt.eps) go to 10
         a(is) = xz
         w(is) = (2.*(1.-xz**2))/(fk*pkm1)**2
   30 continue
      if (k.ne.kk*2) then
C
C     !  for odd k computation of weight at the equator
C
         a(kk+1) = 0.
         pk = 2./fk**2
         do 40 n=2,k,2
            fn = n
            pk = pk*fn**2/(fn-1.)**2
   40    continue
         w(kk+1) = pk
      endif
C
C     !  complete the sets of abscissas and weights, using the symmetry.
C
      do 60 n=1,kk
         l = k + 1 - n
         a(l) = -a(n)
         w(l) = w(n)
   60 continue
      return
      end
