#include "params.h"
C----------------------------------------------------------------------
C  permute.f
C
C  Part of program that decomposes spectral and 
C  fourier domains by producing local index arrays.
C
C  This is a revised version which returns to the load
C  balanced ordering used in the transpose-sequential FFT.  
C
C----------------------------------------------------------------------
C
C This routine can be called for versions of the code that
C use the transpose-sequential FFT.  It orders the m coeficients
C in a way that is most amedable to load balancing in the
C spectral transform.
C
C modified 2/95 by P. Worley to not reorder when the Fourier
C coefficients are not decomposed, to use the updated
C ordering that supports not transposing truncated wavenumbers,
C and rewritten in fortran:
C
C       Using double transpose/serial FFT. Thus all wavenumbers 
C       are local in decomposed Fourier grid initially, but next 
C       transpose will decompose over the wavenumber dimension. Reorder
C       the wavenumbers first to load balance the subsequent Legendre 
C       transform.
C
C       Calculate ordering of untruncated wavenumbers to balance 
C       assignment of long and short columns of spectral coefficients:
C       Assign wavenumbers 0-(P-1) to processor rows 
C       (P-1)-0, wavenumbers P-(2P-1) to rows 0-(P-1), etc., 
C       changing direction of wrap map every P wavenumbers. 
C       (Ordering courtesy of S. Barros and T. Kauranne.) 
C       The last wrap is modified to "left justify" the last row
C       of wavenumbers, assigning them to lower numbered processors.
C
C       This code is not as general as that in PSTSWM, in that it 
C       assumes that wavenumber nfc-1 is truncated.
C
C----------------------------------------------------------------------
      subroutine permute(wavdex, nlfcs, lfcdex, nprocs, nllon)
C
#ifdef PP_PCCM
      implicit none
      integer nprocs, nllon
      integer wavdex((nllon * nprocs +2)/2)
      integer nlfcs(0:nprocs-1), lfcdex(0:nprocs-1)
# if (defined(PP_FFTX) && (PP_NPROC_LN - 1))
      integer nglon, ngfc, ngmm
      integer p,i
      integer nxtdex, nfclim1, nfclim2, first, second

      nglon = nllon * nprocs
      ngfc = (nglon + 2)/2
#ifdef PP_TRIANG_TRUNC
      ngmm = PP_TRIANG_TRUNC + 1
#endif
#ifdef PP_RHOMB_TRUNC
      ngmm = PP_RHOMB_TRUNC + 1
#endif
C
      nfclim1 = ngmm-1 - mod(ngmm-1,nprocs)
      nfclim2 = ngmm-1

      nxtdex = 0
      do p=0,nprocs-1
C
        lfcdex(p) = nxtdex+1
        nlfcs(p) = 0
        first  = nprocs-1 - p
        second = nprocs   + p
        do while (first .lt. nfclim1)
C
          nxtdex = nxtdex + 1
          wavdex(nxtdex) = first
          nlfcs(p) = nlfcs(p) + 1
          first = first + 2*nprocs
C
          if (second .lt. nfclim1) then
            nxtdex = nxtdex + 1
            wavdex(nxtdex) = second
            nlfcs(p) = nlfcs(p) + 1
            second = second + 2*nprocs
          endif
C
        enddo
C
        if (nfclim2 .ge. nfclim1) then
          nxtdex = nxtdex + 1
          wavdex(nxtdex) = nfclim2
          nlfcs(p) = nlfcs(p) + 1
          nfclim2 = nfclim2 - 1
        endif
C
      enddo
C      
C     Finish definition of wavdex by assigning truncated wavenumbers
C     to the end.
      do i=ngmm,ngfc-1
        nxtdex = nxtdex + 1
        wavdex(nxtdex) = i
      enddo
# else
      integer i, nxtdex, p
      integer nllonh, ngfc
      integer mdex
      logical wave0
c
      nllonh = nllon/2
      ngfc = (nllon * nprocs + 2)/2
c
      nxtdex=0
      do p=0,nprocs-1
c
        wave0 = .false.
        lfcdex(p) = nxtdex+1
        nlfcs(p) = 0
        do i=0,nllonh-1
          nxtdex = nxtdex + 1
          wavdex(nxtdex) = mdex(i, p, nprocs, nllonh)
          if (wavdex(nxtdex) .eq. 0) wave0 = .true.
          nlfcs(p) = nlfcs(p) + 1
        enddo
        if (wave0) then
c         largest wavenumber unpacked into last of local wavenumbers
c         of processor containing wavenumber 0
          nxtdex = nxtdex + 1
          wavdex(nxtdex) = ngfc - 1
          nlfcs(p) = nlfcs(p) + 1
        endif
      enddo
# endif
#endif
      return
      end

