#include "params.h"

#ifdef COUPLED
      subroutine pccm3uw(rsf,his,rnl,fin,time_inv2,tilen,pccm_comm)
#else
      program pccm3uw
#endif

C------------------------------------------------------------------------
C
C Entry point for PCCM3UW  (PCCM2 with CCM3 physics)
C 
C Originally called ccm2.F in pccm2.1 distribution.
C
C-----------------------------NOTICE------------------------------------
C 
C                     see README.PCCM2
C
C---------------------------Code history--------------------------------
C
C Original version:  L. Bath
C Standardized:      L. Bath, June 1992
C MIMD Parallelization: J. Drake, P.Worley, J.Michalakes, I.Foster, Sept.95

C Changes for coupling to FOAM:  C. Schafer, R. Jacob, April-Sept 1995
C Additions of CCM3 physics: R. Jacob, Dec 1996
C
C------------------------------------------------------------------------
      implicit none
C--------------------------------commons---------------------------------
#include "comctl.com"
C------------------------------------------------------------------------
#include "comtim.com"

#ifdef PP_PCCM
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
#include "restart_fil.com"
#ifdef SIMPLE_OUTPUT
#include "comhst.com"
      character holdnhfil*11
#endif

      real tot_time
#else
      integer m_pin		! declaration for below
      data m_pin /0/
#endif
C-----------------------------local variables----------------------------
      character*8 cdate,
     $            ctime
      integer     lhbuf         ! Length of history tape buffer
      logical*4   exist
C-----------------------------externals----------------------------------
      character*8 date, clock, hchar
#ifdef COUPLED
      integer pccm_comm
#include "pcoupled.com"
      integer rsf,his,rnl,fin,tilen,ic
      character time_inv2(256)
      character*256 time_inv
#endif
#  ifdef RS6000
      integer iswall,iewall,totcpu
#  endif
      external blkdat,   ! Block data routine to initialize common vars
     $         lunits,   ! Initialize logical units
     $         preset,   ! Set namelist variables
     $         data,     ! Read namelist
     $         inital,   ! Initial run
     $         resume,   ! Restart/regeneration run
     $         intbnd,   ! Read boundary datasets
     $         inti,     ! Parameterization initializations
     $         stepon,   ! Time stepping control
     $         date,     ! Return current date
     $         time,     ! Return current time
     $         hchar     ! Convert hollerith string to character
cc

#ifdef COUPLED
      rsfreq=rsf
      hisfreq=his
      finished=fin
      rnlng=rnl
      do ic=1,tilen
        time_inv(ic:ic)=time_inv2(ic)
      enddo
#endif

#ifdef PP_PCCM
#ifdef COUPLED
      call parlyz(pccm_comm)
#else
      call parlyz
#endif
#endif

#ifdef PP_PCCM
C   Change standard output and standard error to the files
C   pccm.out and pccm.error
      call change_dir()
      call error_dup(m_pin)
#endif
C
C Print Model heading
C
      write(6,2000)
 2000 format('1'/1x,60('-')/25x,'NCAR CCM'/1x,60('-'))
C
C Fetch and print current date and time
C
      cdate = date()
      ctime = clock()
      write(6,3000) cdate ,ctime
      write(6,4000)
 3000 format(7x,'DATE  ',a8,8x,'TIME  ',a8)
 4000 format(1x,60('-'))
C
C Initialize logical unit numbers
C
      call lunits
      write(6,4000)
C
C Set defaults then override with user-specified input
C
#ifdef COUPLED
      call preset(rsfreq,hisfreq,rnlng)
#else
      call preset
#endif

#ifdef COUPLED
      call data(time_inv,tilen)
#else
      call data
#endif
C
C Set pointers in common /comgrd/. Must be initialized before restart
C data is read.
C
      call points
      if (.not.nlres) then
C
C Initial run: define physical initial conditions
C
         call inital
      else
C
C Restart run, position files
C
         call resume(lhbuf)
      endif
C
C Initialize boundary datasets
C
      call intbnd

#ifdef COUPLED
C
C Read coupler restart data
C
      call coup_restrt(time_inv,tilen)
#endif

C
C Set parameterization-specific constants
C
      call inti
C
C Initialize history tape handler
C
      call intht(nlres, lbrnch, exist, lhbuf, parhist)
C
C Invoke driving routine for time integration
C  The following call to stepdrv allocates an appropriately
C  sized history buffer and then calls stepon proper passing it the
C  address of said storage as an argument.  This approach gives run time
C  sizing and allocation of this buffer without the use of POINTER
C  declarations.  
C
#ifdef INCORE_HBUF
      call stepdrv_c(nlres,lbrnch,exist,lhbuf*plat)
#else
      call stepdrv_c(nlres,lbrnch,exist,lhbuf)
#endif
C
C Print final output
C
      write (6,900) nstep
      write (6,901)
  900 format(//'Number of completed timesteps:',i15)
  901 format(/1x,35('-')/' ******* END OF ATM MODEL RUN *******'/
     $     1x,35('-')/)
c
#ifdef SIMPLE_OUTPUT
      if (m_pin .eq. 0) then
	holdnhfil=nhfil(1)
	call move_atmos_his(holdnhfil,0,nstep)
      endif
#endif

      call iwrtend( desc )
      call restart_wclose( fid2 )
#ifdef PP_PCCM
      call commexit
      call close0f()
#endif

      end

