#include "params.h"
      subroutine aphys(nstep   ,doslt   ,lat     ,ztodt   ,geos    ,
     $                 pblh    ,tpert   ,pmid    ,pint    ,pdel    ,
     $                 pmln    ,piln    ,t       ,q       ,ts      ,
     $                 qpert   ,wet     ,prl     ,prc     ,srl     ,
     $                 src     ,rnf     ,dtcon   ,dqcon   ,cnt     ,
     $                 cnb     ,qc      ,cmfdt   ,cmfdq   ,cmfdqr  ,
     $                 cmfmc   ,cmfsl   ,cmflq   )
C-----------------------------------------------------------------------
C
C Adjustment physics
C Apply dry convection, moist convection, and large-scale condensation
C schemes.  Correct invalid (negative or too small) constituent mixing 
C ratios if necessary and report action in unit 6 output.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Hack, G. Taylor, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer nstep                 ! current time step for cmfmca diag.
      logical doslt                 ! SLT vs. spectral transport
      integer lat                   ! latitude index (S->N)
      real ztodt                    ! 2 delta t (model time increment)
      real geos(plond),             ! surface geopotential
     $     pblh(plond),             ! planetary boundary layer height
     $     tpert(plond),            ! thermal temperature excess
     $     pmid(plond,plev),        ! pressures at model levels
     $     pint(plond,plevp),       ! pressure at model interfaces
     $     pdel(plond,plev),        ! delta-p (pressure depth)
     $     pmln(plond,plev),        ! alog(pmid)
     $     piln(plond,plevp),       ! alog(pint)
     $     rpdel(plond,plevp)       ! 1/pdel
C
C Input/Output arguments
C
      real t(plond,plev),           ! temperature (K)
     $     q(plond,plev,pcnst),     ! constituent mixing ratio field
     $     ts(plond),               ! surface temperature
     $     qpert(plond,pcnst),      ! thermal humidity & constituent excess
     $     wet(plond)               ! soil moisture
C
C Output arguments
C
      real prl(plond),              ! large-scale precipitation rate
     $     prc(plond),              ! convective-scale preciptn rate
     $     srl(plond),              ! L.S. precipitation rate (snow)
     $     src(plond),              ! C.S. precipitation rate (snow)
     $     rnf(plond),              ! runoff
     $     dtcon(plond,plev),       ! dT/dt due to convective adjustments
     $     dqcon(plond,plev,pcnst), ! dq/dt due to convective adjustments
     $     cnt(plond),              ! top level of convective activity
     $     cnb(plond),              ! bottom level of convective activity
     $     qc(plond,plev),          ! dq/dt due to rainout terms
     $     cmfdt(plond,plev),       ! dT/dt due to moist convection
     $     cmfdq(plond,plev),       ! dq/dt due to moist convection
     $     cmfdqr(plond,plev),      ! dq/dt due to moist convective rainout 
     $     cmfmc(plond,plev),       ! moist convection cloud mass flux
     $     cmfsl(plond,plev),       ! moist convection lw stat energy flux
     $     cmflq(plond,plev)        ! moist convection total water flux

C  add for Zhang McFarlane Convection
      real zmdt(plond,plev)         ! Zhang convective temperature tendency
      real zmdq(plond,plev)         ! Zhang convective moisture tendency

C
C---------------------------Local workspace-----------------------------
C
      real gz(plond,plev),          ! geopotential
     $     tv(plond,plev),          ! virtual temperature array
     $     rztodt,                  ! 1./ztodt
C Add for Zhang McFarlane Convection
     $     zi(plond,plevp),         ! Height above sfc in interface
     $     zm(plond,plev)           ! Height above sfc in mid-layer
      real precc2(plond)            ! Convective-scale preciptn rate
      real cnt2(plond)              ! Top level of convective activity
      real cnb2(plond)              ! Bottom level of convective activity
      real qc2(plond,plev)          ! dq/dt due to rainout terms
      real cmfdqr2(plond,plev)      ! dq/dt due to moist convective rainout
      real cmfmc2(plond,plev)       ! Moist convection cloud mass flux
      real cmfsl2(plond,plev)       ! Moist convection lw stat energy flux
      real cmflq2(plond,plev)       ! Moist convection total water flux
      real tpert2(plond)            ! Perturbation T
      real qpert2(plond,pcnst)      ! Perturbation q
      real dlf(plond,plev)          ! Detraining cld H20 from convection
      real pflx(plond,plev)         ! Conv rain flux thru out btm of lev

      integer
     $     i,k,m                    ! longitude, level, constituent indices
      external resetr,
     $         qneg2,
     $         virtem,
     $         geomid,
     $         dadadj,
     $         cmfmca,
     $         cond,
     $         qneg3
C
C-----------------------------------------------------------------------
C
C Zero out precip and convective fields before accumulating terms
C
      call resetr(prl,plond,0.)
      call resetr(prc,plond,0.)
      call resetr(srl,plond,0.)
      call resetr(src,plond,0.)
      call resetr(qc    ,plond*plev,0.)
      call resetr(cmfdt ,plond*plev,0.)
      call resetr(cmfdq ,plond*plev,0.)
      call resetr(cmfdqr,plond*plev,0.)
      call resetr(cmfmc ,plond*plev,0.)
      call resetr(cmfsl ,plond*plev,0.)
      call resetr(cmflq ,plond*plev,0.)

      call resetr(zmdt  ,plond*plev,0.)
      call resetr(zmdq  ,plond*plev,0.)

C
C Store initial temperature, and constituent fields for later computation 
C of changes due to convective processes (dry, moist, and large-scale)
C
      do k=1,plev
         do i=1,plon
            dtcon(i,k) = t(i,k)
            dqcon(i,k,1) = q(i,k,1)
         end do
      end do
      do m=2,pcnst
         do k=1,plev
            do i=1,plon
               dqcon(i,k,m) = q(i,k,m)
            end do
         end do
      end do
C
C Adjust negative mixing ratios--only necessary with spectral transport
C
      if (.not.doslt) call qneg2(q)
C
C Calculate geopotential for cmfmca (requires virtual temperature)
C
      call virtem(t       ,q       ,zvir    ,tv      )
      call zmid(piln    ,pmln    ,rair    ,gravit   ,tv      ,
     $          zm      )
      call zint(piln    ,pmln    ,rair    ,gravit  ,tv      ,
     $          zm      ,zi      )

C
C Dry adjustment
C
      call dadadj(lat     ,pmid    ,pint    ,pdel    ,t       ,
     $            q       )
C
C Since the PBL doesn-t pass constituent perturbations, they 
C are zeroed here for input to the moist convection routine
C
      do m=2,pcnst
         do i=1,plon
            qpert(i,m) = 0.0
         end do
      end do

C
C              --------- Moist convection -------------
C
C Begin with Zhang-McFarlane (1996) convection parameterization
C Calls to qneg3 are to guard against problems, but probably unnecessary
C
       call qneg3('APHYS bz' ,lat       ,q(1,1,1) )
       call conv_ccm(t       ,q        ,prc       ,cnt     ,cnb     ,
     $               pblh    ,zm       ,geos      ,zi      ,zmdq   ,
     $               zmdt    ,pmid     ,pint      ,pdel    ,ts      ,
     $               .5*ztodt ,cmfmc     ,cmfdqr  ,nstep   ,lat     ,
     $               tpert    ,qpert     ,dlf     ,pflx    )
       call qneg3('APHYS az' ,lat,      q(1,1,1) )
C
C Convert mass flux from reported mb/s to kg/m^2/s
C
      do k=1,plev
        do i=1,plon
	  cmfmc(i,k) = cmfmc(i,k) * 100./gravit
        enddo
      enddo
C
c Add production of rain by conv_ccm to qc.  PJR says the vertical loop 
c indices are inconsistent.
c

      do k=2,plev
        do i=1,plon
	    qc(i,k) = qc(i,k) + (pflx(i,k)-pflx(i,k-1))*gravit/pdel(i,k)
        enddo
      enddo
C
      do i=1,plon
        tpert2(i) = 0.
      end do
      do m=1,pcnst
        do i=1,plon
           qpert2(i,m) = qpert(i,m)
	end do
      end do

C
C Call Hack (1994) convection scheme to deal with shallow/mid-level convection
C Begin by zeroing local copies of mass flux, energy fluxes, etc.
C
      call resetr(cmfmc2 ,plond*plev,0.)
      call resetr(cmfdqr2,plond*plev,0.)
      call resetr(cmfsl2 ,plond*plev,0.)
      call resetr(cmflq2 ,plond*plev,0.)
      call resetr(qc2    ,plond*plev,0.)
      do k=1,plev
         do i=1,plon
            rpdel(i,k) = 1./pdel(i,k)
         end do
      end do  


      call cmfmca(lat     ,nstep   ,ztodt   ,pmid    ,pdel    ,
     $            rpdel   ,zm      ,tpert2  ,qpert2  ,geos    ,
     $            pblh    ,t       ,q       ,cmfdt   ,cmfdq   ,
     $            cmfmc2  ,cmfdqr2 ,cmfsl2  ,cmflq2  ,precc2  ,
     $            qc2     ,cnt2    ,cnb2    )

c
c Merge shallow/mid-level output with prior results from Zhang-McFarlane
c
      do i=1,plon
        prc(i) = prc(i) + precc2(i)
        if (cnt2(i) .lt. cnt(i)) cnt(i) = cnt2(i)
        if (cnb2(i) .gt. cnb(i)) cnb(i) = cnb2(i)
      end do
c
      do k=1,plev
        do i=1,plon
	  cmfmc(i,k)  = cmfmc(i,k)  + cmfmc2(i,k)
	  cmfdqr(i,k) = cmfdqr(i,k) + cmfdqr2(i,k)
	  cmfsl(i,k)  = cmfsl(i,k)  + cmfsl2(i,k)
	  cmflq(i,k)  = cmflq(i,k)  + cmflq2(i,k)
	  qc(i,k)     = qc(i,k)     + qc2(i,k)
	end do
      end do


C
C Large-scale (stable) condensation
C Evaporation of stratiform precipitation is included in this call
C

      call cond(ztodt   ,pmid    ,pdel    ,t       ,q       ,
     $          qc      ,prl     ) 
C
C Compute rate of temperature change due to convective processes
C
      rztodt = 1./ztodt
      do k=1,plev
         do i=1,plon
            dtcon(i,k) = (t(i,k) - dtcon(i,k))*rztodt
         end do
      end do
C
C Check for constituents < qmin. Set to minimum value and print message
C
      call qneg3('APHYS   ',lat,q(1,1,1))
C
C Compute rate of constituent change due to convective processes
C
      do m=1,pcnst
         do k=1,plev
            do i=1,plon
               dqcon(i,k,m) = (q(i,k,m) - dqcon(i,k,m))*rztodt
            end do
         end do
      end do
C
      return
      end
