#        include "params.h"
C     ******************************************************************
C     Program:    radxfr.F
C
C     Desription: subroutine to rearrange variables assocated with
C                 radcsw.  This routine swaps every other data column
C                 with the processor p_nproc_ln / 2 processors away.
C                 The purpose of this is to reduce the imbalance
C                 associated with the shortwave radiation calculations.
C                 The algorithm scatters the columns exposed to sunlight
C                 and thus balances the computational load.
C
C     CVS: $Source: /homes/jacob/foam-cvs/pccm3uw/pccm3uwsrc/physics/ccm3_rad/radxfr.F,v $
C          $Date: 2002/07/16 16:27:15 $
C          $Revision: 1.2 $
C          $State: Exp $
C     ******************************************************************

      subroutine radxfr(lat,     oro,     sndpth,  pint,    h2ommr,
     $                  cld,     clwp,    o3mmr,   coszrs,  solin,
     $                  qrs,     fsns,    fsnt,    fsnsc,   fsntc,
     $		        t,   aermmr, rh,   rel,  rei,  fice,
     $			albs,  albl,  albsd, albld,
     $                  dir)

         implicit none

C        *** Include Common Files ***
#        include "prgrid.com"
#        include "crdalb.com"
#        include "pspect.com"
#        include "comparll.com"
#        include "comtim.com"
#        include "comctl.com"
#        include "palgor.com"


C        *** Declare Arguments ***
         integer lat               !latitude index
         integer oro(plond)        !land/ocean/seaice flag used by 'radalb'
         real sndpth(plond)        !snow depth used by 'radalb'
         real pint(plond,plevp)    !interface pressure 
         real h2ommr(plond,plevp)  !specific humidity (h2o mass mixing ratio)
         real cld(plond,plevp)     !fractional cloud cover
         real clwp(plond,plev)     !layer liquid water path
         real o3mmr(plond,plev)    !ozone mass mixing ratio
         real coszrs(plond)        !cosine solar zenith angle
         real solin(plond)         !incident solar flux
         real qrs(plond,plev)      !solar heating rate
         real fsns(plond)          !surface absorbed solar flux
         real fsnt(plond)          !total column absorbed solar flux
         real fsnsc(plond)         !clear sky surface absorbed solar flux
         real fsntc(plond)         !clear sky total column absorbed solar flux
	 real t(plond,plev)	   ! model level temperatures
         logical dir               !direction (true=forward, false=backward)
c CCM3 addition
	 real aermmr(plond,plev)   ! level aerosol mass mixing ratio
         real rel(plond,plev)      ! liquid effective drop size (microns)
	 real rei(plond,plev)      ! ice effective drop size (microns)
	 real fice(plond,plev)     ! fractional ice content within cloud
         real rh(plond,plev)      ! level relative humidity (fraction)


      real albs(plond),    ! Srf alb for direct rad   0.2-0.7 micro-ms
     $     albl(plond),    ! Srf alb for direct rad   0.7-5.0 micro-ms
     $     albsd(plond),   ! Srf alb for diffuse rad  0.2-0.7 micro-ms
     $     albld(plond)    ! Srf alb for diffuse rad  0.7-5.0 micro-ms


C        *** Declare Local Constants ***
         integer plon2             !number of columns to be moved (plon / 2)
         integer buf_size          !send/receive buffer size in words

         parameter (plon2 = plon / 2)
         parameter (buf_size = (23 + 3 * plevp + 9 * plev) * plon2)

C        *** Declare Local Variables ***
         integer lon
         integer lev
         real rbuf(buf_size)       !receive communication buffer 
         real sbuf(buf_size)       !send communication buffer
         integer off               !buffer offset
         integer twin              !processor swapping data with
         integer rlth              !maximum message length
         integer slth              !maximum message length
         integer order             !order of swap
         integer commopt           !communication option to use in swap

C        *** Calculate twin-s ID number and message size ***
         twin = mapmesh(mod(m_pin_ln + p_nproc_ln / 2,p_nproc_ln),
     &                  m_pin_lt)
         rlth = buf_size * R_SIZE
         if (m_pin .ge. twin) then
           order = 1
         else
           order = 2
         endif

C        *** Register receive buffer ***
         if (p_lbalopt .eq. 2) then
C          error
           commopt = 0
         else
           commopt = p_lbalopt
         endif
         call swap1(commopt, p_lbalprot, .true., order, m_pin, 
     &              m_type, twin, rlth, rbuf)

C        *** Pack Data ***
         off = 0
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = vegtyp(2 * lon, lat)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = rghnss(2 * lon, lat)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = evapf(2 * lon, lat)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = vevapf(2 * lon, lat)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = snwjan(2 * lon, lat)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = snwjly(2 * lon, lat)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = float(oro(2 * lon))	! jm fix dbl prec bug 950918
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = sndpth(2 * lon)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = albs(2 * lon)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = albl(2 * lon)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = albsd(2 * lon)
         enddo
         do lon = 1, plon2
            off = off + 1
            sbuf(off) = albld(2 * lon)
         enddo

         do lon = 1, plon2
            off = off + 1
            sbuf(off) = coszrs(2 * lon)
         enddo

         if (.not. dir) then
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = solin(2 * lon)
            enddo
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = fsns(2 * lon)
            enddo
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = fsnt(2 * lon)
            enddo
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = fsnsc(2 * lon)
            enddo
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = fsntc(2 * lon)
            enddo
         endif
         do lev = 1, plevp
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = pint(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plevp
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = h2ommr(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plevp
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = cld(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = clwp(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = o3mmr(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = t(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = aermmr(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = rh(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = rel(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = rei(2 * lon, lev)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               sbuf(off) = fice(2 * lon, lev)
            enddo
         enddo
         if (.not. dir) then
            do lev = 1, plev
               do lon = 1, plon2
                  off = off + 1
                  sbuf(off) = qrs(2 * lon, lev)
               enddo
            enddo
         endif
	 slth = off * R_SIZE

C        *** Send to and receive from Processor 180 Degrees Away ***
         call swap2(commopt, p_lbalprot, .true., order, m_pin, 
     &              m_type, twin, slth, sbuf, rlth, rbuf)

C        *** Unpack Data ***
         off = 0
         do lon = 1, plon2
            off = off + 1
            vegtyp(2 * lon, lat) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            rghnss(2 * lon, lat) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            evapf(2 * lon, lat) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            vevapf(2 * lon, lat) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            snwjan(2 * lon, lat) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            snwjly(2 * lon, lat) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            oro(2 * lon) = int(rbuf(off)+.1)	! jm fix dbl prec. bug 950918
         enddo
         do lon = 1, plon2
            off = off + 1
            sndpth(2 * lon) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            albs(2 * lon) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            albl(2 * lon) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            albsd(2 * lon) = rbuf(off)
         enddo
         do lon = 1, plon2
            off = off + 1
            albld(2 * lon) = rbuf(off)
         enddo

         do lon = 1, plon2
            off = off + 1
            coszrs(2 * lon) = rbuf(off)
         enddo

         if (.not. dir) then
            do lon = 1, plon2
               off = off + 1
               solin(2 * lon) = rbuf(off)
            enddo
            do lon = 1, plon2
               off = off + 1
               fsns(2 * lon) = rbuf(off)
            enddo
            do lon = 1, plon2
               off = off + 1
               fsnt(2 * lon) = rbuf(off)
            enddo
            do lon = 1, plon2
               off = off + 1
               fsnsc(2 * lon) = rbuf(off)
            enddo
            do lon = 1, plon2
               off = off + 1
               fsntc(2 * lon) = rbuf(off)
            enddo
         endif
         do lev = 1, plevp
            do lon = 1, plon2
               off = off + 1
               pint(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plevp
            do lon = 1, plon2
               off = off + 1
               h2ommr(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plevp
            do lon = 1, plon2
               off = off + 1
               cld(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               clwp(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               o3mmr(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               t(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               aermmr(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               rh(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               rel(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               rei(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         do lev = 1, plev
            do lon = 1, plon2
               off = off + 1
               fice(2 * lon, lev) = rbuf(off)
            enddo
         enddo
         if (.not. dir) then
            do lev = 1, plev
               do lon = 1, plon2
                  off = off + 1
                  qrs(2 * lon, lev) = rbuf(off)
               enddo
            enddo
         endif

C        *** Complete send operation ***
         call swap3(commopt, p_lbalprot, m_pin, m_type, twin, rlth,
     &              rbuf)
         m_type = m_type + p_nproc

         return
      end

