#include "params.h"
      subroutine rfinti(cpwvx,cpairx)
C-----------------------------------------------------------------------
C
C Initialization of time independent fields for Rayleigh friction
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville
C Standardized:      J. Rosinski, June 1992
C Reviewed:          P. Rasch, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Commons----------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comrf.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real cpwvx,      ! specific heat of water vapor at const. pressure
     $     cpairx      ! specific heat of dry air
C
C---------------------------Local workspace-----------------------------
C
      real hb,         ! base height for tanh (lnp m)
     $     hs,         ! e-folding depth for the tanh profile (lnp m)
     $     hscale,     ! approximate atmospheric scale height (lnp m)
     $     rayla       ! time scale in 1/seconds at the zero point of 
C                      ! the tanh profile height "hb"
      real secpdy,     ! seconds per day
     $     tscmax      ! inverse max time scale (1./days) for which R. 
C                      ! fric. is applied
      integer k        ! level index
C
C-----------------------------------------------------------------------
C
C Memory transfers
C
      cpair = cpairx
C
C Derived constants
C
      cpvir = cpwvx/cpairx - 1.
      secpdy = 86400.
C
C Characteristics of the rayliegh friction
C
      rayla = 1.0/1.5

C Add more rayleigh drag if running at low resolution
C to reduce large winds which are causing bandij errors
C RLJ

#if (PP_TRIANG_TRUNC==21) || (PP_RHOMB_TRUNC==15)
      rayla = 100.0*1.0/1.5
#endif

      rayla = rayla/secpdy      ! convert hours to seconds
      hscale = 7.0e3
      hb = 75.0e3
      hs = 09.0e3
      tscmax = 0.001
C
C compute raleigh friction coefficient at all levels
C
      do k=1,plev
         rlghco(k) = rayla*
     $        (1.0 + tanh((-hscale*alog(hyam(k) + hybm(k)) - hb)/hs))
      end do
C
C Set highest level for rayleigh friction
C Find lowest level for rayleigh friction based on tscmax
C
      ntoprl = 1
      if (plev.eq.1) ntoprl = 0
      do nbotrl=0,plev-1
         if(rlghco(nbotrl+1) .le. tscmax/secpdy ) go to 30
      end do
      nbotrl = plev
   30 continue
C
C Zero out all coeffs with longer timescale than tscmax.
C
      if (nbotrl.lt.plev) then
         do k=nbotrl+1,plev
            rlghco(k) = 0.0
         end do
      end if
      write(6,800) nbotrl,ntoprl,rlghco
      return
C
C Formats
C
  800 format(//,1x,26('-----') /,
     $       30X, 'RAYLIEGH FRICTION',/,
     $       ' BOTTOM LEVEL: NBOTRL=',I3,' NTOPRL=',I3,/,
     $       ' COEFFICIENTS ',/,1P,(1X,10E12.4))
      end
