#include "params.h"
      subroutine cpslhy (phis    ,ps      ,t       ,psl     )
C-----------------------------------------------------------------------
C
C CCM1 hybrid coord version
C Compute sea level pressure for a latitude line
C Algorithm: using the two lowest levels of free atmosphere
C temperature, define a temperature profile that is linear
C in the log of hyam + hybm.
C Warning: this profile definition may not be reasonable
C if the hybrid coordinate base pressure is very different
C than the surface pressure.
C Using this profile, extrapolate linearly
C to the surface and then assume a moist adiabatic lapse rate
C between surface and sea level to determine a  sea-level
C temperature  for evaluating the integral of the hydrostatic
C equation between surface and sea level.
C
C---------------------------Code history--------------------------------
C
C Original version:  Stolen from the processor
C Standardized: Stolen and standardized by John Truesdale      
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Commons----------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "comcon.com"
#include "comhyb.com"
C---------------------------Input arguments-----------------------------
      real phis(plond),         ! surface geopotential height
     $    ps(plond),            ! surface pressure
     $    t(plond,plev)         ! temperature
C---------------------------Output arguments----------------------------
      real psl(plond)           ! sea level pressure
C---------------------------Local variables-----------------------------
C
      real econst,              ! constant of integration for hydostatic eq.
     $     gmoist,              ! moist lapse rate
     $     tsurf ,              ! statement function var for surf temp
     $     pconst               ! lapse rate/grav
C
      integer i                 ! longitude index
C
C---------------------------statement functions-------------------------
C
C statement function to get temperature at surface (extrapolate)
C
      tsurf(i) = t(i,plev) + prsfac * (t(i,plev-1) - t(i,plev))
C-----------------------------------------------------------------------
C
C
C     rair = 287.04, grav = 9.80616
C
      gmoist = 6.5e-3
      econst = gravit / (rair * gmoist)
      pconst = gmoist * rga
C
C Calculate sea level pressure for a latitude
C
      do 20 i=1,plon
         psl(i) = ps(i) * (1. + pconst * phis(i) / tsurf(i)) ** econst
   20 continue
C
      return
      end





