#include "params.h"
      subroutine geomid(pmln    ,piln    ,geos    ,rair    ,tv      ,
     $                  geo     )
C-----------------------------------------------------------------------
C
C Compute the geopotential at layer midpoints using the surface
C geopotential, virtual temperature, and pressure.
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Hack
C Standardized:      L. Buja, June 1992
C Reviewed:          J. Hack, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Commons----------------------------------
#include "pmgrid.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real pmln(plond,plev),    ! Ln(pmid)
     $     piln(plond,plevp),   ! Ln(pint)
     $     geos(plond),         ! Surface geopotential
     $     rair,                ! Gas constant for dry air
     $     tv(plond,plev)       ! Virtual temperature
C
C Output arguments
C
      real geo(plond,plev)      ! Geopotential at midpoints
C
C---------------------------Local variables-----------------------------
C
      integer i,                ! Longitude index
     $        k,                ! Level index
     $        l                 ! Level index
C
C-----------------------------------------------------------------------
C
C Diagonal term of hydrostatic equation
C
      do k=1,plev-1
         do i=1,plon
            geo(i,k) = geos(i) + rair*tv(i,k)*0.5*
     $                (pmln(i,k+1) - pmln(i,k))
         end do
      end do
      k=plev
      do i=1,plon
         geo(i,k) = geos(i) + rair*tv(i,k)*(piln(i,k+1)-pmln(i,k))
      end do
C
C Bottom level term of hydrostatic equation
C
      do k=1,plev-1
         l=plev
         do i=1,plon
            geo(i,k) = geo(i,k) + rair*tv(i,l)*
     $                (piln(i,l+1) - 0.5*(pmln(i,l-1) + pmln(i,l)))
         end do
      end do
C
C Interior terms of hydrostatic equation
C
      do k=1,plev-2
         do l=k+1,plev-1
            do i=1,plon
               geo(i,k) = geo(i,k) + rair*tv(i,l)*0.5*
     $                    (pmln(i,l+1) - pmln(i,l-1))
            end do
         end do
      end do
C
      return
      end
