#include <params.h>
      subroutine omcalc(rcoslat ,d       ,u       ,v       ,dpsl    ,
     $                  dpsm    ,pmid    ,pdel    ,pmln    ,piln    ,
     $                  rpmid   ,rpdel   ,pbot    ,omga    )
C-----------------------------------------------------------------------
C
C Calculate vertical pressure velocity (omga = dp/dt)
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          B. Boville, D. Williamson, August 1992
C
C-----------------------------------------------------------------------
c
c $Id: omcalc.F,v 1.1.1.1 2002/03/20 22:16:33 jacob Exp $
c $Author: jacob $
c
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include <pmgrid.com>
#include <pspect.com>
C------------------------------Commons----------------------------------
#include <comhyb.com>
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real rcoslat,              ! 1 / cos(lat)
     $     d(plond,plev),        ! divergence
     $     u(plond,plev),        ! zonal wind * cos(lat)
     $     v(plond,plev),        ! meridional wind * cos(lat)
     $     dpsl(plond),          ! longitudinal component of grad ln(ps)
     $     dpsm(plond),          ! latitudinal component of grad ln(ps)
     $     pmid(plond,plev),     ! mid-level pressures
     $     pdel(plond,plev),     ! layer thicknesses (pressure)
     $     pmln(plond,plev),     ! alog (mid-level pressure)
     $     piln(plond,plevp),    ! alog (interface pressure)
     $     rpmid(plond,plev),    ! 1./pmid
     $     rpdel(plond,plev),    ! 1./pdel
     $     pbot(plond)           ! bottom interface pressure
C
C Output arguments
C
      real omga(plond,plev)      ! vertical pressure velocity
C
C---------------------------Local workspace-----------------------------
C
      real hkk(plond),           ! diagonal element of hydrostatic matrix
     $     hlk(plond),           ! super diagonal element
     $     suml(plond),          ! partial sum over l = (1, k-1)
     $     vgpk,                 ! v dot grad ps
     $     tmp                   ! vector temporary
      integer
     $     i,k                   ! longitude, level indices
C
C-----------------------------------------------------------------------
C
C First evaluate the expressions for omega/p, then rescale to omega at
C the end.
C
C-----------------------------------------------------------------------
C
C Zero partial sum
C
      do i=1,plon
         suml(i) = 0.
      end do
C
C Pure pressure part: top level
C
      do i=1,plon
         hkk(i) = 0.5*rpmid(i,1)
         omga(i,1) = -hkk(i)*d(i,1)*pdel(i,1)
         suml(i) = suml(i) + d(i,1)*pdel(i,1)
      end do
C
C sum(k)(v(j)*ps*grad(lnps)*db(j)) part. Not normally invoked since 
C the top layer is normally a pure pressure layer.
C
      if (1.ge.nprlev) then
         do i=1,plon
            vgpk = rcoslat*(u(i,1)*dpsl(i) + v(i,1)*dpsm(i))*pbot(i)
            tmp = vgpk*hybd(1)
            omga(i,1) = omga(i,1) + hybm(1)*rpmid(i,1)*vgpk -
     $                              hkk(i)*tmp
            suml(i) = suml(i) + tmp
         end do
      end if
C
C Integrals to level above bottom
C
      do k=2,plev-1
C
C Pure pressure part
C
         do i=1,plon
            hkk(i) = 0.5*rpmid(i,k)
            hlk(i) =     rpmid(i,k)
            omga(i,k) = -hkk(i)*d(i,k)*pdel(i,k) - hlk(i)*suml(i)
            suml(i) = suml(i) + d(i,k)*pdel(i,k)
         end do
C
C v(j)*grad(lnps) part
C
         if (k.ge.nprlev) then
            do i=1,plon
               vgpk = rcoslat*(u(i,k)*dpsl(i) + v(i,k)*dpsm(i))*pbot(i)
               tmp = vgpk*hybd(k)
               omga(i,k) = omga(i,k) + hybm(k)*rpmid(i,k)*vgpk -
     $                                 hkk(i)*tmp
               suml(i) = suml(i) + tmp
            end do
         end if
      end do
C
C Pure pressure part: bottom level
C
      do i=1,plon
         hkk(i) = 0.5*rpmid(i,plev)
         hlk(i) =     rpmid(i,plev)
         omga(i,plev) = -hkk(i)*d(i,plev)*pdel(i,plev) - hlk(i)*suml(i)
      end do
C
C v(j)*grad(lnps) part
C
      if (plev.ge.nprlev) then
         do i=1,plon
            vgpk = rcoslat*(u(i,plev)*dpsl(i) + v(i,plev)*dpsm(i))*
     $             pbot(i)
            omga(i,plev) = omga(i,plev) +
     $                     hybm(plev)*rpmid(i,plev)*vgpk -
     $                     hkk(i)*vgpk*hybd(plev)
         end do
      end if
C
C The above expressions give omega/p. Rescale to omega.
C
      do k=1,plev
         do i=1,plon
            omga(i,k) = omga(i,k)*pmid(i,k)
         end do
      end do
C
      return
      end
