#include "params.h"
      subroutine sstan(lat     ,oro     ,ioro    ,ts      ,tssub   ,
     $                 sndpth  )
C-----------------------------------------------------------------------
C
C Update sea surface temperatures (sst-s) and sea ice distribution
C
C Assume that the sst data exists in a two dimensional field
C encoded as follows:
C
C     Land               values greater than 140
C
C     Ocean without      values degrees celcius (greater than -1.90C)
C      sea ice
C
C     Ocean with         values less than -1.90C
C      sea ice
C
C New sea ice has a constant 0.5 cm value for snow cover prescribed
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      L. Buja, June 1992
C Reviewed:          J. Hack, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C-----------------------------------------------------------------------
      real tsice                ! Freezing point of sea ice degrees C
      parameter (tsice = -1.90)
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comsst.com"
C-----------------------------------------------------------------------
#include "comtsc.com"
C-----------------------------------------------------------------------
#include "crdalb.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer lat               ! Latitude index
C
C input/output arguments
C
      real oro(plond)           ! Surface type flag array
      integer
     $     ioro(plond)          ! nint(oro(i))
      real ts(plond),           ! Surface temperature
     $     tssub(plond,plevmx), ! Sub/srf tmps over ocean set to sst:
     $     sndpth(plond)        ! Snow cover (liquid water equivalent)
C
C---------------------------Local variables-----------------------------
C
      integer i,                ! Longitude index
     $        n                 ! Sub/srf level index
C-----------------------------------------------------------------------
C
C Update the model arrays
C
      do 100 i=1,plon
C
C Find ocean (and lake) points, whether ice covered or not.
C
         if (sst(i,lat).le.140.) then ! SST says this is ocean/sea ice.
            if (ioro(i).eq.1) then    ! ORO says this is a land point.
               print *,' '
               print *,'SSTAN: Disagreement between the SST and ORO '
     $                ,'fields whether this point is land or not.'
               print *,'SSTAN: Data for lon,lat,ioro,sst= '
     $                ,i,lat,ioro(i),sst(i,lat)
               call endrun
            end if
C Set sub/surface temperatures over ice-free ocean to be ocean sst:
C Set snow depth and surface roughness length also.
C
            if (ioro(i).ne.2) then
               do n=1,plevmx
                  tssub(i,n) = sst(i,lat) + 273.16
               end do
               rghnss(i,lat) = zzocen
               sndpth(i) = 0.000
            else
               sndpth(i) = 0.005
               rghnss(i,lat) = zzsice
            end if
C
C Sea ice point; check to see if this pnt was previusly sea ice (oro=2)
C If this is a new ice point, set the temperatures, oro flag, roughness
C length, and snow depth. Otherwise (already was an ice point), leave 
C the current values alone.
C
            if (sst(i,lat).le.tsice) then
               if (ioro(i).ne.2) then
                  ts(i) = sst(i,lat) + 273.16
                  oro(i) = 2.0
                  ioro(i) = 2
                  do n=1,plevmx
                     tssub(i,n) = sst(i,lat) + 273.16
                  end do
                  rghnss(i,lat) = zzsice
                  sndpth(i) = 0.005
                end if
C
C Ice free ocean
C
            else
               ts(i) = sst(i,lat) + 273.16
               oro(i) = 0.0
               ioro(i) = 0
               sndpth(i) = 0.0
            end if
         end if
  100 continue
C
      return
      end
