#include "params.h"
      subroutine tphys1(lat     ,clat    ,coslat  ,hbuf    ,lhbuf   ,
     $                                                     ztodt   ,
     $                 oro     ,ioro    ,phis    ,pmidm1  ,pintm1  ,
     $                 pdelm1  ,u       ,v       ,q       ,omga    ,
     $                 t2      ,fu      ,fv      ,um1     ,vm1     ,
     $                 tm1     ,qm1     ,psm1    ,pblh    ,qpert   ,
     $                 tpert   ,ts      ,sndpth  ,cnt     ,cnb     ,
     $                 qrs     ,qrl     ,flwds   ,fsns    ,plol    ,
     $                 plos    ,tssub   ,ws      ,cnvprc  ,cmfmc)
C-----------------------------------------------------------------------
C
C Tendency physics
C Call physics subroutines.  Dump appropriate fields to history tape
C
C----------------------------Code History-------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, J. Hack, August 1992
C
C physics split for coupling: C. Schafer, R. Jacob, April 1995
C additions to work with CCM3: R. Jacob, December 1996
C
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comsgh.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
C-----------------------------------------------------------------------

#ifdef COUPLED
#include "pccmcoup.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     lat,                   ! model latitude index
     $     lhbuf                  ! history buffer length
      real clat,                  ! current latitude(radians)
     $     coslat                 ! cosine latitude
      real hbuf(*),               ! history buffer
     $     ztodt,                 ! two times model timestep (2 delta-t)
     $     oro(plond)             ! sfc type flag (ocean,land,sea ice)
      integer
     $     ioro(plond)            ! nint(oro(i))
      real phis(plond),           ! surface geopotential height
     $     pmidm1(plond,plev),    ! midpoint pressures
     $     pintm1(plond,plevp),   ! interface pressures
     $     pdelm1(plond,plev),    ! layer pressure depth
     $     u(plond,plev),         ! u wind
     $     v(plond,plev),         ! v wind
     $     q(plond,plev),         ! specific humidity
     $     omga(plond,plev),      ! vertical pressure velocity
     $     t2(plond,plev),        ! temperature tendency
     $     fu(plond,plev),        ! accumulated u tendency (from tphys)
     $     fv(plond,plev),        ! accumulated v tendency (from tphys)
     $     um1(plond,plev),       ! u horizontal wind component
     $     vm1(plond,plev),       ! v horizontal wind component
     $     tm1(plond,plev),       ! temperature
     $     qm1(plond,plev,pcnst), ! specific humidity and constituents
     $     psm1(plond)            ! surface pressure

c  new input field for CCM3 cldint  comes from aphys by way of
c  physics.F
      real cmfmc(plond,plev)    ! Convective mass flux--m sub c

C
C Output arguments
C
      real pblh(plond),         ! planetary boundary layer height
     $     qpert(plond,pcnst),  ! moisture/constituent perturbation (PBL)
     $     tpert(plond),        ! temperature perturbation (PBL)
     $     ts(plond),           ! surface (skin) temperature
     $     sndpth(plond),       ! snow depth (liquid water equivalent)
     $     cnt(plond),          ! top level of convective activity
     $     cnb(plond),          ! lowest level of convective activity
     $     qrs(plond,plev),     ! shortwave heating rate
     $     qrl(plond,plev),     ! longwave  heating rate
     $     flwds(plond),        ! surface longwave down flux
     $     fsns(plond),         ! surface solar absorbed flux
     $     plol(plond,plevp),   ! o3 pressure weighted absorber path
     $     plos(plond,plevp),   ! o3 absorber path
     $     tssub(plond,plevmx), ! surface/sub surface temperatures
     $     ws(plond),           ! soil moisture(water equivalent < wssat)
     $     cnvprc(plond)        ! convective precipitation rate
C
C---------------------------Local workspace-----------------------------
C
      real up1(plond,plev),    ! u-wind after vdiff & rfric
     $     vp1(plond,plev),    ! v-wind after vdiff & rfric
     $     clwp(plond,plev),   ! presribed cloud liq. h2o path 
     $     cld(plond,plevp),   ! fraction cloud cover
     $     effcld(plond,plevp),! effective cloud=cld*emis
     $     emis(plond,plev),   ! cloud longwave emissivity
     $     clc(plond),         ! total convective cloud (cloud scheme)
     $     cflx(plond,pcnst,plat),  ! surface constituent flux (kg/m^2/s)
     $     dtv(plond,plev),    ! t tendency from vertical diffusion
     $     duv(plond,plev),    ! u tendency from vertical diffusion
     $     dvv(plond,plev),    ! v tendency from vertical diffusion
     $     shflx(plond),       ! sensible heat flux (w/m^2)
     $     lhflx(plond),       ! latent  heat flux (w/m^2)
     $     psomc(plond,plev,plat),  ! (psm1/pmidm1)**cappa
     $     taux(plond),        ! x surface stress (zonal)
     $     tauy(plond),        ! y surface stress (meridional)
     $     thm(plond,plev,plat),    ! potential temperature 
     $     tvm(plond,plev),    ! virtual temperature 
     $     zm(plond,plev,plat),     ! height of midpoints (above surface) 
     $     rpdeli(plond,plev,plat), ! 1./(pmid(k+1)-pmid(k))
     $     rpdel(plond,plev,plat),  ! 1./(pint(k+1)-pint(k))
     $     cltot(plond),       ! diagnostic total cloud cover
     $     cllow(plond),       !       -     low  cloud cover
     $     clmed(plond),       !       -     mid  cloud cover
     $     clhgh(plond),       !       -     hgh  cloud cover
     $     pmln(plond,plev,plat),   ! ln(pmidm1)
     $     piln(plond,plevp,plat),  ! ln(pintm1)
     $     srcsnk(plond,plev,pcnst,plat), ! src/sink for spec. hum. + constituents
     $     srfrad(plond,plat)       ! net surface radiative flux (watts/m**2)
      integer
     $     i,k,                ! longitude, level indices
     $     m                   ! constituent index
C
C     real actlwp(plond,plev), ! actual cloud liquid h2o path 
C    $     totlwp(plond)       ! actual column cloud liquid h2o path 
C     real sum
      real cloudup(plond,plev) ! cloud fraction during omega < 0.

c new fields for CCM3.  Derived in new cldint and passed to new radctl
      real rel(plond,plev)      ! Liquid cloud particle effective radius
      real rei(plond,plev)      ! Ice effective drop size (microns)
      real fice(plond,plev)     ! Fractional ice content within cloud

C
C------------------------------Externals--------------------------------
C
      external
     $     resetr,             ! memory clear
     $     cmpsnw,             ! compute snow
     $     zmid,               ! compute level heights (above surface)
     $     sstan,              ! sst update
     $     virtem,             ! virtual temperature calculation
     $     cldint,             ! cloud parameterization interface
     $     radctl,             ! radiation interface
     $     pottem,             ! potential temperature
     $     srfint,             ! surface fluxes/temperature interface
     $     copyac,             ! copy data between vectors
     $     vdintr,             ! vertical diffusion interface
     $     rfintr,             ! rayleigh friction interface
     $     mgwintr,             ! gravity wave drag interface
     $     outfld              ! history file handler
C
C-----------------------------------------------------------------------
C
C Common block to hold info from tphys1 to tphys2-CMS
      common/tphys_hold/ cflx,psomc,thm,zm,rpdeli,rpdel,pmln,
     $                   piln,srcsnk,srfrad
C-----------------------------------------------------------------------

C Initialize source/sink array to zero for all constituents.
C
      call resetr(srcsnk(1,1,1,lat),plond*plev*pcnst,0.)
C
C Precompute frequently used variables
C
      do k=1,plev
         do i=1,plon
            pmln(i,k,lat)  = alog(pmidm1(i,k))
            piln(i,k,lat)  = alog(pintm1(i,k))
            rpdel(i,k,lat) = 1./pdelm1(i,k)
         end do
      end do
      do k=1,plev-1
         do i=1,plon
            rpdeli(i,k,lat) = 1./(pmidm1(i,k+1) - pmidm1(i,k))
         end do
      end do
C
C Add reference pressure to perturbation surface pressure (to obtain 
C total surface pressure).  Complete computation of piln
C
      do i=1,plon
         psm1(i)       = psm1(i) + psr
         piln(i,plevp,lat) = alog(pintm1(i,plevp))
      end do
C
C Update prescribed sea surface temperatures and sea ice distribution
C
#ifndef COUPLED
      if (anncyc .and. mod(nstep,itsst).eq.0) then
         call sstan(lat     ,oro     ,ioro    ,ts      ,tssub   ,
     $              sndpth  )
      end if
#endif
C
C Update prescribed snow cover every time step
C
#ifndef COUPLED
      call cmpsnw(lat     ,clat    ,ioro    ,sndpth)
#endif
C
C Useful diagnostics--
C Virtual temperatures, potential temperatures, and model level heights
C for use in radiation, surface, and diffusion calculations
C
      call virtem(tm1     ,qm1     ,zvir    ,tvm     )
      call pottem(pintm1  ,pmidm1  ,tm1     ,cappa   ,psomc(1,1,lat),
     $            thm(1,1,lat) )
      call zmid(piln(1,1,lat) ,pmln(1,1,lat) ,rair  ,gravit  ,tvm,
     $          zm(1,1,lat)   )
      if (dosw .or. dolw) then
C
C Compute cloud properties for input to radiation
C
C Call new CCM3 version
         call cldint(coslat ,pmidm1 ,rpdeli(1,1,lat),tm1  ,qm1   ,
     $               omga    ,cnt     ,cnb     ,cnvprc  ,
     $               piln(1,1,lat)  ,
     $               pmln(1,1,lat) ,phis ,tvm  ,zm(1,1,lat), cld ,
     $               clc     ,clwp    ,emis    ,effcld  ,cltot   ,
     $               cllow   ,clmed   ,clhgh   ,pdelm1  ,cmfmc   ,
     $		     oro     ,sndpth  ,rel     ,rei     ,fice    ,
     $		     psm1)
c        call cldint(coslat ,pmidm1 ,rpdeli(1,1,lat),tm1  ,qm1   ,
c    $               omga    ,cnt     ,cnb     ,cnvprc  ,
c    $               piln(1,1,lat)  ,
c    $               pmln(1,1,lat) ,phis ,tvm  ,zm(1,1,lat), cld ,
c    $               clc     ,clwp    ,emis    ,effcld  ,cltot   ,
c    $               cllow   ,clmed   ,clhgh   )
C
C Dump cloud field information to history tape buffer (diagnostics)
C
         call outfld('CLOUD   ',cld    ,plond,lat,hbuf,lhbuf)
         call outfld('EFFCLD  ',effcld ,plond,lat,hbuf,lhbuf)
         call outfld('CNVCLD  ',clc    ,plond,lat,hbuf,lhbuf)
         call outfld('CLDTOT  ',cltot  ,plond,lat,hbuf,lhbuf)
         call outfld('CLDLOW  ',cllow  ,plond,lat,hbuf,lhbuf)
         call outfld('CLDMED  ',clmed  ,plond,lat,hbuf,lhbuf)
         call outfld('CLDHGH  ',clhgh  ,plond,lat,hbuf,lhbuf)
C
C Special diagnostic cloud fields:
C
c         call outfld('SETLWP  ',clwp   ,plond,nrow,hbuf,lhbuf)
C
C Outfield special monthly cloud diagnostics
C      
         if (ninavg(1).eq.'Q') then
            do k=1,plev
               do i=1,plon
                  cloudup(i,k) = 0.
                  if (omga(i,k).lt.0.) then
                     cloudup(i,k) = cld(i,k)
                  end if
               end do
            end do
            call outfld('CLOUDUP ',cloudup ,plond,lat,hbuf,lhbuf)
         end if
C
C Compute actual liquid water path
C
C         do i=1,plon
C           sum = 0.
C           do k=1,plev
C             actlwp(i,k) = cld(i,k)*clwp(i,k)
C             sum         = sum + cld(i,k)*clwp(i,k)
C           end do
C           totlwp(i) = sum
C         end do
C         call outfld('CLDLWP  ',actlwp ,plond,nrow,hbuf,lhbuf)
C         call outfld('TOTLWP  ',totlwp ,plond,nrow,hbuf,lhbuf)
C
C Complete radiation calculations
C
c the new CCM3 version adds four fields to the input:
c coslat rel rei fice
         call radctl(hbuf    ,lhbuf   ,
     $               lat     ,clat    ,ioro    ,sndpth  ,
     $               ts      ,pmidm1  ,pintm1  ,pmln(1,1,lat),
     $               piln(1,1,lat) ,
     $               tm1     ,qm1     ,cld     ,effcld  ,clwp    ,
     $               plol    ,plos    ,fsns    ,qrs     ,qrl     ,
     $               flwds   ,coslat  ,rel     ,rei    ,fice)
      end if
C
C Add radiative heating rates to temperature tendency
C
      do k=1,plev
         do i=1,plon
            t2(i,k) = t2(i,k) + qrs(i,k) + qrl(i,k)
         end do
      end do
C
C Compute net surface radiative flux for use by surface temperature code.
C Note that units have already been converted to mks in RADCTL.  Since
C fsns and flwds are in the buffer, array values will be carried across
C timesteps when the radiation code is not invoked.
C
      do i=1,plon
         srfrad(i,lat) = fsns(i) + flwds(i)
      end do
      call outfld('SRFRAD  ',srfrad(1,lat),plond,lat,hbuf,lhbuf)
C
C Zero out all surface constituent fluxes initially
C
      do m=1,pcnst
         do i=1,plon
            cflx(i,m,lat) = 0.0
         end do
      end do
C
#ifdef COUPLED
      do i=1,plon
        to_coupler(i,lat,1)=tm1(i,plev)
        to_coupler(i,lat,2)=tm1(i,plev-1)
        to_coupler(i,lat,3)=psm1(i)
        to_coupler(i,lat,4)=pmidm1(i,plev)
        to_coupler(i,lat,5)=um1(i,plev)
        to_coupler(i,lat,6)=vm1(i,plev)
        to_coupler(i,lat,7)=qm1(i,plev,1)
        to_coupler(i,lat,8)=thm(i,plev,lat)
        to_coupler(i,lat,9)=zm(i,plev,lat)
        to_coupler(i,lat,10)=flwds(i)
        to_coupler(i,lat,13)=fsns(i)
      enddo
#endif

      return
      end




      subroutine tphys2(lat     ,clat    ,coslat  ,hbuf    ,lhbuf   ,
     $                                                     ztodt   ,
     $                 oro     ,ioro    ,phis    ,pmidm1  ,pintm1  ,
     $                 pdelm1  ,u       ,v       ,q       ,omga    ,
     $                 t2      ,fu      ,fv      ,um1     ,vm1     ,
     $                 tm1     ,qm1     ,psm1    ,pblh    ,qpert   ,
     $                 tpert   ,ts      ,sndpth  ,cnt     ,cnb     ,
     $                 qrs     ,qrl     ,flwds   ,fsns    ,plol    ,
     $                 plos    ,tssub   ,ws      ,cnvprc  )
C-----------------------------------------------------------------------
C
C Tendency physics
C Call physics subroutines.  Dump appropriate fields to history tape
C
C----------------------------Code History-------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, J. Hack, August 1992
C
C physics split for coupling: C. Schafer, R. Jacob, April 1995
C additions to work with CCM3: R. Jacob, December 1996
C
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comsgh.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
C-----------------------------------------------------------------------

#ifdef COUPLED
#include "pccmcoup.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     lat,                   ! model latitude index
     $     lhbuf                  ! history buffer length
      real clat,                  ! current latitude(radians)
     $     coslat                 ! cosine latitude
      real hbuf(*),               ! history buffer
     $     ztodt,                 ! two times model timestep (2 delta-t)
     $     oro(plond)             ! sfc type flag (ocean,land,sea ice)
      integer
     $     ioro(plond)            ! nint(oro(i))
      real phis(plond),           ! surface geopotential height
     $     pmidm1(plond,plev),    ! midpoint pressures
     $     pintm1(plond,plevp),   ! interface pressures
     $     pdelm1(plond,plev),    ! layer pressure depth
     $     u(plond,plev),         ! u wind
     $     v(plond,plev),         ! v wind
     $     q(plond,plev),         ! specific humidity
     $     omga(plond,plev),      ! vertical pressure velocity
     $     t2(plond,plev),        ! temperature tendency
     $     fu(plond,plev),        ! accumulated u tendency (from tphys)
     $     fv(plond,plev),        ! accumulated v tendency (from tphys)
     $     um1(plond,plev),       ! u horizontal wind component
     $     vm1(plond,plev),       ! v horizontal wind component
     $     tm1(plond,plev),       ! temperature
     $     qm1(plond,plev,pcnst), ! specific humidity and constituents
     $     psm1(plond)            ! surface pressure
C
C Output arguments
C
      real pblh(plond),         ! planetary boundary layer height
     $     qpert(plond,pcnst),  ! moisture/constituent perturbation (PBL)
     $     tpert(plond),        ! temperature perturbation (PBL)
     $     ts(plond),           ! surface (skin) temperature
     $     sndpth(plond),       ! snow depth (liquid water equivalent)
     $     cnt(plond),          ! top level of convective activity
     $     cnb(plond),          ! lowest level of convective activity
     $     qrs(plond,plev),     ! shortwave heating rate
     $     qrl(plond,plev),     ! longwave  heating rate
     $     flwds(plond),        ! surface longwave down flux
     $     fsns(plond),         ! surface solar absorbed flux
     $     plol(plond,plevp),   ! o3 pressure weighted absorber path
     $     plos(plond,plevp),   ! o3 absorber path
     $     tssub(plond,plevmx), ! surface/sub surface temperatures
     $     ws(plond),           ! soil moisture(water equivalent < wssat)
     $     cnvprc(plond)        ! convective precipitation rate
C
C---------------------------Local workspace-----------------------------
C
      real up1(plond,plev),    ! u-wind after vdiff & rfric
     $     vp1(plond,plev),    ! v-wind after vdiff & rfric
     $     clwp(plond,plev),   ! presribed cloud liq. h2o path 
     $     cld(plond,plevp),   ! fraction cloud cover
     $     effcld(plond,plevp),! effective cloud=cld*emis
     $     emis(plond,plev),   ! cloud longwave emissivity
     $     clc(plond),         ! total convective cloud (cloud scheme)
     $     cflx(plond,pcnst,plat),  ! surface constituent flux (kg/m^2/s)
     $     dtv(plond,plev),    ! t tendency from vertical diffusion
     $     duv(plond,plev),    ! u tendency from vertical diffusion
     $     dvv(plond,plev),    ! v tendency from vertical diffusion
     $     shflx(plond),       ! sensible heat flux (w/m^2)
     $     lhflx(plond),       ! latent  heat flux (w/m^2)
     $     psomc(plond,plev,plat),  ! (psm1/pmidm1)**cappa
     $     taux(plond),        ! x surface stress (zonal)
     $     tauy(plond),        ! y surface stress (meridional)
     $     thm(plond,plev,plat),    ! potential temperature 
     $     tvm(plond,plev),    ! virtual temperature 
     $     zm(plond,plev,plat),     ! height of midpoints (above surface) 
     $     rpdeli(plond,plev,plat), ! 1./(pmid(k+1)-pmid(k))
     $     rpdel(plond,plev,plat),  ! 1./(pint(k+1)-pint(k))
     $     cltot(plond),       ! diagnostic total cloud cover
     $     cllow(plond),       !       -     low  cloud cover
     $     clmed(plond),       !       -     mid  cloud cover
     $     clhgh(plond),       !       -     hgh  cloud cover
     $     pmln(plond,plev,plat),   ! ln(pmidm1)
     $     piln(plond,plevp,plat),  ! ln(pintm1)
     $     srcsnk(plond,plev,pcnst,plat), ! src/sink for spec. hum. + constituents
     $     srfrad(plond,plat)       ! net surface radiative flux (watts/m**2)
      integer
     $     i,k,                ! longitude, level indices
     $     m                   ! constituent index
C
C     real actlwp(plond,plev), ! actual cloud liquid h2o path 
C    $     totlwp(plond)       ! actual column cloud liquid h2o path 
C     real sum
      real cloudup(plond,plev) ! cloud fraction during omega < 0.
C
C------------------------------Externals--------------------------------
C
      external
     $     resetr,             ! memory clear
     $     cmpsnw,             ! compute snow
     $     zmid,               ! compute level heights (above surface)
     $     sstan,              ! sst update
     $     virtem,             ! virtual temperature calculation
     $     cldint,             ! cloud parameterization interface
     $     radctl,             ! radiation interface
     $     pottem,             ! potential temperature
     $     srfint,             ! surface fluxes/temperature interface
     $     copyac,             ! copy data between vectors
     $     vdintr,             ! vertical diffusion interface
     $     rfintr,             ! rayleigh friction interface
     $     mgwintr,             ! gravity wave drag interface
     $     outfld              ! history file handler
C
C-----------------------------------------------------------------------
C
C Common block to hold info from tphys1 to tphys2-CMS
      common/tphys_hold/ cflx,psomc,thm,zm,rpdeli,rpdel,pmln,
     $                   piln,srcsnk,srfrad
C-----------------------------------------------------------------------

C Call srfint if no coupler. -RLJ

#ifndef COUPLED
      call srfint(lat     ,ioro    ,psm1    ,sndpth  ,um1(1,plev)  ,
     $            vm1(1,plev)      ,tm1(1,plev)      ,qm1(1,plev,1),
     $            thm(1,plev,lat)  ,zm(1,plev,lat)       ,
     $            pmidm1  ,srfrad(1,lat) ,ts   ,tssub   ,shflx   ,
     $            lhflx   ,cflx(1,1,lat),ws   ,taux  ,tauy    )
#else
      do i=1,plon
        ts(i)=from_coupler(i,lat,1)
        do k=1,plevmx
          tssub(i,k)=from_coupler(i,lat,k+1)
        enddo
        shflx(i)=from_coupler(i,lat,6)
        lhflx(i)=from_coupler(i,lat,7)
        cflx(i,1,lat)=from_coupler(i,lat,8)
        ws(i)=from_coupler(i,lat,9)
        taux(i)=from_coupler(i,lat,10)
        tauy(i)=from_coupler(i,lat,11)
        oro(i)=from_coupler(i,lat,12)
        sndpth(i)=from_coupler(i,lat,13)
      enddo
#endif
C
C Dump sfc flux and sfc temperature information to history tape buffer
C
      call outfld('SHFLX   ',shflx    ,plond,lat,hbuf,lhbuf)
      call outfld('LHFLX   ',lhflx    ,plond,lat,hbuf,lhbuf)
      call outfld('QFLX    ',cflx(1,1,lat),plond,lat,hbuf,lhbuf)
      do m=2,pcnst
         call outfld(sflxnam(m),cflx(1,m,lat),plond,lat,hbuf,lhbuf)
      end do
      call outfld('TAUX    ',taux     ,plond,lat,hbuf,lhbuf)
      call outfld('TAUY    ',tauy     ,plond,lat,hbuf,lhbuf)
      do k=1,plevmx
         call outfld(tsnam(k),tssub(1,k),plond,lat,hbuf,lhbuf)
      end do
C
C Add contribution of src/snk tendency to constituent fields
C (excluding moisture).
C
      do m=2,pcnst
         do k=1,plev
            do i=1,plon
               qm1(i,k,m) = qm1(i,k,m) + ztodt*srcsnk(i,k,m,lat)
            end do
         end do
      end do
C
C Vertical diffusion/pbl calculation
C
      call vdintr(hbuf    ,lhbuf   ,
     $            pmidm1  ,pintm1  ,psomc(1,1,lat)   ,thm(1,1,lat) ,
     $            zm(1,1,lat) ,ts  ,rpdel(1,1,lat),rpdeli(1,1,lat) ,
     $            um1     ,vm1     ,tm1     ,
     $            taux    ,tauy    ,shflx   ,cflx(1,1,lat),ztodt   ,
     $            lat     ,qm1     ,fu      ,fv      ,t2      ,
     $            dtv     ,duv     ,dvv     ,up1     ,vp1     ,
     $            pblh    ,tpert   ,qpert   )
C
C Rayleigh friction
C
      call rfintr(q       ,u       ,v       ,ztodt   ,fu      ,
     $            fv      ,dtv     ,duv     ,dvv     ,t2      ,
     $            up1     ,vp1     )
C
C Gravity wave drag
C
c     call gwintr(hbuf    ,lhbuf   ,
c    $            um1     ,vm1     ,tm1     ,ioro    ,
c    $            fu      ,fv      ,sgh(1,lat)       ,pmidm1  ,
c    $            pintm1  , pmln(1,1,lat),
c    $            piln(1,1,lat) ,rpdel(1,1,lat) ,rpdeli(1,1,lat),lat ,
c    $            t2      ,dtv     ,qm1     )
      call mgwintr(um1    ,vm1     ,tm1     ,qm1     ,sgh(1,lat),
     $            pmidm1  ,pintm1  ,pdelm1  ,rpdel(1,1,lat),
     $            piln(1,1,lat)  ,
     $            zm(1,1,lat) ,pblh    ,ztodt   ,hbuf      ,lhbuf ,
     $		  lat     ,
     $            clat    ,fu      ,fv      ,t2      ,dtv     )

C
C Dump diffusive tendencies to history tape buffer
C
      call outfld ('DTV     ',dtv,plond,lat,hbuf,lhbuf)
C$$$      call outfld ('DUV     ',duv,plond,nrow,hbuf,lhbuf)
C$$$      call outfld ('DVV     ',dvv,plond,nrow,hbuf,lhbuf)
C
C Make surface pressure the perturbation surface pressure again
C
      do i=1,plon
         psm1(i) = psm1(i) - psr
      end do
      return
      end
