#include "params.h"
      subroutine aqsat(t       ,p       ,es      ,qs        ,ii      ,
     $                 ilen    ,kk      ,kstart  ,kend      )
C-----------------------------------------------------------------------
C
C Utility procedure to look up and return saturation vapor pressure from 
C precomputed table, calculate and return saturation specific humidity 
C (g/g),for input arrays of temperature and pressure (dimensioned ii,kk)
C This routine is useful for evaluating only a selected region in the 
C vertical.
C 
C---------------------------Code history--------------------------------
C
C Original version: J. J. Hack, February 1990
C Reviewed:         J. J. Hack, August 1992 
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer ii,            ! I dimension of arrays t, p, es, qs
     $        kk             ! K dimension of arrays t, p, es, qs 
      real t(ii,kk),         ! Temperature
     $     p(ii,kk)          ! Pressure
      integer ilen,          ! length of vectors in I direction
C                            ! which are assumed to start at 1
     $        kstart,        ! starting location in K direction
     $        kend           ! ending location in K direction
C
C Output arguments
C
      real es(ii,kk),        ! Saturation vapor pressure
     $     qs(ii,kk)         ! Saturation specific humidity
C
C---------------------------Local workspace-----------------------------
C
      real omeps             ! 1 - 0.622
      integer i,             ! i index
     $        k              ! k index
C
C-----------------------------------------------------------------------
#include "eslookup.com"
C-----------------------------------------------------------------------
C
      omeps = 1.0 - epsqs
      do k=kstart,kend
         do i=1,ilen
            es(i,k) = estblf(t(i,k))
C
C Saturation specific humidity
C
            qs(i,k) = epsqs*es(i,k)/(p(i,k) - omeps*es(i,k))
C   
C The following check is to avoid the generation of negative values
C that can occur in the upper stratosphere and mesosphere
C
            qs(i,k) = amin1(1.0,qs(i,k))
C
            if (qs(i,k) .lt. 0.0) then
               qs(i,k) = 1.0
               es(i,k) = p(i,k)
            end if
         end do
      end do
C
      return
      end
