#include "params.h"
      subroutine zmid(piln    ,pmln    ,rair    ,gravit  ,tv      ,
     $                z       )
C-----------------------------------------------------------------------
C
C Compute the geopotential height *ABOVE THE SURFACE* at layer
C midpoints from the virtual temperatures and pressures.
C
C---------------------------Code history--------------------------------
C
C Original version:  B. Boville, D. Williamson, January 1990
C Standardized:      L. Buja, June 1992
C Reviewed:          J. Hack, B. Boville, August 1992
C
C-----------------------------------------------------------------------
      implicit none
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      real piln(plond,plevp),    ! Log interface pressures
     $     pmln(plond,plev),     ! Log midpoint pressures
     $     rair,                 ! Gas constant for dry air
     $     gravit,               ! Acceleration of gravity
     $     tv(plond,plev)        ! Virtual temperature
C
C Output arguments
C
      real z(plond,plev)         ! Height above surface at midpoints
C
C---------------------------Local variables-----------------------------
C
      integer i,                 ! Longitude index
     $        k,                 ! Level index
     $        l                  ! Level index
      real rog                   ! Rair / gravit
C
C Diagonal term of hydrostatic equation
C
      rog = rair/gravit
      do k=1,plev-1
         do i=1,plon
            z(i,k) = rog*tv(i,k)*0.5*(pmln(i,k+1) - pmln(i,k))
         end do
      end do
      do i=1,plon
         z(i,plev) = rog*tv(i,plev)*(piln(i,plevp) - pmln(i,plev))
      end do
C
C Bottom level term of hydrostatic equation
C
      do  k=1,plev-1
         do i=1,plon
            z(i,k) = z(i,k) + rog*tv(i,plev)*(piln(i,plevp) -
     $                        0.5*(pmln(i,plev-1) + pmln(i,plev)))
         end do
      end do
C
C Interior terms of hydrostatic equation
C
      do k=1,plev-2
         do l=k+1,plev-1
            do i=1,plon
               z(i,k) = z(i,k) + rog*(tv(i,l))*
     $                           0.5*(pmln(i,l+1) - pmln(i,l-1))
            end do
         end do
      end do
C
      return
      end
