#include "params.h"
      subroutine cubydr(nf,fint,wdy,jdp,fyb,fyt)
c
      implicit none
#include "pmgrid.com"
#include "parslt.com"
      integer jxdm,ngls
#ifdef PP_PCCM
      parameter( jxdm= 2*jintmx + p_lato2 + 2*nxpt,
     *            ngls = plon*plev )
#else
      parameter( jxdm= 2*jintmx + plat + 2*nxpt,
     *            ngls = plon*plev )
#endif
c
      integer nf,jdp(ngls)
      real fint(ngls,npdy,nf),wdy(4,2,jxdm),
     *     fyb(ngls,nf),fyt(ngls,nf)
c
c...Compute Lagrangian cubic derivative estimates at both ends of the
c   intervals in the y coordinate (unequally spaced) containing the
c   departure points for the latitude slice being forecasted.
c
c...On entry.
c
c  nf      Number of fields being interpolated.
c  fint    (fint(i,k,j,n),j=1,npdy) contains the x interpolants at each
c          latitude needed for the y derivative estimates at the endpoin
c          of the interval that contains the departure point for grid
c          point (i,k).  The last index of fint allows for interpolation
c          of multiple fields.  fint is generated by a call to herxin.
c  wdy     Weights for Lagrange cubic derivative estimates on the
c          unequally spaced latitude grid.  If grid interval j (in
c          extended array) is surrounded by a 4 point stencil, then
c          the derivative at the "bottom" of the interval uses the
c          weights wdy(1,1,j),wdy(2,1,j), wdy(3,1,j), and wdy(4,1,j).
c          The derivative at the "top" of the interval uses wdy(1,2,j),
c          wdy(2,2,j), wdy(3,2,j), and wdy(4,2,j).
c  jdp     jdp(i,k) is the index of the y-interval that contains the
c          departure point corresponding to global grid point (i,k) in t
c          latitude slice being forecasted.
c          Suppose yb contains the y-coordinates of the extended array
c          and ydp(i,k) is the y-coordinate of the departure point
c          corresponding to grid point (i,k).  Then,
c                yb(jdp(i,k)) .le. ydp(i,k) .lt. yb(jdp(i,k)+1) .
c
c...On return.
c
c  fyb     fyb(i,k,.) is the derivative at the bottom of the y interval
c          that contains the departure point of global grid point (i,k).
c  fyt     fyt(i,k,.) is the derivative at the top of the y interval
c          that contains the departure point of global grid point (i,k).
c
c...Parameters needed.
c
c  plon    Number of longitudes in global grid.
c  plev    Number of levels in global grid.
c  jxdm    Number of latitudes in extended array.
c  npdy    The number of points in the stencil for computing the y
c          derivatives.
c  ngls   Number of points in one latitude slice of global grid.
c
      integer i,k,n
      real wdytem(ngls,4,2)
c
c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c For sake of efficiency,
c   if (nf .eq. 1) then
c      run first piece of code
c   else
c      run second piece of code
c   endif
c
c-----------------------------------------------------------------------
c
      if( nf .eq. 1) then
#ifdef CRAY
         fyb(:,1) = wdy(1,1,jdp)*fint(:,1,1)
     *            + wdy(2,1,jdp)*fint(:,2,1)
     *            + wdy(3,1,jdp)*fint(:,3,1)
     *            + wdy(4,1,jdp)*fint(:,4,1)
         fyt(:,1) = wdy(1,2,jdp)*fint(:,1,1)
     *            + wdy(2,2,jdp)*fint(:,2,1)
     *            + wdy(3,2,jdp)*fint(:,3,1)
     *            + wdy(4,2,jdp)*fint(:,4,1)
#else
         do 50     i = 1,ngls
            fyb(i,1) = wdy(1,1,jdp(i))*fint(i,1,1)
     *               + wdy(2,1,jdp(i))*fint(i,2,1)
     *               + wdy(3,1,jdp(i))*fint(i,3,1)
     *               + wdy(4,1,jdp(i))*fint(i,4,1)
            fyt(i,1) = wdy(1,2,jdp(i))*fint(i,1,1)
     *               + wdy(2,2,jdp(i))*fint(i,2,1)
     *               + wdy(3,2,jdp(i))*fint(i,3,1)
     *               + wdy(4,2,jdp(i))*fint(i,4,1)
   50    continue
#endif
      else
c
c     ...Load temp arrays with weights for Lagrangian cubic derivative
c        estimates on unequally spaced grid.
c
         do 70          i = 1,ngls
            wdytem(i,1,1) = wdy(1,1,jdp(i))
            wdytem(i,2,1) = wdy(2,1,jdp(i))
            wdytem(i,3,1) = wdy(3,1,jdp(i))
            wdytem(i,4,1) = wdy(4,1,jdp(i))
            wdytem(i,1,2) = wdy(1,2,jdp(i))
            wdytem(i,2,2) = wdy(2,2,jdp(i))
            wdytem(i,3,2) = wdy(3,2,jdp(i))
            wdytem(i,4,2) = wdy(4,2,jdp(i))
   70    continue
c
c     ...Loop over fields.
c
         do 100       n = 1,nf
            do 80     i = 1,ngls
               fyb(i,n) = wdytem(i,1,1)*fint(i,1,n)
     *                  + wdytem(i,2,1)*fint(i,2,n)
     *                  + wdytem(i,3,1)*fint(i,3,n)
     *                  + wdytem(i,4,1)*fint(i,4,n)
               fyt(i,n) = wdytem(i,1,2)*fint(i,1,n)
     *                  + wdytem(i,2,2)*fint(i,2,n)
     *                  + wdytem(i,3,2)*fint(i,3,n)
     *                  + wdytem(i,4,2)*fint(i,4,n)
   80       continue
  100    continue
c
      endif
c
      return
      end
