#include "params.h"
      subroutine g2spos(jgc,dttmp,lam,phib,phi,cosphi,sinphi,
     $                  upr,vpr,lamgc,phigc,lamsc,phisc)
c
      implicit none
#include "pmgrid.com"
#include "parslt.com"
      integer pgls
      parameter(pgls=plon*plev)
      real fac
#ifdef WORD4 
      parameter ( fac = 1. - 1.e-06 )
#else
      parameter ( fac = 1. - 1.e-12 )
#endif
c
      integer jgc
      real dttmp,lam(plon),phib(platd),phi,cosphi,sinphi,
     $     upr(pgls),vpr(pgls),
     $     lamgc(pgls),phigc(pgls),
     $     lamsc(pgls),phisc(pgls)
c
c...Transform position coordinates for a set of points, each of which is
c   associated with a grid point in a global latitude slice, from local
c   geodesic to spherical coordinates.
c
c...On entry.
c
c  jgc    Latitude index in global grid of the latitude slice with which
c         the set of points is associated.
c  dttmp  Time step over which midpoint/endpoint trajectory is
c         calculated (seconds).
c  lam    Longitude coordinates of the global grid points in spherical
c         system.  The grid points in the global array are the reference
c         points for the local geodesic systems.
c  phib   Latitude values for the extended grid.
c  phi    Latitude coordinate of the global latitude slice in spherical
c         system.
c  cosphi cosine( phi )
c  sinphi sine( phi )
c  upr    zonal      velocity at departure point in local geodesic coord
c  vpr    Meridional velocity at departure point in local geodesic coord
c  lamgc  Longitude coordinate of points in geodesic coordinates.
c  phigc  Latitude coordinate of points in geodesic coordinates.
c
c...On return.
c
c  lamsc  Longitude coordinate of points in spherical coordinates.
c  phisc  Latitude coordinate of points in spherical coordinates.
c
c...Parameters required.
c
c  plon   Number of longitudes in global array.
c  plev   Number of levels in global array.
c  plat   Number of latitudes in global array.
c
c
      integer i,ii,k,nval,indx(pgls)
      real pi,twopi, pi2, sgnphi
      real sphigc, cphigc, clamgc, slam2, phipi2,
     *     slamgc(pgls), dlam(pgls)
      real coeff,distmx,dist(pgls)
c
      real tmp
c
      pi     = 4.*atan(1.)
      twopi  = pi*2.
      pi2    = pi/2.
      coeff  = (1.1*dttmp)**2
      distmx = (sign(pi2,phi) - phi)**2/coeff
      sgnphi = sign( 1., phi )
c
      do 10      i = 1,pgls
         sphigc    = sin( phigc(i) )
         cphigc    = cos( phigc(i) )
         slamgc(i) = sin( lamgc(i) )
         clamgc    = cos( lamgc(i) )
c
         phisc(i) = asin( ( sphigc*cosphi +
     *                        cphigc*sinphi*clamgc)*fac )
#ifdef PP_PCCM
         if(phisc(i) .ge. phib(j1+p_lato2))
     $                          phisc(i) = phib(j1+p_lato2)*fac
#else
         if(phisc(i) .ge. phib(j1+plat))
     $                          phisc(i) = phib(j1+plat)*fac
#endif
c
         dlam(i) = asin( ( slamgc(i)*cphigc/cos( phisc(i) ) )*fac )
c
c     ...Compute estimated trajectory distance based upon winds
c        alone
c
         dist(i) = upr(i)**2 + vpr(i)**2
   10 continue
c
c     Determine which trajectories may have crossed over pole
c
      call whenfgt(pgls,dist,1,distmx,indx,nval)
c
c     Check that proper branch of arcsine is used for calculation of
c     dlam for those trajectories which may have crossed over pole.
c
CDIR$ IVDEP
      do 20  ii = 1,nval
         i      = indx(ii)
         slam2  = slamgc(i)**2
         phipi2 = asin( ( sqrt( ( slam2 - 1. )/
     *            ( slam2 - 1./cosphi**2 ) ) )*fac )
         if( sgnphi*phigc(i) .gt. phipi2 )
     $      dlam(i) = sign( pi, lamgc(i) ) - dlam(i)
   20 continue
c
      do 100   k = 1,plev
         do 50 i = 1,plon
            ii = i + (k-1)*plon
            lamsc(ii) = lam(i) + dlam(ii)
c
c        ...Normalize longitude to be in the range [0, twopi).
c
#ifdef PP_PCCM
c        ...Parallel code allows extension beyond twopi
	   if( p_nproc_ln.eq.1) then
#endif
            if( lamsc(ii) .ge. twopi ) lamsc(ii) = lamsc(ii) - twopi
            if( lamsc(ii) .lt. 0.0 )   lamsc(ii) = lamsc(ii) + twopi
#ifdef PP_PCCM
	   endif
#endif
 50      continue
 100  continue
c
      return
      end
