#include "params.h"
      subroutine heryin(nf,fint,fyb,fyt,y,dy,ydp,jdp,fdp)
c
      implicit none
#include "pmgrid.com"
#include "parslt.com"
      integer jxdm,ngls
#ifdef PP_PCCM
      parameter( jxdm= 2*jintmx + p_lato2 + 2*nxpt,
     *           ngls = plon*plev )
#else
      parameter( jxdm= 2*jintmx + plat + 2*nxpt,
     *           ngls = plon*plev )
#endif
c
      integer nf,jdp(ngls)
      real fint(ngls,npdy,nf),fyb(ngls,nf),fyt(ngls,nf),
     *     y(jxdm),dy(jxdm),ydp(ngls),fdp(ngls,nf)
c
c...For each departure point in the latitude slice to be forecasted,
c   interpolate (using unequally spaced Hermite cubic formulas) the
c   x interpolates to the y value of the departure point.
c
c...On entry.
c
c  nf      Number of fields being interpolated.
c  fint    (fint(i,k,j,n),j=npdy/2,npdy/2 + 1) contains the x interpolan
c          at the endpoints of the y-interval that contains the departur
c          point for grid point (i,k).  The last index of fint allows fo
c          interpolation of multiple fields.  fint is generated by a cal
c          to herxin.
c  fyb     fyb(i,k,.) is the derivative at the "bottom" of the y-interva
c          that contains the departure point of grid point (i,k).
c          fyb is generated by a call to cubydr.
c  fyt     fyt(i,k,.) is the derivative at the "top" of the y-interval
c          that contains the departure point of grid point (i,k).
c          fyt is generated by a call to cubydr.
c  y       y-coordinate (latitude) values in the extended array.
c  dy      Increment in the y-coordinate value for each interval in the
c          extended array.
c  ydp     ydp(i,k) is the y-coordinate of the departure point that
c          corresponds to global grid point (i,k) in the latitude slice
c          being forecasted.
c  jdp     jdp(i,k) is the index of the y-interval that contains the
c          departure point corresponding to global grid point (i,k) in t
c          latitude slice being forecasted.
c          Note that
c                y(jdp(i,k)) .le. ydp(i,k) .lt. y(jdp(i,k)+1) .
c
c...On return.
c
c  fdp     Horizontally interpolated field values at the departure point
c          for the latitude slice being forecasted.
c
c...Parameters required.
c
c  plon    Number of longitudes in global grid.
c  plev    Number of levels in global grid.
c  jxdm    Number of latitudes in extended array.
c  npdy    The number of points in the stencil for computing the y
c          derivatives.
c  ngls    Number of points in latitude slice of global grid.
c
c
      integer i,k,jb,jt,n
      real dyj(ngls),yb(ngls),yt(ngls),
     *     hb(ngls),ht(ngls),dhb(ngls),dht(ngls)
c
      jb = npdy/2
      jt = jb + 1
c
         do 10 i=1,ngls
            dyj(i) = dy(jdp(i))
            yb(i) = ( y(jdp(i)+1) - ydp(i) )/dyj(i)
            yt(i) = 1. - yb(i)
            hb(i) = ( 3.0 - 2.0*yb(i) )*yb(i)**2
            ht(i) = ( 3.0 - 2.0*yt(i) )*yt(i)**2
            dhb(i) = -dyj(i)*( yb(i) - 1. )*yb(i)**2
            dht(i) =  dyj(i)*( yt(i) - 1. )*yt(i)**2
   10    continue
c
c     ...Loop over fields.
      do 100  n=1,nf
c
            do 80 i=1,ngls
               fdp(i,n) = fint(i,jb,n)*hb(i) + fyb(i,n)*dhb(i)
     *                  + fint(i,jt,n)*ht(i) + fyt(i,n)*dht(i)
   80       continue
c
  100 continue
c
      return
      end
