#include "params.h"
      subroutine sphdep(jcen,jgc,dt,ra,iterdp,locgeo,
     *                  ub,uxl,uxr,vb,lam,phib,lbasiy,
     *                  lammp,phimp,lamdp,phidp,idp,jdp,
     *                  nxpt_a)
c
      implicit none
#include "pmgrid.com"
#include "pspect.com"
#include "comparll.com"
#include "parslt.com"
      integer ibdm,kbdm,jxdm,istart
#ifdef PP_PCCM
      parameter( ibdm= plon + 1 + 2*nxpt, kbdm=plev,
     *           jxdm= p_lato2 +  2*nxpt + 2*jintmx,
     *           istart= nxpt + 1  )
      integer nxpt_a(plev)
#else
      parameter( ibdm= plon + 1 + 2*nxpt, kbdm=plev,
     *           jxdm= plat +     2*nxpt + 2*jintmx,
     *           istart= nxpt + 1  )
#endif
c
      integer jcen,jgc,iterdp,idp(plon,plev),jdp(plon,plev)
      real ub(ibdm,kbdm*(3+pcnst),jxdm),uxl(ibdm,kbdm,2,jxdm),
     $     uxr(ibdm,kbdm,2,jxdm),
     *     vb(ibdm,kbdm*(3+pcnst),jxdm),
     *     dt,ra,lam(ibdm),phib(jxdm),lbasiy(4,2,jxdm),
     *     lammp(plon,plev),phimp(plon,plev),
     *     lamdp(plon,plev),phidp(plon,plev)
      logical locgeo
c
c-----------------------------------------------------------------------
c
c...Compute departure points for semi-Lagrangian transport on surface
c   of sphere using midpoint quadrature.
c   Computations are done in:
c
c      1) "local geodesic"   coordinates for "locgeo" = .true.
c      2) "global spherical" coordinates for "locgeo" = .false.
c
c-----------------------------------------------------------------------
c
c...On entry.
c
c  jcen    Index in extended grid corresponding to latitude being foreca
c  jgc     Latitude index in global grid of the latitude slice being for
c  dt      Time interval that parameterizes the parcel trajectory.
c  ra      Reciprocal of radius of earth.
c  iterdp  Number of iterations used for departure point calculation.
c  ub      Longitudinal velocity components in spherical coordinates.
c  locgeo  Logical flag to indicate computation in "local geodesic" or
c          "global spherical" space
c  uxl     x-derivatives at the left edge of each interval contained in
c  uxr     x-derivatives at the right edge of each interval contained in
c  vb      Latitudinal velocity components in spherical coordinates.
c  lam     Longitude values for the extended grid.
c  phib    Latitude values for the extended grid.
c  lbasiy  Weights for Lagrange cubic interpolation on the
c          unequally spaced latitude grid.
c  lammp   Longitude coordinates of the trajectory mid-points of the
c          parcels that correspond to the global grid points contained
c          in the latitude slice being forecast.  On entry lammp
c          is an initial guess.
c  phimp   Latitude coordinates of the trajectory mid-points of the
c          parcels that correspond to the global grid points contained
c          in the latitude slice being forecast.  On entry phimp
c          is an initial guess.
c
c...On return.
c
c  lammp   Longitude coordinates of the trajectory mid-points of the
c          parcels that correspond to the global grid points contained
c          in the latitude slice being forecast.
c  phimp   Latitude coordinates of the trajectory mid-points of the
c          parcels that correspond to the global grid points contained
c          in the latitude slice being forecast.
c  lamdp   Longitude coordinates of the departure points that correspond
c          to the global grid points contained in the latitude slice bei
c          forecast.  lamdp is constrained so that
c                     0.0 .le. lamdp(i) .lt. 2*pi .
c  phidp   Latitude coordinates of the departure points that correspond
c          to the global grid points contained in the latitude slice bei
c          forecast.  If phidp does not satisfy the constraint
c          phib(jcen-jintmx) .le. phidp(i) .lt. phib(jcen+jintmx), then
c          an abort will be called by subroutine bandij.
c  idp     Longitude index of departure points.  This index points into
c          the extended arrays, e.g.,
c                   lam(idp) .le. lamdp .lt. lam(idp+1) .
c  jdp     Latitude index of departure points.  This index points into
c          the extended arrays, e.g.,
c                   phib(jdp) .le. phidp .lt. phib(jdp+1) .
c
c...Parameters required.
c
c  plon    Number of longitudes in global grid.
c  plev    Number of levels in global grid.
c  ibdm    Number of longitudes in extended grid.
c  kbdm    Number of levels in extended grid.
c  jxdm    Number of latitudes in extended grid.
c  istart  Longitude index of point corresponding to lambda=0 in
c          extended array.
c
      integer iter,i,k
      real ump(plon,plev),vmp(plon,plev),upr(plon,plev),
     $     vpr(plon,plev),
     *     dlam,phicen,cphic,sphic,lampr(plon,plev),phipr(plon,plev),
     $     uvmp(plon,plev,2),fint(plon,plev,npdy,2),
     $     finc,dttmp
      equivalence ( uvmp(1,1,1),ump(1,1) )
     $          , ( uvmp(1,1,2),vmp(1,1) )
      external s2gphi,bandij,hrvint,s2gvel,trajmp,trjgl,trjmps,g2spos
c
#ifdef PP_PCCM
      real pi
      pi = 4.*atan(1.)
c     ...Interval length in equally spaced longitude grid.
      dlam = 2.*pi/float(p_glon)
#else
      dlam   = lam(nxpt+2) - lam(nxpt+1)
#endif
      phicen = phib(jcen)
      cphic  = cos( phicen )
      sphic  = sin( phicen )
c
c     Convert latitude coordinates of trajectory midpoints from
c     spherical to local geodesic basis.
c
      if( locgeo )call s2gphi(lam(istart),cphic,sphic,lammp,phimp,phipr)
c
c     Loop over departure point iterates.
c
      do 100  iter=1,iterdp
c
c     ...Compute midpoint indicies.
c
#ifdef PP_PCCM
         call bandij(dlam,phib,lammp,phimp,idp,jdp,nxpt_a)
#else
         call bandij(dlam,phib,lammp,phimp,idp,jdp)
#endif
c
c     ...Interpolate velocity fields to midpoint locations
c        (Lagrangian bicubic interpolation; no limiters applied to
c        the velocity field derivatives).
C        Last arg to hrintp=dimensioning construct--JR
c
         call hrvint(2,ub,uxl,uxr,lam,lbasiy,lammp,phimp,
     *               idp,jdp,fint,uvmp,3+pcnst)
C
C Put u/v on unit sphere
C
         do k = 1,plev
            do i = 1,plon
               uvmp(i,k,1) = uvmp(i,k,1)*ra
               uvmp(i,k,2) = uvmp(i,k,2)*ra
            end do
         end do
c
c     ...For local geodesic:
c
c          a) Convert velocity coordinates at trajectory midpoints from
c             spherical coordinates to local geodesic coordinates,
c          b) Estimate midpoint parcel trajectory,
c          c) Convert back to spherical coordinates
c
c     ...Else, for global spherical
c
c             Estimate midpoint trajectory with no conversions
c
         if ( locgeo ) then
            call s2gvel(ump,vmp,lam(istart),cphic,sphic,lammp,phimp,
     *                  upr,vpr)
            call trajmp(dt,upr,vpr,phipr,lampr)
            dttmp = 0.5*dt
            call g2spos(jgc,dttmp,lam(istart),phib,phicen,cphic,sphic,
     $                  upr,vpr,lampr,phipr,lammp,phimp)
         else
            call trjmps(dt,ump,vmp,phimp,phipr,lampr)
            finc = 1.
            call trjgl(jgc,finc,phicen,lam(istart),phib,lampr,phipr,
     $                  lammp,phimp)
         endif
c
 100  continue
c
c     Compute departure points in geodesic coordinates, and convert back
c     to spherical coordinates.
c
c     Else, compute departure points directly in spherical coordinates
c
      if (locgeo ) then
         do 120         k = 1,plev
            do 110      i = 1,plon
               lampr(i,k) = 2.*lampr(i,k)
               phipr(i,k) = 2.*phipr(i,k)
  110       continue
  120    continue
         dttmp = dt
         call g2spos(jgc,dttmp,lam(istart),phib,phicen,cphic,sphic,
     $               upr,vpr,lampr,phipr,lamdp,phidp)
      else
         finc = 2.
         call trjgl(jgc,finc,phicen,lam(istart),phib,lampr,phipr,
     $               lamdp,phidp)
      endif
c
c     Compute departure point indicies.
c
#ifdef PP_PCCM
      call bandij(dlam,phib,lamdp,phidp,idp,jdp,nxpt_a)
#else
      call bandij(dlam,phib,lamdp,phidp,idp,jdp)
#endif
c
      return
      end
