#include "params.h"
       subroutine meshmax0(  vec, tmp, idim, mmtype )
       implicit none
c-----------------------------------------------------------------------------
c  Maximize the vector over all processors in logical mesh
#include "palgor.com"
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
#if ( defined(INTERVAL_TIMER) )
#include "timers.com"
#endif
c-----------------------------------------------------------------------------
c
c     !  input parameters
c
      integer idim, mmtype
      real vec(idim), tmp(idim)
c
c     ! local storage
c
      integer base, base2
#ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_gmax, 0 )
#endif
c     First, maximize over columns
      if (p_nproc_lt .gt. 1) then
c
c       ! update message tags
c
        base   = mmtype
        base2  = mmtype + p_nproc
c
c       ! compute column vector max
c
        if ((p_sumopt .le. 13) .and. (p_sumopt .ge. 10)) then
          call ringmax(p_sumopt-10, p_sumprot, p_nproc_lt, m_pin,
     &                 m_pin_dwn, m_pin_up, m_pin_lt, base, base2, 
     &                 idim, tmp, vec)
        elseif ((p_sumopt .eq. 20) .or. (p_sumopt .eq. 21)) then
          call halfmax(p_sumopt-20, p_sumbufs, p_sumprot, p_nproc_lt, 
     &                 mapcol, m_pin_lt, base, base2, p_exchsz, idim, 
     &                 tmp, vec)
        else
C         illegal communication option specified
          write(0,100) m_pin_lt, p_sumopt
  100     format (/,' fatal error in subroutine meshmax0 ',/,
     &              ' illegal parallel algorithm specified ',/,
     &              ' procid = ',I4,' p_sumopt = ',I4)
          stop
c
        endif
c
      endif
c
c     Next, maximize over rows
      if (p_nproc_ln .gt. 1) then
c
c       ! update message tags
c
        base   = mmtype + 2*p_nproc
        base2  = mmtype + 3*p_nproc
c
c       ! compute row vector max
c
        if ((p_sumopt .le. 13) .and. (p_sumopt .ge. 10)) then
          call ringmax(p_sumopt-10, p_sumprot, p_nproc_ln, m_pin,
     &                 m_pin_lft, m_pin_rht, m_pin_ln, base, base2, 
     &                 idim, tmp, vec)
        elseif ((p_sumopt .eq. 20) .or. (p_sumopt .eq. 21)) then
          call halfmax(p_sumopt-20, p_sumbufs, p_sumprot, p_nproc_ln, 
     &                 maprow, m_pin_ln, base, base2, p_exchsz, idim, 
     &                 tmp, vec)
        else
C         illegal communication option specified
          write(0,100) m_pin_ln, p_sumopt
          stop
C
        endif
c
      endif
#ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_gmax, 0 )
#endif
      return
      end

      subroutine rowmaxi0(vec, tmp, idim, mmtype)
      implicit none
c-----------------------------------------------------------------------------
c  Sums the vector of processors in the row of the processor mesh
#include "palgor.com"
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
#if ( defined(INTERVAL_TIMER) || defined(PICL_TRACING) )
#include "timers.com"
#endif
c-----------------------------------------------------------------------------
c
c     !  input parameters
c
      integer idim, mmtype
      integer vec(idim), tmp(idim)
c
c     ! local storage
c
      integer base, base2
      character*4 savetype
#ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_rowmax, 0 )
#endif
c
      if (p_nproc_ln .gt. 1) then
c
c       Save current data type and set new one
c
        call getdataf(savetype)
        call setdataf('integer')
c
c       ! update message tags
c
        base   = mmtype
        base2  = mmtype + p_nproc
c
c       ! compute global vector sum
c
        if ((p_sumopt .le. 13) .and. (p_sumopt .ge. 10)) then
          call ringimax(p_sumopt-10, p_sumprot, p_nproc_ln, m_pin,
     &                 m_pin_lft, m_pin_rht, m_pin_ln, base, base2, 
     &                 idim, tmp, vec)
        elseif ((p_sumopt .eq. 20) .or. (p_sumopt .eq. 21)) then
          call halfimax(p_sumopt-20, p_sumbufs, p_sumprot, p_nproc_ln, 
     &                 maprow, m_pin_ln, base, base2, p_exchsz, idim, 
     &                 tmp, vec)
        else
C         illegal communication option specified
          write(0,100) m_pin_ln, p_sumopt
  100     format (/,' fatal error in subroutine rowmaxi0 ',/,
     &              ' illegal parallel algorithm specified ',/,
     &              ' procid = ',I4,' p_sumopt = ',I4)
          stop
C
        endif
c
      endif
c
c Restore default data type
c
      call setdataf(savetype)
#ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_rowmax, 0 )
#endif
c
      return
      end


