#include "params.h"
C     ******************************************************************
C     Subroutine:  interval_timer
C     Description: subroutine to act as stop watch for determining the 
C                  run-time of specified subroutines.
C     Arguments:   mode - specifies whether the timer is to be started
C                    (TIMER_START) or stopped (TIMER_STOP)
C                  section - section of code to time.  A constant should
C                    defined in timers.com for each routine to be timed.
C     ******************************************************************
      subroutine interval_timer(mode, section, dosync)
         implicit none

         integer mode, section

#include "timers.com"

         double precision istop             !timer value when stopped
         integer dosync                     !sync before starting timer
         double precision clock0f           !current timer value

         if (section .lt. 1 .or. section .gt. NUM_TIMERS) then
#ifdef PP_ERROR
            write (0,40) section
#endif
 40         format ('ERROR: INVALID TIMER SECTION ',I2,'!!!')
#ifdef PP_ERROR
            write (0,*) 'TIMERS: ',(timer_started(section),
     &       section = 1, NUM_TIMERS)
#endif
	       return
         endif

         if (mode .eq. TIMER_STOP) then
            istop = clock0f()
            if (.not. timer_started(section)) then
#ifdef PP_ERROR
               write (0,10) section
#endif
 10            format ('ERROR: TIMER ',I2,' HAS NOT BEEN STARTED!!!')
	          return
            endif
            timer_started(section) = .false.
            timer_total(section) = timer_total(section) + istop -
     &       timer_begin(section)
         else if (mode .eq. TIMER_START) then
            if (timer_started(section)) then
#ifdef PP_ERROR
               write (0,20) section
#endif
 20            format ('ERROR: TIMER ',I2,' IS ALREADY RUNNING!!!')
	       return
            endif
            if (dosync .ne. 0) then
               call sync0f
            endif
            timer_started(section) = .true.
            timer_begin(section) = clock0f()
         else
#ifdef PP_ERROR
            write (0,30) mode, section
#endif
 30         format ('ERROR: MODE ',I2,' IS NOT VALID FOR TIMER ',I2,
     &       '!!!')
         endif

      end

C     ******************************************************************
C     Subroutine:  reset_timer
C     Description: subroutine to clear the timer accumulator array
C     Arguments:   none
C     ******************************************************************
      subroutine reset_timer(section)
         implicit none
#include "timers.com"
         integer section                   !section being cleared

         timer_total(section) = 0.0
         timer_started(section) = .false.

      end

C     ******************************************************************
C     Subroutine:  dump_timer
C     Description: subroutine to dump the timers to standard error
C     Arguments:   timestep - time step being dumped
C     ******************************************************************

      subroutine dump_timer(timestep,section)

         implicit none

         integer timestep, section
#include "timers.com"
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"

         if (timer_started(section)) then
               write (0,20) section
 20            format ('ERROR: TIMER ',I2,' IS ACTIVE, DUMP ABORTED!!!')
	       return
            endif

         write (0,10) timer_names(section), m_pin, timestep,
     &          timer_total(section) * 1000.0
10       format(1x, '*** ',a15,
     &        '  pin: ',i3,
     &        '   ts: ',i3,
     &        '   time: ',f12.3)

         return

      end






