#include "params.h"
C----------------------------------------------------------------------
C  BACKWARD FULL-TRANSPOSE FFT
C                     FFT
C  (nllon_p+2,nvect) <--- (mxlfc,nvect)
C
C  Arguments:
C     fc_i    : Input and output array.
C     work    : Work array; size is 2 x size(fc_i).
C     nvect   : Number of FFTs in block
C
C Ian Foster, Argonne National Laboratory, January 1993.
c -----
c rewritten to use PSTSWM software, Pat Worley, ORNL, FEB 1995.
C----------------------------------------------------------------------
      subroutine bwd_fftx(fc_i,work1,work2,nvect)

#ifdef PP_FFTX
      implicit none
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
#include "comparll.com"
#include "comfft.com"
#include "comxpose.com"
#include "palgor.com"

      integer jump
#ifdef ESSL_FFT
      parameter ( jump = p_glon + 2 )
#else
      parameter ( jump = p_glon )
#endif

      integer nvect
      real fc_i(plond, nvect )
      real work1(plond, nvect )
      real work2(jump, * )
      integer i
      integer ntmp1, ntmp2, mxlvect, nlvect
      integer nlvects(0:p_nproc_ln-1)
c
      if ( .not. xpose_init ) call xpose_ini
c
      if (p_nproc_ln .gt. 1) then
C       Compute actual number of vectors; may be 0:
        ntmp1 = nvect/p_nproc_ln
        ntmp2 = mod(nvect,p_nproc_ln)
        if (ntmp2 .gt. 0) then
          mxlvect = ntmp1 + 1
        else
          mxlvect = ntmp1
        endif
        do i=0,ntmp2-1
          nlvects(i) = ntmp1+1
        enddo
        do i=ntmp2,p_nproc_ln-1
          nlvects(i) = ntmp1
        enddo
        nlvect = nlvects(m_pin_ln)
c
c       reorder and transpose fields
        call reorder3(fc_i, work1, plond, nlmm, nvect)
        call transpose(p_tfftopt, p_tfftbufs, p_tfftprot,
     &                 p_nproc_ln, maprow, m_pin_ln, m_type,
     &                 +2, 2, ptrm+1, nvect, 1, 1,
     &                 nlfcs, nlvects, jump/2, work1,
     &                 work2, work2(1,mxlvect+1))
        m_type = m_type+p_nproc
        call zerofc(ptrm+1, jump/2, jump/2, 
     &              nlvect, ordinv_s, work2(1,mxlvect+1))
c
c       serial real Fourier transform
        if (nlvect .gt. 0) then
c         "pack" the last Fourier coefficient into the first
c         complex value - assuming that last wavenumber is 
c         always truncated, and that wavenumber 0 is always the
c         first wavenumber
          do i=mxlvect+1,mxlvect+nlvect
            work2(2,i) = 0.0
          enddo
#ifdef ESSL_FFT
          call esslft(work2(1,mxlvect+1),work2,1,jump,
     &                p_glon, nlvect,+1)
#else
          call RFT_235(0, 0, 1, maprow(m_pin_ln), 0,
     &                 m_type, m_type, +1, p_glon,
     &                 nlvect, p_glon, mxtrigs, trig, 
     &                 work2, work2(1,mxlvect+1))
#endif
        endif
c
c       reorder and transpose fields
        call reorder4(work2(1,mxlvect+1),work2,jump,p_glon,nlvect) 
        call transpose(p_tfftopt, p_tfftbufs, p_tfftprot,
     &                 p_nproc_ln, maprow, m_pin_ln, m_type,
     &                 +1, 1, nvect, p_glon, 1, 1,
     &                 nlvects, nllons, plond, work2,
     &                 work2(1,mxlvect+1), fc_i)
        m_type = m_type+p_nproc
      else
c       serial real Fourier transform
        if (nvect .gt. 0) then
c         "pack" the last Fourier coefficient into the first
c         complex value - assuming that the last wavenumber is always
c         truncated
          do i=1,nvect
            fc_i(2*m_pwave0_dex,i)  = 0.0
          enddo
#ifdef ESSL_FFT
          call esslft( fc_i,work2,1,plond,plon, nvect,+1)
#else
          call RFT_235(0, 0, 1, maprow(m_pin_ln), 0,
     &                 m_type, m_type, +1, plon,
     &                 nvect, plond, mxtrigs, trig, 
     &                 work2, fc_i)
#endif
        endif
      endif
c
#endif
      return
      end

#ifdef PP_FFTX
C ------------------------------------------------------------
C REORDER3(from,to,nlfcd,nlfc,nvect)
C 
C FROM(nlfcd,nvect) ->
C	TO(nlfc,nvect)
C ------------------------------------------------------------
      subroutine reorder3(from,to,nlfc2d,nlfc,nvect)
      implicit none
      integer nvect,nlfc,nlfc2d
      real from(nlfc2d,nvect)
      real to(2,nlfc,nvect)
      integer i,j
      
      do j = 1,nvect
        do i = 1,nlfc
          to(1,i,j) = from(2*i-1,j)
          to(2,i,j) = from(2*i  ,j)
        enddo
      enddo
      
      return
      end
C ------------------------------------------------------------


C ------------------------------------------------------------
C REORDER4(from,to,mxlon,nlon,count)
C 
C FROM(mxlon,count) -> TO(count,nlon)
C ------------------------------------------------------------
      subroutine reorder4(from,to,mxlon,nlon,count)
      implicit none
      integer mxlon,nlon,count
      real from(mxlon,count)
      real to(count,nlon)
      integer i,j
      
      do j = 1,count
        do i = 1,nlon
          to(j,i) = from(i,j)
        enddo
      enddo
c      
      return
      end

C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
      SUBROUTINE ZEROFC(NLMM, NLFC, MXLFC, NLVECT, ORDER, FC)
C
C This routine zeroes the truncated Fourier coefficients, needed before
C calculating the inverse real FFT.
C
C called by: DZPUV, ILTRNS, ITRNPE, RFTLON
C calls: 
C
C---- Implicit None ----------------------------------------------------
C
      IMPLICIT NONE
C
C---- Arguments --------------------------------------------------------
C
C     Input
C
C number of untruncated Fourier coefficients
      INTEGER NLMM
C total number of Fourier coefficients
      INTEGER NLFC
C dimensions of array of Fourier coefficients
      INTEGER MXLFC, NLVECT
C array defining ordering of wavenumbers
      INTEGER ORDER(NLFC)
C
C     Output
C
C array whose truncated Fourier coefficients are to be zeroed
      COMPLEX FC(MXLFC,NLVECT)
C
C---- Local Variables --------------------------------------------------
C
C loop indices
      INTEGER I, J
C
C---- Executable Statements --------------------------------------------
C
C     Zero the tail of the complex coefficient spectrum.
      DO J = 1,NLVECT
        DO I = NLMM+1,NLFC
          FC(ORDER(I),J) = (0.0,0.0)
        ENDDO
      ENDDO
C
      RETURN
      END
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
#endif
