#include "params.h"
      subroutine dyn(irow    ,doslt   ,grt1    ,grq1    ,grz1    ,
     $               grd1    ,grfu1   ,grfv1   ,grut1   ,gruq1   ,
     $               grt2    ,grq2    ,grz2    ,grd2    ,grfu2   ,
     $               grfv2   ,grut2   ,gruq2   )
C-----------------------------------------------------------------------
C
C Combine undifferentiated and longitudinally differentiated Fourier
C coefficient terms for later use in the Gaussian quadrature
C
C Computational note: Index "2*m-1" refers to the real part of the
C complex coefficient, and "2*m" to the imaginary.
C
C The naming convention is as follows:
C  - t, q, d, z refer to temperature, specific humidity, divergence
C     and vorticity
C  - "1" suffix to an array => symmetric component of current latitude pair
C  - "2" suffix to an array => antisymmetric component
C
C---------------------------Code history--------------------------------
C
C Original version:  J. Rosinski
C Standardized:      J. Rosinski, June 1992
C Reviewed:          D. Williamson, B. Boville, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#ifdef PP_PCCM
#include "comparll.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer irow                ! latitude pair index
      logical doslt               ! slt or spectral transport
C
C Input/output arguments
C
#ifdef PP_PCCM
      real grt1(plon,plev),    ! sym. undifferentiated term in t eqn.
     $     grq1(plon,plev),    ! sym. undifferentiated term in q eqn.
     $     grz1(plon,plev),    ! sym. undifferentiated term in z eqn.
     $     grd1(plon,plev),    ! sym. undifferentiated term in d eqn.
     $     grfu1(plon,plev),   ! sym. nonlinear terms in u eqn.
     $     grfv1(plon,plev),   ! sym. nonlinear terms in v eqn.
     $     grut1(plon,plev),   ! sym. lambda derivative term in t eqn.
     $     gruq1(plon,plev)    ! sym. lambda derivative term in q eqn.
      real grt2(plon,plev),    ! antisym. undifferentiated term in t eqn.
     $     grq2(plon,plev),    ! antisym. undifferentiated term in q eqn.
     $     grz2(plon,plev),    ! antisym. undifferentiated term in z eqn.
     $     grd2(plon,plev),    ! antisym. undifferentiated term in d eqn.
     $     grfu2(plon,plev),   ! antisym. nonlinear terms in u eqn.
     $     grfv2(plon,plev),   ! antisym. nonlinear terms in v eqn.
     $     grut2(plon,plev),   ! antisym. lambda derivative term in t eqn.
     $     gruq2(plon,plev)    ! antisym. lambda derivative term in t eqn.
C
C---------------------------Local workspace-----------------------------
C
      real zxm(plon),             ! m*2dt/(a*cos(lat)**2)
     $     zrcsj,                 ! 1./(a*cos(lat)**2)
     $     ztdtrc                 ! 2dt/(a*cos(lat)**2), 1dt/..... at nstep=0
      integer
     $     m,                     ! Fourier index
     $     k                      ! level index
      integer mm		  ! added for parallel Fourier indexing
#else
      real grt1(2*pmmax,plev),    ! sym. undifferentiated term in t eqn.
     $     grq1(2*pmmax,plev),    ! sym. undifferentiated term in q eqn.
     $     grz1(2*pmmax,plev),    ! sym. undifferentiated term in z eqn.
     $     grd1(2*pmmax,plev),    ! sym. undifferentiated term in d eqn.
     $     grfu1(2*pmmax,plev),   ! sym. nonlinear terms in u eqn.
     $     grfv1(2*pmmax,plev),   ! sym. nonlinear terms in v eqn.
     $     grut1(2*pmmax,plev),   ! sym. lambda derivative term in t eqn.
     $     gruq1(2*pmmax,plev)    ! sym. lambda derivative term in q eqn.
      real grt2(2*pmmax,plev),    ! antisym. undifferentiated term in t eqn.
     $     grq2(2*pmmax,plev),    ! antisym. undifferentiated term in q eqn.
     $     grz2(2*pmmax,plev),    ! antisym. undifferentiated term in z eqn.
     $     grd2(2*pmmax,plev),    ! antisym. undifferentiated term in d eqn.
     $     grfu2(2*pmmax,plev),   ! antisym. nonlinear terms in u eqn.
     $     grfv2(2*pmmax,plev),   ! antisym. nonlinear terms in v eqn.
     $     grut2(2*pmmax,plev),   ! antisym. lambda derivative term in t eqn.
     $     gruq2(2*pmmax,plev)    ! antisym. lambda derivative term in t eqn.
C
C---------------------------Local workspace-----------------------------
C
      real zxm(pmmax),            ! m*2dt/(a*cos(lat)**2)
     $     zrcsj,                 ! 1./(a*cos(lat)**2)
     $     ztdtrc                 ! 2dt/(a*cos(lat)**2), 1dt/..... at nstep=0
      integer
     $     m,                     ! Fourier index
     $     k                      ! level index
#endif
C
C-----------------------------------------------------------------------
C
C Set constants
C
      zrcsj = 1./(cs(irow)*rearth)
      if (nstep.eq.0) then
         ztdtrc = dtime*zrcsj
      else
         ztdtrc = twodt*zrcsj
      end if
C
C Combine constants with Fourier wavenumber m
C
#ifdef PP_PCCM
      do mm=m_fwavbeg,m_fwavend
         m = m_fwavdex(mm)
         zxm(m) = ztdtrc*float(m_mtrue(mm))
      end do
#else
      do m=1,pmmax
         zxm(m) = ztdtrc*xm(m)
      end do
#endif
C
C Combine undifferentiated and longitudinal derivative terms for
C later use in Gaussian quadrature
C
      do k=1,plev
#ifdef PP_PCCM
         do mm=m_fwavbeg,m_fwavend
            m = m_fwavdex(mm)
#else
         do m=1,pmmax
#endif
            grt1(2*m-1,k) = grt1(2*m-1,k) + zxm(m)*grut1(2*m,k)
            grt1(2*m,k)   = grt1(2*m,k)   - zxm(m)*grut1(2*m-1,k)
            grd1(2*m-1,k) = grd1(2*m-1,k) - zxm(m)*grfu1(2*m,k)
            grd1(2*m,k)   = grd1(2*m,k)   + zxm(m)*grfu1(2*m-1,k)
            grz1(2*m-1,k) = grz1(2*m-1,k) - zxm(m)*grfv1(2*m,k)
            grz1(2*m,k)   = grz1(2*m,k)   + zxm(m)*grfv1(2*m-1,k)
C
            grt2(2*m-1,k) = grt2(2*m-1,k) + zxm(m)*grut2(2*m,k)
            grt2(2*m,k)   = grt2(2*m,k)   - zxm(m)*grut2(2*m-1,k)
            grd2(2*m-1,k) = grd2(2*m-1,k) - zxm(m)*grfu2(2*m,k)
            grd2(2*m,k)   = grd2(2*m,k)   + zxm(m)*grfu2(2*m-1,k)
            grz2(2*m-1,k) = grz2(2*m-1,k) - zxm(m)*grfv2(2*m,k)
            grz2(2*m,k)   = grz2(2*m,k)   + zxm(m)*grfv2(2*m-1,k)
         end do
      end do
      if (.not.doslt) then
         do k=1,plev
#ifdef PP_PCCM
            do mm=m_fwavbeg,m_fwavend
               m = m_fwavdex(mm)
#else
            do m=1,pmmax
#endif
               grq1(2*m-1,k) = grq1(2*m-1,k) + zxm(m)*gruq1(2*m,k)
               grq1(2*m,k)   = grq1(2*m,k)   - zxm(m)*gruq1(2*m-1,k)
               grq2(2*m-1,k) = grq2(2*m-1,k) + zxm(m)*gruq2(2*m,k)
               grq2(2*m,k)   = grq2(2*m,k)   - zxm(m)*gruq2(2*m-1,k)
            end do
         end do
      end if
      return
      end
