#include "params.h"
      subroutine xform( irow    ,lat     ,tdt     ,ps      ,psm1    ,
     $                  u3      ,u3m1    ,v3      ,v3m1    ,t3      ,
     $                  t3m1    ,q3      ,q3m1    ,etadot  ,psslt   ,
     $                  etamid  ,ztodt   ,vmax    ,vmaxt   ,vcour   ,
     $                  detam   ,grlps1  ,grt1    ,grq1    ,grz1    ,
     $                  grd1    ,grfu1   ,grfv1   ,grut1   ,gruq1   ,
     $                  grvt1   ,grvq1   ,grrh1   ,grlps2  ,grt2    ,
     $                  grq2    ,grz2    ,grd2    ,grfu2   ,grfv2   ,
     $                  grut2   ,gruq2   ,grvt2   ,grvq2   ,grrh2   ,
     $                                    psm2    ,bpstr   ,um2     ,
     $                  vm2     ,tm2     ,qm2     ,qminus  ,vort    ,
     $                  vortm1  ,vorttl  ,div     ,divm1   ,divtl   ,
     $                  dpsl    ,dpsm    ,ts      ,ws      ,sn      ,
     $                  oro     ,rnfp1   ,phis    ,pblht   ,qpert   ,
     $                  tpert   ,duh     ,dvh     ,dth     ,qrs     ,
     $                  qrl     ,flwds   ,fsns    ,plos    ,plol    ,
#ifndef BLOCK_FFT
     $                  tssub   )
#else
     $                  tssub   ,fft_buffer, ip )
#endif
C-----------------------------------------------------------------------
C
C Control adjustment physics, time filter, history tape write-up,
C tendency physics, non-linear dynamical terms, FFT and combine terms
C in preparation for Fourier -> spectral quadrature.
C
C The naming convention is as follows:
C  - prefix gr contains grid point values before FFT and Fourier
C     coefficients after
C  - t, q, d, z and ps refer to temperature, specific humidity,
C     divergence, vorticity and surface pressure
C  - "1" suffix to an array => symmetric component current latitude pair
C  - "2" suffix to an array => antisymmetric component.
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          D. Williamson, J. Hack, August 1992
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "pmgrid.com"
#include "pagrid.com"
#include "pspect.com"
C------------------------------Commons----------------------------------
#include "comcon.com"
C-----------------------------------------------------------------------
#include "comctl.com"
C-----------------------------------------------------------------------
#include "comfft.com"
C-----------------------------------------------------------------------
#include "comhed.com"
C-----------------------------------------------------------------------
#include "comhst.com"
C-----------------------------------------------------------------------
#include "comhyb.com"
C-----------------------------------------------------------------------
#include "comlun.com"
C-----------------------------------------------------------------------
#include "commap.com"
C-----------------------------------------------------------------------
#include "commss.com"
C-----------------------------------------------------------------------
#include "comqfl.com"
C-----------------------------------------------------------------------
#include "comslt.com"
C-----------------------------------------------------------------------
#include "comspe.com"
C-----------------------------------------------------------------------
#include "comtfc.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "comtrcnm.com"
#ifdef INTERVAL_TIMER
#include "timers.com"
#endif
C------------------------------Arguments--------------------------------
C
C Input arguments
C
      integer
     $     irow,                     ! latitude pair index
     $     lat                       ! latitude index for S->N storage
      real tdt,                      ! two delta t (unless nstep<2)
     $     ps(plond),                ! surface pressure (time n)
     $     psm1(plond),              ! surface pressure (time n-1)
     $     u3(plond,plev),           ! u-wind (time n)
     $     u3m1(plond,plev),         ! u-wind (time n-1)
     $     v3(plond,plev),           ! v-wind (time n)
     $     v3m1(plond,plev),         ! v-wind (time n-1)
     $     t3(plond,plev),           ! temperature (time n)
     $     t3m1(plond,plev),         ! temperature (time n-1)
     $     q3(plond,plev,pcnst),     ! constituent conc(time n  :h2o first)
     $     q3m1(plond,plev,pcnst),   ! constituent conc(time n-1:h2o first)
     $     etadot(plon,plevp),       ! vertical motion (3-d used by slt)
     $     psslt(plond),             ! surface pressure (2-d used by slt)
     $     etamid(plev),             ! midpoint values of eta (a+b)
     $     ztodt,                    ! 2*timestep unless nstep = 0
     $     vmax(plev),               ! maximum wind speed squared (m^2/s^2)
     $     vmaxt(plev),              ! maximum truncated wind speed (m^2/s^2)
     $     vcour(plev),              ! maximum Courant number in vert.
     $     detam(plev)               ! intervals between vert.levels
#ifdef PP_PCCM
c parallel fft needs entire width
      real grlps1(plon),          ! sym. undiff. term in lnps eqn.
     $     grlps2(plon),          ! antisym undiff. term in lnps eqn.
     $     grt1(plon,plev),       ! sym. undiff. term in t eqn.
     $     grt2(plon,plev),       ! antisym. undiff. term in t eqn.
     $     grq1(plon,plev),       ! sym. undiff. term in q eqn.
     $     grq2(plon,plev),       ! antisym. undiff. term in q eqn.
     $     grz1(plon,plev),       ! sym. undiff. term in z eqn.
     $     grz2(plon,plev),       ! antisym. undiff. term in z eqn.
     $     grd1(plon,plev),       ! sym. undiff. term in d eqn.
     $     grd2(plon,plev),       ! antisym. undiff. term in d eqn.
     $     grfu1(plon,plev),      ! sym. nonlinear terms in u eqn.
     $     grfu2(plon,plev),      ! antisym. nonlinear terms in u eqn.
     $     grfv1(plon,plev),      ! sym. nonlinear terms in v eqn.
     $     grfv2(plon,plev),      ! antisym. nonlinear terms in v eqn.
     $     grut1(plon,plev),      ! sym. lambda deriv. term in t eqn.
     $     grut2(plon,plev),      ! antisym. lambda deriv. term in t eqn.
     $     gruq1(plon,plev),      ! sym. lambda deriv. term in q eqn.
     $     gruq2(plon,plev),      ! antisym. lambda deriv. term in q eqn.
     $     grvt1(plon,plev),      ! sym. mu derivative term in t eqn.
     $     grvt2(plon,plev),      ! antisym. mu deriv. term in t eqn.
     $     grvq1(plon,plev),      ! sym. mu derivative term in q eqn.
     $     grvq2(plon,plev),      ! antisym. mu deriv. term in q eqn.
     $     grrh1(plon,plev),      ! sym. del**2 term in d eqn.
     $     grrh2(plon,plev)       ! antisym. del**2 term in d eqn.
#else
      real grlps1(2*pmmax),          ! sym. undiff. term in lnps eqn.
     $     grlps2(2*pmmax),          ! antisym undiff. term in lnps eqn.
     $     grt1(2*pmmax,plev),       ! sym. undiff. term in t eqn.
     $     grt2(2*pmmax,plev),       ! antisym. undiff. term in t eqn.
     $     grq1(2*pmmax,plev),       ! sym. undiff. term in q eqn.
     $     grq2(2*pmmax,plev),       ! antisym. undiff. term in q eqn.
     $     grz1(2*pmmax,plev),       ! sym. undiff. term in z eqn.
     $     grz2(2*pmmax,plev),       ! antisym. undiff. term in z eqn.
     $     grd1(2*pmmax,plev),       ! sym. undiff. term in d eqn.
     $     grd2(2*pmmax,plev),       ! antisym. undiff. term in d eqn.
     $     grfu1(2*pmmax,plev),      ! sym. nonlinear terms in u eqn.
     $     grfu2(2*pmmax,plev),      ! antisym. nonlinear terms in u eqn.
     $     grfv1(2*pmmax,plev),      ! sym. nonlinear terms in v eqn.
     $     grfv2(2*pmmax,plev),      ! antisym. nonlinear terms in v eqn.
     $     grut1(2*pmmax,plev),      ! sym. lambda deriv. term in t eqn.
     $     grut2(2*pmmax,plev),      ! antisym. lambda deriv. term in t eqn.
     $     gruq1(2*pmmax,plev),      ! sym. lambda deriv. term in q eqn.
     $     gruq2(2*pmmax,plev),      ! antisym. lambda deriv. term in q eqn.
     $     grvt1(2*pmmax,plev),      ! sym. mu derivative term in t eqn.
     $     grvt2(2*pmmax,plev),      ! antisym. mu deriv. term in t eqn.
     $     grvq1(2*pmmax,plev),      ! sym. mu derivative term in q eqn.
     $     grvq2(2*pmmax,plev),      ! antisym. mu deriv. term in q eqn.
     $     grrh1(2*pmmax,plev),      ! sym. del**2 term in d eqn.
     $     grrh2(2*pmmax,plev)       ! antisym. del**2 term in d eqn.
#endif
      real psm2(plond),              ! surface pressure (time n-2)
     $     bpstr(plond),             ! storage next to vortm1 for ps transfm
     $     um2(plond,plev),          ! u-wind (time n-2)
     $     vm2(plond,plev),          ! v-wind (time n-2)
     $     tm2(plond,plev),          ! temperature (time n-2)
     $     qm2(plond,plev,pcnst),    ! constituent-h2o first (time n-2)
     $     qminus(plond,plev,pcnst), ! saves q being advected in slt code
     $     vort(plond,plev),         ! vorticity (time n)
     $     vortm1(plond,plev),       ! vorticity (time n-1)
     $     vorttl(plond,plev),       ! vorticity: half time-filtered
     $     div(plond,plev),          ! divergence (time n)
     $     divm1(plond,plev),        ! divergence (time n-1)
     $     divtl(plond,plev)         ! divergence: half time-filtered
      real dpsl(plond),              ! longitude derivative of lnps (time n)
     $     dpsm(plond),              ! latitude  derivative of lnps (time n)
     $     ts(plond),                ! surface temperature
     $     ws(plond),                ! surface wetness
     $     sn(plond),                ! equivalent liquid water snow depth
     $     oro(plond),               ! land/ocean/sea ice flag
     $     rnfp1(plond),             ! runoff
     $     phis(plond),              ! surface geopotential
     $     pblht(plond),             ! PBL height
     $     qpert(plond,pcnst)        ! moisture/constituent perturb. (PBL)
      real tpert(plond),             ! temperature perturbation (PBL)
     $     duh(plond,plev),          ! horizontal diffusion of u-momentum
     $     dvh(plond,plev),          ! horizontal diffusion of v-momentum
     $     dth(plond,plev),          ! horizontal temperature diffusion
     $     qrs(plond,plev),          ! shortwave radiative heating rate 
     $     qrl(plond,plev),          ! longwave  radiative heating rate 
     $     flwds(plond),             ! surface longwave down flux
     $     fsns(plond),              ! surface absorbed solar flux
     $     plos(plond,plevp),        ! O3 path length (cm)
     $     plol(plond,plevp),        ! O3 pressure-weighted pathlength(cm)
     $     tssub(plond,plevmx)       ! sub-surface soil temperatures
#ifdef BLOCK_FFT
      real fft_buffer(plond,*)
      integer ip
#endif
C
C---------------------------Local workspace-----------------------------
C
      real cnt(plond),               ! top of convective layer
     $     cnb(plond),               ! bottom of convective layer
     $     qc(plond,plev),           ! q tendency due to total rainout
     $     vqfcst(plond,plev,pcnst), ! q tendency due to vert. adv. (slt)
     $     pmid(plond,plev),         ! pressure at model levels (time n)
     $     pmidm1(plond,plev),       ! pressure at model levels (time n-1)
     $     rpmid(plond,plev),        ! 1./pmid
     $     pmln(plond,plev),         ! alog(pmid)
     $     pint(plond,plevp),        ! pressure at model interfaces (n  )
     $     pintm1(plond,plevp),      ! pressure at model interfaces (n-1)
     $     pdel(plond,plev),         ! pdel(k)   = pint  (k+1)-pint  (k)
     $     pdelm1(plond,plev),       ! pdelm1(k) = pintm1(k+1)-pintm1(k)
     $     rpdel(plond,plev),        ! 1./pdel
     $     piln(plond,plevp)         ! alog(pint)
      real est(plond,plevp),         ! sat vap pressure
     $     qsat(plond,plevp),        ! saturation specific humidity
     $     psl(plond)                ! sea level pressure
#ifdef CRAY
      real work1((plon+1)*pcray)  ! Workspace for fft (Cray)
      real work2(1)
#else
# ifndef BLOCK_FFT
#  if (defined(PP_PCCM))
#   if (defined(PP_FFTX))
c      transpose parallel fft work space
       real work1(plond,7*plev+1)
       real work2(p_glon+2,((7*plev+1)/p_nproc_ln + 1)*2) 
#   else
c      distributed parallel fft work space
       real work1(plond,7*plev+1)
       real work2(1)
#   endif
#  else
       real work1((plon+1)*(7*plev+1))   ! sequential definition
       real work2(1)
#  endif
# endif
#endif
      real pstar(plond),             ! ps + psr (time n)
     $     pstarm(plond),            ! ps + psr (time n-1)
     $     pstarmln(plond),          ! alog(pstarm)
     $     t2(plond,plev),           ! nonlinear term - temperature
     $     engy(plond,plev)          ! kinetic energy
      real xnlin(plndlv*7)           ! non-linear terms (equivalence 
C                                    !  region for following arrays to 
C                                    !  optimize fft performance)
#ifndef BLOCK_FFT
      real xnlin_sv(plndlv*7,plat)  ! save these quantities for all lat (JM)
      common /xxxx/ xnlin_sv
#endif
      real fu(plond,plev),           ! nonlinear term - u momentum eqn.
     $     fv(plond,plev),           ! nonlinear term - v momentum eqn.
     $     ut(plond,plev),           ! (u*T) - heat flux - zonal
     $     vt(plond,plev),           ! (v*T) - heat flux - meridional
     $     drhs(plond,plev),         ! RHS of divergence eqn. (del^2 term)
     $     uq(plond,plev),           ! (u*q) - water flux - zonal
     $     vq(plond,plev)            ! (v*q) - water flux - meridional
#if ( defined SUN )
C
C Stupid Sun compiler won-t allow equivalence for stack variables => make
C them static.
C
      static xnlin,fu,fv,ut,vt,drhs,uq,vq
#endif
C
C The following equivalences are for optimal fft performance
C
      equivalence (fu,xnlin(1)),
     $            (fv,xnlin(1+1*plndlv)),
     $            (ut,xnlin(1+2*plndlv)),
     $            (vt,xnlin(1+3*plndlv)),
     $            (drhs,xnlin(1+4*plndlv)),
     $            (uq,xnlin(1+5*plndlv)),
     $            (vq,xnlin(1+6*plndlv))
C
      real dqfx3(plond,plev,pcnst),  ! q tendency due to mass adjustment
     $     dtcond(plond,plev),       ! dT/dt due to convective adjustment
     $     dqcond(plond,plev,pcnst), ! dq/dt due to convective adjustment
     $     cmfdt(plond,plev),        ! convective temperature tendency
     $     cmfdq(plond,plev),        ! convective moisture tendency
     $     cmfdqr(plond,plev),       ! dq/dt due to convective rainout
     $     cmfmc(plond,plev),        ! convective mass flux--m sub c
     $     cmfsl(plond,plev),        ! liquid water static energy flux
     $     cmflq(plond,plev)         ! total water flux
      real ftem(plond,plev)          ! temporary workspace
      real omga(plond,plev),         ! vertical velocity
     $     precl(plond),             ! large scale stable precip rate
     $     precc(plond),             ! convective precip rate
     $     precsl(plond),            ! snow component large scale precip
     $     precsc(plond)             ! snow component convective precip
C      real utend(plond,plev),        ! du/dt
C     $     vtend(plond,plev),        ! dv/dt
C     $     ttend(plond,plev),        ! dT/dt
C     $     qtend(plond,plev,pcnst),  ! dq/dt
C     $     pstend(plond),            ! d(ps)/dt
      real wind,                     ! u**2 + v**2 (m/s)
     $     lvcour,                   ! local vertical courant number
     $     om2eps,                   ! 1. - 2.*eps for time filter
     $     dtdz                      ! dt/detam(k)
      integer
     $     ioro(plond),              ! nint(oro(i))
     $     i,k,kk,m,                 ! longitude,level,constituent indices
     $     ilen,                     ! length to copy to avoid overrun
     $     isign,                    ! flag indicates transform direction
     $     ntr,                      ! number of transforms to perform
     $     itape,
     $     kfld
      real ta(plond,plev,pcnst),     ! total advection of constituents
     $     ddivdt(plond,plev),       ! temporary workspace
     $     ddpn(plond),              ! complete sum of d*delta p
     $     vpdsn(plond),             ! complete sum V dot grad(ln(ps)) delta b
     $     dqdt(plond,plev)          ! moisture tendency (spectral transport)
      real dpslat(plond,plev),       ! Pressure gradient term 
     $     dpslon(plond,plev)        ! Pressure gradient term 
      real coslat,                   ! cosine(latitude)
     $     rcoslat,                  ! 1./cosine(latitude)
     $     rhypi,                    ! 1./hypi(plevp)
     $     utfac,                    ! Only Byron knows
     $     vtfac                     ! Only Byron knows
      real numomgup(plond,plev),     ! number of times wind directed +z
     $     omegaup(plond,plev),      ! omega during omega < 0.
     $     tvm(plond,plev),          ! virtual temperature 
     $     zm(plond,plev)            ! height of midpoints (above surface) 
C
C------------------------------Externals--------------------------------
C
      external
     $     resetr,  ! reset real vector
     $     copyac,  ! copy data between vectors
     $     omcalc,  ! calc. vertical pressure velocity
     $     aphys,   ! apply adjustment physics
     $     plevs0,  ! define interface/midpoint pressures
     $     writup,  ! history tape write
     $     tphys,   ! drives model tendency physics (not adjustment physics)
     $     grmult,  ! nonlinear dynamics in grid space
     $     dyn,     ! assemble terms for quadrature
     $     zmid,    ! compute level heights (above surface)
     $     virtem,  ! virtual temperature calculation
     $     stats,   ! diagnostic statistics
     $     qneg3,
     $     fft991
C
C-----------------------------------------------------------------------
C
#ifdef BLOCK_FFT
#  ifdef INTERVAL_TIMER
      call interval_timer( 1, TIMER_FwdFFT, 0 )
#  endif
         call copyac( fft_buffer( 1, ip ), xnlin, plndlv*5 )
         ip = ip + 5*plev
         call copyac( fft_buffer( 1, ip ), t3m1,  plndlv )
         ip = ip + plev
         call copyac( fft_buffer( 1, ip ), psm1,  plond )
         ip = ip + 1
         call copyac( fft_buffer( 1, ip ), vortm1,  plndlv )
         ip = ip + plev
         call copyac( fft_buffer( 1, ip ), divm1,   plndlv )
         ip = ip + plev
#  ifdef INTERVAL_TIMER
      call interval_timer( 0, TIMER_FwdFFT, 0 )
#  endif
#else
         call copyac( xnlin_sv(1,lat), xnlin, plndlv*7 )
#endif
#ifndef BLOCK_FFT

#  ifdef INTERVAL_TIMER 
         call interval_timer( 1, TIMER_FwdFFT, 0 )
#  endif
         isign = -1
C
C FFT of undifferentiated terms
C !!!! NOTE CONTIGUITY ASSUMPTIONS !!!!
C First: copy ps to location contiguous to vorticity for transform
C
         call copyac(psm1,bpstr,plon)
C
C t (slt) or t & q (spectral transport)
C
         ntr = 2*plev
         if (doslt) ntr = plev
         call rftlon(t3m1, work1, work2, plond, plon, ntr, isign)
C
C ps,vort,div
C
         ntr = 2*plev + 1
         call rftlon(bpstr, work1, work2, plond, plon, ntr, isign)
C
C
C Copy transformed surface pressure back
C
         call copyac(bpstr,psm1,plon)
C
C FFT of longitudinally and latitudinally differentiated terms,
C (includes u*q,v*q if spectral, excludes if semi-lagrangian)
C
         ntr = 7*plev
         if (doslt) ntr = 5*plev
         call rftlon(xnlin, work1, work2, plond, plon, ntr, isign)
#  ifdef INTERVAL_TIMER
         call interval_timer( 0, TIMER_FwdFFT, 0 )
#  endif
#endif
         if (lat.gt.plat/2) then
C
C First of latitude pair (N. hemisphere), save Fourier coefficients
C to use after second of pair (S. hemisphere) is complete to calculate
C symmetric and antisymmetric components.
C
#ifdef PP_PCCM
            call copyac(psm1,grlps2,plon)
#else
            call copyac(psm1,grlps2,2*pmmax)
#endif
            do k=1,plev
#ifdef PP_PCCM
               do i=1,plon
#else
               do i=1,2*pmmax
#endif
                  grt2(i,k) = t3m1(i,k)
                  grfu2(i,k) = fu(i,k)
                  grfv2(i,k) = fv(i,k)
                  grut2(i,k) = ut(i,k)
                  grvt2(i,k) = vt(i,k)
                  grrh2(i,k) = drhs(i,k)
                  grz2(i,k) = vortm1(i,k)
                  grd2(i,k) = divm1(i,k)
               end do
            end do
            if (.not.doslt) then
               do k=1,plev
#ifdef PP_PCCM
                  do i=1,plon
#else
                  do i=1,2*pmmax
#endif
                     grq2(i,k) = q3m1(i,k,1)
                     gruq2(i,k) = uq(i,k)
                     grvq2(i,k) = vq(i,k)
                  end do
               end do
            end if
         else
C
C Second of latitude pair (S. hemisphere) complete, compute symmetric
C and antisymmetric parts.
C Antisymmetric (suffix = 2) = 0.5*(N-S)
C Symmetric (suffix = 1) = A+S = (0.5*(N-S) + S) = 0.5*(N+S)
C
#ifdef PP_PCCM
            do i=1,plon
#else
            do i=1,2*pmmax
#endif
               grlps2(i) = 0.5*(grlps2(i) - psm1(i))
               grlps1(i) = grlps2(i) + psm1(i)
            end do
            do k=1,plev
#ifdef PP_PCCM
               do i=1,plon
#else
               do i=1,2*pmmax
#endif
                  grt2(i,k) = 0.5*(grt2(i,k) - t3m1(i,k))
                  grz2(i,k) = 0.5*(grz2(i,k) - vortm1(i,k))
                  grd2(i,k) = 0.5*(grd2(i,k) - divm1(i,k))
C
                  grt1(i,k) = grt2(i,k) + t3m1(i,k)
                  grz1(i,k) = grz2(i,k) + vortm1(i,k)
                  grd1(i,k) = grd2(i,k) + divm1(i,k)
C
                  grfu2(i,k) = 0.5*(grfu2(i,k) - fu(i,k))
                  grfv2(i,k) = 0.5*(grfv2(i,k) - fv(i,k))
                  grut2(i,k) = 0.5*(grut2(i,k) - ut(i,k))
                  grvt2(i,k) = 0.5*(grvt2(i,k) - vt(i,k))
                  grrh2(i,k) = 0.5*(grrh2(i,k) - drhs(i,k))
C
                  grfu1(i,k) = grfu2(i,k) + fu(i,k)
                  grfv1(i,k) = grfv2(i,k) + fv(i,k)
                  grut1(i,k) = grut2(i,k) + ut(i,k)
                  grvt1(i,k) = grvt2(i,k) + vt(i,k)
                  grrh1(i,k) = grrh2(i,k) + drhs(i,k)
               end do
            end do
            if (.not.doslt) then
               do k=1,plev
#ifdef PP_PCCM
                  do i=1,plon
#else
                  do i=1,2*pmmax
#endif
                     grq2(i,k) = 0.5*(grq2(i,k) - q3m1(i,k,1))
                     grq1(i,k) = grq2(i,k) + q3m1(i,k,1)
C
                     gruq2(i,k) = 0.5*(gruq2(i,k) - uq(i,k))
                     grvq2(i,k) = 0.5*(grvq2(i,k) - vq(i,k))
C
                     gruq1(i,k) = gruq2(i,k) + uq(i,k)
                     grvq1(i,k) = grvq2(i,k) + vq(i,k)
                  end do
               end do
            end if
C
C Combine undifferentiated and longitudinally differentiated Fourier
C coefficient terms for this (irow=1,plat/2) latitude pair
C for later use in the Gaussian quadrature.
C
            call dyn(irow    ,doslt   ,grt1    ,grq1    ,grz1    ,
     $               grd1    ,grfu1   ,grfv1   ,grut1   ,gruq1   ,
     $               grt2    ,grq2    ,grz2    ,grd2    ,grfu2   ,
     $               grfv2   ,grut2   ,gruq2   )
         end if
c
c -- The section of calls to outfld that was here
c    has been moved into physics.F
C
      return
      end
