#include "params.h"
      subroutine pcoszrs(coszrs)
C-----------------------------------------------------------------------
C compute the cosine of the zenith angle on the high resolution
C surface grid.
C  Based on CCM2 radinp.F
C
C radalb.F:
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, B. Briegleb, August 1992
C
C Modified for FOAM: R. Jacob, Jan 1995
C-----------------------------------------------------------------------
      implicit none
C-----------------------------------------------------------------------
#include "prgrid.com"
C-----------------------------------------------------------------------
#include "comtim.com"
C-----------------------------------------------------------------------
#include "crdcon.com"
C-----------------------------------------------------------------------
#include "pcoupled.com"
C-----------------------------------------------------------------------
C
C
C------------------Output variables----------------------------------
      real coszrs(uwlon,uwlat)   ! Cosine solar zenith angle

C------------------Local variables----------------------------------
      real ulats(uwlat)
      real eccf,                 ! Earth-sun distance factor
     $     theta,                ! Earth orbit seasonal angle in radians
     $     delta,                ! Solar declination angle  in radians
     $     sinc,                 ! Sine   of latitude
     $     phi,
     $     cosc,                 ! Cosine of latitude
     $     sind,                 ! Sine   of declination
     $     cosd,                 ! Cosine of declination
C    additional variables for new eccentricity calculation.
     $     anper,   ! #degrees betw vernal equinox and perihelion
     $     ecc,     ! eccentricity
     $     tiltd,   ! tilt angle(obliquity)
     $     dayp,    ! #days from jan1(modern) to date of perihelion
     $     tiltr,omega,coef1,coef2,coef3,xnu,denom,xcel

      integer i,m,j
C-----------------------------------------------------------------------

C Compute solar distance factor and cosine solar zenith angle using
C day value where a round day (such as 213.0) refers to 0z at
C Greenwich longitude.
C
C   
C DON'T USE formulas from Paltridge, G.W. and C.M.R. Platt 1976: Radiative
C Processes in Meterology and Climatology, Elsevier Scientific
C Publishing Company, New York  p. 57, p. 62,63.
C
C INSTEAD:
C Use general earth orbital formulas(e.g Smart,1977).
C Formulas adapted by R. Gallimore (gallimor@fig.aos.wisc.edu). 
C Key input parameters are anper=#degrees between vernal 
C equinox and perihelion measured counterclockwise to earth's 
C orbit, ecc=eccentricity,
C tiltd= tilt angle(oblquity) of earth to plane of the eccliptic,
C dayp=number  of days from jan1(modern) to date of perihelion
C
#undef CALDEF
#ifdef MODSOL
C MODERN VALUES BELOW
      data anper/77.9610/
      data ecc/0.016724/
      data tiltd/23.4463/
      data dayp/2.00/
# define CALDEF
#endif

#ifdef SOL3KY
C 3000YRBP VALUES BELOW
      data anper/128.9253/
      data ecc/0.017824/
      data tiltd/23.8205/
      data dayp/310.767/
# define CALDEF
#endif
 
#ifdef SOL6KY
C 6000YRBP VALUES BELOW
      data anper/179.1304/
      data ecc/0.018682/
      data tiltd/24.1054/
      data dayp/258.993/
# define CALDEF
#endif
 
#ifdef SOL8KY
C 8000YRBP VALUES BELOW
      data anper/212.2577/
      data ecc/0.019110/
      data tiltd/24.2092/
      data dayp/224.649/
# define CALDEF
#endif

#ifdef SOL11KY
C 11000YRBP VALUES BELOW
      data anper/261.5920/
      data ecc/0.019529/
      data tiltd/24.2012/
      data dayp/174.280/
# define CALDEF
#endif
 
#ifdef SOL21KY
C 21000YRBP VALUES BELOW
      data anper/65.5750/
      data ecc/0.018994/
      data tiltd/22.9490/
      data dayp/14.4850/
# define CALDEF
#endif
 
#ifdef SOL126KY
C 126000YRBP VALUES BELOW
      data anper/248.7659/
      data ecc/0.039710/
      data tiltd/23.9281/
      data dayp/185.037/
# define CALDEF
#endif

#ifndef CALDEF
^^^^^^^^^^^^^^^^^ Solar Calendar not defined.  See params.h ^^^^^^^^^^^^^^^^^^^^
#endif
C
C Compute tiltr,omega, and xnu for solar declination calculation
c and eccentricity calc
      tiltr=2.*pie*tiltd/360.
      omega=360.-anper
      coef1=2.*ecc-1./4.*ecc**3
      coef2=5./4.*ecc**2
      coef3=13./12.*ecc**3
      denom=(1.-ecc**2)**2
      theta=2.*(pie/dayspy)*(calday-dayp)
      xnu=theta+coef1*sin(theta)+coef2*sin(2.*theta)+coef3*sin(3.*theta)

C Compute eccentricity factor (sun-earth distance factor)
C
      eccf=(1.+ecc*cos(xnu))**2/denom

C Solar declination in radians:
C
      xcel=xnu+2.*pie*omega/360.
      delta=asin(sin(tiltr)*sin(xcel))


C Compute local cosine solar zenith angle,
C
      sind = sin(delta)
      cosd = cos(delta)

C compute latitude values
      ulats(1) = -(pie/2.) + ((pie/uwlat)/2.)
      do i=2, uwlat
       ulats(i) = ulats(i-1) + (pie/uwlat)
      enddo
C
C Calday is the calender day for Greenwich, including fraction
C of day; the fraction of the day represents a local time at
C Greenwich; to adjust this to produce a true instantaneous time
C For other longitudes, we must correct for the local time change:
C local time based on the longitude and day of year
C then compute the local cosine solar zenith angle
C
c first longitude isn't centered on Greenwich so adjust with
c an extra 0.5 -RLJ
      do m = 1,2
      do j=plats(m),plate(m)
      sinc = sin(ulats(j))
      cosc = cos(ulats(j))
      do i=plons,plone
         phi = calday + ((float(i-1) + 0.5)/float(uwlon))
         coszrs(i,j) = sinc*sind - cosc*cosd*cos(2.*pie*phi)
      enddo
      enddo
      enddo

      return
      end
