c $Header: /homes/jacob/foam-cvs/pcoupler/pcoupler.F,v 1.3 2002/09/22 16:21:54 jacob Exp $
#include "params.h"
c
c  FOAM coupler to let PCCM interact with other models.
c
c  This program coordinates the coupling between the atmosphere and the
c  surface.  The surface data is provided by the ocean, land and
c  sea ice models.
c
c  This coupling assumes two rectangular grids which
c  cover the whole globe:  a low-resolution atmosphere grid and
c  a high resolution surface grid shared by the ocean, land, and
c  sea ice models.   A third grid, called the overlap grid, is
c  formed by laying the other two on top of one another.
c
c  The FOAM coupling scheme does no interpolation of state variables
c  onto different grids.  Instead, fluxes are computed on the
c  overlap grid.
c
c  Coupling is achieved by recasting any calculation involving
c  both a surface and an atmosphere variable to reference the
c  overlap grid.  Fluxes are then accumulated on the overlap
c  grid and area averaged to either the atmosphere or high
c  resolution surface grid.
c
c  To deal with with the distribution of physical variables by
c  PCCM (each processor handles points from both hemispheres),
c  all arrays are full-global and the loop indices are calculated
C  to only cover the corresponding portion of the high-resolution
c  surface.   Parallel communication is done to fill in values
c  needed from other processors because of the uneven overlap
c  of the two grids.
c  
c  Initial Version of parallel coupler:  R. Jacob, C. Schafer Oct, 1995.
c  Cleanup and frozen for FOAM1.0  :  R.Jacob Dec, 1996
c  Changes for Colocated ocean:  C. Schafer June, 1997
c  Extensive Refactoring:  R. Jacob Jan, 2000
c
c Notes:  Some of the defines are leftover from testing and
c  should now be always on in params.h
c ICE_MODEL--activate sea ice model
c ACTICE-- sea ice participates in freshwater budget.
c SIMPLE_OUTPUT --sorts output into directories.

      subroutine pcoupler()

      implicit none

c  Include some common blocks from the atmosphere model
c------------------------------------------------
#include "pmgrid.com"
c------------------------------------------------
#include "comctl.com"
c------------------------------------------------
#include "pspect.com"
c------------------------------------------------
#include "comtim.com"
c------------------------------------------------
#include "comtsc.com"
c------------------------------------------------
#include "comparll.com"
c------------------------------------------------
#include "pcoupled.com"
c------------------------------------------------

C  Include some new common blocks for the coupler
c------------------------------------------------
#include "povrlp2a.com"
c------------------------------------------------
#include "pcplrstr.com"
c------------------------------------------------
#ifdef FOAM_LPJ
#include "plpj.com"
#endif
c------------------------------------------------
#include "hiscom.com"
c------------------------------------------------
#include "pccmcoup.com"
c------------------------------------------------

c time keeping data
      integer milliclock

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    data on the low resolution atmosphere grid
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  collect the following data from the Atmosphere.
      real tbot(p_glon,p_glat) ! bottom level temp
      real tbot1(p_glon,p_glat) ! next-to-bottom level temp
      real ubot(p_glon,p_glat) ! bottom level u wind
      real vbot(p_glon,p_glat) ! bottom level v wind
      real qbot(p_glon,p_glat) ! bottom level specific humidity
      real srfrad(p_glon,p_glat) ! surface radiative heat flux
      real fsns(p_glon,p_glat) ! surface solar absorbed flux
      real flwds(p_glon,p_glat) ! surface longwave down flux
      real precc(p_glon,p_glat)  ! convective precip (m/s)
      real precl(p_glon,p_glat)  ! large scale precip (m/s)
      real psm1(p_glon,p_glat) ! surface pressure
      real pmidm1(p_glon,p_glat) ! bottom level pressure
      real thbot(p_glon,p_glat) ! bottom level potential temp
      real zbot(p_glon,p_glat) ! bottom level height above surface

C form these atmo res variables for local use
      real tot_prec(p_glon,p_glat)  ! total precip
      real aicecov(p_glon,p_glat)   ! area of sea-ice cover


c  Data to pass back to Atmosphere.
c  NOTE:  some of the data passed back to the atmosphere
c (such as atssub) are no longer used anywhere in the
c Atmosphere model.  They are included for purposes of
c filling the atmosphere history file.
c
      real ashflx(p_glon,p_glat)   ! sensible heat flux
      real alhflx(p_glon,p_glat)   ! latent heat flux
      real aqflx(p_glon,p_glat)    ! surface water flux
      real ataux(p_glon,p_glat)    ! wind stress
      real atauy(p_glon,p_glat)    ! wind stress
      real aws(p_glon,p_glat)	   ! sfc wetness
      real aalbs(p_glon,p_glat)	   ! shortwave direct albedo
      real aalbl(p_glon,p_glat)	   ! longwave direct albedo
      real aalbsd(p_glon,p_glat)   ! shortwave diffuse albedo
      real aalbld(p_glon,p_glat)   ! longwave diffuse albedo
      real afsul(p_glon,p_glat)    ! longwave radiation from surface
      real ats(p_glon,p_glat)      ! interpolated surface temp
      real atssub(p_glon,p_glat,plevmx) ! 4 layer temps
      real asndpth(p_glon,p_glat)  !  snow depth


C  save some of the above in RESTART file.  Declare
C  arrays for holding result of parallel gather.
      real aalbs2(p_glon,p_glat)   ! albedos
      real aalbl2(p_glon,p_glat)	    
      real aalbsd2(p_glon,p_glat)	    
      real aalbld2(p_glon,p_glat)	    
      real afsul2(p_glon,p_glat)    ! full longwave field for restart

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Data on the high-resultion surface/ocean/sea ice grid
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c river model data
      real orunoff(uwlon,uwlat)	    ! runoff into ocean


c local surface variables
      real ts(uwlon,uwlat)      ! unified surface temp field (K)
      real ts2,sst2
      real snwmlt(uwlon,uwlat)	! snow melt amount (m)
      real exws(uwlon,uwlat)    ! excess surface moisture (m)
      real runoff(uwlon,uwlat)  ! runoff (m)
      real ltheat(uwlon,uwlat)  ! latent heat 
      logical snowing(uwlon,uwlat)  ! is it snowing
      real oalbs(uwlon,uwlat)   ! Srf alb for direct rad  0.2-0.7 micron
      real oalbl(uwlon,uwlat)   ! Srf alb for direct rad  0.7-5.0 
      real oalbsd(uwlon,uwlat)  ! Srf alb for diffuse rad 0.2-0.7
      real oalbld(uwlon,uwlat)  ! Srf alb for diffuse rad 0.7-5.0 

#ifdef CSIMICE
c local sea ice variables
      real fwfice(uwlon,uwlat)  ! freshwater flux from sea ice
      real PminE(uwlon,uwlat)   ! Precip minus evap (kg/m2/s)
#endif

C ice albedoes
      real oalbsi(uwlon,uwlat)   ! Srf alb for direct rad  0.2-0.7 micron
      real oalbli(uwlon,uwlat)   ! Srf alb for direct rad  0.7-5.0 
      real oalbsdi(uwlon,uwlat)  ! Srf alb for diffuse rad 0.2-0.7
      real oalbldi(uwlon,uwlat)  ! Srf alb for diffuse rad 0.7-5.0 
      
c ocean albedoes
      real oalbso(uwlon,uwlat)   ! Srf alb for direct rad  0.2-0.7 micron
      real oalblo(uwlon,uwlat)   ! Srf alb for direct rad  0.7-5.0 
      real oalbsdo(uwlon,uwlat)  ! Srf alb for diffuse rad 0.2-0.7
      real oalbldo(uwlon,uwlat)  ! Srf alb for diffuse rad 0.7-5.0 

      real fbt(uwlon,uwlat)	! heat flow between ocean and sea ice
      real coszrs(uwlon,uwlat)  !cosine of the zenith angle
      logical doalb             ! update albedos flag

      logical checkice,firstcall
      save checkice,firstcall

C ocean fluxes need to be accumulated over several atmosphere
C time steps before communication with the ocean.  Declare arrays.
C  these fields may be undefined over land
      real oheat(uwlon,uwlat)   ! currnet total accumulated heat flux into ocean
      real otaux(uwlon,uwlat)   ! accumulated wind stress over ocean
      real otauy(uwlon,uwlat)   ! accumulated wind stress

C  also in pcplrstr common block:  oqflx  ! accumulated moisture flux
      save otaux,otauy,oheat

c The only interpolated atmospheric variables
      real oprec(uwlon,uwlat)   ! Atmos. precip fields on the ocean grid
      real ofsns(uwlon,uwlat)   ! Atmos. absorbed solar flux
      real oflwds(uwlon,uwlat)  ! surface longwave down flux
      real osrfrad(uwlon,uwlat) ! Atmos. downward radiation flux
      real otbot(uwlon,uwlat)   ! temperature of bottom atmo layer
      real otbot1(uwlon,uwlat)  ! temperature of next atmo layer

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C data on the OVERLAP grid.
C These quantities must be found on the overlap grid for
C  the FOAM coupling scheme
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C  ocean-atmosphere fluxes
      real qflxo(uwlon,uwlat,maxoverlaps)	! latent heat flux
      real lhfo(uwlon,uwlat,maxoverlaps)	! latent heat flux
      real shfo(uwlon,uwlat,maxoverlaps)	! sensible heat flux
      real tauxo(uwlon,uwlat,maxoverlaps)
      real tauyo(uwlon,uwlat,maxoverlaps)
      real ofnt(uwlon,uwlat)
      real ofntemp,tmpqflxo


C  ice-atmosphere fluxes
      real qflxi(uwlon,uwlat,maxoverlaps)	! latent heat flux
      real lhfi(uwlon,uwlat,maxoverlaps)	! latent heat flux
      real shfi(uwlon,uwlat,maxoverlaps)	! sensible heat flux
      real tauxi(uwlon,uwlat,maxoverlaps)
      real tauyi(uwlon,uwlat,maxoverlaps)
#ifdef CSIMICE
      real ifnt(uwlon,uwlat)
      real dfntdti(uwlon,uwlat,maxoverlaps)
      real ifntdt(uwlon,uwlat)
      real ifntemp,tmpqflxi
#endif

C unified surface-atmosphere fluxes.
      real qflx(uwlon,uwlat,maxoverlaps)	! water flux
      real lhflx(uwlon,uwlat,maxoverlaps)	! latent heat flux
      real shflx(uwlon,uwlat,maxoverlaps)	! sensible heat flux
      real taux(uwlon,uwlat,maxoverlaps)        ! x wind-stress
      real tauy(uwlon,uwlat,maxoverlaps)        ! y wind-stress

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  other variables      
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C  physical variables
      real odelt	!  ocean model timestep
      real tratio       !  ratio of atmosphere/ocen model timestep
      real rhoh2o 	! density of water

C For parallel communication
c define number of high-res variables passed during boundary swap
#define varcnt 33
      integer llon,colm
      integer wordz
      logical zero_out
      real passlat((plon+1)*2,NTOCOUP)
      real recvlat((plon+1)*2,NTOCOUP)

      real passulat((puwlon+2)*2,varcnt)
      real recvulat((puwlon+2)*2,varcnt)
      real passulon(puwlat*2,varcnt)
      real recvulon(puwlat*2,varcnt)

      real passlon(plat,NTOCOUP)
      real recvlon(plat,NTOCOUP)

C For communication with ocean
C  Gather ocean fluxes from all nodes into these arrays
C
      real oheat2(uwlon,uwlat),oqflx2(uwlon,uwlat)
      real otaux2(uwlon,uwlat),otauy2(uwlon,uwlat)
c arrays of info for communication with ocean model
      real to_ocn(uwlat,uwlon,4)
      real from_ocn(uwlat,uwlon,2)

C For history file writing
      integer stephisfreq
      character hisname*22
#ifdef FOAM_LPJ
      character hislpj*22
#endif
      integer ioro2(uwlon,uwlat)
      real tssub2(uwlon,uwlat,4),sndpth2(uwlon,uwlat)
      real thickn2(uwlon,uwlat),ws2(uwlon,uwlat)
      real fice2(uwlon,uwlat),tsnow2(uwlon,uwlat)
      real tice2(uwlon,uwlat,2),tinter2(uwlon,uwlat)
      real volr2(uwlon,uwlat),oprec2(uwlon,uwlat)
      real shf2(uwlon,uwlat),lhf2(uwlon,uwlat)
      real evp2(uwlon,uwlat),rnf2(uwlon,uwlat)
      real ornf2(uwlon,uwlat),olw2(uwlon,uwlat)
      real snm2(uwlon,uwlat),orad2(uwlon,uwlat)
      real meltp2(uwlon,uwlat)
#ifdef FOAM_LPJ
      real laet2(uwlon,uwlat),legrd2(uwlon,uwlat)
      real dphen2(uwlon,uwlat,npft),fpcgrid2(uwlon,uwlat,npft)
      real disturb2(uwlon,uwlat,npft),anpp2(uwlon,uwlat,npft)
      real litag2(uwlon,uwlat,npft),litbg2(uwlon,uwlat,npft)
      real lmind2(uwlon,uwlat,npft),smind2(uwlon,uwlat,npft)
      real rmind2(uwlon,uwlat,npft),hmind2(uwlon,uwlat,npft)
      real nind2(uwlon,uwlat,npft),height2(uwlon,uwlat,npft)
      real fsns2(uwlon,uwlat)
#endif

C For Restart file writing
      integer steprsfreq
      character resname*21
      real oqflx3(uwlon,uwlat)
      integer ioro3(uwlon,uwlat)
      integer oioro3(uwlon,uwlat)
      real tssub3(uwlon,uwlat,4),sndpth3(uwlon,uwlat)
      real thickn3(uwlon,uwlat),ws3(uwlon,uwlat)
      real lwuphf2(uwlon,uwlat)
      real lwuphfi3(uwlon,uwlat),lwuphfo3(uwlon,uwlat)
      real meltp3(uwlon,uwlat),fice3(uwlon,uwlat)
      real volr3(uwlon,uwlat),oprec3(uwlon,uwlat)
      real sst3(uwlon,uwlat),delts3(uwlon,uwlat)
      real fluxout2(uwlon,uwlat),sfluxin2(uwlon,uwlat)
      real oro2(uwlon,uwlat),tsnow3(uwlon,uwlat)
      real qstor3(uwlon,uwlat)

#ifdef FOAM_LPJ
c  for lpj restart file writing
      character reslpj*21
      real aet3(uwlon,uwlat,npft),aeq3(uwlon,uwlat,npft)
      real suph3(uwlon,uwlat,npft),gp3(uwlon,uwlat,npft)
      real dphen3(uwlon,uwlat,npft)
      real fpcgrid3(uwlon,uwlat,npft),aetd3(uwlon,uwlat,npft)
      real aeqd3(uwlon,uwlat,npft),gdd3(uwlon,uwlat,npft)
      real aphen3(uwlon,uwlat,npft),litag3(uwlon,uwlat,npft)
      real litbg3(uwlon,uwlat,npft),height3(uwlon,uwlat,npft)
      real lmind3(uwlon,uwlat,npft),smind3(uwlon,uwlat,npft)
      real rmind3(uwlon,uwlat,npft),hmind3(uwlon,uwlat,npft)
      real anpp3(uwlon,uwlat,npft),nind3(uwlon,uwlat,npft)
      real mgpp3(uwlon,uwlat,npft),lrespm3(uwlon,uwlat,npft)
      real srespm3(uwlon,uwlat,npft),rrespm3(uwlon,uwlat,npft)
      real gddtw3(uwlon,uwlat,npft),awscal3(uwlon,uwlat,npft)
      integer alfd3(uwlon,uwlat,npft),ipr3(uwlon,uwlat,npft)
      integer ifr3(uwlon,uwlat,npft)
      real fb3(uwlon,uwlat),wu3(uwlon,uwlat),wl3(uwlon,uwlat)
      real laet3(uwlon,uwlat),kfast3(uwlon,uwlat),kslow3(uwlon,uwlat)
      real litdcm3(uwlon,uwlat),cpfast3(uwlon,uwlat)
      real cpslow3(uwlon,uwlat),apr3(uwlon,uwlat),gddclim3(uwlon,uwlat)
      real tsm3(uwlon,uwlat,12),wum3(uwlon,uwlat,12)
      real tsmmin3(uwlon,uwlat,20)
#endif

C  for generating filenames
      integer stfnum
      character itf*8

C  misc temporary variables
      integer i,j,k,l,m,n,ic,jc,kc,iw,jw
      real radtmp,prectmp,tmp,fsntmp,flwtmp
      real otbottmp, otbot1tmp
      real tmpqflx,tmpshflx,tmplhflx
      real tmptaux,tmptauy
#ifdef FOAM_LPJ
      integer pft
#endif

C diagnostics
      real maxprec,maxatmo(17),minatmo(17)
      real maxocean(12)
      real minocean(12)

c   Externals
      external priver,pradalb,pflxoce,
     $ ccm2lnd_drv

c Note:  these arrays were used in old method which
C had coupler on seperate nodes.  Data between atmosphere
C and ocean now passed by common blocks
c arrays of info for communication with atmosphere model
c
c     real to_atmos(p_glat,p_glon,NFROMCOUP)
c     real from_atmos(p_glat,p_glon,NTOCOUP)

      data checkice /.TRUE./
      data firstcall /.TRUE./


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C   Begin pcoupler code
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c  Set constants
      odelt = 21600.   ! Ocean model timestep
      tratio = odelt/dtime
      rhoh2o = 1.e3
      iw=0
      jw=0

      stephisfreq = hisfreq*TSPDAY
      steprsfreq = rsfreq*TSPDAY


#ifdef WORD4
      wordz=4
#else
      wordz=8
#endif

c number of columns of ocean grid data to exchange
      colm =2
     
c     zero out some fields
      do j=1,p_glat
      do i=1,p_glon
	  afsul(i,j) = 0.
	  aalbs(i,j) = 0.
	  aalbl(i,j) = 0.
	  aalbsd(i,j) = 0.
	  aalbld(i,j) = 0.
          ubot(i,j)=0.
          vbot(i,j)=0.
	  fsns(i,j)=0.
	  flwds(i,j)=0.
	  srfrad(i,j)=0.
      enddo
      enddo

      doalb = .FALSE.
c 
c see if its time to do an albedo update.  Note that this
c is done one timestep before a radiation calculation.
c
      if(nstep.eq.0 .or. iradsw.eq.1 .or.
     $ (mod(nstep,iradsw).eq.0 .and. nstep.ne.1)) then
       doalb = .TRUE.
      endif

      if(firstcall) then


      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
        icesteps(i,j)=0
        oheat(i,j)=0.
	oqflx(i,j)=0.
       enddo
       enddo
      enddo

      firstcall=.FALSE.

      endif

      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
	snwmlt(i,j)=0.
	exws(i,j)=0.
	fbt(i,j)=0.
	oprec(i,j) = 0.
	ofsns(i,j) = 0.
	oflwds(i,j) = 0.
        runoff(i,j) = 0.
	orunoff(i,j) = 0.
	osrfrad(i,j) = 0.
#ifdef CSIMICE
	PminE(i,j) = 0.
 	fwfice(i,j)=0.
#endif
        if(doalb) then
	 oalbs(i,j) = 0.
	 oalbl(i,j) = 0.
	 oalbsd(i,j) = 0.
	 oalbld(i,j) = 0.
	 oalbsi(i,j) = 0.
	 oalbli(i,j) = 0.
	 oalbsdi(i,j) = 0.
	 oalbldi(i,j) = 0.
	 oalbso(i,j) = 0.
	 oalblo(i,j) = 0.
	 oalbsdo(i,j) = 0.
	 oalbldo(i,j) = 0.
        endif
      enddo
      enddo
      enddo

c form the unified surface temp field
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
         ts(i,j) = tssub(i,j,1)
       enddo
      enddo
      enddo

c  calculate the cosine of the zenith angle
      call pcoszrs(coszrs)

#ifdef PROFILE
      call mark_event(19,1)
#endif


#ifdef PROFILE
      call mark_event(20,1)
#endif

c
c
c  wait for values from atmosphere
C NOTE:  This code no longer used.  Values from
C Atmosphere are passed in by common block. -RLJ
C
c       call receive_from_atmosphere(p_glat,p_glon,p_nproc_lt,
c    $             p_nproc_ln,NTOCOUP,from_atmos,100)
c     call bcast0(from_atmos,p_glat*p_glon*4*NTOCOUP,,0)


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Communication with atmosphere
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C 
c Copy the to_coupler array into the proper place of the full 
c arrays
      jc = 1
      do m =1,2
      do j=ccmplats(m),ccmplate(m)
	ic = 1
        do i=ccmplons,ccmplone
          tbot(i,j)=to_coupler(ic,jc,1)
          tbot1(i,j)=to_coupler(ic,jc,2)
          psm1(i,j)=to_coupler(ic,jc,3)
          pmidm1(i,j)=to_coupler(ic,jc,4)
          ubot(i,j)=to_coupler(ic,jc,5)
          vbot(i,j)=to_coupler(ic,jc,6)
          qbot(i,j)=to_coupler(ic,jc,7)
          thbot(i,j)=to_coupler(ic,jc,8)
          zbot(i,j)=to_coupler(ic,jc,9)
          flwds(i,j)=to_coupler(ic,jc,10)
          tot_prec(i,j)=to_coupler(ic,jc,11)
          precc(i,j)=to_coupler(ic,jc,12)
          fsns(i,j)=to_coupler(ic,jc,13)
	  ic = ic + 1
        enddo
	jc = jc + 1
      enddo
      enddo

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  PARALLEL  Comm.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Make sure every surface point considered on this processor
C will know the values of atmospheric variables above it by
C doing some data exchange.  Use a catch-all send/recieve buffer.
C
C Because of constraint that the surface and atmosphere are
C decomposed on the same number of processors in roughly
C the same sized pieces, the overlap will only be on
C one side.  So only pass in one direction.
ccccccccccccccccccccccccccccccccccccccc
c  East West passing of Atmo. arrays
ccccccccccccccccccccccccccccccccccccccc
c everybody load the leftmost longitude into array for passing
      do j=1,NTOCOUP
      do i=1,plat
      passlon(i,j)=to_coupler(1,i,j)
      enddo
      enddo
      
#ifdef DAY_TIMER
      if (mod(nstep,TSPDAY) .eq. 0) then
        call barrier0()
        write(0,*) '***',nstep,' 1 ','2',milliclock()
      endif
#endif
 
      if (mod(m_pin_ln,2) .eq. 0)then
       call send0(passlon,plat*NTOCOUP*wordz,500,m_pin_lft)
       call recv0(recvlon,plat*NTOCOUP*wordz,500)
      else
       call recv0(recvlon,plat*NTOCOUP*wordz,500)
       call send0(passlon,plat*NTOCOUP*wordz,500,m_pin_lft)
      endif

      jc = 1
      do m =1,2
      do j=ccmplats(m),ccmplate(m)
	  llon = ccmplone + 1
	  if(llon .gt. p_glon) llon=llon-p_glon

          tbot(llon,j)=recvlon(jc,1)
          tbot1(llon,j)=recvlon(jc,2)
          psm1(llon,j)=recvlon(jc,3)
          pmidm1(llon,j)=recvlon(jc,4)
          ubot(llon,j)=recvlon(jc,5)
          vbot(llon,j)=recvlon(jc,6)
          qbot(llon,j)=recvlon(jc,7)
          thbot(llon,j)=recvlon(jc,8)
          zbot(llon,j)=recvlon(jc,9)
          flwds(llon,j)=recvlon(jc,10)
          tot_prec(llon,j)=recvlon(jc,11)
          precc(llon,j)=recvlon(jc,12)
          fsns(llon,j)=recvlon(jc,13)
	  jc = jc + 1
      enddo
      enddo

ccccccccccccccccccccccccccccccccccccccc
c  North South passing of Atmos arrays
ccccccccccccccccccccccccccccccccccccccc
c everybody load the innermost latitudes into array for passing
      do j=1,NTOCOUP

      do i=1,plon
      passlat(i,j)=to_coupler(i,plat/2,j)
      passlat(i+plon+1,j)=to_coupler(i,(plat/2)+1,j)
      enddo
c load up the corners
      passlat(plon+1,j)= recvlon(plat/2,j)
      passlat((2*plon)+2,j)= recvlon((plat/2)+1,j)

      enddo

c this send is only one-way

c polar row only has to send
      if(m_pin_lt .eq. 0) then
       call send0(passlat,((plon*2)+2)*NTOCOUP*wordz,400,m_pin_up)
      endif
c equator row only has to receive
      if( m_pin_lt .eq. (p_nproc_lt-1)) then
       call recv0(recvlat,((plon*2)+2)*NTOCOUP*wordz,400)
      endif

c pair up the send and receives for the middle rows
      if( (m_pin_lt .ne. 0) .and. (m_pin_lt .ne. (p_nproc_lt-1) )) then
      if (mod(m_pin_lt,2) .eq. 0)then
       call send0(passlat,((plon*2)+2)*NTOCOUP*wordz,400,m_pin_up)
       call recv0(recvlat,((plon*2)+2)*NTOCOUP*wordz,400)
      else
       call recv0(recvlat,((plon*2)+2)*NTOCOUP*wordz,400)
       call send0(passlat,((plon*2)+2)*NTOCOUP*wordz,400,m_pin_up)
      endif
      endif

c everyone but the polar row unpack the lat info
      if(m_pin_lt .ne. 0) then
      ic=1
      do i=ccmplons,ccmplone+1
	 llon= i
	 if(llon .gt. p_glon) llon=llon-p_glon

         tbot(llon,ccmplats(1)-1)=recvlat(ic,1)
         tbot(llon,ccmplate(2)+1)=recvlat(plon+ic+1,1)
         tbot1(llon,ccmplats(1)-1)=recvlat(ic,2)
         tbot1(llon,ccmplate(2)+1)=recvlat(plon+ic+1,2)
         psm1(llon,ccmplats(1)-1)=recvlat(ic,3)
         psm1(llon,ccmplate(2)+1)=recvlat(plon+ic+1,3)
         pmidm1(llon,ccmplats(1)-1)=recvlat(ic,4)
         pmidm1(llon,ccmplate(2)+1)=recvlat(plon+ic+1,4)
         ubot(llon,ccmplats(1)-1)=recvlat(ic,5)
         ubot(llon,ccmplate(2)+1)=recvlat(plon+ic+1,5)
         vbot(llon,ccmplats(1)-1)=recvlat(ic,6)
         vbot(llon,ccmplate(2)+1)=recvlat(plon+ic+1,6)
         qbot(llon,ccmplats(1)-1)=recvlat(ic,7)
         qbot(llon,ccmplate(2)+1)=recvlat(plon+ic+1,7)
         thbot(llon,ccmplats(1)-1)=recvlat(ic,8)
         thbot(llon,ccmplate(2)+1)=recvlat(plon+ic+1,8)
         zbot(llon,ccmplats(1)-1)=recvlat(ic,9)
         zbot(llon,ccmplate(2)+1)=recvlat(plon+ic+1,9)
         flwds(llon,ccmplats(1)-1)=recvlat(ic,10)
         flwds(llon,ccmplate(2)+1)=recvlat(plon+ic+1,10)
         tot_prec(llon,ccmplats(1)-1)=recvlat(ic,11)
         tot_prec(llon,ccmplate(2)+1)=recvlat(plon+ic+1,11)
         precc(llon,ccmplats(1)-1)=recvlat(ic,12)
         precc(llon,ccmplate(2)+1)=recvlat(plon+ic+1,12)
         fsns(llon,ccmplats(1)-1)=recvlat(ic,13)
         fsns(llon,ccmplate(2)+1)=recvlat(plon+ic+1,13)
	 ic = ic + 1
      enddo
      endif

c
c  build the srfrad field as done in tphys.F
      do j=1,p_glat
      do i=1,p_glon
	srfrad(i,j)=fsns(i,j) + flwds(i,j)
      enddo
      enddo

#ifdef PROFILE
      call mark_event(20,0)
#endif

c  transform precip field and srfrad to fine grid
c
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
	  radtmp = 0.
	  fsntmp = 0.
	  flwtmp = 0.
	  prectmp = 0.
	  otbottmp = 0.
	  otbot1tmp = 0.
c
c if this ocean cell isn't split, its easy
	  if(overlaps(j,i) .eq. 1) then
	    l = ctoulon(j,i,1)
	    m = ctoulat(j,i,1)
	    oprec(i,j) = tot_prec(l,m)
	    ofsns(i,j) = fsns(l,m)
	    oflwds(i,j) = flwds(l,m)
	    osrfrad(i,j) =  srfrad(l,m)
	    otbot(i,j) = tbot(l,m)
	    otbot1(i,j) = tbot1(l,m)

	  else
c do an area-weighted add of the prec,srfrad 
	    do k=1,overlaps(j,i)
	     l = ctoulon(j,i,k)
	     m = ctoulat(j,i,k)
	     prectmp = prectmp +
     $       tot_prec(l,m)*cweight(j,i,k)

	     fsntmp = fsntmp +
     $       fsns(l,m)*cweight(j,i,k)

	     flwtmp = flwtmp +
     $       flwds(l,m)*cweight(j,i,k)

	     radtmp = radtmp +
     $       srfrad(l,m)*cweight(j,i,k)

             otbottmp = otbottmp + 
     $	     tbot(l,m)*cweight(j,i,k)

             otbot1tmp = otbot1tmp + 
     $	     tbot1(l,m)*cweight(j,i,k)
	    enddo
	    oprec(i,j) = prectmp/uwarea(j)
	    ofsns(i,j) = fsntmp/uwarea(j)
	    oflwds(i,j) = flwtmp/uwarea(j)
	    osrfrad(i,j) = radtmp/uwarea(j)
	    otbot(i,j) = otbottmp/uwarea(j)
	    otbot1(i,j) = otbot1tmp/uwarea(j)

          endif

       enddo
      enddo
      enddo
c
c  set snow flag.  It snows if surface temp and bottom two 
c  atmosphere temps are below freezing
c
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
         snowing(i,j) = (ts(i,j).lt.tmelt) .and. (otbot(i,j).lt.tmelt)
     $                  .and. (otbot1(i,j).lt.tmelt)
       enddo
      enddo
      enddo

c
c set latent heat for evaporation
c
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
	   ltheat(i,j) = latvap
       enddo
      enddo
      enddo
        
#ifdef FOAM_LPJ
CCCCCCCCCCCCCCCCCCCCCCCC
C  Call the CCM2 land model modified for LPJ
CCCCCCCCCCCCCCCCCCCCC
      call ccm2lndlpj_drv(
c  atmosphere input variables
     $ psm1  ,pmidm1, ubot, vbot, tbot, qbot,thbot,
     $ zbot, srfrad,
c  surface input
     $ snowing,ltheat,oprec,doalb,coszrs,
c surface input for LPJ
     $ osrfrad,oflwds,
c surface input/output
     $ ts,
c hydrology output
     $ snwmlt,exws,
c surface output
     $ fbt,oalbs,oalbl,oalbsd,oalbld,
C overlap grid output
     $ qflx,lhflx,shflx,taux,tauy)
C also references the following pcplrstr.com variables
C  uses: sst, lwuphf,ioro
C  modifies: tssub,sndpth,ws,delts

#else

CCCCCCCCCCCCCCCCCCCCCCCC
C  Call the CCM2 land model
CCCCCCCCCCCCCCCCCCCCC

      call ccm2lnd_drv(
c  atmosphere input variables
     $ psm1  ,pmidm1, ubot, vbot, tbot, qbot,thbot,
     $ zbot, srfrad,
c  surface input
     $ snowing,ltheat,oprec,doalb,coszrs,
c surface input/output
     $ ts,
c hydrology output
     $ snwmlt,exws,
c surface output
     $ fbt,oalbs,oalbl,oalbsd,oalbld,
C overlap grid output
     $ qflx,lhflx,shflx,taux,tauy)
C also references the following pcplrstr.com variables
C  uses: sst, lwuphf,ioro
C  modifies: tssub,sndpth,ws,delts

#endif


C  Calculate ocean-atmosphere sensible and latent heat fluxes
C  and wind stress.
      call pflxoce(pmidm1, ubot, vbot,
     $             tbot,   qbot, thbot, zbot,
     $             shfo,lhfo,tauxo,tauyo)



#ifdef CSIMICE 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC    CSM Sea Ice model CSIM 2.2.6
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


c find the ice-atmosphere fluxes
      call pflxsice(pmidm1, ubot, vbot,
     $             tbot,   qbot, thbot, zbot,ts,
     $             shfi,lhfi,tauxi,tauyi,dfntdti)

c form the sea ice heat forcing field
      do n=1,2
      do j=plats(n),plate(n)
      do i=plons,plone

         if(uwmask(i,j).eq.0) then

          ifnt(i,j) = 0.
          ifntdt(i,j) = 0.
          ifntemp = 0.
          tmpqflxo = 0.
          tmpqflxi = 0.
          do k=1,overlaps(j,i)
            qflxi(i,j,k) = lhfi(i,j,k)/(latvap+latice)
            qflxo(i,j,k) = lhfo(i,j,k)/latvap

            tmpqflxi = tmpqflxi + qflxi(i,j,k)*cweight(j,i,k)
            tmpqflxo = tmpqflxo + qflxo(i,j,k)*cweight(j,i,k)

            ifntemp= shfi(i,j,k)+lhfi(i,j,k)+lwuphfi(i,j)
     $         -oflwds(i,j)

            ifnt(i,j) = ifnt(i,j) + ifntemp*cweight(j,i,k) 
            ifntdt(i,j) = ifntdt(i,j) +
     $        dfntdti(i,j,k)*cweight(j,i,k)

          enddo
          
c  form ice heat forcing.
          ifnt(i,j) = ifnt(i,j)/uwarea(j)
          ifntdt(i,j) = ifntdt(i,j)/uwarea(j)

c Because of how moisture flux is handled in CSIM sea ice,
c add P-E forcing to ocean here.  *Before* sea ice call.
c Use current value of fice.  This value is also used to
c evaporate water from ice.  positive flux is adjusted
c with fice for later addition as snow or pass-through precip.
c After csmice_drv, ice melt/formation already has fice in it.
c

          oqflx(i,j) = oqflx(i,j) +  (1.0-fice(i,j))*
c    $    (oprec(i,j)*rhoh2o*1.102 - tmpqflxo/uwarea(j))
     $    (oprec(i,j)*rhoh2o - tmpqflxo/uwarea(j))

          PminE(i,j)= (oprec(i,j)*rhoh2o) - (tmpqflxi/uwarea(j))

#ifdef CSIMICE
	  if((i.eq.iw).and.(j.eq.jw)) then
           debb(1)=sst(i,j)
           debb(2)=ioro(i,j)
           debb(3)=meltp(i,j)
           debb(4)=thickn(i,j)
           debb(5)=fice(i,j)
           debb(6)=ifnt(i,j)
           debb(7)=calday
          endif
#endif
       endif

      enddo
      enddo
      enddo


CCCCCCCCCCCCCCCCCCCCCCCC
C  Call the CSIM 2.2.6 sea ice model
CCCCCCCCCCCCCCCCCCCCC
      call csmice_drv(
c surface input
     $ ifnt,ifntdt,ofsns,coszrs,doalb,
     $ PminE,
c surface input/output
     $ ts,   
c surface output
     $ fbt,oalbsi,oalbli,oalbsdi,oalbldi,
c hydrology output
     $ fwfice,snwmlt)

#endif


#ifdef SIMSICE
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CC    The FOAM 1.0-1.4 4-layer Simple Sea Ice Model
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


CCCCCCCCCCCCCCCCCCCCCCCC
C  Call the sea ice model: uses same formulation as
C  the 4 layer land model so looks the same.
CCCCCCCCCCCCCCCCCCCCC
      call simsice_drv(
c  atmosphere input variables
     $ psm1  ,pmidm1, ubot, vbot, tbot, qbot,thbot,
     $ zbot,srfrad,
c  surface input
     $ snowing,ltheat,oprec,doalb,coszrs,
c surface input/output
     $ ts,checkice,
c hydrology output
     $ snwmlt,exws,
c surface output
     $ fbt,oalbsi,oalbli,oalbsdi,oalbldi,
C overlap grid output
     $ qflxi,lhfi,shfi,tauxi,tauyi)
C also references the following pcplrstr.com variables
C  uses: sst, lwuphf, ioro
C  modifies: tssub,sndpth,ws,delts

#endif
	 

      do n=1,2
      do j=plats(n),plate(n)
      do i=plons,plone

C  add runoff from land model to runoff.
          if(ioro(i,j) .eq. 1) then
             runoff(i,j) = runoff(i,j) + exws(i,j)
	  endif

      if(uwmask(i,j) .eq. 0) then
#ifdef SIMSICE
      tmpqflxo = 0.
#endif
      ofntemp = 0.
      ofnt(i,j) = 0.

          do k=1,overlaps(j,i)

#ifdef SIMSICE
            qflxo(i,j,k) = lhfo(i,j,k)/latvap
	    tmpqflxo = tmpqflxo + qflxo(i,j,k)*cweight(j,i,k)
#endif

            ofntemp = ofsns(i,j)+oflwds(i,j)
     $        -shfo(i,j,k)-lhfo(i,j,k)-lwuphfo(i,j)
	    ofnt(i,j) = ofnt(i,j) + ofntemp*cweight(j,i,k)

          enddo

c find ocean heat forcing and accumulate
c for CSIMICE, fbt already has fice factor
c for SIMSICE, fbt is 0 if fice is 0.
	  ofnt(i,j) = ofnt(i,j)/uwarea(j)
	  oheat(i,j)=oheat(i,j) + (1.0-fice(i,j))*ofnt(i,j) 
     $	      + fbt(i,j)

#ifdef CSIMICE
	if((i.eq.iw).and.(j.eq.jw)) then
	  write(0,*)"HEAT",oheat(i,j),fice(i,j),ofnt(i,j),fbt(i,j)
         debb(8)=fice(i,j)
         debb(9)=thickn(i,j)
         debb(10)=ofnt(i,j)
         debb(11)=fbt(i,j)
         debb(12)=oheat(i,j)
         debb(13)=(1.0-fice(i,j))*ofnt(i,j)+fbt(i,j)
	 debb(39)=ts(i,j)
	 debb(40)=fwfice(i,j)
	 debb(41)=otbot(i,j)
	 debb(42)=qstor(i,j)
	 do k=1,42
	 write(59,*)debb(k)
	 enddo
        endif
#endif


C  finish ocean water forcing.
#ifdef SIMSICE
C  Add precip-evap to ocean moisture flux.  For ocean
C  under sea ice, add any excess moisture or snowdepth
C  that has piled up on the ice as a flux.
         if( ioro(i,j) .eq. 0) then
	  oqflx(i,j) = oqflx(i,j) + oprec(i,j)*rhoh2o 
     $	   -tmpqflxo/uwarea(j)
         endif
         if(ioro(i,j) .eq. 2) then
	  oqflx(i,j) = oqflx(i,j) + (exws(i,j)*rhoh2o/dtime)
	 endif
#endif
#ifdef CSIMICE
C already added p-e to ocean above.  Now add the effects
C of sea ice melting which already include an fice factor
         oqflx(i,j) = oqflx(i,j) + fwfice(i,j)
#endif

      endif
      enddo
      enddo
      enddo


C merge ocean-atmosphere fluxes into unified surface-atmosphere
C  flux fields
      do n=1,2
      do j=plats(n),plate(n)
      do i=plons,plone

      if(uwmask(i,j) .eq. 0) then
       do k=1,overlaps(j,i)
          lhflx(i,j,k) = (1.-fice(i,j))*lhfo(i,j,k) + 
     $                  fice(i,j)*lhfi(i,j,k) 

          qflx(i,j,k)=  (1.-fice(i,j))*qflxo(i,j,k) +
     $                  fice(i,j)*qflxi(i,j,k)

          shflx(i,j,k) = (1.-fice(i,j))*shfo(i,j,k) + 
     $                  fice(i,j)*shfi(i,j,k) 
          taux(i,j,k) = (1.-fice(i,j))*tauxo(i,j,k) + 
     $                  fice(i,j)*tauxi(i,j,k) 
          tauy(i,j,k) = (1.-fice(i,j))*tauyo(i,j,k) + 
     $                  fice(i,j)*tauyi(i,j,k) 

       enddo
      endif


      enddo
      enddo
      enddo


c  find new upward longwave flux.
c  ts will be land surface temp (or snow surface) over land.
c  sst over ocean
c  ice surface temp (or snow surface) over sea ice.
      if(nstep.eq.0 .or. iradlw.eq.1 .or.
     $ (mod(nstep,iradlw).eq.0 .and. nstep.ne.1)) then

      do n=1,2
      do j=plats(n),plate(n)
      do i=plons,plone
        ts2 = ts(i,j) * ts(i,j)
	sst2 =sst(i,j) * sst(i,j)

        if(uwmask(i,j) .eq. 1) then
	  lwuphf(i,j) = stebol*ts2*ts2
        else
	  lwuphfi(i,j) = stebol*ts2*ts2
	  lwuphfo(i,j) = stebol*sst2*sst2
          lwuphf(i,j) = (1.-fice(i,j))*lwuphfo(i,j) +
     $      fice(i,j)*lwuphfi(i,j)
	endif
	 
      enddo
      enddo
      enddo

      endif

c  call the river model; find new river flux
c  into ocean.

#ifdef PROFILE
      call mark_event(17,1)
#endif

#ifdef RIVER
      call priver(runoff,orunoff)
C also references the following pcplrstr.com variables
C modifies:  volr,sfluxin,fluxout

#endif

#ifdef PROFILE
      call mark_event(17,0)
#endif

      if(doalb) then

c Calculate albedoes over entire high-res surface
       call pradalb(coszrs,oalbso,
     $ 	oalblo, oalbsdo, oalbldo)
C also references the following pcplrstr.com variables
C  uses: sndpth,ioro

c merge the ocean and ice albedoes
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
       if(uwmask(i,j) .eq.0) then
       oalbs(i,j)=(1.-fice(i,j))*oalbso(i,j) +
     $       fice(i,j)*oalbsi(i,j)

       oalbl(i,j)=(1.-fice(i,j))*oalblo(i,j) +
     $       fice(i,j)*oalbli(i,j)

       oalbsd(i,j)=(1.-fice(i,j))*oalbsdo(i,j) +
     $       fice(i,j)*oalbsdi(i,j)

       oalbld(i,j)=(1.-fice(i,j))*oalbldo(i,j) +
     $       fice(i,j)*oalbldi(i,j)

       endif
       enddo
      enddo
      enddo



      endif

c
c Transform fluxes from OVERLAP to ocean grid for adding to history.
C  Accumulate ocean fluxes for next communication with ocean
c
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
          tmpshflx = 0.
	  tmplhflx = 0.
	  tmpqflx = 0.
	  tmptaux = 0.
	  tmptauy = 0.

C  add up pieces of overlap grid that form this i,j point
	  do k=1,overlaps(j,i)
            tmpshflx = tmpshflx + shflx(i,j,k)*cweight(j,i,k)
	    tmplhflx = tmplhflx + lhflx(i,j,k)*cweight(j,i,k)
	    tmpqflx = tmpqflx + qflx(i,j,k)*cweight(j,i,k)
	    tmptaux = tmptaux + taux(i,j,k)*cweight(j,i,k)
	    tmptauy = tmptauy + tauy(i,j,k)*cweight(j,i,k)
	  enddo

c  add runoff to history 
          his_rnf(i,j) = his_rnf(i,j) + (runoff(i,j)/stephisfreq)

c add long wave out to history
	  his_olw(i,j) = his_olw(i,j) + (lwuphf(i,j)/stephisfreq)

c add net downward radiation to history
	  his_orad(i,j) = his_orad(i,j) + (osrfrad(i,j)/stephisfreq)

c add sensible heat flux to history
	  his_shf(i,j) = his_shf(i,j) +
     $     ((tmpshflx/uwarea(j))/stephisfreq)

c add latent heat flux to history
	  his_lhf(i,j) = his_lhf(i,j) +
     $     ((tmplhflx/uwarea(j))/stephisfreq)
	 
c dd evaportaion to history
	  his_evp(i,j) = his_evp(i,j) +
     $     ((tmpqflx/(uwarea(j)*rhoh2o))/stephisfreq)

c  add river runoff to ocean freshwater flux
 	  oqflx(i,j) = oqflx(i,j) + orunoff(i,j)*rhoh2o

c  add river runoff to history
          his_ornf(i,j) = his_ornf(i,j) + (orunoff(i,j)/stephisfreq)

c accumulate ocean wind stress
	  otaux(i,j) = otaux(i,j) - tmptaux/uwarea(j)
	  otauy(i,j) = otauy(i,j) - tmptauy/uwarea(j)

       enddo
      enddo
      enddo

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  PARALLEL  Comm.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  To form the atmosphere fluxes from the overlap grid.
C  will need to do some data exchange so that each
C  atmosphere cell handled by this processors knows the
C  values of all overlap grid pieces underneath it.
C
c  Update the the following arrays:
c  shflx,lhflx,qflx,taux,tauy,lwuphf,ioro,ws,ts
c  tssub,sndpth, albedos
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

#ifdef UPSHOT
      call log_event(11,m_pin,'StrtGath',8)
#endif

#ifdef PROFILE
      call mark_event(20,1)
#endif

cccccccccccccccccccccccccccccccccccccc
c  East West passing of UW arrays
cccccccccccccccccccccccccccccccccccccc
c everybody load the rightmost longitude into array
      jc = 1

      do n=1,colm
      do m=1,2
      do i=plats(m),plate(m)

      do k=1,4
      passulon(jc,k) = shflx(plone+1-n,i,k)
      passulon(jc,4+k) = lhflx(plone+1-n,i,k)
      passulon(jc,8+k) = qflx(plone+1-n,i,k)
      passulon(jc,12+k) = taux(plone+1-n,i,k)
      passulon(jc,16+k) = tauy(plone+1-n,i,k)
      enddo
      passulon(jc,21) = lwuphf(plone+1-n,i)
      passulon(jc,22) = ws(plone+1-n,i)
      passulon(jc,23) = ts(plone+1-n,i)
      passulon(jc,24) = tssub(plone+1-n,i,1)
      passulon(jc,25) = tssub(plone+1-n,i,2)
      passulon(jc,26) = tssub(plone+1-n,i,3)
      passulon(jc,27) = tssub(plone+1-n,i,4)
      passulon(jc,28) = sndpth(plone+1-n,i)
      passulon(jc,29) = ioro(plone+1-n,i)
      passulon(jc,30) = oalbs(plone+1-n,i)	
      passulon(jc,31) = oalbl(plone+1-n,i)	
      passulon(jc,32) = oalbsd(plone+1-n,i)	
      passulon(jc,33) = oalbld(plone+1-n,i)	
      jc =jc + 1
      enddo
      enddo
      enddo
      
      if (mod(m_pin_ln,2) .eq. 0)then
       call send0(passulon,colm*puwlat*varcnt*wordz,550,
     $       m_pin_rht)
       call recv0(recvulon,colm*puwlat*varcnt*wordz,550)
      else
       call recv0(recvulon,colm*puwlat*varcnt*wordz,550)
       call send0(passulon,colm*puwlat*varcnt*wordz,550,
     $ 		m_pin_rht)
      endif

      jc = 1
      do n=1,colm
      do m=1,2
      do i=plats(m),plate(m)

      llon=plons-n
      if(llon .le. 0) llon=llon+uwlon

      do k=1,4
      shflx(llon,i,k) = recvulon(jc,k)
      lhflx(llon,i,k) = recvulon(jc,4+k)
      qflx(llon,i,k) = recvulon(jc,8+k)
      taux(llon,i,k) = recvulon(jc,12+k)
      tauy(llon,i,k) = recvulon(jc,16+k)
      enddo

      lwuphf(llon,i) = recvulon(jc,21)
      ws(llon,i) = recvulon(jc,22)
      ts(llon,i) = recvulon(jc,23)
      tssub(llon,i,1) = recvulon(jc,24)
      tssub(llon,i,2) = recvulon(jc,25)
      tssub(llon,i,3) = recvulon(jc,26)
      tssub(llon,i,4) = recvulon(jc,27)
      sndpth(llon,i) = recvulon(jc,28)
      ioro(llon,i) = recvulon(jc,29)
      oalbs(llon,i) = recvulon(jc,30)
      oalbl(llon,i) = recvulon(jc,31)
      oalbsd(llon,i) = recvulon(jc,32)
      oalbld(llon,i) = recvulon(jc,33)
      jc =jc +1
      enddo
      enddo
      enddo

ccccccccccccccccccccccccccccccccccccccc
c  North South passing of UW arrays
ccccccccccccccccccccccccccccccccccccccc
c everybody load the outermost latitudes into array for passing
      ic = 1
      do i=plons-colm,plone
      llon=i
      if(llon .le. 0) llon=llon+uwlon

c mt fixed loop bounds bug
      do k=1,4
c
      passulat(ic,k)=shflx(llon,plats(1),k)
      passulat(ic+puwlon+colm,k)=shflx(llon,plate(2),k)

      passulat(ic,4+k)=lhflx(llon,plats(1),k)
      passulat(ic+puwlon+colm,4+k)=lhflx(llon,plate(2),k)

      passulat(ic,8+k)=qflx(llon,plats(1),k)
      passulat(ic+puwlon+colm,8+k)=qflx(llon,plate(2),k)

      passulat(ic,12+k)=taux(llon,plats(1),k)
      passulat(ic+puwlon+colm,12+k)=taux(llon,plate(2),k)

      passulat(ic,16+k)=tauy(llon,plats(1),k)
      passulat(ic+puwlon+colm,16+k)=tauy(llon,plate(2),k)

      enddo

      passulat(ic,21)=lwuphf(llon,plats(1))
      passulat(ic+puwlon+colm,21)=lwuphf(llon,plate(2))

      passulat(ic,22)=ws(llon,plats(1))
      passulat(ic+puwlon+colm,22)=ws(llon,plate(2))

      passulat(ic,23)=ts(llon,plats(1))
      passulat(ic+puwlon+colm,23)=ts(llon,plate(2))

       do k=1,4
       passulat(ic,23+k)=tssub(llon,plats(1),k)
       passulat(ic+puwlon+colm,23+k)=tssub(llon,plate(2),k)
       enddo

      passulat(ic,28)=sndpth(llon,plats(1))
      passulat(ic+puwlon+colm,28)=sndpth(llon,plate(2))

      passulat(ic,29)=ioro(llon,plats(1))
      passulat(ic+puwlon+colm,29)=ioro(llon,plate(2))
      passulat(ic,30)=oalbs(llon,plats(1))
      passulat(ic+puwlon+colm,30)=oalbs(llon,plate(2))
      passulat(ic,31)=oalbl(llon,plats(1))
      passulat(ic+puwlon+colm,31)=oalbl(llon,plate(2))
      passulat(ic,32)=oalbsd(llon,plats(1))
      passulat(ic+puwlon+colm,32)=oalbsd(llon,plate(2))
      passulat(ic,33)=oalbld(llon,plats(1))
      passulat(ic+puwlon+colm,33)=oalbld(llon,plate(2))
      ic = ic +1
      enddo

c equator row only has to send
      if( m_pin_lt .eq. (p_nproc_lt-1)) then
       call send0(passulat,(puwlon+colm)*2*varcnt*wordz,
     $       650,m_pin_dwn)
      endif
c polar row only has to receive
      if(m_pin_lt .eq. 0) then
       call recv0(recvulat,(puwlon+colm)*2*varcnt*wordz,650)
      endif

c pair up the send and receives for the middle rows
      if( (m_pin_lt .ne. 0) .and. 
     $      (m_pin_lt .ne. (p_nproc_lt-1))) then
      if(mod(m_pin_lt,2) .eq. 0)then
       call send0(passulat,(puwlon+colm)*2*varcnt*wordz,
     $ 650,m_pin_dwn)
       call recv0(recvulat,(puwlon+colm)*2*varcnt*wordz,650)
      else
       call recv0(recvulat,(puwlon+colm)*2*varcnt*wordz,650)
       call send0(passulat,(puwlon+colm)*2*varcnt*wordz,
     $ 650,m_pin_dwn)
      endif
      endif


c if not the equator row, unpack the recv array
      if( m_pin_lt .ne. (p_nproc_lt-1)) then
      ic = 1

      do i=plons-colm,plone

      llon=i
      if(llon .le. 0) llon=llon+uwlon

      do k=1,4
      shflx(llon,plate(1)+1,k)=recvulat(ic,k)
      shflx(llon,plats(2)-1,k)=recvulat(puwlon+ic+colm,k)

      lhflx(llon,plate(1)+1,k)=recvulat(ic,4+k)
      lhflx(llon,plats(2)-1,k)=recvulat(puwlon+ic+colm,4+k)

      qflx(llon,plate(1)+1,k)=recvulat(ic,8+k)
      qflx(llon,plats(2)-1,k)=recvulat(puwlon+ic+colm,8+k)
     
      taux(llon,plate(1)+1,k)=recvulat(ic,12+k)
      taux(llon,plats(2)-1,k)=recvulat(puwlon+ic+colm,12+k)

      tauy(llon,plate(1)+1,k)=recvulat(ic,16+k)
      tauy(llon,plats(2)-1,k)=recvulat(puwlon+ic+colm,16+k)

      enddo
     
 
      lwuphf(llon,plate(1)+1)=recvulat(ic,21)
      lwuphf(llon,plats(2)-1)=recvulat(puwlon+ic+colm,21)
 
      ws(llon,plate(1)+1)=recvulat(ic,22)
      ws(llon,plats(2)-1)=recvulat(puwlon+ic+colm,22)
 
      ts(llon,plate(1)+1)=recvulat(ic,23)
      ts(llon,plats(2)-1)=recvulat(puwlon+ic+colm,23)

       do k=1,4 
       tssub(llon,plate(1)+1,k)=recvulat(ic,23+k)
       tssub(llon,plats(2)-1,k)=recvulat(puwlon+ic+colm,23+k)
       enddo
      
      sndpth(llon,plate(1)+1)=recvulat(ic,28)
      sndpth(llon,plats(2)-1)=recvulat(puwlon+ic+colm,28)
 
      ioro(llon,plate(1)+1)=recvulat(ic,29)
      ioro(llon,plats(2)-1)=recvulat(puwlon+ic+colm,29)
      oalbs(llon,plate(1)+1)=recvulat(ic,30)
      oalbs(llon,plats(2)-1)=recvulat(puwlon+ic+colm,30)
      oalbl(llon,plate(1)+1)=recvulat(ic,31)
      oalbl(llon,plats(2)-1)=recvulat(puwlon+ic+colm,31)
      oalbsd(llon,plate(1)+1)=recvulat(ic,32)
      oalbsd(llon,plats(2)-1)=recvulat(puwlon+ic+colm,32)
      oalbld(llon,plate(1)+1)=recvulat(ic,33)
      oalbld(llon,plats(2)-1)=recvulat(puwlon+ic+colm,33)
      ic = ic +1
      enddo

      endif

#ifdef UPSHOT
      call log_event(12,m_pin,'EndGath',7)
#endif

#ifdef PROFILE
      call mark_event(20,0)
#endif

#ifdef DIAGNOSTICS
c     if(m_pin .eq. 0) then
      do i=1,12
	maxocean(i) = 0.
	minocean(i) = 1e9
      enddo
      do i=1,uwlon
      do j=1,uwlat
          maxocean(1) = amax1(maxocean(1),ts(i,j))
          maxocean(2) = amax1(maxocean(2),ws(i,j))
          maxocean(4) = amax1(maxocean(4),runoff(i,j))
          maxocean(5) = amax1(maxocean(5),sndpth(i,j))

          minocean(1) = amin1(minocean(1),ts(i,j))
          minocean(2) = amin1(minocean(2),ws(i,j))
          minocean(4) = amin1(minocean(4),runoff(i,j))
          minocean(5) = amin1(minocean(5),sndpth(i,j))
      enddo
      enddo
      write(6,*)'ts      = ',minocean(1),maxocean(1)
      write(6,*)'ws      = ',minocean(2),maxocean(2)
      write(6,*)'runoff  = ',minocean(4),maxocean(4)
      write(6,*)'sndpth  = ',minocean(5),maxocean(5)
      endif
#endif

c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c build the flux fields for the atmosphere 
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
#ifdef DIAGNOSTICS
      do k=1,17
	 maxatmo(k) = 0.
	 minatmo(k) = 1e9
      enddo
#endif
      jc = 1
      do kc=1,2
      do j=ccmplats(kc),ccmplate(kc)
	  ic = 1
       do i=ccmplons,ccmplone
          ashflx(i,j) = 0.
	  alhflx(i,j) = 0.
	  aqflx(i,j) = 0.
	  afsul(i,j) = 0.
	  aalbs(i,j) = 0.
	  aalbl(i,j) = 0.
	  aalbsd(i,j) = 0.
	  aalbld(i,j) = 0.
	  ataux(i,j) = 0.
	  atauy(i,j) = 0.
	  aoro(i,j) = 0.
	  ats(i,j) = 0.
	  aws(i,j) = 0.
	  atssub(i,j,1) = 0.
	  atssub(i,j,2) = 0.
	  atssub(i,j,3) = 0.
	  atssub(i,j,4) = 0.
	  asndpth(i,j) = 0.
	  aicecov(i,j) = 0.
	  do k=1,coverlaps(j,i)
	    l = utoclon(j,i,k)
	    m = utoclat(j,i,k)
	    n = utocwch(j,i,k)
	    ashflx(i,j) = ashflx(i,j) + shflx(l,m,n)*cweight(m,l,n)
	    alhflx(i,j) = alhflx(i,j) + lhflx(l,m,n)*cweight(m,l,n)
	    aqflx(i,j) = aqflx(i,j) + qflx(l,m,n)*cweight(m,l,n)
	    afsul(i,j) = afsul(i,j) + lwuphf(l,m)*cweight(m,l,n)
	    ataux(i,j) = ataux(i,j) + taux(l,m,n)*cweight(m,l,n)
	    atauy(i,j) = atauy(i,j) + tauy(l,m,n)*cweight(m,l,n)
#ifdef ICE_MODEL
	    if(ioro(l,m) .eq. 2) then
	     aicecov(i,j) = aicecov(i,j) + cweight(m,l,n)
            endif
#else
	    aoro(i,j) = aoro(i,j) + ioro(l,m)*cweight(m,l,n)
#endif
	    ats(i,j) = ats(i,j) + ts(l,m)*cweight(m,l,n)
	    aws(i,j) = aws(i,j) + ws(l,m)*cweight(m,l,n)
	    if(doalb) then
	    aalbs(i,j) = aalbs(i,j) + oalbs(l,m)*cweight(m,l,n)
	    aalbl(i,j) = aalbl(i,j) + oalbl(l,m)*cweight(m,l,n)
	    aalbsd(i,j) = aalbsd(i,j) + oalbsd(l,m)*cweight(m,l,n)
	    aalbld(i,j) = aalbld(i,j) + oalbld(l,m)*cweight(m,l,n)
	    endif
	    atssub(i,j,1) = atssub(i,j,1) + tssub(l,m,1)*cweight(m,l,n)
	    atssub(i,j,2) = atssub(i,j,2) + tssub(l,m,2)*cweight(m,l,n)
	    atssub(i,j,3) = atssub(i,j,3) + tssub(l,m,3)*cweight(m,l,n)
	    atssub(i,j,4) = atssub(i,j,4) + tssub(l,m,4)*cweight(m,l,n)
	    asndpth(i,j) = asndpth(i,j) + sndpth(l,m)*cweight(m,l,n)
	  enddo
#ifdef ICE_MODEL
c  If there's sea-ice, check for land which we assume is covered
c  with snow, and add it to ice area cover.  This is done to fix
c  problems where a ccm2 grid is half land-half ocean and thus may
c  be counted as open ocean when the ocean part is covered with ice
	  if (aicecov(i,j) .gt. 0.) then
	  do k=1,coverlaps(j,i)
	    l = utoclon(j,i,k)
	    m = utoclat(j,i,k)
	    n = utocwch(j,i,k)
	      if(ioro(l,m) .eq. 1) then
		aicecov(i,j) = aicecov(i,j) + cweight(m,l,n)
	      endif
          enddo
	  endif
#endif

	  ashflx(i,j) = ashflx(i,j)/ccm2area(j)
	  alhflx(i,j) = alhflx(i,j)/ccm2area(j)
	  aqflx(i,j) = aqflx(i,j)/ccm2area(j)

c convert afsul to cgs units
	  afsul(i,j) = (afsul(i,j)/ccm2area(j))*1000.

	  ataux(i,j) = ataux(i,j)/ccm2area(j)
	  atauy(i,j) = atauy(i,j)/ccm2area(j)

 	  aoro(i,j) = ccm2mask(i,j)
#ifdef ICE_MODEL
	  if (ccm2mask(i,j) .eq. 0.) then
	   if((aicecov(i,j)/ccm2area(j)) .gt. 0.5) then
		aoro(i,j) = 2.
	   endif
          endif
#endif

	  ats(i,j) = ats(i,j)/ccm2area(j)

	  aws(i,j) = aws(i,j)/ccm2area(j)
c 	  if (ccm2mask(i,j) .lt. 1.) aws(i,j) = 0.

	  asndpth(i,j) = asndpth(i,j)/ccm2area(j)
c	  if (ccm2mask(i,j) .lt. 1.) asndpth(i,j) = 0.

	  atssub(i,j,1) = atssub(i,j,1)/ccm2area(j)
	  atssub(i,j,2) = atssub(i,j,2)/ccm2area(j)
	  atssub(i,j,3) = atssub(i,j,3)/ccm2area(j)
	  atssub(i,j,4) = atssub(i,j,4)/ccm2area(j)
	    aalbs(i,j) = aalbs(i,j)/ccm2area(j)
	    aalbl(i,j) = aalbl(i,j)/ccm2area(j)
	    aalbsd(i,j) = aalbsd(i,j)/ccm2area(j)
	    aalbld(i,j) = aalbld(i,j)/ccm2area(j)

#ifdef DIAGNOSTICS
	  maxatmo(1) = amax1(maxatmo(1),ats(i,j))
	  maxatmo(2) = amax1(maxatmo(2),atssub(i,j,1))
	  maxatmo(3) = amax1(maxatmo(3),atssub(i,j,2))
	  maxatmo(4) = amax1(maxatmo(4),atssub(i,j,3))
	  maxatmo(5) = amax1(maxatmo(5),atssub(i,j,4))
	  maxatmo(6) = amax1(maxatmo(6),ashflx(i,j))
	  maxatmo(7) = amax1(maxatmo(7),alhflx(i,j))
	  maxatmo(8) = amax1(maxatmo(8),aqflx(i,j))
	  maxatmo(9) = amax1(maxatmo(9),aws(i,j))
	  maxatmo(10) = amax1(maxatmo(10),ataux(i,j))
	  maxatmo(11) = amax1(maxatmo(11),atauy(i,j))
	  maxatmo(15) = amax1(maxatmo(15),aoro(i,j))
	  maxatmo(16) = amax1(maxatmo(16),asndpth(i,j))
	  maxatmo(17) = amax1(maxatmo(17),afsul(i,j))

	  minatmo(1) = amin1(minatmo(1),ats(i,j))
	  minatmo(2) = amin1(minatmo(2),atssub(i,j,1))
	  minatmo(3) = amin1(minatmo(3),atssub(i,j,2))
	  minatmo(4) = amin1(minatmo(4),atssub(i,j,3))
	  minatmo(5) = amin1(minatmo(5),atssub(i,j,4))
	  minatmo(6) = amin1(minatmo(6),ashflx(i,j))
	  minatmo(7) = amin1(minatmo(7),alhflx(i,j))
	  minatmo(8) = amin1(minatmo(8),aqflx(i,j))
	  minatmo(9) = amin1(minatmo(9),aws(i,j))
	  minatmo(10) = amin1(minatmo(10),ataux(i,j))
	  minatmo(11) = amin1(minatmo(11),atauy(i,j))
	  minatmo(15) = amin1(minatmo(15),aoro(i,j))
	  minatmo(16) = amin1(minatmo(16),asndpth(i,j))
	  minatmo(17) = amin1(minatmo(17),afsul(i,j))
#endif


          from_coupler(ic,jc,1)=ats(i,j)
          do k=1,plevmx
            from_coupler(ic,jc,k+1)=atssub(i,j,k)
          enddo
          from_coupler(ic,jc,6)=ashflx(i,j)
          from_coupler(ic,jc,7)=alhflx(i,j)
          from_coupler(ic,jc,8)=aqflx(i,j)
          from_coupler(ic,jc,9)=aws(i,j)
          from_coupler(ic,jc,10)=ataux(i,j)
          from_coupler(ic,jc,11)=atauy(i,j)
          from_coupler(ic,jc,12)=aoro(i,j)
          from_coupler(ic,jc,13)=asndpth(i,j)
          from_coupler(ic,jc,14)=afsul(i,j)
          from_coupler(ic,jc,15)=aalbs(i,j)
          from_coupler(ic,jc,16)=aalbl(i,j)
          from_coupler(ic,jc,17)=aalbsd(i,j)
          from_coupler(ic,jc,18)=aalbld(i,j)
	  ic = ic +1
       enddo
       jc = jc +1
      enddo
      enddo

c     if (m_pin .eq. 0) then
c       call send_to_atmosphere(p_glat,p_glon,p_nproc_lt,p_nproc_ln,
c    $                      17,to_atmos,400)
c     endif

#ifdef DIAGNOSTICS
      write(6,*)'ats       ',minatmo(1),maxatmo(1)
      write(6,*)'atssub1 = ',minatmo(2),maxatmo(2)
      write(6,*)'atssub2 = ',minatmo(3),maxatmo(3)
      write(6,*)'atssub3 = ',minatmo(4),maxatmo(4)
      write(6,*)'atssub4 = ',minatmo(5),maxatmo(5)
      write(6,*)'ashflx =  ',minatmo(6),maxatmo(6)
      write(6,*)'alhflx =  ',minatmo(7),maxatmo(7)
      write(6,*)'aqflx =   ',minatmo(8),maxatmo(8)
      write(6,*)'aws =     ',minatmo(9),maxatmo(9)
      write(6,*)'ataux =   ',minatmo(10),maxatmo(10)
      write(6,*)'atauy =   ',minatmo(11),maxatmo(11)
      write(6,*)'aoro =    ',minatmo(15),maxatmo(15)
      write(6,*)'asndpth = ',minatmo(16),maxatmo(16)
      write(6,*)'afsul =   ',minatmo(17),maxatmo(17)
#endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  end of calculation of atmosphere flux fields
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c   ACCUMULATE into HISTORY buffers
      do n=1,2
      do j=plats(n),plate(n)
      do i=plons,plone
       do k=1,4
	his_tssub(i,j,k)= his_tssub(i,j,k)+(tssub(i,j,k)/stephisfreq)
       enddo
       his_sndpth(i,j) = his_sndpth(i,j)+(sndpth(i,j)/stephisfreq)
       his_thickn(i,j) = his_thickn(i,j)+(thickn(i,j)/stephisfreq)
       his_fice(i,j) = his_fice(i,j)+(fice(i,j)/stephisfreq)
       his_melt(i,j) = his_melt(i,j)+(meltp(i,j)/stephisfreq)
       his_tsnow(i,j) = his_tsnow(i,j)+tsnow(i,j)
       his_tice(i,j,1) = his_tice(i,j,1)+tice(i,j,1)
       his_tice(i,j,2) = his_tice(i,j,2)+tice(i,j,2)
       his_tinter(i,j) = his_tinter(i,j)+tinter(i,j)
       if(fice(i,j) .gt. 0) then
         icesteps(i,j) = icesteps(i,j) + 1
       endif
       his_ws(i,j) = his_ws(i,j) + (ws(i,j)/stephisfreq)
       his_volr(i,j) = his_volr(i,j) + (volr(i,j)/stephisfreq)
       his_oprec(i,j) = his_oprec(i,j)+(oprec(i,j)/stephisfreq)
       his_snm(i,j) = his_snm(i,j) + (snwmlt(i,j)/stephisfreq)
       his_oro(i,j) = ioro(i,j)

#ifdef FOAM_LPJ
c add in net solar flux absorbed at surface to history
       his_fsns(i,j)=his_fsns(i,j) + (ofsns(i,j)/stephisfreq)
c
c  add in lpj coupler fields
c
       his_laet(i,j)=his_laet(i,j) + (laettot(i,j)/stephisfreq)
       his_legrd(i,j)=his_legrd(i,j) + ((tmplhflx/uwarea(j)-
     $ laettot(i,j))/stephisfreq)
c
c  add in lpj dphen and fpcgrid fields
c
       do pft=1,npft
      his_dphen(i,j,pft)=his_dphen(i,j,pft)+(dphen(i,j,pft)/stephisfreq)
c     his_fpcgrid(i,j,pft)=his_fpcgrid(i,j,pft)+(fpcgrid(i,j,pft)/
c    $ stephisfreq)
       enddo
#endif

      enddo
      enddo
      enddo

#ifndef STANDALONE_OCEAN
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    BEGIN OCEAN STEP
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if ( (mod(nstep,int(tratio)) .eq. 0) .and.
     $		(nstep .ne. 0)) then
#ifdef UPSHOT
      call log_event(4,m_pin,'EndCoupl',8)
      call log_event(17,m_pin,'StrtCplOcn',10)
#endif

ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Update the the following arrays:
c  oheat,oqflx,otaux,otauy
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

#ifdef ICE_MODEL
      do n=1,2
      do j= plats(n),plate(n)
      do i= plons,plone
	if(ioro(i,j) .eq. 2) then
	  otaux(i,j) = otaux(i,j)/15.
	  otauy(i,j) = otauy(i,j)/15.
        endif
      enddo
      enddo
      enddo
#endif
      checkice = .TRUE.

#ifdef UPSHOT
      call log_event(19,m_pin,'StrtGath',8)
#endif


#ifdef PROFILE
      call mark_event(20,1)
#endif
      do j=1,uwlat
        do i=1,uwlon
          zero_out=.false.
          if (j .lt. plats(1)) then
            zero_out=.true.
          else if (j .gt. plate(1) .and. j .lt. plats(2)) then
            zero_out=.true.
          else if (j .gt. plate(2)) then
            zero_out=.true.
          else if (i .lt. plons .or. i .gt. plone) then
            zero_out=.true.
          endif
          if (zero_out) then
	  oheat(i,j) = 0.
	  oqflx(i,j) = 0.
	  otaux(i,j) = 0.
	  otauy(i,j) = 0.
	  endif
       enddo
      enddo

      call reduce_sum(oheat,oheat2,uwlon*uwlat)
      call reduce_sum(oqflx,oqflx2,uwlon*uwlat)
      call reduce_sum(otaux,otaux2,uwlon*uwlat)
      call reduce_sum(otauy,otauy2,uwlon*uwlat)

#ifdef UPSHOT
      call log_event(20,m_pin,'EndGath',7)
#endif


cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    ONLY PASS FLUXES AND GET TEMPS IF I'M ROOT
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if(m_pin .eq. 0) then

#ifdef UPSHOT
      call log_event(9,m_pin,'StrtSend',8)
#endif
 
c
c  divide fluxes by tratio to give average flux over the
c  ocean time step.
c
      do j=1,uwlon
      do i=1,uwlat

        to_ocn(i,j,1)=oheat2(j,i)/tratio
	his_toocn(j,i,1)=his_toocn(j,i,1)+oheat2(j,i)

	to_ocn(i,j,2)=oqflx2(j,i)/tratio
	his_toocn(j,i,2)=his_toocn(j,i,2)+oqflx2(j,i)

	to_ocn(i,j,3)=otaux2(j,i)/tratio
	his_toocn(j,i,3)=his_toocn(j,i,3)+otaux2(j,i)

	to_ocn(i,j,4)=otauy2(j,i)/tratio
	his_toocn(j,i,4)=his_toocn(j,i,4)+otauy2(j,i)
      enddo
      enddo

      call send_to_ocean(uwlat,uwlon,Nrows,Ncols,4,to_ocn,200)

#ifdef UPSHOT
      call log_event(10,m_pin,'EndSend',7)
#endif

      endif
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    END OF FLUX PASSING
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

#ifdef COLOCATED_OCEAN

#ifdef UPSHOT
      call log_event(18,m_pin,'EndCplOcn',9)
      call log_event(5,m_pin,'StrtOcn',7)
#endif

C This is where the ocean will be called
        call ocean_step()

#ifdef UPSHOT
      call log_event(6,m_pin,'EndOcn',6)
      call log_event(17,m_pin,'StrtCplOcn',10)
#endif

#endif


c
c  everybody zero the fluxes
c
      do j=1,uwlat
      do i = 1,uwlon
        oqflx(i,j) = 0.
        oqflx2(i,j) = 0.
        otaux(i,j) = 0.
        otauy(i,j) = 0.
        otaux2(i,j) = 0.
        otauy2(i,j) = 0.
	oheat(i,j) = 0.
	oheat2(i,j) = 0.
      enddo
      enddo
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    ONLY GET NEW SSTS ON NODE 0
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c insert routines to receive new sst
c

      if(m_pin .eq. 0) then

#ifdef UPSHOT
      call log_event(7,m_pin,'StrtRecv',8)
#endif

      call receive_from_ocean(uwlat,uwlon,Nrows,Ncols,2,from_ocn,300)

#ifdef UPSHOT
      call log_event(8,m_pin,'EndRecv',7)
#endif

      endif


#ifdef UPSHOT
      call log_event(13,m_pin,'StrtBcast',9)
#endif

      call bcast0(from_ocn,uwlat*uwlon*4*2,10,0)

#ifdef UPSHOT
      call log_event(14,m_pin,'EndBcast',8)
#endif

c load in new sst's from ocean model
      do n=1,2
      do i= plats(n),plate(n)
        do j= plons,plone
          sst(j,i)=from_ocn(i,j,1)
          meltp(j,i)=from_ocn(i,j,2)
	if((i.eq.jw).and.(j.eq.iw)) then
          write(0,*)"FROMOCN",meltp(j,i)
	endif
        enddo
      enddo
      enddo

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    END OF SST GET
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Update the the following arrays:
c  sst
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

c
c  build global surface temp field from ocean sst, 
c	
      do n=1,2
      do j=plats(n),plate(n)
      do i=plons,plone
	if(ioro(i,j) .eq. 0) then
	  ts(i,j) = sst(i,j)
	  tssub(i,j,1) = sst(i,j)
	  tssub(i,j,2) = sst(i,j)
	  tssub(i,j,3) = sst(i,j)
	  tssub(i,j,4) = sst(i,j)
        endif
      enddo
      enddo
      enddo

#ifdef PROFILE
      call mark_event(20,0)
#endif
#ifdef UPSHOT
      call log_event(18,m_pin,'EndCplOcn',9)
      call log_event(3,m_pin,'StartCpl',8)
#endif

      endif 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c    END OF OCEAN MODEL STEP
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
#endif 

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c write HISTORY file
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if ( (mod(nstep,stephisfreq) .eq. 0) .and.
     $      (nstep .ne. 0) ) then
#ifdef PROFILE
      call mark_event(10,1)
#endif
#ifdef UPSHOT
      call log_event(21,m_pin,'StrtGath',8)
#endif

     
      if (m_pin .eq. 0) then
      stfnum = finished + hisfreq*hiscount
      write(itf,'(i8)') stfnum + 10000000

      hisname = 'history.coupl.'//itf(2:8)

#ifdef SIMPLE_OUTPUT
      open(91,file='history/coupl/'//hisname,
     $form='unformatted')
#else
      open(91,file=hisname,
     $form='unformatted')
#endif

#ifdef FOAM_LPJ
      hislpj = 'history.colpj.'//itf(2:8)
# ifdef SIMPLE_OUTPUT
      open(58,file='history/coupl/'//hislpj,
     $form='unformatted')
# else
      open(58,file=hislpj,
     $form='unformatted')
# endif
#endif

      endif

      do j=1,uwlat
        do i=1,uwlon
          zero_out=.false.
          if (j .lt. plats(1)) then
            zero_out=.true.
          else if (j .gt. plate(1) .and. j .lt. plats(2)) then
            zero_out=.true.
          else if (j .gt. plate(2)) then
            zero_out=.true.
          else if (i .lt. plons .or. i .gt. plone) then
            zero_out=.true.
          endif
          if (zero_out) then
	    his_sndpth(i,j)=0.
	    his_thickn(i,j)=0.
            his_fice(i,j)=0.
            his_melt(i,j)=0.
            his_tice(i,j,1)=0.
            his_tice(i,j,2)=0.
            his_tsnow(i,j)=0.
            his_tinter(i,j)=0.
            his_shf(i,j) = 0.
	    his_lhf(i,j) = 0.
	    his_evp(i,j) = 0.
	    his_orad(i,j) = 0.
	    his_olw(i,j) = 0.
	    his_rnf(i,j) = 0.
	    his_ornf(i,j) = 0.
	    his_snm(i,j) = 0.
	    his_volr(i,j)=0.
	    his_oro(i,j)=0
	    his_ws(i,j)=0.

            sndpth(i,j)=0.
            thickn(i,j)=0.
            ws(i,j)=0.
            volr(i,j)=0.
            fluxout(i,j)=0.
            sfluxin(i,j)=0.
            oro(i,j)=0.
            ioro(i,j)=0
	    icesteps(i,j)=0
            do k=1,4
	      his_tssub(i,j,k)=0.
              tssub(i,j,k)=0.
            enddo
#ifdef FOAM_LPJ
            his_fsns(i,j)=0.
            his_laet(i,j)=0.
            his_legrd(i,j)=0.
       do pft=1,npft
           his_dphen(i,j,pft)=0.0
           his_fpcgrid(i,j,pft)=0.0
           his_disturb(i,j,pft)=0.0
           his_anpp(i,j,pft)=0.0
           his_litag(i,j,pft)=0.0
           his_litbg(i,j,pft)=0.0
           his_lmind(i,j,pft)=0.0
           his_smind(i,j,pft)=0.0
           his_rmind(i,j,pft)=0.0
           his_hmind(i,j,pft)=0.0
           his_nind(i,j,pft)=0.0
           his_height(i,j,pft)=0.0
       enddo
#endif
          endif

	  if(icesteps(i,j) .gt. 0) then
	    his_tice(i,j,1)=his_tice(i,j,1)/icesteps(i,j)
	    his_tice(i,j,2)=his_tice(i,j,2)/icesteps(i,j)
	    his_tsnow(i,j)=his_tsnow(i,j)/icesteps(i,j)
	    his_tinter(i,j)=his_tinter(i,j)/icesteps(i,j)
	  endif
        enddo
      enddo

      call ireduce_sum(his_oro,ioro2,uwlon*uwlat)
      call reduce_sum(his_tssub,tssub2,uwlon*uwlat*4)
      call reduce_sum(his_sndpth,sndpth2,uwlon*uwlat)
      call reduce_sum(his_thickn,thickn2,uwlon*uwlat)
      call reduce_sum(his_fice,fice2,uwlon*uwlat)
      call reduce_sum(his_melt,meltp2,uwlon*uwlat)
      call reduce_sum(his_tice,tice2,uwlon*uwlat*2)
      call reduce_sum(his_tsnow,tsnow2,uwlon*uwlat)
      call reduce_sum(his_tinter,tinter2,uwlon*uwlat)
      call reduce_sum(his_ws,ws2,uwlon*uwlat)
      call reduce_sum(his_volr,volr2,uwlon*uwlat)
      call reduce_sum(his_oprec,oprec2,uwlon*uwlat)
      call reduce_sum(his_shf,shf2,uwlon*uwlat)
      call reduce_sum(his_lhf,lhf2,uwlon*uwlat)
      call reduce_sum(his_evp,evp2,uwlon*uwlat)
      call reduce_sum(his_rnf,rnf2,uwlon*uwlat)
      call reduce_sum(his_ornf,ornf2,uwlon*uwlat)
      call reduce_sum(his_snm,snm2,uwlon*uwlat)
      call reduce_sum(his_orad,orad2,uwlon*uwlat)
      call reduce_sum(his_olw,olw2,uwlon*uwlat)
#ifdef FOAM_LPJ
      call reduce_sum(his_fsns,fsns2,uwlon*uwlat)
      call reduce_sum(his_laet,laet2,uwlon*uwlat)
      call reduce_sum(his_legrd,legrd2,uwlon*uwlat)
      call reduce_sum(his_dphen,dphen2,uwlon*uwlat*npft)
      call reduce_sum(his_fpcgrid,fpcgrid2,uwlon*uwlat*npft)
      call reduce_sum(his_disturb,disturb2,uwlon*uwlat*npft)
      call reduce_sum(his_anpp,anpp2,uwlon*uwlat*npft)
      call reduce_sum(his_litag,litag2,uwlon*uwlat*npft)
      call reduce_sum(his_litbg,litbg2,uwlon*uwlat*npft)
      call reduce_sum(his_lmind,lmind2,uwlon*uwlat*npft)
      call reduce_sum(his_smind,smind2,uwlon*uwlat*npft)
      call reduce_sum(his_rmind,rmind2,uwlon*uwlat*npft)
      call reduce_sum(his_hmind,hmind2,uwlon*uwlat*npft)
      call reduce_sum(his_nind,nind2,uwlon*uwlat*npft)
      call reduce_sum(his_height,height2,uwlon*uwlat*npft)
#endif


#ifdef UPSHOT
      call log_event(22,m_pin,'EndGath',7)
#endif

      if (m_pin .eq. 0) then
      do j=1,uwlat
	do i=1,uwlon
	 do k=1,4
	 his_toocn(j,i,k)=his_toocn(j,i,k)/stephisfreq
	 enddo
        enddo
      enddo


#ifdef FOAM_LPJ
      write(91) ioro2,tssub2,sndpth2,thickn2,ws2,volr2,oprec2,
     $ his_toocn,shf2,lhf2,laet2,legrd2,evp2,rnf2,snm2,ornf2,orad2,
     $ olw2,fice2,tsnow2,tinter2,tice2,meltp2,fsns2
      close(91)
      write(58) ioro2,dphen2,fpcgrid2,disturb2,anpp2,litag2,litbg2,
     $ lmind2,smind2,rmind2,hmind2,nind2,height2
      close(58)
#else
      write(91) ioro2,tssub2,sndpth2,thickn2,ws2,volr2,oprec2,
     $ his_toocn,shf2,lhf2,evp2,rnf2,snm2,ornf2,orad2,olw2,
     $ fice2,tsnow2,tinter2,tice2,meltp2
      close(91)
#endif
      hiscount = hiscount + 1

      endif

c Zero the HISTORY buffers
      do j=1,uwlat
	do i=1,uwlon
	 do k=1,4
	 his_toocn(j,i,k)=0.
	 his_tssub(i,j,k)=0.
	 enddo
         his_sndpth(i,j)=0.
	 his_thickn(i,j)=0.
         his_fice(i,j)=0.
         his_melt(i,j)=0.
         his_tice(i,j,1)=0.
         his_tice(i,j,2)=0.
         his_tsnow(i,j)=0.
         his_tinter(i,j)=0.
	 his_ws(i,j)=0.
	 his_volr(i,j)=0.
	 his_oprec(i,j)=0.
	 his_oro(i,j)=0
	 his_evp(i,j)=0.
	 his_shf(i,j)=0.
	 his_lhf(i,j)=0.
	 his_snm(i,j)=0.
	 his_ornf(i,j)=0.
	 his_orad(i,j)=0.
	 his_olw(i,j)=0.
	 his_rnf(i,j)=0.
	 icesteps(i,j)=0
#ifdef FOAM_LPJ
         his_fsns(i,j)=0.
         his_laet(i,j)=0.
         his_legrd(i,j)=0.
         do pft=1,npft
           his_dphen(i,j,pft)=0.
         enddo
#endif
        enddo
      enddo
#ifdef PROFILE
      call mark_event(10,0)
#endif
      endif

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c       WRITE RESTART DATA
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      if ( (mod(nstep,steprsfreq) .eq. 0) .and.
     $      (nstep .ne. 0) ) then
      do j=1,uwlat
        do i=1,uwlon
          zero_out=.false.
          if (j .lt. plats(1)) then
            zero_out=.true.
          else if (j .gt. plate(1) .and. j .lt. plats(2)) then
            zero_out=.true.
          else if (j .gt. plate(2)) then
            zero_out=.true.
          else if (i .lt. plons .or. i .gt. plone) then
            zero_out=.true.
          endif
          if (zero_out) then
            sndpth(i,j)=0.
            thickn(i,j)=0.
            meltp(i,j)=0.
            fice(i,j)=0.
            tsnow(i,j)=0.
            qstor(i,j)=0.
            ws(i,j)=0.
            volr(i,j)=0.
	    lwuphf(i,j)=0.
	    lwuphfi(i,j)=0.
	    lwuphfo(i,j)=0.
	    sst(i,j)=0.
            fluxout(i,j)=0.
            sfluxin(i,j)=0.
            oro(i,j)=0.
            ioro(i,j)=0
	    oioro(i,j) = 0
	    oqflx(i,j) = 0.
	    delts(i,j)=0.
            do k=1,4
              tssub(i,j,k)=0.
            enddo
#ifdef FOAM_LPJ
c add in lpj fields needed for restart
      do pft=1,npft
       aet(i,j,pft)=0.0
       aeq(i,j,pft)=0.0
       suph(i,j,pft)=0.0
       gp(i,j,pft)=0.0
       dphen(i,j,pft)=0.0
       fpcgrid(i,j,pft)=0.0
       aetd(i,j,pft)=0.0
       aeqd(i,j,pft)=0.0
       gdd(i,j,pft)=0.0
       aphen(i,j,pft)=0.0
       litter_ag(i,j,pft)=0.0
       litter_bg(i,j,pft)=0.0
       height(i,j,pft)=0.0
       lm_ind(i,j,pft)=0.0
       sm_ind(i,j,pft)=0.0
       rm_ind(i,j,pft)=0.0
       hm_ind(i,j,pft)=0.0
       anpp(i,j,pft)=0.0
       nind(i,j,pft)=0.0
       mgpp(i,j,pft)=0.0
       lrespm(i,j,pft)=0.0
       srespm(i,j,pft)=0.0
       rrespm(i,j,pft)=0.0
       gddtw(i,j,pft)=0.0
       awscal(i,j,pft)=0.0
       aleafdays(i,j,pft)=0
       ipresent(i,j,pft)=0
       ifrost(i,j,pft)=0
        enddo
       fbare(i,j)=0.0
       wu(i,j)=0.0
       wl(i,j)=0.0
       laettot(i,j)=0.0
       k_fast_ave(i,j)=0.0
       k_slow_ave(i,j)=0.0
       litter_decom_ave(i,j)=0.0
       cpool_fast(i,j)=0.0
       cpool_slow(i,j)=0.0
       aprec(i,j)=0.0
       gddclim(i,j)=0.0
        do m=1,12
       tsm(i,j,m)=0.0
       wum(i,j,m)=0.0
        enddo
        do m=1,20
       tsm_min20(i,j,m)=0.0
        enddo
#endif
          endif
        enddo
      enddo

#ifdef PROFILE
      call mark_event(11,1)
#endif
#ifdef UPSHOT
      call log_event(21,m_pin,'StrtGath',8)
#endif
      call ireduce_sum(ioro,ioro3,uwlon*uwlat)
      call ireduce_sum(oioro,oioro3,uwlon*uwlat)
      call reduce_sum(tssub,tssub3,uwlon*uwlat*4)
      call reduce_sum(sndpth,sndpth3,uwlon*uwlat)
      call reduce_sum(thickn,thickn3,uwlon*uwlat)
      call reduce_sum(ws,ws3,uwlon*uwlat)
      call reduce_sum(lwuphf,lwuphf2,uwlon*uwlat)
      call reduce_sum(lwuphfi,lwuphfi3,uwlon*uwlat)
      call reduce_sum(lwuphfo,lwuphfo3,uwlon*uwlat)
      call reduce_sum(afsul,afsul2,p_glon*p_glat)
      call reduce_sum(aalbs,aalbs2,p_glon*p_glat)
      call reduce_sum(aalbl,aalbl2,p_glon*p_glat)
      call reduce_sum(aalbsd,aalbsd2,p_glon*p_glat)
      call reduce_sum(aalbld,aalbld2,p_glon*p_glat)
      call reduce_sum(volr,volr3,uwlon*uwlat)
      call reduce_sum(sst,sst3,uwlon*uwlat)
      call reduce_sum(delts,delts3,uwlon*uwlat)

      call reduce_sum(oro,oro2,uwlon*uwlat)
      call reduce_sum(fluxout,fluxout2,uwlon*uwlat)
      call reduce_sum(sfluxin,sfluxin2,uwlon*uwlat)
      call reduce_sum(oqflx,oqflx3,uwlon*uwlat)
      call reduce_sum(meltp,meltp3,uwlon*uwlat)
      call reduce_sum(fice,fice3,uwlon*uwlat)
      call reduce_sum(tsnow,tsnow3,uwlon*uwlat)
      call reduce_sum(qstor,qstor3,uwlon*uwlat)
#ifdef FOAM_LPJ
c
c needed for lpj restart
      call reduce_sum(aet,aet3,uwlon*uwlat*npft)
      call reduce_sum(aeq,aeq3,uwlon*uwlat*npft)
      call reduce_sum(suph,suph3,uwlon*uwlat*npft)
      call reduce_sum(gp,gp3,uwlon*uwlat*npft)
      call reduce_sum(dphen,dphen3,uwlon*uwlat*npft)
      call reduce_sum(fpcgrid,fpcgrid3,uwlon*uwlat*npft)
      call reduce_sum(aetd,aetd3,uwlon*uwlat*npft)
      call reduce_sum(aeqd,aeqd3,uwlon*uwlat*npft)
      call reduce_sum(gdd,gdd3,uwlon*uwlat*npft)
      call reduce_sum(aphen,aphen3,uwlon*uwlat*npft)
      call reduce_sum(litter_ag,litag3,uwlon*uwlat*npft)
      call reduce_sum(litter_bg,litbg3,uwlon*uwlat*npft)
      call reduce_sum(height,height3,uwlon*uwlat*npft)
      call reduce_sum(lm_ind,lmind3,uwlon*uwlat*npft)
      call reduce_sum(sm_ind,smind3,uwlon*uwlat*npft)
      call reduce_sum(rm_ind,rmind3,uwlon*uwlat*npft)
      call reduce_sum(hm_ind,hmind3,uwlon*uwlat*npft)
      call reduce_sum(anpp,anpp3,uwlon*uwlat*npft)
      call reduce_sum(nind,nind3,uwlon*uwlat*npft)
      call reduce_sum(mgpp,mgpp3,uwlon*uwlat*npft)
      call reduce_sum(lrespm,lrespm3,uwlon*uwlat*npft)
      call reduce_sum(srespm,srespm3,uwlon*uwlat*npft)
      call reduce_sum(rrespm,rrespm3,uwlon*uwlat*npft)
      call reduce_sum(gddtw,gddtw3,uwlon*uwlat*npft)
      call reduce_sum(awscal,awscal3,uwlon*uwlat*npft)
      call ireduce_sum(aleafdays,alfd3,uwlon*uwlat*npft)
      call ireduce_sum(ipresent,ipr3,uwlon*uwlat*npft)
      call ireduce_sum(ifrost,ifr3,uwlon*uwlat*npft)

      call reduce_sum(fbare,fb3,uwlon*uwlat)
      call reduce_sum(wu,wu3,uwlon*uwlat)
      call reduce_sum(wl,wl3,uwlon*uwlat)
      call reduce_sum(laettot,laet3,uwlon*uwlat)
      call reduce_sum(k_fast_ave,kfast3,uwlon*uwlat)
      call reduce_sum(k_slow_ave,kslow3,uwlon*uwlat)
      call reduce_sum(litter_decom_ave,litdcm3,uwlon*uwlat)
      call reduce_sum(cpool_fast,cpfast3,uwlon*uwlat)
      call reduce_sum(cpool_slow,cpslow3,uwlon*uwlat)
      call reduce_sum(aprec,apr3,uwlon*uwlat)
      call reduce_sum(gddclim,gddclim3,uwlon*uwlat)
      call reduce_sum(tsm,tsm3,uwlon*uwlat*12)
      call reduce_sum(wum,wum3,uwlon*uwlat*12)
      call reduce_sum(tsm_min20,tsmmin3,uwlon*uwlat*20)
#endif



#ifdef UPSHOT
      call log_event(22,m_pin,'EndGath',7)
#endif

      if (m_pin .eq. 0) then
      stfnum = finished + rsfreq*rescount
      write(itf,'(i8)') stfnum + 10000000
      resname = 'restart.coupl.'//itf(2:8)
#ifdef SIMPLE_OUTPUT
      open(90,file='restart/coupl/'//resname,
     $form='unformatted')
#else
      open(90,file=resname,
     $form='unformatted')
#endif

#ifdef FOAM_LPJ
      reslpj = 'restart.colpj.'//itf(2:8)
# ifdef SIMPLE_OUTPUT
      open(57,file='restart/coupl/'//reslpj,
     $form='unformatted')
# else
      open(57,file=reslpj,
     $form='unformatted')
# endif
#endif

#ifdef WRITE1_4
      write(90) tssub3,sndpth3,thickn3,ws3,oro2,ioro3,
     $          volr3,fluxout2,sfluxin2,lwuphf2,afsul2,
     $		aalbs2,aalbl2,aalbsd2,aalbld2,	
     $		sst3,delts3,oioro3,oqflx3
#else

      write(90) tssub3,sndpth3,thickn3,ws3,oro2,ioro3,
     $          volr3,fluxout2,sfluxin2,lwuphf2,afsul2,
     $		aalbs2,aalbl2,aalbsd2,aalbld2,	
     $		sst3,delts3,oioro3,meltp3,fice3,tsnow3,
     $          qstor3,lwuphfi3,lwuphfo3
#endif
      close(90)

#ifdef FOAM_LPJ
      write(57) aet3,aeq3,suph3,gp3,dphen3,fpcgrid3,aetd3,aeqd3,gdd3,
     $ aphen3,litag3,litbg3,height3,lmind3,smind3,rmind3,hmind3,
     $ anpp3,nind3,mgpp3,lrespm3,srespm3,rrespm3,gddtw3,awscal3,
     $ alfd3,ipr3,ifr3,fb3,wu3,wl3,laet3,kfast3,kslow3,litdcm3,
     $ cpfast3,cpslow3,apr3,gddclim3,tsm3,wum3,tsmmin3
      close(57)
#endif

      rescount = rescount + 1
      endif

#ifdef PROFILE
      call mark_event(11,0)
#endif
      endif


 9999 continue
#ifdef PROFILE
      call mark_event(19,0)
#endif
      return
      end
