#include "params.h"
      subroutine pradalb(coszrs, oalbs    ,
     $                  oalbl    ,oalbsd   ,oalbld   )
C-----------------------------------------------------------------------
C
C Compute surface albedos for ocean
C
C Computes surface albedos for direct/diffuse incident radiation for
C two spectral intervals:
C   s = 0.2-0.7 micro-meters
C   l = 0.7-5.0 micro-meters
C
C Uses knowledge of surface type to specify albedo, as follows:
C
C Ocean           Uses solar zenith angle to compute albedo for direct
C                 radiation; diffuse radiation values constant; albedo
C                 independent of spectral interval and other physical
C                 factors such as ocean surface wind speed.
C
C For more details , see Briegleb, Bruce P., 1992: Delta-Eddington
C Approximation for Solar Radiation in the NCAR Community Climate Model,
C Journal of Geophysical Research, Vol 97, D7, pp7603-7612).
C
C---------------------------Code history--------------------------------
C
C Original version:  CCM1
C Standardized:      J. Rosinski, June 1992
C Reviewed:          J. Kiehl, B. Briegleb, August 1992

C reduced down to ocean only:  R. Jacob, Feb, 2000
C
C-----------------------------------------------------------------------
      implicit none
#if ( defined RS6000 )
c      implicit automatic (a-z)
#endif
C------------------------------Parameters-------------------------------
#include "prgrid.com"
#include "pcoupled.com"
#include "pcplrstr.com"
C------------------------------Arguments--------------------------------
C
C  input from uwmask,sndpth in common block
      real coszrs(uwlon,uwlat)   ! Cosine solar zenith angle
C
C Output arguments
C
      real oalbs(uwlon,uwlat),    ! Srf alb for direct rad   0.2-0.7 micro-ms
     $     oalbl(uwlon,uwlat),    ! Srf alb for direct rad   0.7-5.0 micro-ms
     $     oalbsd(uwlon,uwlat),   ! Srf alb for diffuse rad  0.2-0.7 micro-ms
     $     oalbld(uwlon,uwlat)    ! Srf alb for diffuse rad  0.7-5.0 micro-ms
C
C---------------------------Local variables-----------------------------
C
      integer i,j,m,n,      ! Longitude index
     $        ii,          ! Lngtd indx pnts satisfying certain condtns
     $        npts         ! Number of computation points
      integer indx(uwlon)       ! Longitude index array (land)

C
C-----------------------------------------------------------------------
C
C  See if theres any oceanpoints here to do
      npts = 0
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
         if(uwmask(i,j) .eq. 0) then
          npts =npts +1
         endif
       enddo
      enddo
      enddo
      if(npts .eq. 0) return


      do n=1,2
      do j=plats(n),plate(n)

       npts = 0
       do i=plons,plone
         if(uwmask(i,j) .eq. 0) then
          npts =npts +1
          indx(npts)=i
         endif
       enddo

      do ii=1,npts
          i=indx(ii)
C Local points over ice-free ocean with incident solar radiation:
C
C Ocean albedos function of solar zenith angle only,and
C independent of spectral interval:
         if( coszrs(i,j).ge.0.) then
         oalbl(i,j)  = (.026/(coszrs(i,j)**1.7 + .065)) +
     $              (.15*(coszrs(i,j) - 0.10)*
     $                   (coszrs(i,j) - 0.50)*
     $                   (coszrs(i,j) - 1.00)  )
         oalbs(i,j)  = oalbl(i,j)
         oalbld(i,j) = 0.06
         oalbsd(i,j) = 0.06
	 endif
      enddo

      enddo
      enddo
C   
      return
      end
