c $Header: /homes/jacob/foam-cvs/simsice/simsice_drv.F,v 1.2 2002/07/08 19:41:30 jacob Exp $
#include "params.h"
c
c This program calls the simple sea ice model used in FOAM1.0-1.4
c
c Heat and momentum fluxes and temperature are calculated as
c in the land model but the "land type" is set to sea ice.
c Appropriate consants for thermal properties are already
c included.    Total thickness is 2m.
c
c For the heat flux between the bottom of the sea ice and
c the ocean, the predicted sst from the ocean model is used
c instead of a constant.  See posrftsb.F
c
c The bucket hydrology scheme from FOAM 1.0-1.4 land model is also
c implemented over the sea ice.  Any moisture or snow over the
c preset limits is allowed to fall through.
c
c Sea ice growth/melting is not modeled.  Ice appears when SST
c falls below freezing.  So that the SST can go above freezing,
c the sea ice is also allowed to go above
c feezing so that the ocean below may warm by conduction.  This
c captures the phase of the seasonal cycle of sea ice fairly well.
c
c call posrfflx -- FOAM's version of the CCM2 land/ice surface flux
c  routine srfflx.F
c call posrftsb  -- FOAM's version of 4 layer model
c  
c Initial Version of simple sea ice model:  R. Jacob Oct, 1995.
c extensive refactoring: R. Jacob Jan, 2000.
c

      subroutine simsice_drv(
c  atmosphere input variables
     $ psm1  ,pmidm1, ubot, vbot, tbot, qbot,thbot,
     $ zbot,srfrad,
c  surface input
     $ snowing,ltheat,oprec,doalb,coszrs,
c  by common block: sst,lwuphf,ioro
c surface input/output
     $ ts,checkice,
c  by common block: tssub
c hydrology output
     $ snwmlt,exws,
c  by common block: sndpth, ws
c surface output
     $ fbt,oalbs,oalbl,oalbsd,oalbld,
c  by common block: delts
c surface output
C overlap grid output
     $ qflx,lhflx,shflx,taux,tauy)


      implicit none

c------------------------------------------------
#include "pmgrid.com"
c------------------------------------------------
#include "comctl.com"
c------------------------------------------------
#include "pspect.com"
c------------------------------------------------
#include "comtim.com"
c------------------------------------------------
#include "comtsc.com"
c------------------------------------------------
#include "comparll.com"
c------------------------------------------------
#include "pcoupled.com"
c------------------------------------------------
#include "pcplrstr.com"
c------------------------------------------------

c atmosphere input
      real psm1(p_glon,p_glat) ! surface pressure
      real pmidm1(p_glon,p_glat) ! bottom level pressure
      real ubot(p_glon,p_glat) ! bottom level u wind
      real vbot(p_glon,p_glat) ! bottom level v wind
      real tbot(p_glon,p_glat) ! bottom level temp
      real qbot(p_glon,p_glat) ! bottom level specific humidity
      real thbot(p_glon,p_glat) ! bottom level potential temp
      real zbot(p_glon,p_glat) ! bottom level height above surface
      real srfrad(p_glon,p_glat)

c  surface input
      logical snowing(uwlon,uwlat)
      real ltheat(uwlon,uwlat)
      real oprec(uwlon,uwlat)   ! Atmos. precip fields on the ocean grid
      real coszrs(uwlon,uwlat)  !cosine of the zenith angle
      logical doalb             ! update albedos flag

c input/output
      real ts(uwlon,uwlat)    ! non-adjusted srf/sub temps
      logical checkice
c hydrology output
      real snwmlt(uwlon,uwlat)     ! heat flow between ocean and sea ice
      real exws(uwlon,uwlat)     ! excess surface moisture

c  surface output
      real fbt(uwlon,uwlat)     ! heat flow between ocean and sea ice
      real oalbs(uwlon,uwlat)   ! Srf alb for direct rad  0.2-0.7 micron
      real oalbl(uwlon,uwlat)   ! Srf alb for direct rad  0.7-5.0
      real oalbsd(uwlon,uwlat)  ! Srf alb for diffuse rad 0.2-0.7
      real oalbld(uwlon,uwlat)  ! Srf alb for diffuse rad 0.7-5.0


c overlap grid output
      real qflx(uwlon,uwlat,maxoverlaps)        ! water flux
      real lhflx(uwlon,uwlat,maxoverlaps)       ! latent heat flux
      real shflx(uwlon,uwlat,maxoverlaps)       ! sensible heat flux
      real taux(uwlon,uwlat,maxoverlaps)
      real tauy(uwlon,uwlat,maxoverlaps)



c  local variables
      integer isrftyp(uwlon)  ! surface "soil" type
      real wet(uwlon)     ! surface wetness (fraction)
      real z0(uwlon)      ! surface roughness lenght for fluxes

      real alpha(uwlon)   ! ts term from top layer linear equ
      real beta(uwlon)    ! rhs term from top layer linear equ
      real gamma(uwlon)   ! t2 term from top layer linear equ
      real eexc(uwlon)    ! excess thermal energy after snow melt
      real tpthck(uwlon)  ! top layer geometrical thickness
      real tsbsf(uwlon,plevmx)    ! non-adjusted srf/sub temps
      real oldmst(uwlon)  ! old ws value
      logical logic(uwlon)	! surface wetness logical
      real tsin(uwlon)    ! save ts
      real ofnt(uwlon)    ! net surface flux for input conditions
      real odfntdt(uwlon)
c albedo local variables
      real tlslo            ! low threshold temp for snow albedo
      real tlshi            ! high threshold temp for snow albedo
      real snwsh,           ! high temp snow albedo for 0.2-0.7 micro-meters
     $     snwlh,           ! high temp snow albedo for 0.7-5.0 micro-meters
     $     snwll,           ! low temp snow albedo for 0.7-5.0 micro-meters
     $     snwsl,            ! low temp snow albedo for 0.2-0.7 micro-meters
     $     sices,          ! Sea ice albedo for 0.2-0.7 micro-meters
     $     sicel           ! Sea ice albedo for 0.7-5.0 micro-meters

      parameter (snwsl  = 0.90,
     $           snwsh  = 0.55,
     $           snwll  = 0.60,
     $           snwlh  = 0.35,
     $           sices = 0.70,
     $           sicel = 0.50)



      real salbs(uwlon),   ! Snow alb for direct rad  0.2-0.7 micro-ms
     $     salbl(uwlon),   ! Snow alb for direct rad  0.7-5.0 micro-ms
     $     salbsd(uwlon),  ! Snow alb for diffuse rad  0.2-0.7 micro-ms
     $     salbld(uwlon),  ! Snow alb for diffuse rad  0.7-5.0 micro-ms
     $     frsnow(uwlon),  ! Horizontal fraction of snow cover
     $     snwhgt,         ! Physical snow height
     $     rghsnw          ! Roughness for horizontal snow cover fractn


c overlap grid local variables
      real fnt(uwlon,uwlat,maxoverlaps)  ! net srf flx tmp (ts)
      real dfntdt(uwlon,uwlat,maxoverlaps)  ! net srf flx tmp deriv (ts)

      real lhf(uwlon,uwlat,maxoverlaps) ! initial latent heat flux
      real shf(uwlon,uwlat,maxoverlaps) ! initial sensible heat flux
      real dlhf(uwlon,uwlat,maxoverlaps)        ! ts deriv for latent heat flux
      real dshf(uwlon,uwlat,maxoverlaps)        ! ts deriv for sensible heat flu


c physical variables
      real rhoice     ! density of ice
      real rhoh2o     ! density of water
      real snwmax     ! maximum snow depth in liquid water equiv

c externals
      external posrfflx,posrftsb
      real cvmgt
#if ( defined SGI) || ( defined INTEL)
      external cvmgt
#endif

      integer indx(uwlon)       ! Longitude index array (ice)
      integer npts              ! Number of sea ice points
      integer ii              ! Longitude indices 


      integer n,i,j,k,l,m
      real emelt,mflux,tmp,wsnorm,zfac
      real snrun
      real tsice  ! freezing point of sea -ice
      real odelt  ! ocean time step
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Begin ice model calculations
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      
      rhoh2o= 1.e3
      rhoice = .92*rhoh2o
      snwmax = 1.0
      odelt = 21600.
      tsice = 273.15 - 1.8


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  check for sea ice formation/desctruction
C
C
C  if an ocean communication happened on the last
C  timestep then checkice will be true
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if(checkice) then
      do n=1,2
      do j= plats(n),plate(n)
      do i= plons,plone
       oioro(i,j)=ioro(i,j)
c if not land
       if(uwmask(i,j) .ne. 1) then

c if sst is below freezing
         if((sst(i,j) .le. tsice) .or. 
     $	    (meltp(i,j) .gt. 0.)) then
c if not already sea ice        
           if(ioro(i,j) .ne. 2) then
c   then declare this point to be sea ice.
c   and set the tssub fields.
            ioro(i,j) = 2
            oro(i,j) = 2.
            thickn(i,j) = 2.
	    fice(i,j)=1.
            ts(i,j) = sst(i,j)
            tssub(i,j,1) = sst(i,j)
            tssub(i,j,2) = sst(i,j)
            tssub(i,j,3) = sst(i,j)
            tssub(i,j,4) = sst(i,j)
           endif
         else
c if above freezing then set to ice free
c and reset tssub fields to be sst.
           ioro(i,j) = 0
           oro(i,j) = 0.
           thickn(i,j) = 0.
	   fice(i,j)=0.
           ts(i,j) = sst(i,j)
           tssub(i,j,1) = sst(i,j)
           tssub(i,j,2) = sst(i,j)
           tssub(i,j,3) = sst(i,j)
           tssub(i,j,4) = sst(i,j)
         endif
       endif

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  try to account for formation/destruction 
C  of sea ice on water/snow on ice.
C  Dump any water or snowmelt
C  which has accumulated on ice into the ocean.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

       if(uwmask(i,j) .eq. 0) then
c see if ice melted
        if((oioro(i,j) - ioro(i,j)) .eq. 2) then
          oqflx(i,j) = oqflx(i,j) + (sndpth(i,j) +
     $    ws(i,j))*rhoh2o/odelt
          sndpth(i,j) = 0.
          ws(i,j) = 0.
        endif
       endif

      enddo
      enddo
      enddo

C  don't check ice on next timestep. 
      checkice = .FALSE.

      endif


C  See if theres any sea ice points here to do
      npts = 0
      do n=1,2
      do j=plats(n),plate(n)
       do i=plons,plone
         if(ioro(i,j) .eq. 2) then
          npts =npts +1
         endif
       enddo
      enddo
      enddo
      if(npts .eq. 0) return


      do n=1,2
      do j=plats(n),plate(n)

       npts = 0
       do i=plons,plone
         if(ioro(i,j) .eq. 2) then
          npts =npts +1
          indx(npts)=i
         endif
       enddo


      do ii=1,npts
          i=indx(ii)
c  Update surface type
          isrftyp(i) = 2

C set the roughness length everywhere
          z0(i) = zzsice

       enddo

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Hydrology Model:  Part 1 of 3
C----------------------------------------
C  distribute precip onto either soil moisture or
C  snow depth.  Calculate wetness factor for
C  latent heat flux calculation
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      do ii=1,npts
          i=indx(ii)
c
c  update surface wetness/snowdepth with precip fields
c

           if (.not.snowing(i,j)) then
               ws(i,j) = ws(i,j) + oprec(i,j) * dtime
c
c  if below freezing, add precip to snow depth (liquid water equiv)
c
           else if (snowing(i,j)) then
               sndpth(i,j) = sndpth(i,j) + oprec(i,j) * dtime
           endif

c
c compute runoff from rain if bucket has overflowed
c
         if(ws(i,j) .gt. wssat) then
           exws(i,j)= ws(i,j) - wssat
           ws(i,j) = wssat
         endif


c modify latent heat for snow covered, sea ice or land ice surfaces

           ltheat(i,j) = latvap + latice

c  Update surface wetness fraction
c  

           wet(i) = 1.0

      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End of hydrology model part one
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC



C Calculate fluxes of heat,water, and momentum between
C sea ice and atmosphere.

#ifdef PROFILE
      call mark_event(22,1)
#endif 

      call posrfflx(indx,npts,j,z0,psm1,pmidm1,ubot,
     $       vbot, tbot, qbot, thbot, zbot,
     $       srfrad, lwuphf(1,j),ts(1,j),ltheat(1,j), wet, 
     $       fnt,dfntdt, shf, dshf, lhf, dlhf,
     $       taux, tauy)


#ifdef PROFILE
      call mark_event(22,0)
#endif



c
c  contruct the fnt dfntdt fields for the ocean resolution
c
      do ii=1,npts
          i=indx(ii)
          ofnt(i) = 0.
          odfntdt(i) = 0.

          do k=1,overlaps(j,i)

            ofnt(i) = ofnt(i) + fnt(i,j,k)*cweight(j,i,k)
   
            odfntdt(i) = odfntdt(i) + dfntdt(i,j,k)*cweight(j,i,k)
          enddo
          ofnt(i) = ofnt(i)/uwarea(j)
          odfntdt(i) = odfntdt(i)/uwarea(j)
      enddo


c
c use diffusion equation in sub-surface along with
c backward implicit scheme to forecast surface temp
c
      do ii=1,npts
          i=indx(ii)
           tsin(i) = ts(i,j)
        do k=1,plevmx
           tsbsf(i,k) = tssub(i,j,k)
        enddo
       enddo

#ifdef PROFILE
      call mark_event(21,1)
#endif 

      call posrftsb(indx,npts,isrftyp,ofnt,odfntdt,
     $     sndpth(1,j),
     $     sst(1,j),tsbsf,alpha,beta,
     $     gamma,tpthck,fbt(1,j))


#ifdef PROFILE
      call mark_event(21,0)
#endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Hydrology Model:  Part 2 of 3
C----------------------------------------
C  adjust predicted suface temp down if it can melt snow
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c
c Melt snow with the warm ground and cool off the ground
c
c if snow cover present and forecasted temp is above
c melting temp, then energy available for snow melt
c

      do ii=1,npts
          i=indx(ii)
        if(sndpth(i,j) .gt. 0. .and. (tsbsf(i,1) .gt. tmelt)) then
         emelt = -gamma(i) + beta(i) -(alpha(i)*tmelt)
         snwmlt(i,j) = (emelt*dtime*tpthck(i))/(latice*rhoice)
         eexc(i) = emelt - sndpth(i,j)*latice*rhoice/
     $             (tpthck(i)*dtime)
         tsbsf(i,1) = tmelt
        endif
      enddo

c
c Don't melt more snow than available
c
      do ii=1,npts
          i=indx(ii)
       if(snwmlt(i,j) .gt. sndpth(i,j)) then
         snwmlt(i,j) = sndpth(i,j)
         tsbsf(i,1) = tmelt + eexc(i)/alpha(i)
       endif
      enddo

c  update snow cover and surface moisture with snow melt
c     
      do ii=1,npts
          i=indx(ii)

          sndpth(i,j) = sndpth(i,j) - snwmlt(i,j)
          ws(i,j) = ws(i,j) + snwmlt(i,j)

c  this is the last ws mod before evap. so save it.
          oldmst(i) = ws(i,j) + sndpth(i,j)
      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End of hydrology model part 2 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c
c  modification to update surface temperatures:
c if sea ice and
c if temps of any layer is over melting point,
c reset temperature to melting point.

c  FOAM1.0: Don't do this for sea ice because this simple sea
c  ice model allows temps to go above freezing so that
c  heat can move by conduction into the ocean and
c  warm it during melting season.  -rlj
c     do ii=1,npts
c         i=indx(ii)
c       if(isrftyp(i) .eq. 3) then
c        do k=1,plevmx
c          tsbsf(i,k) = amin1(tsbsf(i,k),tmelt)
c        enddo
c       endif
c     enddo


c
c  update surface temps over ice surfaces 
c    
      do ii=1,npts
          i=indx(ii)
          ts(i,j) = tsbsf(i,1)
#ifdef SST_FROM_FILE
          if(ts(i,j) .gt. tmelt) ts(i,j)=tmelt
#endif
      enddo

c
c update subsurface temps
c
      do k=1,plevmx
      do ii=1,npts
          i=indx(ii)
          tssub(i,j,k) = tsbsf(i,k)
#ifdef SST_FROM_FILE
          if(tssub(i,j,k) .gt. tmelt) tssub(i,j,k)=tmelt
#endif
      enddo
      enddo

c set CSMICE variables
      do ii=1,npts
	tice(i,j,1)=0.
	tice(i,j,2)=0.
	tinter(i,j)=0.
	tsnow(i,j)=0.
      enddo


c
c  find delts over sea ice
      do ii=1,npts
          i=indx(ii)
         delts(i,j) = ts(i,j) - tsin(i)
      enddo

c
c Define latent and sensible heat flux on the overlap grid
c
      do ii=1,npts
          i=indx(ii)
       do k=1,overlaps(j,i)
          lhflx(i,j,k) = lhf(i,j,k) + dlhf(i,j,k)*delts(i,j)
          qflx(i,j,k) = lhflx(i,j,k)/ltheat(i,j)
          shflx(i,j,k) = shf(i,j,k) + dshf(i,j,k)*delts(i,j)
       enddo

      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  Hydrology Model:  Part 3 of 3
C----------------------------------------
C  evaporate water and/or sublimate snow
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

c
c evaporate water from sea ice surfaces
c
      do ii=1,npts
          i=indx(ii)

c first find the total moisture flux in this grid cell
c qflx is in kg/m^2 s
         tmp = 0.
         mflux = 0.
         do k=1,overlaps(j,i)
           tmp = tmp + qflx(i,j,k)*cweight(j,i,k)
         enddo
c     now find the total moisture evaporated in meters
crlj There is a factor of rhoh2o missing from these qflx caculations.
crlj  Otherwise the units don't work.
         mflux = tmp*dtime/(uwarea(j)*rhoh2o)

          if(sndpth(i,j) .gt. 0.) then
c first evaporate the snow.
            sndpth(i,j) = sndpth(i,j) - mflux
c if evaporation more than available snow, reduce surface moisture
            if(sndpth(i,j) .lt. 0.) then
              ws(i,j) = ws(i,j) + sndpth(i,j)
              sndpth(i,j) = 0.0
            endif

          else
c evaporate the surface moisture
            ws(i,j) = ws(i,j) - mflux
          endif

c if more evaporation than available surface moisture, set surface
c moisture to zero and reduce evaporative loss accordingly.
c maximum possible evap is a function of the total ws

c NOTE: for sea ice, there's always enough moisture so don't
c  reset qflx.

c         if(ws(i,j) .lt. 0.0) then
c            ws(i,j) = 0.0
c 
c            do k=1,overlaps(j,i)
c              qflx(i,j,k) = oldmst(i)*rhoh2o/dtime
c        change lhflx to reflect changes in qflx
c              lhflx(i,j,k) = qflx(i,j,k)*ltheat(i,j)

c            enddo
c         endif
      enddo

c
c  take away any moisture over the bucket depth
c  or snow over the snow maximum.  The excess will
c  be dumped into the ocean beneath the ice.
c
      do ii=1,npts
          i=indx(ii)
	  snrun = 0.
	  tmp = amax1(ws(i,j)-wssat,0.)
	  ws(i,j) = ws(i,j) - tmp

 	   if (sndpth(i,j) .gt. snwmax) then
	      snrun = sndpth(i,j) - snwmax
	      sndpth(i,j) = snwmax
           endif
	   exws(i,j) = exws(i,j) + tmp + snrun

c  if ws is negative here, that means evap was more than available
c  snow or bucket moisture.  Take the remainder out of the ocean.
	   if (ws(i,j) .lt. 0.) then
	     exws(i,j) = ws(i,j)
	     ws(i,j)=0.
           endif
      enddo
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C  End of hydrology model part 3 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
c  Update albedos
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      if(doalb) then
       tlslo = tmelt -5.
       tlshi = tmelt

       do ii=1,npts
        i=indx(ii)
C
C  Set sea ice albedos
C
         if( coszrs(i,j).gt.0.0 ) then
          oalbs(i,j)  = sices
          oalbl(i,j)  = sicel
          oalbsd(i,j) = oalbs(i,j)
          oalbld(i,j) = oalbl(i,j)
         endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C Albedo adjustment for snowcover
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C
C
      if( sndpth(i,j).gt.0. .and. coszrs(i,j).gt.0. ) then

C  set visible snow albedos
        zfac = amax1(0., 1.5/(1. + 4. * coszrs(i,j)) - 0.5)

        tmp = (snwsh*(tssub(i,j,1)-tlslo) + snwsl*(tlshi -tssub(i,j,1)))
     $   / (tlshi - tlslo)
        tmp = amin1(snwsl, amax1(snwsh, tmp))
        salbs(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
        salbsd(i) = tmp

C  set near-infrared (longwave) snow albedos
        tmp = (snwlh*(tssub(i,j,1)-tlslo) + snwll*(tlshi -tssub(i,j,1)))
     $   / (tlshi - tlslo)
        tmp = amin1(snwll, amax1(snwlh, tmp))
        salbl(i) = amin1(0.99, tmp + (1. - tmp)*zfac)
        salbld(i) = tmp


      endif
C
C Compute fraction of horizonal surface covered with snow:
C
         if(sndpth(i,j) .le. 0.) then
            snwhgt    = 0.0
            rghsnw    = 0.0
            frsnow(i) = 0.0
         else
C Change snow fraction to use the correct
C snow equivalent depth factor and also
C use total roughness.  RLJ,RG
C           snwhgt    = 20. * sndpth(i,j)
C           rghsnw    = amax1(zzsice,0.25)
C           frsnow(i) = snwhgt/(rghsnw + snwhgt)

            snwhgt    = snwedp * sndpth(i,j)
	    rghsnw    = zzsice
            frsnow(i) = snwhgt/(rghsnw + snwhgt)
         end if
C
C  Now modify albedoes for snowcover
C
         if( sndpth(i,j).gt.0. .and. coszrs(i,j).gt.0. ) then
          oalbs(i,j)  = oalbs(i,j) *(1.-frsnow(i)) +
     $      salbs(i) *frsnow(i)
          oalbl(i,j)  = oalbl(i,j) *(1.-frsnow(i)) +
     $      salbl(i) *frsnow(i)
          oalbsd(i,j) = oalbsd(i,j)*(1.-frsnow(i)) +
     $      salbsd(i)*frsnow(i)
          oalbld(i,j) = oalbld(i,j)*(1.-frsnow(i)) +
     $      salbld(i)*frsnow(i)
         endif
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C End of albedo adjustment for snowcover.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

       enddo
      endif


      enddo
      enddo

      return
      end
